// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/kernel/BBlockingComplete.java,v 1.3 1999/10/01 07:52:23 tp079541 Exp $

package au.edu.swin.synergy.kernel;

import java.util.EventObject;
import au.edu.swin.synergy.debug.Debug;

/**
 * A generic completion object which allows blocking until completion occures
 * Possible extention is to only accept one completion call, or only kSuccess
 * @see BCommandEvent
 * @concurrency this class is thread safe and uses a blocking wait
 * @copyright 1999 Synergy Software
 * @rref
 * @author John Newbigin
 */


public class BBlockingComplete extends BCommandComplete
{
	private BBoolean ready;
	private BCommandEvent event = null;
	private int status = 0;

	public BBlockingComplete()
	{
		ready = new BBoolean(false);
	}

    public void doComplete( BCommandEvent fEvent, int fStatus )
	{
		Debug.doPrintln("BlockingComplete being completed");
		event = fEvent;
		status = fStatus;
		Debug.doPrintln("Blocking complete status = " + fStatus);
		synchronized(ready)
		{
			ready.setValue(true);
			ready.notify();
		}
	}

	public BCommandEvent getEvent()
	{
		return event;
	}

	public int getStatus()
	{
		return status;
	}

	public boolean waitForEvent()
	{
		synchronized(ready)
		{
			Debug.doPrintln("Waiting for the completion to be called");
			if(ready.getValue() == false)
			{
				try
				{
					ready.wait();
				}
				catch(InterruptedException e)
				{
					Debug.doPrintln("Wait interrupted while waiting for user : " + e);
					//???
				}
			}
		}
		return true;
	}
}

