//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/lib/BColorCombo.java,v 1.4 1999/10/07 08:58:47 tp079511 Exp $

package au.edu.swin.synergy.lib;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;

import au.edu.swin.synergy.debug.Debug;

/**
 * Combo box for choosing colors
 * 
 * @author Shane Fitzgibbon
 * @copyright Synergy Software 1999
 */

public class BColorCombo  extends JComboBox
{

	public final static Color kNoColour = new Color ( 254 );
	
	/**
	 * Constructor for the color combo
	 */
	public BColorCombo()
	{
//		super();
		this ( false );
	}

	/**
	* @param none Indicates whether a "none" option should be given in combo.
	*/
	public BColorCombo ( boolean none )
	{
		// if this is even needed...
		super ();
		addDefaultColors( none );
	}

	/**
	 * adds all the default java colors into the combo box
	 */
	private void addDefaultColors( boolean noColour )
	{
		if ( noColour )
		{
			this.addItem ( kNoColour );
		}

		this.addItem( Color.black );
		this.addItem( Color.blue );
		this.addItem( Color.cyan );
		this.addItem( Color.darkGray );
		this.addItem( Color.gray );
		this.addItem( Color.green );
		this.addItem( Color.lightGray );
		this.addItem( Color.magenta );
		this.addItem( Color.orange );
		this.addItem( Color.pink );
		this.addItem( Color.red );
		this.addItem( Color.white );
		this.addItem( Color.yellow );
		this.setRenderer( new BCellRenderer() );
		Debug.doPrintln( "Color Combo renderer is: " + this.getRenderer() );
	}

	/**
	 * Gets the color that is currently displayed in the combo box
	 *
	 * @returns the selected color
	 */
	
	public Color getSelectedColor()
	{
		return (Color) this.getSelectedItem();
	}

	/**
	 * Inner class that determines how the combo box contents are displayed
	 */
	class BCellRenderer extends JLabel implements ListCellRenderer {
		public BCellRenderer() {
		  setOpaque(true);
		}
			Color colour;
			Color outline;
     /**
	  * creates an image icon that has a rect filled with the color that
	  * is being displayed
	  * @returns a label that contains the image icon
	  */
		public Component getListCellRendererComponent(
			 JList list,
			Object value,
			int index,
			boolean isSelected,
			boolean cellHasFocus)
		{
			if ( value == kNoColour )
			{
				colour = getBackground ();
				outline = getBackground ();

				Debug.doPrintln ( "Outline = " + outline.toString () );
				Debug.doPrintln ( "Colour = " + colour.toString () );


				this.setText ( "-None-" );
				this.setHorizontalTextPosition ( ( int )JLabel.CENTER_ALIGNMENT );
			}
			else
			{
				this.setText ( "" );
				outline = Color.black;
				colour = ( Color ) value;
			}

			setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
			Image img = new BufferedImage(60, 12, BufferedImage.TYPE_INT_RGB);
			Graphics graphics = img.getGraphics();
			graphics.setColor( outline );
			graphics.drawRect( 1, 1, 60, 12 );
			graphics.setColor( colour );
			graphics.fillRect( 1, 1, 58, 10 );
			ImageIcon icon = new ImageIcon( img );
			setIcon( icon );
			//setText(value.toString());
			//setBackground( (Color) value );
			setBackground(isSelected ? list.getSelectionBackground() : Color.lightGray);
			//setForeground(isSelected ? Color.white : Color.black);

			return this;
		}
	}

	

}


