

package au.edu.swin.synergy.lib;

import java.io.InputStream;
import java.io.IOException;

/**
 * This class will allow calls like read() and readLine()
 * on the command line input stream.
 *
 * @author Tristan Austin
 */
public class BInputReader
{
    /**
     * The input stream to read from
     */
    private InputStream in;

    /**
     * Creates a new BInputRead to read from the
     * stream given.
     *
     * @in  The input stream to read from
     */
    public BInputReader(InputStream in)
    {
        this.in = in;
    }

    /**
     * This will read until it finds a carriage return signfied
     * by a '\n' or the end of the input stream is reached
     * at which point an IOException will be thrown.
     * <p>
     * '\r' characters will be ignored.
     *
     * @return The line read
     * @throws IOException if the end of the input stream is reached
     */
    public String readLine() throws IOException
    {
        StringBuffer line = new StringBuffer();
        int lineNumber = 0;
        int character;

        //Keep looping indefinitely
        while(true)
        {
            //Read the next character
            character = this.read();

            //If we get a new line, exit the loop
            if(character == '\n')
            {
                break;
            }
            else if(character == '\r')
            {
                //If we get an '\r' ignore it
            }
            else if(character == -1)
            {//If we get a -1 it's the end of the input stream
                //throw new IOException("End of stream");
				return null;
            }
            else
            {//If we get anything else, append it to the line
                line.append((char)character);
            }
        }

        //Return the line read
        return line.toString();
    }

    /**
     * Reads the next character in the input stream and
     * returns it as an int.
     *
     * @return The next character as an int
     * @throws IOException when an io error occurs
     */
    public int read() throws IOException
    {
        return in.read();
    }
}
