//$

package au.edu.swin.synergy.lib;

import javax.swing.*;
import javax.swing.text.*;

/**
 * Extends JTextArea and overrides the isManagingFocus() method so that
 * it returns false.  This means that Tab key events will result in the focus
 * transferred, instead of inserting a tab into the text area.
 *
 * @author Matthew Evans
 */
public class BTextArea extends JTextArea
{
  /**
  * Constructs a new TextArea, with default settings (default model, null string,
  * rows/columns = 0)
  */
  public BTextArea()
  {
    super();
  }

  /**
   * Constructs a new TextArea with the specified text displayed.
   * A default model is created and rows/columns are set to 0.
   *
   * @param text the text to be displayed, or null
   */
  public BTextArea(String text)
  {
    super(text);
  }

  /**
   * Constructs a new empty TextArea with the specified number of
   * rows and columns.  A default model is created, and the initial
   * string is null.
   *
   * @param rows the number of rows >= 0
   * @param columns the number of columns >= 0
   */
  public BTextArea(int rows, int columns)
  {
    super(rows, columns);
  }

  /**
   * Constructs a new TextArea with the specified text and number
   * of rows and columns.  A default model is created.
   *
   * @param text the text to be displayed, or null
   * @param rows the number of rows >= 0
   * @param columns the number of columns >= 0
   */
  public BTextArea(String text, int rows, int columns)
  {
    super (text, rows, columns);
  }

  /**
   * Constructs a new JTextArea with the given document model, and defaults
   * for all of the other arguments (null, 0, 0).
   *
   * @param doc  the model to use
   */
  public BTextArea(Document doc)
  {
    super (doc);
  }

  /**
   * Constructs a new JTextArea with the specified number of rows
   * and columns, and the given model.  All of the constructors
   * feed through this constructor.
   *
   * @param doc the model to use, or create a default one if null
   * @param text the text to be displayed, null if none
   * @param rows the number of rows >= 0
   * @param columns the number of columns >= 0
   */
  public BTextArea(Document doc, String text, int rows, int columns)
  {
    super (doc, text, rows, columns);
  }



  /**
   * Overrides isManagingFocus() in JTextArea, so that it returns true instead
   * of true.
   *
   * @return false, to indicate that this implementation of a text area does not
   * manage the focus, and therefore, tab events will not be consumed in 
   * BTextArea.
   */
  public boolean isManagingFocus()
  {
    return false;
  }
}