//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/lib/BTreeModel.java,v 1.10 1999/10/08 05:20:38 tp082177 Exp $

package au.edu.swin.synergy.lib;

import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import java.util.Vector;

import au.edu.swin.synergy.lib.BTreeNode;
import au.edu.swin.synergy.debug.Debug;


/**
 * A reusable tree model that can contain any implementaion of a TreeNode
 *
 * @author Matthew Evans
 */
public class BTreeModel implements BTreeModelInterface, BTreeNode
{
  /**
   * This is the list of listeners that need to be notified
   * of the changes to the tree
   */
  private Vector treeListeners = new Vector();

  /**
   * The collection of branches in the tree
   */
  protected Vector branches = new Vector();

  /**
   * This is the name of the item being represented by this model
   */
  private String name = null;

  /**
   * Creates a new tree model, and gives it a name
   *
   * @param name the name of the model
   */
  public BTreeModel(String name)
  {
    this.name = name;
  }

  /**
   * Sets the name of the tree model to the title specified. This will be reflected
   * on the screen immediately
   *
   * @param   newPackageName  The name of the package
   */
    public void setName(String newUserName)
    {
       this.name = newUserName;

       Debug.doPrintln("BTreeModel: Setting name of tree", Debug.kWarningLevel);
       //this.fireStructureChanged();
    }

    public String getName()
    {
       return this.name;
    }

    public void addBranch (BTreeNode branch)
    {
       branches.addElement(branch);
	   nodeWasInserted( this, branches.size() - 1, branch );
    }

    public void removeBranch (BTreeNode branch)
    {
	  int index = branches.indexOf(branch);	

      branches.removeElement(branch);
  	  nodeWasRemoved(this, index, branch);
    }

    public Vector getBranches()
    {
       return this.branches;
    }

	/* public void addNode(BTreeNode parent, BTreeNode child)
	 {
	 	 parent.add(child);
		 //slack way; change later
		 this.fireStructureChanged();
	 }

	 public void removeNode(BTreeNode parent, BTreeNode child)
	 {
		 parent.remove(child);
		 //slack way; change later
		 this.fireStructureChanged();
	 }*/
	 
      //////////////Tree Model stuff
      public void addTreeModelListener(TreeModelListener l)
      {
         this.treeListeners.add( l );
      }

      public Object getChild(Object parent, int index)
      {
         BTreeNode branch = (BTreeNode)parent;
         return branch.getChildAt( index );
      }

      public int getChildCount(Object parent)
      {
         BTreeNode branch = (BTreeNode)parent;
         return branch.getChildCount();
      }

      public int getIndexOfChild(Object parent, Object child)
      {
         BTreeNode branch = (BTreeNode)parent;
         return branch.getIndexOf( child );
      }

      public Object getRoot()
      {
         return this;
      }

      public boolean isLeaf(Object node)
      {
         BTreeNode branch = (BTreeNode)node;
         return branch.isLeaf();
      }

      public void removeTreeModelListener(TreeModelListener listener)
      {
         this.treeListeners.remove( listener );
      }

      public void valueForPathChanged(TreePath path, Object newValue)
      {
         Debug.doPrintln( "Value for Path Changed", Debug.kWarningLevel );
      }

	  public void nodeWasInserted( BTreeNode node, int index, BTreeNode added )
	  {
		Object [] path = BTreeUtil.getPathToRoot( node, 0 );
		
		Object [] children = new Object[1];
		int [] indices = new int[1];
		children[0] = added;
		indices[0] = index;
		TreeModelEvent me = new TreeModelEvent( this, path, indices, children );
		BTreeUtil.fireNodesInserted( node, me );
	  }

	  public void nodeWasRemoved( BTreeNode node, int index, BTreeNode removed )
	  {
		Object [] path = BTreeUtil.getPathToRoot( node, 0 );
		Object [] children = new Object[1];
		int [] indices = new int[1];
		children[0] = removed;
		indices[0] = index;
		TreeModelEvent me = new TreeModelEvent( this, path, indices, children );
		BTreeUtil.fireNodesRemoved( node, me );		
	  }

      protected void fireStructureChanged()
      {
			Debug.doPrintln("fsc1");
         TreePath treePath = new TreePath( this );
         TreeModelListener listener = null;
	      Debug.doPrintln("fsc2");

         TreeModelEvent me = new TreeModelEvent( this, treePath.getPath() );

	      Debug.doPrintln("fsc3");

         for ( int i = 0; i < this.treeListeners.size(); i++ )
         {
				Debug.doPrintln("fsc4");
            listener = (TreeModelListener)this.treeListeners.elementAt( i );
            listener.treeStructureChanged( me );
         }

		   Debug.doPrintln("fsc5");
      }

		protected void fireNodesInserted()
      {
			Debug.doPrintln("fsc1");
         TreePath treePath = new TreePath( this );
         TreeModelListener listener = null;
	      Debug.doPrintln("fsc2");

         TreeModelEvent me = new TreeModelEvent( this, treePath.getPath() );

	      Debug.doPrintln("fsc3");

         for ( int i = 0; i < this.treeListeners.size(); i++ )
         {
				Debug.doPrintln("fsc4");
            listener = (TreeModelListener)this.treeListeners.elementAt( i );
            listener.treeNodesInserted( me );
         }

		   Debug.doPrintln("fsc5");
      }

		protected void fireNodesRemoved()
      {
			Debug.doPrintln("fsc1");
         TreePath treePath = new TreePath( this );
         TreeModelListener listener = null;
	      Debug.doPrintln("fsc2");

         TreeModelEvent me = new TreeModelEvent( this, treePath.getPath() );

	      Debug.doPrintln("fsc3");

         for ( int i = 0; i < this.treeListeners.size(); i++ )
         {
				Debug.doPrintln("fsc4");
            listener = (TreeModelListener)this.treeListeners.elementAt( i );
            listener.treeNodesRemoved( me );
         }

		   Debug.doPrintln("fsc5");
      }

	  public void fireInserted( TreeModelEvent e )
	  {
		TreeModelListener l = null;
		int size, i;
		size = this.treeListeners.size();
		for ( i = 0; i < size; i++ )
		{
			l = (TreeModelListener)this.treeListeners.elementAt(i);
			l.treeNodesInserted( e );
		}
	  }

	  public void fireRemoved (TreeModelEvent e)
	  {
		TreeModelListener l = null;
		int size, i;
		size = this.treeListeners.size();
		for ( i = 0; i < size; i++ )
		{
			l = (TreeModelListener)this.treeListeners.elementAt(i);
			l.treeNodesRemoved( e );
		}
	  }
	  
      /**
       * Returns the name of the workspace, so it can be displayed on the tree for
       * the root node
       *
       * @return the name of the workspace
       */
      public String toString()
      {
        return this.name;
      }


   ///////////BTreeNodeStuff
		public boolean add(BTreeNode branch)
		{
      	 branches.addElement(branch);
			 branch.setParent(this);
	       this.fireStructureChanged();
		    return true;
		}

		public boolean remove(BTreeNode branch)
		{
	      branches.removeElement(branch);
   	   this.fireStructureChanged();
			return true;
		}
		
      public boolean isLeaf()
      {
         return (branches.size() <= 0);
      }

      public Object getChildAt( int index )
      {
         return branches.elementAt(index);
      }

      public int getChildCount()
      {
         return branches.size();
      }

      public Object getParent()
      {
         return null;
      }

      public int getIndexOf( Object node )
      {
         for (int i=0; i<branches.size(); i++)
         {
            if (branches.elementAt(i) == node)
            {
               return i;
            }
         }

        return -1;
      }

      public void setParent( Object parent )
      {
      }

      public void removeAllChildren()
      {
		 while (branches.size() > 0)
       {	
			this.removeBranch( (BTreeNode)branches.elementAt(0) );
		 }
      }
}
