//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/lib/io/BDataFile.java,v 1.11 1999/10/10 02:53:53 tp079519 Exp $

   package au.edu.swin.synergy.lib.io;

   import java.io.FileInputStream;
   import java.io.FileOutputStream;
   import java.io.IOException;
   import java.io.InputStream;
   import java.io.PrintStream;
   import au.edu.swin.synergy.debug.Debug;
   import java.util.Vector;


/**
 *
 * @author John Newbigin
 * @see 
 * @concurrency NA
 * @copyright Synergy Software
 * @rref
 */

   public class BDataFile
   {
   	  /**
	   * This enables a shit load of debug info which can help track down a problem
	   * in load and save code
	   */
   	  private static final boolean debug = false;

      private static final String kIndentString = "  ";
      private static final String kBeginString = "<";
	  private static final String kBeginStringEnd = ">";
      private static final String kEndString = "</";
	  private static final String kEndStringEnd = ">";
      private static final String kNullString = "null";
   
      String fileName = null;
   
      private FileInputStream inputStream = null;
      private FileOutputStream outputStream = null;
   
      private PrintStream printStream = null;
   
      private boolean eof;
      private int indentLevel = 0;
	  private int lineNo = 0;
   
      private String peekedLine = null;
      private boolean levelRead = false;
   
      private Vector dataElements = new Vector();

	  // Static stuff for magic class name maps
	  private static Vector classMap = new Vector();

	  public static class ClassMap
	  {
	  	String className = null;
		String alias = null;
		
	  	public ClassMap(String fClassName, String fAlias)
		{
			className = fClassName;
			alias = fAlias;

			synchronized(classMap)
			{
				classMap.add(this);
			}
			Debug.doPrintln("Got a class name map " + className + " = " + alias);
		}

		private String getClassName()
		{
			return className;
		}

		private String getAlias()
		{
			return alias;
		}

		private boolean aliasEquals(String name)
		{
			return alias.equals(name);
		}
		
		private boolean classEquals(String name)
		{
			return className.equals(name);
		}
	  }

	  /**
	   * This is a generic object which you can use to pass information into the load method
	   * It is not used by the BDataFile in any way
	   */
	   private Object userObject = null;
   
      public BDataFile(String fFileName)
      {
         fileName = fFileName;
      }

	  public String getFileName()
	  {
	  	return fileName;
	  }

	  private String getRealClassName(String name)
	  {
	  	// look through the map and see if there is a class with this alias
		synchronized(classMap)
		{
			for(int i=0; i<classMap.size(); i++)
			{
				ClassMap map = (ClassMap) classMap.get(i);
				if(map.aliasEquals(name))
				{
					return map.getClassName();
				}
			}
		}
		return name;
	  }
   
	  private String getAlias(String name)
	  {
	  	// look through the map and see if there is a class with this name
		synchronized(classMap)
		{
			for(int i=0; i<classMap.size(); i++)
			{
				ClassMap map = (ClassMap) classMap.get(i);
				if(map.classEquals(name))
				{
					return map.getAlias();
				}
			}
		}
		return name;
	  }
	  
      public boolean save(BSaveable object)
      {
	  	 if(object != null)
		 {
	         addTag(kBeginString + getAlias(object.getClass().getName()) + kBeginStringEnd);
	         indentLevel++;
	         boolean result = object.save(this);
	         indentLevel--;
	         addTag(kEndString + getAlias(object.getClass().getName()) + kEndStringEnd);
	         return result;
		  }
		  else
		  {
		  	addTag(kNullString);
			return true;
		  }
      }
   
      public boolean load(BSaveable object)
      {
         indentLevel++;
         levelRead = false;
         boolean result =  object.load(this);
         indentLevel--;
      // skip to the next "end"
         while(true)
         {
            String line = getLine();
            if(line == null)
            {
            // eof...
               break;
            }
            else
            {
               line = line.trim();
            }
            if(line.startsWith(kEndString))
            {
				if(debug)
				{
	               Debug.doPrintln("Got End");
				}
               break;
            }
         }
      
      
         return result;
      }
   
      public BSaveable autoLoad()
      {
	  	 if(debug)
		 {
	         Debug.doPrintln("autoload...");
		 }
         String line = peekLine();
         if(line == null)
         {
		 	if(debug)
			{
	            Debug.doPrintln("EOF reached");
			}
            return null;
         }
         else
         {
            line = line.trim();
         }
		 if(debug)
		 {
	         Debug.doPrintln("Got line " + line);
		}
         if(line.startsWith(kBeginString) && (!line.startsWith(kEndString)))
         {
         // accept this line
            getLine();

			// remove the end of the string
			int index = line.indexOf(kBeginStringEnd);
			if(index == -1)
			{
				Debug.doPrintln("missing " + kBeginStringEnd + " on end of string");
			}
			else
			{
				line = line.substring(0, index);
			}
            try
            {
            // see what kind of object to load
               String className = getRealClassName(line.substring(kBeginString.length()).trim());
			   if(debug)
			   {
	               Debug.doPrintln("Loading a " + className);
			}
               BSaveable object = (BSaveable) Class.forName(className).newInstance();
               load(object);
			 levelRead = false;
			 return object;
            }
               catch(Exception e)
               {
                  Debug.doPrintln("error loading file " , e);
               }
         }
		 else if(line.startsWith(kNullString))
		 {
			 levelRead = false;
		 	return null;
		 }
         else
         {
/*            Debug.doPrintln("looking for a " + kBeginString);
			readDataElements();
			// we will try again, but make sure that there is a begin
			// or we will end up in an infinite loop

			if(peekLine().startsWith(kBeginString))
			{
				return autoLoad();
			}
            Debug.doPrintln("no " + kBeginString + " in this scope");*/

			if(debug)
			{
				Debug.doPrintln("no available " + kBeginString);
			}
		}
			 levelRead = false;
         return null;
      }

	  public void setUserObject(Object fObject)
	  {
	  	userObject = fObject;
	  }

	  public Object getUserObject()
	  {
	  	return userObject;
	  }
   
   
    /**
     * openFileForRead
     * @post the file is opened for reading
     * @return true if the file could be opened, otherwise false
     */
   
      public boolean openFileForRead()
      {
         try
         {
            inputStream = new FileInputStream( fileName );
            eof = false;
            return true;
         }
            catch( IOException e )
            {
				Debug.doPrintln("Error opening file for read", e);
               return false;
            }
      }
   
   
    /**
     * opens a file for writing
     * @post the file is opened for writing
     * @return true if the file could be opened
     * @param append if append is false, then any existing data will be overwritten
     */
   
      public boolean openFileForWrite( boolean append )
      {
         try
         {
            outputStream = new FileOutputStream( fileName, append );
            printStream = new PrintStream(outputStream);
            return true;
         }
            catch( IOException e )
            {
				Debug.doPrintln("Error opening file for write", e);
               return false;
            }
      }
   
   
    /**
     * close an open file
     * @post the file is closed
     * @pre the file is open
     */
   
      public void close()
      {
         try
         {
            if( inputStream != null )
            {
               inputStream.close();
               inputStream = null;
            }
            if( outputStream != null )
            {
               outputStream.close();
               outputStream = null;
            }
         }
            catch( IOException e )
            {
            // ?
            }
      }
   
   
    /**
     * get a line of text from the file
     * @post the file pointer is movel along one line
     * @pre the file is open for reading
     * @return the string, with no new line character, or if the end of the file is reached returns null
     */
   
      private String peekLine()
      {
         peekedLine = getLine();
         return peekedLine;
      }
   
      private String getLine()
      {
         if( eof )
         {
            return null;
         }
      
         if(peekedLine != null)
         {
            String result = peekedLine;
            peekedLine = null;
            return result;
         }
      
         String str = new String();
         while( true )
         {
            int letter;
            try
            {
               letter = inputStream.read();
            }
               catch( Exception e )
               {
                  letter = - 1;
               }
         
            if( letter == - 1 )
            {
               eof = true;
               break;
            }
            else if( letter == 13 )
            {
                // ignore
            }
            else if( letter == 10 )
            {
               break;
            }
            else
            {
               str = str + ( char ) letter;
            }
         
         }
		 lineNo++;
		 if(debug)
		 {
			 Debug.doPrintln("Up to line " + lineNo);
		}
         return str;
      }
   
      public void addTag(String fTag)
      {
         printStream.println(getIndent() + fTag);
      }

		/**
		 * Add a key=data to the file, but only if
		 * data != default
		 * Handling is done for null
		 */
	  public void addData(String fKey, String fData, String fDefault)
	  {
	  	if(fData == null)
		{
			if(fDefault != null)
			{
				addData(fKey, fData);
			}
		}
		else
		{
			if(fDefault == null)
			{
				addData(fKey, fData);
			}
			else
			{
				if(!fData.equals(fDefault))
				{
					addData(fKey, fData);
				}
			}
		}
	  }
   
      public void addData(String fKey, String fData)
      {
	  	if(debug)
		{
		  	Debug.doPrintln("fKey " + fKey);
		  	Debug.doPrintln("fData " + fData);
		}
	  	 if(fKey == null)
		 {
		 	Debug.doPrintln("you can't use a null key");
		 }
		 else
		 {
		 	 if(fData == null)
			 {
			 	fData = "";
			 }
	         printStream.println(getIndent() + fKey + "=" + fData);
		 }
      }
   
      private String getIndent()
      {
         String result = new String();
         for(int i=0; i<indentLevel; i++)
         {
            result = result + kIndentString;
         }
         return result;
      }
   
      public String getData(String fKey, String fDefault)
      {
	  	if(debug)
		{
		  	Debug.doPrintln("Request for key " + fKey);
		}
         if(!levelRead)
         {
            readLevel();
         }
      
      // search through dataElements for the first matching key
         for(int i=0; i<dataElements.size(); i++)
         {
            BDataElement dataElement = (BDataElement) dataElements.get(i);
            if(dataElement.getKey().equals(fKey))
            {
               String result = dataElement.getValue();
               dataElements.remove(dataElement);
			   if(debug)
			   {
				   Debug.doPrintln("found " + result);
				 }
               return result;
            }
         }
      // key not found, return default
	  if(debug)
	  {
	  	Debug.doPrintln("Not found, using default");
	 }	
         return fDefault;
      }
   
      private void readLevel()
      {
         dataElements.clear();

		 readDataElements();
	  }

	  private int readDataElements()
	  {
	  	int count = 0;
      
         while(true)
     
	  {
         
         // read all the key=data pairs until the next begin -or- end
            String line = peekLine();
            if(line == null)
            {
            // eof
               levelRead = true;
               break;
            }
            else
            {
               line = line.trim();
            }
            if(line.startsWith(kBeginString))
            {
               levelRead = true;
               break;
            }
            else if(line.startsWith(kEndString))
            {
               levelRead = true;
               break;
            }
            else if(line.startsWith(kNullString))
            {
               levelRead = true;
               break;
            }
            else
            {
				count++;
            // accept the line
               getLine(); 
            
               BDataElement dataElement = new BDataElement();
               dataElement.setString(line);
            
            // add this pair to the list...
               dataElements.add(dataElement);
            }
         }
		 return count;
      }
   }
