//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/lib/io/BTextdbRecord.java,v 1.6 1999/08/28 10:20:18 tp079519 Exp $

package au.edu.swin.synergy.lib.io;

import java.util.Vector;
import java.lang.String;

/**
 * This class maintains a list of strings which equate to fields in a database record
 * These fields can then be written to a single string which can be stored in a file
 * see BTextdb
 *
 * The format is
 * field 0:field 1:"field with a : in it":etc...
 * see sample test class for details
 *
 * @author John Newbigin
 * @see BTextdb
 * @concurrency na
 * @copyright Synergy Software
 * @rref
 */

public class BTextdbRecord
{
    private String record = null;
    private Vector elements = new Vector();
    private boolean updated = false;
	
    /**
     * Get an element of the record
     * @pre the record data has been set
     * @return a string, containg the data in the requested field
     * @param index a 0 based index into this record
     */
    
    public String getElement( int index )
    {
        String result = ( String ) elements.get( index );
        if( result == null )
        {
            result = "";
        }
        return result;
    }

	
    /**
     * Set the specified element to the specified string
     * @post the element has been updated
     * @pre the record data has been set
     * @param index a 0 based index into this record
	 * @param data the string to set the element to
     */
    
    public void setElement( int index, String data )
    {
        elements.set( index, data );
        updated = false;
    }

	
    /**
     * Return the number of elements in this record
     * @pre the record data has been set
     * @return the number of elements
     */
    
    public int getCount()
    {
        return elements.size();
    }
	
    /**
     * Get the record as a string suitable for writing to file
     * @pre the record data has been set
     * @return a string representation of the record
     */
    
    public String getRecord()
    {
        //        if(!updated)
        {
            buildRecord();
        }
        return record;
    }
	
    /**
     * Sets the data for the record
     * @post the record data is set
     * @param data a string of data read from the database file
     */
    
    public void setRecord( String data )
    {
        record = data;
        parseRecord();
        updated = true;
    }
	
    /**
     * Add a new element to the record
     * @post the new element is added to the end of the element list
     * @param data a string of data to add
     */
    
    public void addElement( String data )
    {
        if( data == null )
        {
            elements.add( "" );
        }
        else
        {
            elements.add( data );
        }
        updated = false;
    }

	
    /**
     * buildRecord creats a record from a list of elements
     */
    
    private void buildRecord()
    {
        record = new String();
        
        for( int i = 0; i < getCount(); i++ )
        {
            String toAdd = new String( getElement( i ) );
            
            if( ( toAdd.indexOf( ':' ) != - 1 ) || ( toAdd.indexOf( '\\' ) != - 1 )
            || ( toAdd.indexOf( '"' ) != - 1 ) || ( toAdd.indexOf( '\n' ) != - 1 ) )
            {
                // escape a few things here
                // " -> \"
                // \ -> \\
                // #10 -> \n
                String escaped = new String();
                for( int j = 0; j < toAdd.length(); j++ )
                {
                    char c = toAdd.charAt( j );
                    if( c == '"' )
                    {
                        escaped = escaped + "\"";
                    }
                    else if( c == '\\' )
                    {
                        escaped = escaped + "\\\\";
                    }
                    else if( c == '\n' )
                    {
                        escaped = escaped + "\\n";
                    }
                    else
                    {
                        escaped = escaped + c;
                    }
                    
                }
                toAdd = "\"" + escaped + "\"";
            }
            
            if( i > 0 )
            {
                record = record + ':';
            }
            record = record + toAdd;
        }
    }
	
    /**
     * Parse a record string and extract the elements
     */
    
    private void parseRecord()
    {
        int state = 0;
        
        elements.clear();
        
        String str = null;
        
        int i = 0;
        
        while( i < record.length() )
        {
            char letter = record.charAt( i );
            
            switch( state )
            {
                case 0 :
                str = new String();
                if( letter == ':' )
                {
                    state = 2;
                }
                else if( letter == '"' )
                {
                    i++;
                    // accept the "
                    state = 3;
                }
                else
                {
                    state = 1;
                }
                break;
                
                case 1 :
                if( letter == ':' )
                {
                    state = 2;
                }
                else
                {
                    str = str + letter;
                    i++;
                }
                break;
                
                case 2 :
                i++;
                // accept the :
                
                // we need to un-escape the string?
                addElement( str );
                state = 0;
                // go back to start
                break;
                
                case 3 :
                if( letter == '\\' )
                {
                    i++;
                    state = 4;
                }
                else if( letter == '"' )
                {
                    i++;
                    state = 1;
                }
                else
                {
                    str = str + letter;
                    i++;
                }
                break;
                
                case 4 :
                if( letter == '\\' )
                {
                    str = str + letter;
                    i++;
                }
                else if( letter == '"' )
                {
                    str = str + letter;
                    i++;
                }
                else if( letter == 'n' )
                {
                    str = str + '\n';
                    i++;
                }
                state = 3;
                break;
            }
        }
        if( str != null )
        {
            addElement( str );
        }
    }
}
;

/*
class test
{
    public static void main(String[] args)
    {
        BTextdbRecord record = new BTextdbRecord();

        record.setRecord("This:is:\"asdf:asdf\":a:test");

        System.out.println("Results:");
        for(int i=0; i<record.getCount(); i++)
        {
            System.out.println(record.getElement(i));
        }
        System.out.println(record.getRecord());
    }
}
*/
