//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/math/BScalar.java,v 1.10 1999/10/15 06:56:31 tp079479 Exp $

package au.edu.swin.synergy.math;

import java.io.Serializable;


/**
 * Class for a generic scalar type..
 * Haven't decided if it should be floating point
 * or fixed point int's yet, so by making it a
 * class i should be able to change it later without
 * having to change everything that uses it.
 *
 * Currently uses a 16:16 fixed point format.
 * so scalar values should never exceed -32768..32767
 *
 * This class doesn't have any debug/log messages printed
 * because it needs to be fast.  It's just provides wrapper
 * methods that mean I don't have to put in shifts and mults
 * in the actual program code.
 *
 * @author Imron Alston
 * @date 23/8/99
 */
public class BScalar implements Serializable
{
	/** value to convert from float to fixed **/
	public static final float kFloatToFixed = 65536.0f;

	/** shift amount for the fixed point format **/
	public static final int kShiftAmount = 16;

	/** the value of one **/
	private static final int kOne = 65536;

	/** the value of half **/
	static final int kHalf = (kOne >> 1);


	/** the current value of this scalar**/
	int value = 0;

	public BScalar( BScalar value )
	{
		this.value = value.value;
	}

	public BScalar( int value )
	{
		this.value = value << BScalar.kShiftAmount;
	}

	public BScalar( float value )
	{
		this.value = (int)( value * BScalar.kFloatToFixed );
	}

	protected BScalar()
	{
		this.value = 0;
	}


	/** sets the value of the Scalar from a float **/
	public final void setValue( float newValue )
	{
		this.value = (int) (newValue * BScalar.kFloatToFixed );
	}

	/** sets the value of the Scalar from an int **/
	public final void setValue( int newValue )
	{
		this.value = newValue << BScalar.kShiftAmount;
	}

	/** sets the value of the Scalar from a Scalar **/
	public final void setValue( BScalar newValue )
	{
		this.value = newValue.value;
	}

	/** sets the value to the x value of the rect **/
	public final void setValueX( BScalarRect rect )
	{
		this.value = rect.rect.x;
	}

	/** sets the value to the y value of the rect **/
	public final void setValueY( BScalarRect rect )
	{
		this.value = rect.rect.y;
	}

	/** sets the value to the x value of the rect **/
	public final void setValueW( BScalarRect rect )
	{
		this.value = rect.rect.width;
	}

	/** sets the value to the y value of the rect **/
	public final void setValueH( BScalarRect rect )
	{
		this.value = rect.rect.height;
	}

	/** sets the actual value of the Scalar, without shifting etc **/
	public final void setActualValue( int newValue )
	{
		this.value = newValue;
	}

	/** gets the integer (truncated) value of this scalar type **/
	public final int getIntValue()
	{
		return (this.value )>> BScalar.kShiftAmount;
	}

	/** gets the floating point value of this scalar type **/
	public final float getFloatValue()
	{
		return (float)this.value / BScalar.kFloatToFixed;
	}

	/** just get the value of the scalar (this is truncated if an int) **/
	public final int getValue()
	{
		return (this.value ) >> BScalar.kShiftAmount;
	}

	/** gets the actual value of the scalar **/
	public final int getActualValue()
	{
		return this.value;
	}

	/** adds another BScalar to this BScalar **/
	public final void add( BScalar amount )
	{
		this.value += amount.value;
	}

	/** adds an int to this BScalar **/
	public final void add( int amount )
	{
		this.value += amount << BScalar.kShiftAmount;
	}

	public static final int add( BScalar op1, BScalar op2 )
	{
		return (op1.value + op2.value ) >> BScalar.kShiftAmount;
	}

	public final void addX( BScalarRect rect )
	{
		this.value += rect.rect.x;
	}

	public final void addY( BScalarRect rect )
	{
		this.value += rect.rect.y;
	}

	public final void addW( BScalarRect rect )
	{
		this.value += rect.rect.width;
	}

	public final void addH( BScalarRect rect )
	{
		this.value += rect.rect.height;
	}


	/** subs a BScalar from this BScalar **/
	public final void sub( BScalar amount )
	{
		this.value -= amount.value;
	}

	/** subs an int from this BScalar **/
	public final void sub( int amount )
	{
		this.value -= amount << BScalar.kShiftAmount;
	}

	/** multiplies the BScalar by an int **/
	public final void mult( int amount )
	{
		this.value = this.value * amount;
	}


	/** multiplies a BScalar by an int and adds the result to value**/
	public final void addMult( BScalar op1, int op2 )
	{
		this.value += op1.value * op2;
	}

	/** multiplies the BScalar by a float **/
	public final void mult( float amount )
	{
		this.value = this.value * (int)(amount * BScalar.kFloatToFixed );
	}

	/** multiplies two BScalar types **/
	public final void mult( BScalar amount )
	{
		this.value = (int)(((long)this.value * (long)amount.value) >> BScalar.kShiftAmount);
	}

	public final void mult( BScalar op1, BScalar op2 )
	{
		this.value = (int)(((long)op1.value * (long)op2.value) >> BScalar.kShiftAmount);
	}

	/** divides a BScalar by an int **/
	public final void div( int amount )
	{
		this.value = this.value / amount;
	}

	
	public final void divByW( BScalarRect rect )
	{
		if ( rect.rect.width != 0 )
		{
			this.value = (int)(((long)this.value << BScalar.kShiftAmount) / rect.rect.width);
		}
	}

	public final void divByH( BScalarRect rect )
	{
		if ( rect.rect.height != 0 )
		{			
			this.value = (int)(((long)this.value << BScalar.kShiftAmount) / rect.rect.height);
		}
	}

	/** divides two BScalar types **/
	public final void div( BScalar amount )
	{
		this.value = (int)(((long)this.value << BScalar.kShiftAmount) / amount.value);
	}

	/** the modulus of the scalar type **/
	public final void mod( int amount )
	{
		this.value = this.value % (amount << BScalar.kShiftAmount);
	}

	/** reverses the sign of the scalar **/
	public final void neg()
	{
		this.value = -this.value;
	}

	/** sets the value of this scalar to the reciprocal of the param **/
	public final void reciprocal( BScalar op1 )
	{
		this.value = (int)(((long)BScalar.kOne << BScalar.kShiftAmount ) / op1.value);
	}

	public final void half()
	{
		this.value >>= 1;
	}

	public final boolean lessThan( BScalar op1 )
	{
		return this.value < op1.value;
	}

	public final boolean lessThanOrEqualTo( BScalar op1 )
	{
		return this.value <= op1.value;
	}

	public final boolean greaterThan( BScalar op1 )
	{
		return this.value > op1.value;
	}

	public final boolean greaterThanOrEqualTo( BScalar op1 )
	{
		return this.value >= op1.value;
	}

	public final String toString()
	{
		return "" + this.getFloatValue();
	}

}