package au.edu.swin.synergy.math;

import java.awt.*;

import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.debug.Debug;
import java.io.Serializable;

/**
 * A point class, that uses scalars as it's
 * internal value...
 *
 * @author Imron Alston
 * @copyright team synergy 1999
 */
public class BScalarPoint implements Serializable, BSaveable
{
	// Saveable constants
	public static final String kKeyX = "x";
	protected static final String kDefaultX = "0.0";

	public static final String kKeyY = "y";
	protected static final String kDefaultY = "0.0";

	static private BDataFile.ClassMap hack = new BDataFile.ClassMap("au.edu.swin.synergy.math.BScalarPoint", "point");

	Point point;

	public BScalarPoint( int x, int y )
	{
		this.point = new Point();
		this.setLocation( x, y );
	}

	public BScalarPoint( Point point )
	{
		this.point = new Point();
		this.setLocation( point );
	}

	public BScalarPoint ( BScalarPoint sPoint )
	{
		this.point = new Point ();
		this.setLocation ( sPoint );
	}

	public BScalarPoint()
	{
		this( 0, 0 );
	}

	public final void setLocation( Point point )
	{
		this.setLocation( point.x, point.y );
	}

	public final void setLocation( int x, int y )
	{
		this.point.x = x << BScalar.kShiftAmount;
		this.point.y = y << BScalar.kShiftAmount;
	}

	public final void setLocation( float x, float y )
	{
		this.point.x = (int)(x * BScalar.kFloatToFixed);
		this.point.y = (int)(y * BScalar.kFloatToFixed);
	}

// ADDED BY PHIL
	public final void setX( float x )
	{
		this.point.x = (int)(x * BScalar.kFloatToFixed);
	}

	public final void setY( float y )
	{
		this.point.y = (int)(y * BScalar.kFloatToFixed);
	}

	public final float getXFloat ()
	{
		return (float)(this.point.x / BScalar.kFloatToFixed);
	}

	public final float getYFloat ()
	{
		return (float)(this.point.y / BScalar.kFloatToFixed);
	}
// END ADDED BY PHIL

	public final void setLocation( BScalarPoint point )
	{
		this.point.x = point.point.x;
		this.point.y = point.point.y;
	}

	public final void setLocation ( BScalarRect rect )
	{
		this.point.x = rect.rect.x;
		this.point.y = rect.rect.y;
	}

    public final void setXYtoWH ( BScalarRect rect )
    {
        this.point.x = rect.rect.width;
        this.point.y = rect.rect.height;    
    }

	public final void setX( BScalarPoint pt )
	{
		this.point.x = pt.point.x;
	}

	public final void setY( BScalarPoint pt )
	{
		this.point.y = pt.point.y;
	}

	public final void incXY( int x, int y )
	{
		this.point.x += x << BScalar.kShiftAmount;
		this.point.y += y << BScalar.kShiftAmount;
	}

	public final void incXY( Point pt )
	{
		this.point.x += pt.x << BScalar.kShiftAmount;
		this.point.y += pt.y << BScalar.kShiftAmount;
	}

	public final void decXY( int x, int y )
	{
		this.point.x -= x << BScalar.kShiftAmount;
		this.point.y -= y << BScalar.kShiftAmount;
	}

	public final void incXY( BScalar x, BScalar y )
	{
		this.point.x += x.value;
		this.point.y += y.value;
	}

	public final void incXY( BScalarPoint pt )
	{
		this.point.x += pt.point.x;
		this.point.y += pt.point.y;
	}

	public final void incXY( BScalarRect rect )
	{
		this.point.x += rect.rect.x;
		this.point.y += rect.rect.y;
	}

	public final void incX( BScalarPoint pt )
	{
		this.point.x += pt.point.x;
	}

	public final void incY( BScalarPoint pt )
	{
		this.point.y += pt.point.y;
	}

	public final void decX( BScalarPoint pt )
	{
		this.point.x -= pt.point.x;
	}

	public final void decY( BScalarPoint pt )
	{
		this.point.y -= pt.point.y;
	}

	public final void decY( int y )
	{
		this.point.y -= y << BScalar.kShiftAmount;
	}

	public final void incX( int x )
	{
		this.point.x += x << BScalar.kShiftAmount;
	}

	public final void decXY( BScalarPoint pt )
	{
		this.point.x -= pt.point.x;
		this.point.y -= pt.point.y;
	}

	public final void decXY( Point pt )
	{
		this.point.x -= pt.x << BScalar.kShiftAmount;
		this.point.y -= pt.y << BScalar.kShiftAmount;
	}

	public final void decXY( BScalarRect rect )
	{
		this.point.x -= rect.rect.x;
		this.point.y -= rect.rect.y;
	}

	public final void decWH( BScalarRect rect )
	{
		this.point.x -= rect.rect.width;
		this.point.y -= rect.rect.height;
	}

	public final void incW( BScalarRect rect )
	{
		this.point.x += rect.rect.width;
	}

	public final void divXYByWH( BScalarRect rect )
	{
		this.point.x = (int)(((long)this.point.x << BScalar.kShiftAmount) / rect.rect.width);
		this.point.y = (int)(((long)this.point.y << BScalar.kShiftAmount) / rect.rect.height);
	}

	public final void div( BScalarPoint pt )
	{
		this.point.x = (int)(((long)this.point.x << BScalar.kShiftAmount) / pt.point.x);		
		this.point.y = (int)(((long)this.point.y << BScalar.kShiftAmount) / pt.point.y);		
	}

	public final void divXByX( BScalarPoint pt )
	{
		this.point.x = (int)(((long)this.point.x << BScalar.kShiftAmount) / pt.point.x);		
	}

	public final void divYByY( BScalarPoint pt )
	{
		this.point.y = (int)(((long)this.point.y << BScalar.kShiftAmount) / pt.point.y);		
	}

	public final void scale( BScalar scale )
	{
		this.point.x = (int)(((long)this.point.x * (long)scale.value) >> BScalar.kShiftAmount);
		this.point.y = (int)(((long)this.point.y * (long)scale.value) >> BScalar.kShiftAmount);
	}

	public final void scale( BScalarPoint scale )
	{
		this.point.x = (int)(((long)this.point.x * (long)scale.point.x) >> BScalar.kShiftAmount);
		this.point.y = (int)(((long)this.point.y * (long)scale.point.y) >> BScalar.kShiftAmount);
	}

	public final void getLocation( Point point )
	{
		point.x = (this.point.x ) >> BScalar.kShiftAmount;
		point.y = (this.point.y ) >> BScalar.kShiftAmount;
	}

	public final int getX()
	{
		return (point.x ) >> BScalar.kShiftAmount;
	}

	public final int getY()
	{
		return (point.y ) >> BScalar.kShiftAmount;
	}

	public final void half()
	{
		this.point.x >>= 1;
		this.point.y >>= 1;
	}

	public final void snap( int spacing )
	{
		int tmp = spacing << BScalar.kShiftAmount;
		if ( this.point.x < 0 )
		{
			this.point.x += this.point.x % tmp;
		}
		else
		{
			this.point.x -= this.point.x % tmp;
		}

		if ( this.point.y < 0 )
		{
			this.point.y += this.point.y % tmp;
		}
		else
		{
			this.point.y -= this.point.y % tmp;
		}
	}

	public final String toString()
	{
		return "BSP: x(" + (this.point.x / BScalar.kFloatToFixed) + ") y(" + (this.point.y / BScalar.kFloatToFixed) +")";
	}

	public boolean xLessThan( int x )
	{
		x <<= BScalar.kShiftAmount;

		return this.point.x < x;
	}

	public boolean xGreaterThan( int x )
	{
		x <<= BScalar.kShiftAmount;

		return this.point.x > x;
	}

	public boolean yGreaterThan( int y )
	{
		y <<= BScalar.kShiftAmount;

		return this.point.y > y;
	}

	public void clamp( int min, int max )
	{
		min <<= BScalar.kShiftAmount;
		max <<= BScalar.kShiftAmount;

		if ( point.x < min ) point.x = min;
		if ( point.x > max ) point.x = max;
		if ( point.y < min ) point.y = min;
		if ( point.y > max ) point.y = max;
	}

	public void clamp( BScalarPoint pt )
	{
		
		if ( point.x < 0 ) point.x = 0;
		if ( point.x > pt.point.x ) point.x = pt.point.x;
		if ( point.y < 0 ) point.y = 0;
		if ( point.y > pt.point.y ) point.y = pt.point.y;
	}


	public boolean load( BDataFile file )
	{
		Debug.doPrintln("BScalarPoint load");
		float temp;
		 
		temp = new Float(file.getData( kKeyX, kDefaultX )).floatValue();
		point.x = (int)(temp * BScalar.kFloatToFixed);

		temp = new Float(file.getData( kKeyY, kDefaultY )).floatValue();
		point.y = (int)(temp * BScalar.kFloatToFixed);
		return true;
	}

	public boolean save( BDataFile file )
	{
		Debug.doPrintln("BScalarPoint save");
		float temp;

		temp = (float)this.point.x / BScalar.kFloatToFixed;
		file.addData(kKeyX, Float.toString( temp ), kDefaultX);
		temp = (float)this.point.y / BScalar.kFloatToFixed;
		file.addData(kKeyY, Float.toString( temp ), kDefaultY);
		return true;
	}

	/**
	 * Compare this point to another point
	 * @param scalarPoint the point to compare too
	 * @return true if all values are equal
	 * @return otherwise returns false
	 */
	public boolean equals(BScalarPoint scalarPoint)
	{
		return (point.x == scalarPoint.point.x) && (point.y == scalarPoint.point.y);		
	}

	/** reverses the sign of the scalar **/
	public final void neg()
	{
		this.point.x = -this.point.x;
		this.point.y = -this.point.y;
	}

	/** reverses the sign of the scalar **/
	public final void negY()
	{
		this.point.y = -this.point.y;
	}
}
