//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/math/BScalarRect.java,v 1.28 1999/10/17 00:40:51 tp079479 Exp $
package au.edu.swin.synergy.math;

import java.awt.*;

import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.debug.Debug;

import java.util.Vector;

import java.io.Serializable;

/**
 * A rectangle, that uses scalars as it's
 * internal value...
 *
 * @author Imron Alston
 * @copyright team synergy 1999
 */
public class BScalarRect implements Serializable, BSaveable
{
	static private BDataFile.ClassMap hack = new BDataFile.ClassMap("au.edu.swin.synergy.math.BScalarRect", "rect");

	public static final int kPosPos = 0;
	public static final int kPosNeg = 1;
	public static final int kNegPos = 2;
	public static final int kNegNeg = 3;

	//outcode values...
	public static final int kOutNone = 0;
	public static final int kOutTop = 1;
	public static final int kOutBottom = 2;
	public static final int kOutLeft = 4;
	public static final int kOutRight = 8;


	// Saveable constants
	protected static final String kKeyX = "x";
	protected static final String kDefaultX = "0.0";

	protected static final String kKeyY = "y";
	protected static final String kDefaultY = "0.0";

	protected static final String kKeyWidth = "width";
	protected static final String kDefaultWidth = "0.0";

	protected static final String kKeyHeight = "height";
	protected static final String kDefaultHeight = "0.0";


	Rectangle rect;

	public BScalarRect( int x, int y, int width, int height )
	{
		this.rect = new Rectangle();
		this.setBounds( x, y, width, height );
	}

	public BScalarRect( Rectangle rect )
	{
		this.rect = new Rectangle();
		this.setBounds( rect );
	}

	public BScalarRect ( BScalarRect sRect )
	{	
		this.rect = new Rectangle ();
		this.setBounds ( sRect );
	}

	public BScalarRect()
	{
		this( 0, 0, 0, 0);
	}

	public final void setBounds( Rectangle rect )
	{
		this.setBounds( rect.x, rect.y, rect.width, rect.height );
	}

	public final void setBounds( int x, int y, int width, int height )
	{
		this.rect.x = x << BScalar.kShiftAmount;
		this.rect.y = y << BScalar.kShiftAmount;
		this.rect.width = width << BScalar.kShiftAmount;
		this.rect.height = height << BScalar.kShiftAmount;
	}

	public final void setBounds( BScalarRect rect )
	{
		this.rect.x = rect.rect.x;
		this.rect.y = rect.rect.y;
		this.rect.width = rect.rect.width;
		this.rect.height = rect.rect.height;
	}

	public final void setBoundsFromPoly( Vector pt )
	{
		int minx, miny, maxx, maxy;
		int i, size;
		BScalarPoint cur = null;

		size = pt.size();

		if ( size <= 0 ) return;

		minx = maxx = ((BScalarPoint)pt.elementAt(0)).point.x;
		miny = maxy = ((BScalarPoint)pt.elementAt(0)).point.y;

		for ( i = 0; i < size; i++ )
		{
			cur = (BScalarPoint)pt.elementAt( i );

			if ( cur.point.x < minx ) minx = cur.point.x;
			if ( cur.point.x > maxx ) maxx = cur.point.x;
			if ( cur.point.y < miny ) miny = cur.point.y;
			if ( cur.point.y > maxy ) maxy = cur.point.y;
		}

		this.rect.x = minx;
		this.rect.y = miny;
		this.rect.width = maxx - minx;
		this.rect.height = maxy - miny;
	}

	public final void setBoundsFrom4Points( BScalarPoint p1, BScalarPoint p2, BScalarPoint p3, BScalarPoint p4 )
	{
	}

	public final void setBoundsFrom2Points( BScalarPoint p1, BScalarPoint p2 )
	{
		int x1, x2, y1, y2;

		if ( p1.point.x < p2.point.x )
		{
			x1 = p1.point.x;
			x2 = p2.point.x;
		}
		else
		{
			x1 = p2.point.x;
			x2 = p1.point.x;
		}

		if ( p1.point.y < p2.point.y )
		{
			y1 = p1.point.y;
			y2 = p2.point.y;
		}
		else
		{
			y1 = p2.point.y;
			y2 = p1.point.y;
		}


		this.rect.x = x1;
		this.rect.y = y1;
		this.rect.width = x2 - x1;
		this.rect.height = y2 - y1;
	}

	public final void setBoundsFromLine( BScalarRect rect )
	{
		int x1, x2, y1, y2;

		if ( rect.rect.x < rect.rect.width )
		{
			x1 = rect.rect.x;
			x2 = rect.rect.width;
		}
		else
		{
			x1 = rect.rect.width;
			x2 = rect.rect.x;
		}

		if ( rect.rect.y < rect.rect.height )
		{
			y1 = rect.rect.y;
			y2 = rect.rect.height;
		}
		else
		{
			y1 = rect.rect.height;
			y2 = rect.rect.y;
		}


		this.rect.x = x1;
		this.rect.y = y1;
		this.rect.width = x2 - x1;
		this.rect.height = y2 - y1;
	}

	public final void setMinimumWH( int w, int h )
	{
		w <<= BScalar.kShiftAmount;
		h <<= BScalar.kShiftAmount;

		if ( this.rect.width < w ) this.rect.width = w;
		if ( this.rect.height < h ) this.rect.height = h;
	}

	public final void setSize( int width, int height )
	{
		this.rect.width = width << BScalar.kShiftAmount;
		this.rect.height = height << BScalar.kShiftAmount;
	}

	public final boolean intersects( BScalarRect rect )
	{
		return this.rect.intersects( rect.rect );
	}

	public final boolean contains( Point pt )
	{
		int tmpX = pt.x << BScalar.kShiftAmount;
		int tmpY = pt.y << BScalar.kShiftAmount;

		return this.rect.contains( tmpX, tmpY );
	}

	public final boolean contains( BScalarPoint pt )
	{
		return this.rect.contains( pt.point.x, pt.point.y );
	}

	public final boolean contains( BScalarRect rect )
	{
		return this.rect.contains( rect.rect );
	}

	public final void setXY( BScalarPoint pt )
	{
		this.rect.x = pt.point.x;
		this.rect.y = pt.point.y;
	}

	public final void setXY( BScalarRect rect )
	{
		this.rect.x = rect.rect.x;
		this.rect.y = rect.rect.y;
	}

	public final void incXY( int x, int y )
	{
		this.rect.x += x << BScalar.kShiftAmount;
		this.rect.y += y << BScalar.kShiftAmount;
	}

	public final void decXY( int x, int y )
	{
		this.rect.x -= x << BScalar.kShiftAmount;
		this.rect.y -= y << BScalar.kShiftAmount;
	}

	public final void decWH( int x, int y )
	{
		this.rect.width -= x << BScalar.kShiftAmount;
		this.rect.height -= y << BScalar.kShiftAmount;
	}

	public final void decWH( BScalarPoint pt )
	{
		this.rect.width -= pt.point.x;
		this.rect.height -= pt.point.y;
	}

	public final void decXY( BScalarPoint pt )
	{
		this.rect.x -= pt.point.x;
		this.rect.y -= pt.point.y;
	}

	public final void decXY( BScalarRect rect )
	{
		this.rect.x -= rect.rect.x;
		this.rect.y -= rect.rect.y;
	}

	public final void incXY( BScalar x, BScalar y )
	{
		this.rect.x += x.value;
		this.rect.y += y.value;
	}

	public final void incXY( BScalarPoint pt )
	{
		this.rect.x += pt.point.x;
		this.rect.y += pt.point.y;
	}

	public final void incXY( BScalarRect rect )
	{
		this.rect.x += rect.rect.x;
		this.rect.y += rect.rect.y;
	}

	public final void incWH( int w, int h )
	{
		this.rect.width += w << BScalar.kShiftAmount;
		this.rect.height += h << BScalar.kShiftAmount;
	}

	public final void incWH( BScalarPoint pt )
	{
		this.rect.width += pt.point.x;
		this.rect.height += pt.point.y;
	}

	public final void incWDecH( BScalarPoint pt )
	{
		this.rect.width += pt.point.x;
		this.rect.height -= pt.point.y;
	}

	public final void decWIncH( BScalarPoint pt )
	{
		this.rect.width -= pt.point.x;
		this.rect.height += pt.point.y;
	}

	public final void incBounds( BScalarPoint pt )
	{
		this.rect.x += pt.point.x;
		this.rect.y += pt.point.y;
		this.rect.width += pt.point.x;
		this.rect.height += pt.point.y;
	}

	public final void decBoundsByXY( BScalarRect rect )
	{
		this.rect.x -= rect.rect.x;
		this.rect.y -= rect.rect.y;
		this.rect.width -= rect.rect.x;
		this.rect.height -= rect.rect.y;
	}

	public final void setWH( int w, int h )
	{
		this.rect.width = w << BScalar.kShiftAmount;
		this.rect.height = h << BScalar.kShiftAmount;
	}

	public final void setWH( BScalarPoint pt )
	{
		this.rect.width = pt.point.x;
		this.rect.height = pt.point.y;
	}


	public final void scale( BScalar scale )
	{
		this.rect.x = (int)(((long)this.rect.x * (long)scale.value) >> BScalar.kShiftAmount);
		this.rect.y = (int)(((long)this.rect.y * (long)scale.value) >> BScalar.kShiftAmount);
		this.rect.width = (int)(((long)this.rect.width * (long)scale.value) >> BScalar.kShiftAmount);
		this.rect.height = (int)(((long)this.rect.height * (long)scale.value) >> BScalar.kShiftAmount);
	}

	public final void getBounds( Rectangle rect )
	{
		int tempX, tempY;

		tempX = this.rect.x + this.rect.width;
		tempY = this.rect.y + this.rect.height;

		rect.x = (this.rect.x ) >> BScalar.kShiftAmount;
		rect.y = (this.rect.y ) >> BScalar.kShiftAmount;
		rect.width = (tempX >> BScalar.kShiftAmount ) - rect.x;//(this.rect.width ) >> BScalar.kShiftAmount;
		rect.height = (tempY >> BScalar.kShiftAmount) -rect.y;//(this.rect.height ) >> BScalar.kShiftAmount;
	}

	/**
	 * sets the value of this bscalarrect to the smallest sized
	 * rectangle that contains the union of both rects.
	 */
	public final void union( BScalarRect rect )
	{
		int endX, endY;

		endX = Math.max( this.rect.x + this.rect.width, rect.rect.x + rect.rect.width );
		endY = Math.max( this.rect.y + this.rect.height, rect.rect.y + rect.rect.height );

		this.rect.x = Math.min( this.rect.x, rect.rect.x );
		this.rect.y = Math.min( this.rect.y, rect.rect.y );

		this.rect.width = endX - this.rect.x;
		this.rect.height = endY - this.rect.y;
	}

	public final int getX()
	{
		return (rect.x ) >> BScalar.kShiftAmount;
	}

	public final int getY()
	{
		return (rect.y ) >> BScalar.kShiftAmount;
	}

	public final void addXYToWH()
	{
		this.rect.width += this.rect.x;
		this.rect.height += this.rect.y;
	}

	public final int getHeight()
	{
		return (rect.height )>> BScalar.kShiftAmount;
	}

	public final int getWidth()
	{
		return (rect.width ) >> BScalar.kShiftAmount;
	}

	public final String toString()
	{
		return "BSR: x(" + getX() + ") y(" + getY() +") w("+getWidth()+") h("+getHeight() + ")";
	}

	/**
	 * Returns the quadrant of the rectangle...i.e the positive-negative relation
	 * between the the pos, and the size of the rect....
	 */
	public final int getQuadrant()
	{
		if ( rect.width >= 0 && rect.height >= 0 ) return kPosPos;
		else if ( rect.width >= 0 && rect.height < 0 ) return kPosNeg;
		else if ( rect.width < 0 && rect.height >= 0 ) return kNegPos;
		else return kNegNeg;
	}


	public final int outcode( int posX, int posY )
	{
		return outcodeScalar( posX << BScalar.kShiftAmount, posY << BScalar.kShiftAmount );
	}

	public final int outcode( Point pt )
	{
		return outcodeScalar( pt.x << BScalar.kShiftAmount, pt.y << BScalar.kShiftAmount );
	}

	public final int outcode( BScalarPoint pt )
	{
		return outcodeScalar( pt.point.x, pt.point.y );
	}

	//gets the outcode of the scalar...
	//doesn't work for -ve vals of the width/height...
	protected final int outcodeScalar( int posX, int posY )
	{
		int result = kOutNone;

		//x outcode..
		if ( posX <= this.rect.x ) //less than the left plane??
		{
			result |= kOutLeft;
		}
		else if ( posX > this.rect.x + this.rect.width ) // greater than the right plane..
		{
			result |= kOutRight;
		}

		//y outcode..
		if ( posY <= this.rect.y ) //less than the top plane??
		{
			result |= kOutTop;
		}
		else if ( posY > this.rect.y + this.rect.height ) // greater than the bottom plane..
		{
			result |= kOutBottom;
		}

		return result;
	}



	/**
	 * loads the BScalarRect from a BDataFile
	 *
	 * @param file the BDataFile that contains the rect
	 * @returns true if success false if failed
	 */

	public boolean load( BDataFile file )
	{
		Debug.doPrintln("BScalarRect load");

		float temp;

		temp = new Float( file.getData( kKeyX, kDefaultX ) ).floatValue();
		rect.x = (int)(temp * BScalar.kFloatToFixed);

		temp = new Float( file.getData( kKeyY, kDefaultY ) ).floatValue();
		rect.y = (int)(temp * BScalar.kFloatToFixed);

		temp = new Float( file.getData( kKeyHeight, kDefaultHeight ) ).floatValue();
		rect.height = (int)(temp * BScalar.kFloatToFixed);

		temp = new Float( file.getData( kKeyWidth, kDefaultWidth ) ).floatValue();
		rect.width = (int)(temp * BScalar.kFloatToFixed);
		return true;
	}

	/**
	 * saves the rect to a datafile
	 *
	 * @param file the datafile to save the rect in
	 * @returns true if success, false if failure
	 */

	public boolean save( BDataFile file )
	{
		Debug.doPrintln("BScalarRect save");
		float temp;

		temp = (float)rect.x / BScalar.kFloatToFixed;
		file.addData( kKeyX, Float.toString( temp ), kDefaultX );

		temp = (float)rect.y / BScalar.kFloatToFixed;
		file.addData( kKeyY, Float.toString( temp ), kDefaultY );

		temp = (float)rect.height / BScalar.kFloatToFixed;
		file.addData( kKeyHeight, Float.toString( temp ), kDefaultHeight );

		temp = (float)rect.width / BScalar.kFloatToFixed;
		file.addData( kKeyWidth, Float.toString( temp ), kDefaultWidth );
		return true;
	}
}
