
//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/math/BVector2.java,v 1.1 1999/08/31 00:48:42 tp079479 Exp $

package au.edu.swin.synergy.math;

import au.edu.swin.synergy.math.BScalar;
import java.io.Serializable;

/**
 * A vector class for BScalars.  It contains 2 elements.
 * NOTE: this is not a Vector in the java sense, but rather
 * a vector in the computer graphics/mathematical sense.  While they are
 * similar, there are certain differences that I won't elaborate
 * on here.  Just don't think that it's like a normal java.util.Vector.
 *
 * provides methods for multiplying, dividing, scaling, adding and subtracting
 * vectors.
 *
 * The vector array is public, so no function calls are necessary to obtain
 * the values.  Index the array using the predefined constants.
 *
 * @author Imron Alston
 *
 */
public class BVector2  implements Serializable
{
	public static final int kNumElements = 2;

	public static final int kXPos = 0;
	public static final int kYPos = 1;

	public BScalar [] vect = null;

	/**
	 * Creates a new bvector2 from the scalar values given.
	 */
	public BVector2( BScalar x, BScalar y )
	{
		this.vect = new BScalar[BVector2.kNumElements];
		this.vect[ BVector2.kXPos ] = x;
		this.vect[ BVector2.kYPos ] = y;
	}

	/**
	 * Creates a new bvector2 with scalars that have the values
	 * of the ints passed in
	 */
	public BVector2( int x, int y )
	{
		this( new BScalar( x ), new BScalar( y ) );
	}

	/**
	 * Creates a new bvector2 with scalars that have the values
	 * of the floats passed in
	 */
	public BVector2( float x, float y )
	{
		this( new BScalar( x ), new BScalar( y ) );
	}

	/** sets the values of the bvector2 **/
	public void setVals( int x, int y )
	{
		this.vect[ BVector2.kXPos ].setValue(x);
		this.vect[ BVector2.kYPos ].setValue(y);
	}

	/** sets the values of the bvector2 **/
	public void setVals( float x, float y )
	{
		this.vect[ BVector2.kXPos ].setValue(x);
		this.vect[ BVector2.kYPos ].setValue(y);
	}

	/** sets the values of the bvector2 **/
	public void setVals( BScalar x, BScalar y )
	{
		this.vect[ BVector2.kXPos ].setValue(x);
		this.vect[ BVector2.kYPos ].setValue(y);
	}

	/** 
	 * gets the x scalar
	 */
	public final BScalar getXScalar()
	{
		return this.vect[ BVector2.kXPos ];
	}

	/** 
	 * gets the y scalar
	 */
	public final BScalar getYScalar()
	{
		return this.vect[ BVector2.kYPos ];
	}


	/**
	 * multiplies this vector by another vector
	 */
	public final void mult( BVector2 op2 )
	{
		//rather than looping, just unroll the loop..		
		this.vect[0].mult( op2.vect[0] );
		this.vect[1].mult( op2.vect[1] );
	}

	/**
	 * this vector = one vector multiplied by another vector
	 */
	public final void mult( BVector2 op1, BVector2 op2 )
	{
		//rather than looping, just unroll the loop..		
		this.vect[0].mult( op1.vect[0], op2.vect[0] );
		this.vect[1].mult( op1.vect[1], op2.vect[1] );
	}

	/**
	 * divs this vector by another vector
	 */
	public final void div( BVector2 op2 )
	{
		//rather than looping, just unroll the loop..		
		this.vect[0].div( op2.vect[0] );
		this.vect[1].div( op2.vect[1] );
	}

	/**
	 * scales this vector by a floating value.
	 */
	public final void scale( float scale )
	{
		//rather than looping, just unroll the loop..		
		this.vect[0].mult( scale );
		this.vect[1].mult( scale );
	}

	/**
	 * adds another vector to this vector
	 */
	public final void add( BVector2 op2 )
	{
		//rather than looping, just unroll the loop..		
		this.vect[0].add( op2.vect[0] );
		this.vect[1].add( op2.vect[1] );
	}	

	/**
	 * adds int values to the scalar
	 */
	public final void add( int x, int y )
	{
		//rather than looping, just unroll the loop..		
		this.vect[0].add( x );
		this.vect[1].add( y );
	}	

	/**
	 * adds scalar values to the scalar
	 */
	public final void add( BScalar x, BScalar y )
	{
		//rather than looping, just unroll the loop..		
		this.vect[0].add( x );
		this.vect[1].add( y );
	}	

	/**
	 * subs another vector from this vector
	 */
	public final void sub( BVector2 op2 )
	{
		//rather than looping, just unroll the loop..		
		this.vect[0].sub( op2.vect[0] );
		this.vect[1].sub( op2.vect[1] );
	}
}