//$

package au.edu.swin.synergy.plugin.addressbook;

//import au.edu.swin.synergy.lib.io.BSaveable;

import java.util.Vector;
import java.io.*;

/**
 *  BAddressBook represents an entire address book full of BAddress objects.
 *
 *  @author Matthew Evans
 */
public class BAddressBook implements Serializable
{
    /**
     *  Contains all BAddress objects for this address book.
     *
     *  @author Matthew Evans
     */
    private Vector addresses;
    
    /**
     * Initialises the 'addresses' vector.
     */
    public BAddressBook()
    {
        this.addresses = new Vector();
    }
    
    
    /**
     *  Adds a new BAddress object to the 'addresses' vector.
     */
    public synchronized boolean addEntry(BAddress newAddress)
    {
        if (addresses == null)
            return false;
        else
        {
            //first check the addressee does not already exist.
            if (addresses.contains(newAddress))//alreadyExists(newAddress.getAddressee()))
                return false;
            else
            {
                addresses.addElement(newAddress);
                return true;
            }
        }
    }
    
    /**
     *  Removes the BAddress object at entryPos from the 'addreses' vector.
     *
     *  @param entryPos The position of the BAddress object to remove in the 'addresses' vector.
     *  @return true if the entry was successfully removed from the address book; false otherwise.
     */
    public synchronized boolean removeEntry(BAddress address)
    {
        if (addresses == null)
            return false;
        else
        {
            //remove the element, it it exists.
            return addresses.removeElement(address);
        }
    }
    
    /**
     *  Modifies the BAddress object at entryPos.
     *
     *  @param entryPos The position of BAddress object to be modified in the addresses vector.
     *  @param modifiedAddress The new BAddress object, contianing the modified details for the object at entryPos.
     *  @return true if the address was modified successfully; false otherwise.
     */
    public synchronized boolean editEntry(BAddress addressToModify, BAddress modifiedAddress)
    {
        if (addresses == null)
            return false;
        else if (addresses.contains(addressToModify))
        {
            addresses.setElementAt(modifiedAddress, addresses.indexOf(addressToModify));
            return true;
        }
        else
            return false;
    }

    /**
     * This method returns the vector containing the BAddress objects for
     * this BAddressBook object.
     *
     * @return a vector of BAddress objects.
     */
    public Vector getAddresses ()
    {
        if (addresses == null)
            return new Vector();
        else
            return new Vector(addresses);
    }
 
    
    /**
     * Determines whether an addressee of name 'name' already exists.
     *
     * @param name The name to compare against all other addressee names in this address book.
     * @return true if this addressee name already exists.
     */
    private boolean alreadyExists(String name)
    {
        for (int i = 0; i < getAddresses().size(); i++)
        {
            BAddress anAddress = (BAddress)(getAddresses().elementAt(i));
                
            if (anAddress.getName().equals(name))
                return true;
        }
        
        return false;
    }
    
    /**
     * Save's this BAddressBook object to file.
     *
     * @return true if saved successfully; false otherwise.
     */
    public synchronized boolean save (String fileName)
    {
        try
        {
            FileOutputStream fos = new FileOutputStream(fileName);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            
            oos.writeObject(this);
            oos.flush();
            oos.close();
            
            return true;
        }
        catch(IOException ioStuffUp) 
        {
            return false;
        }
    }
    

}
