//
package au.edu.swin.synergy.plugin.addressbook.event;

import au.edu.swin.synergy.plugin.addressbook.BAddress;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;

import java.util.Vector;

/**
 * This implementation of BCommandEvent provides a way for the client to 
 * request address book data from the client and server address books
 *
 * @author Matthew Evans
 */
public class BAddressBookCE extends BCommandEvent
{
   /**
    * Indicates that the client wishes the address book to be provided.
    */
   public static final int kListAddressBook = 1;


	/**
    * Indicates that the client wishes to add the given address.
    */
   public static final int kAddAddress = 2;
   
   /**
    * Indicates that the client wishes to remove the
    * given address.
    */
   public static final int kRemoveAddress = 3;

   /**
    * Indicates the supplied address needs to be updated
    * or added as necessary
    */
   public static final int kUpdateAddress = 4;

   /**
    * The type of request being made
    */
   private int requestType = -1;


	/**
	 * Identifies the address book location that handled this event
	 */
	private int addressBookOrigin;

	/**
    * The address to operate on
    */
   private BAddress address = null;

   /**
    * If the requestType is kUpdateAddress, this is the address to be updated.
    */
   private BAddress modifyAddress = null;

   /**
    * The address book, if required (on kListAddressBook)
    */
   private Vector addressList = null;
   
   /**
    * Create a user request object with the given request type
    *
    * @param   requestType The type of request being made
    */
   public BAddressBookCE(Object source, int requestType, BCommandComplete complete)
   {
      super(source);
      this.requestType = requestType;
      this.setCompletionObject(complete);
   }

   /**
    * Indicates the type of request this is
    *
    * @return The request type
    */
   public int getRequest()
   {
      return this.requestType;
   }

   /**
    * Sets the address to operate on
    */
   public void setAddress(BAddress address)
   {
      this.address = address;
   }

   /**
    * Supplies the address associated with this request
    *
    * @return The address associated with this request
    */
   public BAddress getAddress()
   {
      return this.address;
   }
   
   /**
    * Set's the modifyAddress associated with the kUpdateRequest type.
    *
    * @param The address associated with this request
    */
   public void setModifyAddress(BAddress modifyAddress)
   {
       this.modifyAddress = modifyAddress;
   }
   
   /**
    * Supplies the modifyAddress associated with the kUpdateRequest type.
    *
    * @return The address to modify.
    */
   public BAddress getModifyAddress()
   {
       return this.modifyAddress;
   }

   /**
    * Sets the addressBook (associated with kListAddressBook)
    *
    * @param addressBook the releveant address book
    */
   public void setAddressList(Vector addressList)
   {
      this.addressList = addressList;
   }

   /**
    * Supplies the addressBook (associated with kListAddressBook)
    *
    * @return an address Book provided by a BAddressBookMgr instance
    */
   public Vector getAddressList()
   {
      return this.addressList;
   }

	/**
	 * Set the address book origin (client or server)
	 *
	 * @param origin the origin of the address book that handled this event
	 */
	public void setAddressBookOrigin(int origin)
	{
		this.addressBookOrigin = origin;
	}

	/**
	 *
	 *
	 * @return the origin of the address book that handled this event (client or server)
	 */
	public int getAddressBookOrigin()
	{
		return this.addressBookOrigin;
	}
}
