// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/cmdgui/BCommandGui.java,v 1.29 1999/10/20 07:48:19 tp079210 Exp $

package au.edu.swin.synergy.plugin.cmdgui;

import java.lang.String;
import java.lang.StringBuffer;
import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BBlockingComplete;
import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.event.BBootCE;
import java.lang.InterruptedException;

import au.edu.swin.synergy.plugin.test2.event.BTestCE;
import au.edu.swin.synergy.kernel.BKernel;
import au.edu.swin.synergy.plugin.mod13.event.BAddMenuItemCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddWindowCE;
import au.edu.swin.synergy.plugin.mod13.event.BCloseWindowCE;
import au.edu.swin.synergy.plugin.mod13.event.BSelectWindowCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddStatusCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddPanelCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddMenuCE;
import au.edu.swin.synergy.plugin.mod13.event.BSelectPanelCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddToolBarCE;
import au.edu.swin.synergy.plugin.mod13.event.BRemovePanelCE;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowAdapter;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowEvent;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;

import au.edu.swin.synergy.plugin.editmanager.event.BRegisterUndoCE;
import au.edu.swin.synergy.plugin.editmanager.BUndoManager;

import au.edu.swin.synergy.plugin.dialect.BString;
import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.dialect.event.BDialectChangeCE;

import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.plugin.command.event.BSetCmdStreamCE;
import au.edu.swin.synergy.lib.BStringList;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Font;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.text.Keymap;
import java.awt.event.KeyEvent;

import java.io.InputStream;
import java.io.PrintStream;
import java.io.OutputStream;
import java.lang.Runnable;
import java.net.URL;



/**
 * Command line GUI
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 * @dispatches BSetCmdStreamCE
 * @dispatches BAddWindowCE
 * @processes none
 */

public class BCommandGui extends BPlugin
{

	/**
	 * When the number of lines reached the highwater some lines are deleted
	 */
	private static final int kHighWaterLimit = 1500;

	/**
	 * When lines are deleted, there should be low water lines left
	 */
	private static final int kLowWaterLimit = 1000;

	/**
	 * Everything is layed out on this panel
	 */
    private JPanel panel = new JPanel();

    /**
     * This is the undo manager for the command line
     */
    private BUndoManager undoManager;

	/**
	 * This is where lines of output are dispalied
	 */
    private JTextArea output = new JTextArea();

	/**
	 * This is where lines of input are entered
	 */
    private JTextField input = new JTextField();

	/**
	 * This allows the scrollback buffer
	 */
	private JScrollPane scrollPane = new JScrollPane(output);
    
	/**
	 * This is the input stream that the user commands are inserted into
	 */
    private CommandInput inputStream = new CommandInput();

	/**
	 * This is the output stream where results are read from
	 */
	private CommandOutput outputStream = new CommandOutput();

	/**
	 * This is the font used by the input and output areas
	 */
	private Font font = new Font("monospaced", Font.PLAIN, 12);

    /**
     * This will display the command line if it's not already displayed
     */
    private JButton showButton;

	/**
	 * This is where the command history is stored
	 */
	private BStringList history = new BStringList();

	/**
	 * This is the current index into the history buffer.
	 * 0 means a new line
	 * 1 means the most recent line etc
	 */
	private int historyIndex = 0;

	/**
	 * This is where the current line is stored before being added to the history.
	 */
	private String tempBuffer = null;

    /**
     * Indicates whether the command line is currently showing
     */
    private boolean isShowing = false;

    /**
     * Same as the show button
     */
    JMenuItem showMenuItem = new JMenuItem();

    /**
     * Return a list of plugin dependancies
	 * This is currently not used by the kernel
     * @return a list of plugins such as "au.edu.swin.synergy.plugin.mod13.BWindowManager"
	 * or null if there are no dependancies
     */

    public BStringList getDependancies()
	{
		BStringList result = new BStringList();
		result.add("au.edu.swin.synergy.plugin.command.BCommandLine");
		result.add("au.edu.swin.synergy.plugin.mod13.BWindowManager");
		return result;
	}

    /**
     * Setup the command line GUI plugin
     * @param params no command line parameters are processed by this plugin
     * @post the gui is ready to perform IO
     */

    public void setup(BStringList params)
    {
        panel.setLayout( new BorderLayout() );
        panel.add( input, BorderLayout.SOUTH );
        panel.add( scrollPane, BorderLayout.CENTER );

		output.setEditable(false);
		output.setFont(font);
        output.addFocusListener(new OnFocus());

        input.addActionListener( new OnEnter( input ) );
		input.setFont(font);

		// set the keymaps for up and down arrows
		input.registerKeyboardAction(new OnUpArrow(), KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0), JComponent.WHEN_FOCUSED);
		input.registerKeyboardAction(new OnDownArrow(), KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0), JComponent.WHEN_FOCUSED);
		input.registerKeyboardAction(new OnHelpKey(), KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0), JComponent.WHEN_FOCUSED);
		input.registerKeyboardAction(new OnF3(), KeyStroke.getKeyStroke(KeyEvent.VK_F3, 0), JComponent.WHEN_FOCUSED);
		input.registerKeyboardAction(new OnEscape(), KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_FOCUSED);
        input.addFocusListener(new OnFocus());

		input.registerKeyboardAction(new OnTab(), KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0), JComponent.WHEN_FOCUSED);

        //Get the Icon
        showButton = new JButton(BSwingUtilities.getImage(this, "/au/edu/swin/synergy/plugin/cmdgui/images/cmd.gif"));

        //Get the icon for the show button
        BAddToolBarCE addToolBar = new BAddToolBarCE(this);
        addToolBar.addButton(showButton);
        showButton.addActionListener(new OnShowButton());
        dispatchEvent(addToolBar);

        // tell the command line about it's new IO streams...
        BSetCmdStreamCE streamEvent = new BSetCmdStreamCE( this, inputStream, new PrintStream(outputStream), true );
        dispatchEvent( streamEvent );

        BAddMenuItemCE addMenu = new BAddMenuItemCE(this, showMenuItem, new BString(this, "CommandPrompt"), new BString(this, "fileMenu"));
        showMenuItem.addActionListener(new OnShowButton());
        dispatchEvent(addMenu);

        this.doDialect();
    }

    /**
     * This will show the window the window manager
     */
    public void showWindow()
    {
        BAddWindowCE addWindowEvent = new BAddWindowCE( this, panel, new BString(this, "CommandPrompt" ));
        addWindowEvent.addWindowListener(new BCmdWindowListener());
        panel.setSize(400, 400);
        input.requestDefaultFocus();
        isShowing = true;
        dispatchEvent( addWindowEvent );
    }

    /**
     * Process kernel events
     * @param source the plugin sending the event
	 * @param event the event to process
     */

    public void onBCommandEvent( Object source, BCommandEvent event )
    {
        //Need to register the undo listener with the command line field
        if (event instanceof BBootCE)
        {
            BRegisterUndoCE undoCE = new BRegisterUndoCE(this.input);
    		BBlockingComplete complete = this.dispatchEventAndWait(undoCE);

            //make sure we got a valid response
            if (complete.getStatus() == BCommandComplete.kSuccess)
            {
                //Need to register the undo listener
                this.undoManager = undoCE.getUndoListener();
                this.input.getDocument().addUndoableEditListener(this.undoManager);
            }
        }
        else if (event instanceof BDialectChangeCE)
        {
            this.doDialect();
        }
    }

    /**
     * This will tell the windo wmanager to bring the command line window
     * to the front for whatever reason.
     */
    protected void selectCommandLine()
    {
        BSelectWindowCE selectCE = new BSelectWindowCE(this, panel);

        //Request focus in the command line
        this.dispatchEvent(selectCE);
    }

    /**
     * Sets the screen text to the appropriate values for the current Locale
     */
    protected void doDialect()
    {
        BSwingUtilities.setToolTip(this.showButton, BDialect.getString(this, "showToolTip"));
    }

	/**
	 * When the user presses enter
	 */
    class OnEnter implements ActionListener
    {
        JTextField input;

        OnEnter( JTextField fInput )
        {
            input = fInput;
        }

        public void actionPerformed( ActionEvent e )
        {
			String command = e.getActionCommand();
            Debug.doPrintln( "Action " + command );
            inputStream.addCommand( command );
			if(command.length() > 0)
			{
				history.add(command );
			}
			historyIndex = 0;
            input.setText( "" );
        }
    }
    
	/**
	 * This custom input stream allows commands to be inserted
	 * and then read out as an input stream
	 */
    class CommandInput extends InputStream
    {
        StringBuffer commands = new StringBuffer();

        void addCommand( String command )
        {
            synchronized( commands )
            {
                commands.append( command );
                commands.append( '\n' );
                commands.notify();
            }
        }
        
        public int read()
        {
            int result;
            synchronized( commands )
            {
                if( commands.length() == 0 )
                {
                    try
                    {
                        commands.wait();
                    }
                    catch( InterruptedException e )
                    {
                    }
                }
                result = commands.charAt( 0 );
                commands.delete( 0, 1 );
            }
            return result;
        }
    }
    
	/**
	 * This custom output stream allows written data to be
	 * added to the text area
	 */
    class CommandOutput extends OutputStream
    {
		String line = new String();

        public void write(int b)
        {
			line = line + (char) b;
			if((char)b == '\n')
			{
				flush();
			}
        }

		public void flush()
		{
			javax.swing.SwingUtilities.invokeLater(new DoAddLine(line));
			line = "";
		}
    }

	/**
	 * This is part of the CommandOutput stream
	 * All GUI work has to be done in the AWT thread
	 */
	class DoAddLine implements Runnable
	{
		String line;
		
		DoAddLine(String fLine)
		{
			line = fLine;
		}
		
		public void run()
		{
				output.append(line);

				// see if we should delete some lines...
				if(output.getLineCount() > kHighWaterLimit)
				{
					int lines = output.getLineCount() - kLowWaterLimit;
					Debug.doPrintln("Deleting " + lines + " lines");
					try
					{
						int offset = output.getLineEndOffset(lines);
						output.replaceRange("", 0, offset);
					}
					catch(Exception e)
					{
						Debug.doPrintln("deleting lines", e);
					}
				}

				JScrollBar scrollBar = scrollPane.getVerticalScrollBar();
				scrollBar.setValue(scrollBar.getMaximum());
		}
	}

	/**
	 * Perform command history functions when the user presses up
	 */
	class OnUpArrow extends AbstractAction
	{
		public void actionPerformed(ActionEvent e)
		{
			Debug.doPrintln("Up pressed");
			if(historyIndex == 0)
			{
				tempBuffer = input.getText();
			}
			historyIndex++;
			Debug.doPrintln("" + historyIndex + " " + history.getCount());
			
			int index = history.getCount() - historyIndex;
			
			if(index >= 0)
			{
				input.setText(history.get(index));
			}
			else
			{
				historyIndex--;
			}
			
		}
	}

	/**
	 * Perform command history functions when the user presses down
	 */
	class OnDownArrow extends AbstractAction
	{
		public void actionPerformed(ActionEvent e)
		{
			Debug.doPrintln("Down pressed");
			if(historyIndex > 0)
			{
				historyIndex--;
				
				Debug.doPrintln("" + historyIndex + " " + history.getCount());

				if(historyIndex <= 0)
				{
					input.setText(tempBuffer);
				}
				else
				{

					int index = history.getCount() - historyIndex;
					if(index >= 0)
					{
						input.setText(history.get(index));
					}
				}
			}
		}
	}

	/**
	 * Perform command history functions when the user presses F3
	 */
	class OnF3 extends AbstractAction
	{
		public void actionPerformed(ActionEvent e)
		{
			Debug.doPrintln("F3 pressed");
			
			int index = history.getCount() - 1;
			if(index > 0)
			{
				input.setText(history.get(index));
			}
		}
	}

	/**
	 * When the user presses F1, send a help command for the current command line
	 */
	class OnHelpKey extends AbstractAction
	{
		public void actionPerformed(ActionEvent e)
		{
			Debug.doPrintln("F1 pressed");
            inputStream.addCommand( "help " + input.getText() );
		}
	}

	/**
	 * Clear the current buffer when the user presses Escape
	 */
	class OnEscape extends AbstractAction
	{
		public void actionPerformed(ActionEvent e)
		{
			Debug.doPrintln("ESC pressed");
			input.setText("");
			historyIndex = 0;
		}
	}

	/**
	 * Do command line completion
	 */
	class OnTab extends AbstractAction
	{
		public void actionPerformed(ActionEvent e)
		{
			Debug.doPrintln("TAB pressed");
		}
	}

    /**
     * Selects the window when either field gets the focus. Also
     * sets the current undo editor to this one
     */
    class OnFocus extends FocusAdapter
    {
        public void focusGained(FocusEvent focusEvent)
        {
            selectCommandLine();
        }
    }

    /**
     * Keeps track of when the command line window becomes active and
     * inactive.
     */
    class BCmdWindowListener extends BWindowAdapter
    {
        /**
         * When deactiviated, we need to relinquish the undo manager
         */
        public void windowDeactivated(BWindowEvent windowEvent)
        {
            if (undoManager != null)
            {
                undoManager.relinquishUndoManager();
            }
        }

        /**
         * When selected, tell the undo manager we are currently active
         */
        public void windowActivated(BWindowEvent windowEvent)
        {
            if (undoManager != null)
            {
                undoManager.setUndoManager();
            }

            //Select the command line
            selectCommandLine();
            input.requestFocus();
        }

        /**
         * Called when the window is closed
         */
        public void windowClosed(BWindowEvent windowEvent)
        {
            isShowing = false;
            this.windowDeactivated(windowEvent);
        }
     }

     /**
      * Handles clicks on the show button command
      */
     class OnShowButton implements ActionListener
     {
        public void actionPerformed(ActionEvent actionEvent)
        {
            if (isShowing)
            {
                selectCommandLine();
            }
            else
            {
                showWindow();
            }
        }
     }
}
