// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/conflict/BConflict.java,v 1.9 1999/10/17 22:04:18 tp079519 Exp $

package au.edu.swin.synergy.plugin.conflict;

import java.lang.String;
import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BBlockingComplete;
import au.edu.swin.synergy.kernel.event.BBootCE;
import au.edu.swin.synergy.kernel.event.BShutdownCE;

import au.edu.swin.synergy.plugin.command.BOnCommand;
import au.edu.swin.synergy.plugin.command.event.BRegisterCmdCE;
import au.edu.swin.synergy.plugin.command.event.BGetEnvironmentCE;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.File;
import java.util.Vector;
import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.debug.Debug;

import au.edu.swin.synergy.plugin.modeller.BModel;
import au.edu.swin.synergy.plugin.modeller.BModelElement;
import au.edu.swin.synergy.plugin.modeller.BModelNode;
import au.edu.swin.synergy.plugin.modeller.BModelArc;

import au.edu.swin.synergy.plugin.repository.event.BRegisterTypeCE;
import au.edu.swin.synergy.plugin.repository.BConflictDetect;


/**
 * Conflict resolution for Cohesion Models.
 * This uses an algorithm called
 * Optomistic Concurrent Revisions
 *
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 * @see BRepository
 */

public class BConflict extends BPlugin implements BConflictDetect
{
   	public static final int kNoConflict = 0;
	public static final int kConflictCosmetic = 1;
	public static final int kConflict = 2;
	public static final int kConflictDeleted = 4;
	public static final int kChanged = 8;

 
    public void setup(BStringList params)
    {
   		dispatchEvent( new BRegisterCmdCE( this, "conflict", new OnConflict() ) );
		dispatchEvent( new BRegisterTypeCE( this, "cmo", this, null) );
	}
    
    public void onBCommandEvent( Object source, BCommandEvent event )
    {
    }

	public int doFindConflicts(Vector oldVersions, String destinationFile)
	{
		int result = kNoConflict;
		Debug.doPrintln("merge sequence is");
		for(int i=1; i<oldVersions.size(); i++)
		{
			Debug.doPrintln(((String)oldVersions.get(i - 1)) + " -> " + ((String)oldVersions.get(i)));
		}
		Debug.doPrintln("and save result to " + destinationFile);

		if(oldVersions.size() == 0)
		{
			return result;
		}

		BModel modelA = null;
		BModel modelB = loadModel((String)oldVersions.get(0));
		
		for(int i=1; i<oldVersions.size(); i++)
		{
			modelA = modelB;
			modelB = loadModel((String)oldVersions.get(i));

			result |= mergeModels(modelA, modelB);
		}

		modelA = modelB;
		modelB = loadModel(destinationFile);

		result |= mergeModels(modelA, modelB);

		// write modelB back to file...
		saveModel(destinationFile, modelB);
		
		return result;
	}

	private BModel loadModel(String fFileName)
	{
		File file = new File(fFileName);
		if(!file.exists())
		{
			Debug.doPrintln("File " + fFileName + " does not exist");
			return null;
		}
		try
		{
   		   	Debug.doPrintln("loading model");
		   	BDataFile modelData = new BDataFile(fFileName);
		   	modelData.openFileForRead();
		   	BModel result = (BModel) modelData.autoLoad();
			modelData.close();
			return result; 
		}
		catch(Exception E)
		{
			Debug.doPrintln("Error loading model " + fFileName);
			return null;
		}
	}

	private void saveModel(String fFileName, BModel fModel)
	{
		BDataFile modelData = new BDataFile(fFileName);
		modelData.openFileForWrite(false);
		{
			modelData.save(fModel);
		}
		modelData.close();
	}

	private int mergeModels(BModel oldModel, BModel newModel)
	{
		int result = kNoConflict;
		// we have 2 models, build lists of A, B and C
		// A = all nodes in A but not in B
		// B = all nodes in B but not in A
		// C = nodes common to A and B
		
		// A = oldModel
		// B = newModel

		Vector setA = new Vector();
		Vector setB = new Vector();
		Vector setCo = new Vector();
		Vector setCn = new Vector();


		// find A and C nodes...
		for(int i = 0; i < oldModel.getChildCount(); i++)
		{
			BModelElement oldElement = (BModelElement) oldModel.getChildAt(i);
			BModelElement newElement = (BModelElement) newModel.getElementByID(oldElement.getUniqueID());

			if(newElement == null)
			{
				Debug.doPrintln("Element " + oldElement.getUniqueID() + " has been deleted");
				setA.add(oldElement);
			}
			else
			{
				Debug.doPrintln("Element " + oldElement.getUniqueID() + " is common");
				setCo.add(oldElement);
				setCn.add(newElement);
				
			}
		}

		// now find B and C nodes again.
		for(int i = 0; i < newModel.getChildCount(); i++)
		{
			BModelElement newElement = (BModelElement) newModel.getChildAt(i);
			BModelElement oldElement = (BModelElement) oldModel.getElementByID(newElement.getUniqueID());

			if(oldElement == null)
			{
				Debug.doPrintln("Node " + newElement.getUniqueID() + " is a new node");
				setB.add(newElement);
			}
			else
			{
				// this is a setC node, but
				// these were all added in the first pass
			}
		}

		// print the conflict table...
		int max = setA.size();
		if(setB.size() > max)
		{
			max = setB.size();
		}
		if(setCo.size() > max)
		{
			max = setCo.size();
		}

		Debug.doPrintln("A\tB\tC");
		for(int i=0; i<max; i++)
		{
			String s = "";
			if(i < setA.size())
			{
				s = s + ((BModelElement)setA.get(i)).getUniqueID() + "\t";
			}
			else
			{
				s = s + "\t";
			}
			if(i < setB.size())
			{
				s = s + ((BModelElement)setB.get(i)).getUniqueID() + "\t";
			}
			else
			{
				s = s + "\t";
			}
			if(i < setCo.size())
			{
				s = s + ((BModelElement)setCo.get(i)).getUniqueID() + "\t";
			}
			else
			{
				s = s + "\t";
			}
			Debug.doPrintln(s);
		}

		// all the nodes in A are kConflictDeleted
		for(int i=0; i<setA.size(); i++)
		{
			((BModelElement)setA.get(i)).setConflict(kConflictDeleted);
			result |= kConflictDeleted;

			// we also need to put this element into model B
			newModel.addElement((BModelElement)setA.get(i));
		}



		// all the nodes in B are OK

		// all the nodes in C need to be checked

		for(int i=0; i<setCo.size(); i++)
		{
			BModelElement oldElement = (BModelElement) setCo.get(i);
			BModelElement newElement = (BModelElement) setCn.get(i);

			int changes = newElement.findChanges(oldElement);
					
			if((changes & kConflictCosmetic) != 0)
			{
				Debug.doPrintln("Cosmetic changes");
			}
			if((changes & kConflict) != 0)
			{
				Debug.doPrintln("Real changes");
				// if this node is already changed, then we go to conflict
				// otherwise it is onlt a change
				if((newElement.getConflict() & kChanged) != 0)
				{
					newElement.setConflict(newElement.getConflict() | kConflict);
				}
				else
				{
					newElement.setConflict(newElement.getConflict() | kChanged);
				}
			}
			result |= changes;
		}

		return result;
	}


    class OnConflict implements BOnCommand
    {
        public int onCommand( BStringList params, InputStream in, PrintStream out )
		{
			if(params.getCount() != 3)
			{
				out.println("Wrong number or arguements");
			}
			else
			{
				Debug.doPrintln("Looking for conflicts");
			
				File oldFile = new File(params.get(1));
				if(!oldFile.exists())
				{
					out.println("File " + params.get(1) + " does not exist");
					return 1;
				}
				File newFile = new File(params.get(2));
				if(!newFile.exists())
				{
					out.println("File " + params.get(2) + " does not exist");
					return 1;
				}

				// OK, we have 2 files
				// load up the models
				BModel oldModel = null;
				BModel newModel = null;
				try
				{
		   		   	Debug.doPrintln("loading model");
				   	BDataFile modelData = new BDataFile(params.get(1));
				   	modelData.openFileForRead();
				   	oldModel = (BModel) modelData.autoLoad();
					modelData.close();
				}
				catch(Exception E)
				{
					out.println("Error loading model " + params.get(1));
					return 2;
				}
				try
				{
		   		   	Debug.doPrintln("loading model");
				   	BDataFile modelData = new BDataFile(params.get(2));
				   	modelData.openFileForRead();
				   	newModel = (BModel) modelData.autoLoad();
					modelData.close();
				}
				catch(Exception E)
				{
					out.println("Error loading model " + params.get(2));
					return 2;
				}

				// we are ready to check

				// we have 2 models, build lists of A, B and C
				// A = all nodes in A but not in B
				// B = all nodes in B but not in A
				// C = nodes common to A and B

				// A = oldModel
				// B = newModel

				Vector setA = new Vector();
				Vector setB = new Vector();
				Vector setCo = new Vector();
				Vector setCn = new Vector();


				// find A and C nodes...
				for(int i = 0; i < oldModel.getChildCount(); i++)
				{
					BModelElement oldElement = (BModelElement) oldModel.getChildAt(i);
					BModelElement newElement = (BModelElement) newModel.getElementByID(oldElement.getUniqueID());

					if(newElement == null)
					{
						out.println("Element " + oldElement.getUniqueID() + " has been deleted");
						setA.add(oldElement);
					}
					else
					{
						out.println("Element " + oldElement.getUniqueID() + " is common");
						setCo.add(oldElement);
						setCn.add(newElement);
						
					}
				}

				// now find B and C nodes again.
				for(int i = 0; i < newModel.getChildCount(); i++)
				{
					BModelElement newElement = (BModelElement) newModel.getChildAt(i);
					BModelElement oldElement = (BModelElement) oldModel.getElementByID(newElement.getUniqueID());

					if(oldElement == null)
					{
						out.println("Node " + newElement.getUniqueID() + " is a new node");
						setB.add(newElement);
					}
					else
					{
						// this is a setC node, but
						// these were all added in the first pass
					}
				}

				// print the conflict table...
				int max = setA.size();
				if(setB.size() > max)
				{
					max = setB.size();
				}
				if(setCo.size() > max)
				{
					max = setCo.size();
				}

				Debug.doPrintln("A\tB\tC");
				for(int i=0; i<max; i++)
				{
					String s = "";
					if(i < setA.size())
					{
						s = s + ((BModelElement)setA.get(i)).getUniqueID() + "\t";
					}
					else
					{
						s = s + "\t";
					}
					if(i < setB.size())
					{
						s = s + ((BModelElement)setB.get(i)).getUniqueID() + "\t";
					}
					else
					{
						s = s + "\t";
					}
					if(i < setCo.size())
					{
						s = s + ((BModelElement)setCo.get(i)).getUniqueID() + "\t";
					}
					else
					{
						s = s + "\t";
					}
					Debug.doPrintln(s);
				}

				// all the nodes in A are kConflictDeleted
				for(int i=0; i<setA.size(); i++)
				{
					((BModelElement)setA.get(i)).setConflict(kConflictDeleted);
				}



				// all the nodes in B are OK

				// all the nodes in C need to be checked

				for(int i=0; i<setCo.size(); i++)
				{
					BModelElement oldElement = (BModelElement) setCo.get(i);
					BModelElement newElement = (BModelElement) setCn.get(i);

					int changes = newElement.findChanges(oldElement);
					
					if((changes & kConflictCosmetic) != 0)
					{
						out.println("Cosmetic changes");
					}
					if((changes & kConflict) != 0)
					{
						out.println("Real changes");
						// if this node is already changed, then we go to conflict
						// otherwise it is onlt a change
						if((newElement.getConflict() & kChanged) != 0)
						{
							newElement.setConflict(newElement.getConflict() | kConflict);
						}
						else
						{
							newElement.setConflict(newElement.getConflict() | kChanged);
						}
						
					}
				}

				

/*							// looking for changes
							int changes = newNode.findChanges(oldNode);
							Debug.doPrintln("changes = " + changes);
							if((changes & kConflictCosmetic) > 0)
							{
								out.println("Cosmetic changes");
							}
							if((changes & kConflict) > 0)
							{
								out.println("Real changes");
							}*/

				
			}
			return 0;
		}

        public String getHelp( BStringList params )
        {
            return "Conflict test two models\n" +
			       "Usage: conflict model1.cmo model2.cmo\n"+
				   "The models must be changes of a common version ie. V2 & V1'";
        }

	}
}
