//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/diagram/BDrawablePalette.java,v 1.12 1999/10/15 15:59:51 tp079479 Exp $

package au.edu.swin.synergy.plugin.diagram;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;

import java.util.Vector;

import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;

import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.plugin.diagram.BDrawableButton;
import au.edu.swin.synergy.debug.Debug;


/**
 * class BDrawablePalette - provides a tool palette containing
 * BDrawables, that the user can select from..
 *
 * @author Imron Alston
 * @date 31/7/99
 *
 * @copyright team synergy 1999
 */
public class BDrawablePalette extends JPanel implements ActionListener
{

    /**
     * the default size of the horizontal gap between the buttons on the panel *
     */
    protected static final int kDefaultHGap = 0;

    /**
     * the default size of the vertical gap between the buttons on the panel *
     */
    protected static final int kDefaultVGap = 0;



    /**
     * the panel containing all the buttons *
     */
    protected JPanel buttonPanel = null;

    /**
     * the flow layout of the panel *
     */
    protected FlowLayout flowLayout = null;

    /**
     * a scroll pane for the panel containing all the buttons *
     */
    protected JScrollPane buttonScroll = null;

    /**
     * a button group, so only 1 button is selected at a time.. *
     */
    protected ButtonGroup buttonGroup = null;

    /**
     * Vector holding all the buttons displayed on the panel. *
     */
    protected Vector buttons = null;

    /**
     * Vector containing all of the drawables on the buttons. *
     */
    protected Vector drawables = null;

    /**
     * The currently selected button *
     */
    protected BDrawableButton curSel = null;

	/** the pointer button **/
	protected JToggleButton pointerButton = null;

	protected ImageIcon pointer;


    /**
     * Please complete the missing tags for BGroupPalette
     * @param
     * @return
     * @pre
     * @post
     */


    public BDrawablePalette( int hGap, int vGap )
    {
        super();

		String path = "/au/edu/swin/synergy/plugin/diagram/icons/";

        this.buttonGroup = new ButtonGroup();
        this.buttons = new Vector();
        this.drawables = null;

        this.flowLayout = new FlowLayout( FlowLayout.LEFT, hGap, vGap );
        this.buttonPanel = new JPanel( this.flowLayout );
        this.buttonScroll = new JScrollPane( this.buttonPanel,
        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );

		 pointer = BSwingUtilities.getImage(this, path + "cursor.gif" );
		 this.pointerButton = new JToggleButton( pointer );

		
		this.pointerButton.addActionListener( this );
		this.pointerButton.setPreferredSize( BDrawableButton.kPreferredSize );
		this.buttonGroup.add( this.pointerButton );
		this.buttonPanel.add( this.pointerButton );


        this.setLayout( new BorderLayout() );
        this.add( buttonPanel );

    }
    /**
     * Please complete the missing tags for BGroupPalette
     * @param
     * @return
     * @pre
     * @post
     */

    public BDrawablePalette()
    {
        this( BDrawablePalette.kDefaultHGap, BDrawablePalette.kDefaultVGap );
    }

    /**
     * Gets the currently selected BDrawable
     * Please complete the missing tags for getSelectedDrawable
     * @param
     * @return
     * @pre
     * @post
     */
    public BDrawable getSelectedDrawable()
    {
        BDrawable result = null;

        if( this.curSel != null )
        {
            result = this.curSel.getDrawable();
        }

        return result;
    }

	public void resetCurSel()
	{
		this.pointerButton.doClick();
	}
	public void setDisabled( boolean b )
	{
		int i, size;
		BDrawableButton curButton = null;

		for ( i = 0; i < this.buttons.size(); i++ )
		{
			curButton = (BDrawableButton)this.buttons.elementAt(i);
			curButton.setVisible( b );
		}
	}

    /**
     * Sets the shapes of the buttons in the tool palette to
     * a corresponding shape from a shape vector.
     * If there aren't sufficient buttons, then new buttons
     * are created to fill demand.
     *
     * @author Imron Alston
     * @date 31/7/99
     *
     * @param Vector drawables - a vector containing the BDrawable to set the buttons to.
     *
     * Please complete the missing tags for setGroups
     * @return
     * @pre
     * @post
     */
    public void setDrawables( Vector drawables )
    {
		Debug.doPrintln("Setting drawables");

        BDrawableButton curButton = null;
        BDrawable curDrawable = null;
        int sizeShape, sizeButton;
        int i;

        this.drawables = drawables;

        sizeButton = this.buttons.size();
        sizeShape = this.drawables.size();

        //increase the number of buttons if there aren't enough..
        if( sizeShape > sizeButton )
        {
            for( i = sizeButton; i < sizeShape; i++ )
            {
                //add the button to all the places that need to know about it.
                curButton = new BDrawableButton();
                curButton.addActionListener( this );

                this.buttonGroup.add( curButton );
                this.buttonPanel.add( curButton );
                this.buttons.add( curButton );

                curButton.setPreferredSize( BDrawableButton.kPreferredSize );
            }
        }

        //set the buttons to have their correct shape.
        //sets the shapes to null, when there are more
        //buttons than shapes.
        sizeButton = this.buttons.size();

        for( i = 0; i < sizeButton; i++ )
        {
            curButton = ( BDrawableButton ) this.buttons.elementAt( i );

            if( i < sizeShape )
            {
                curDrawable = ( BDrawable ) this.drawables.elementAt( i );
                curButton.setVisible( true );
            }
            else
            {
                curDrawable = null;
                curButton.setVisible( false );
            }

            curButton.setDrawable( curDrawable );
        }

        this.repaint();

        //help out the poor old garbage collector..
        curButton = null;
        curDrawable = null;
    }

    /**
     * Sets the horizontal gap between
     * the buttons.
     * Please complete the missing tags for setHGap
     * @param
     * @return
     * @pre
     * @post
     */
    public void setHGap( int hGap )
    {
        this.flowLayout.setHgap( hGap );
    }

    /**
     * Sets the vertical gap between
     * the buttons.
     * Please complete the missing tags for setVGap
     * @param
     * @return
     * @pre
     * @post
     */
    public void setVGap( int vGap )
    {
        this.flowLayout.setVgap( vGap );
    }
    /**
     * Please complete the missing tags for reshape
     * @param
     * @return
     * @pre
     * @post
     */

    public void reshape( int x, int y, int w, int h )
    {
        Debug.doPrintln( "resizing" );
        super.reshape( x, y, w, h );

    }


    /**
     * Handler for the pressing of buttons.
     * Please complete the missing tags for actionPerformed
     * @param
     * @return
     * @pre
     * @post
     */
    public void actionPerformed( ActionEvent e )
    {
        Object source;

        source = e.getSource();

		if ( source == this.pointerButton )
		{
			this.curSel = null;
		}
		else if( source instanceof BDrawableButton )
        {
            this.curSel = ( BDrawableButton ) source;
			Debug.doPrintln( "selected a new drawable " + this.curSel.getDrawable().getName(), Debug.kWarningLevel );
        }


        //help out the gc.
        source = null;
    }


    /**
     * Removes any unused buttons from the button vector.
     * Removes the unused buttons from the button panel.
     * Removes the unused buttons from the button group.
     * Calls cleanup on the unused buttons.
     * Trims the button Vector's capacity.
     * Please complete the missing tags for cleanup
     * @param
     * @return
     * @pre
     * @post
     */
    public void cleanup()
    {
        BDrawableButton curButton = null;
        int sizeShape;
        int sizeButton;
        int i;

        if( this.drawables == null )
        {
            sizeShape = 0;
        }
        else
        {
            sizeShape = this.drawables.size();
        }

        sizeButton = this.buttons.size();

        while( this.buttons.size() > sizeShape )
        {
            curButton = ( BDrawableButton ) this.buttons.remove( this.buttons.size() - 1 );
            this.buttonPanel.remove( curButton );
            this.buttonGroup.remove( curButton );

            curButton.removeActionListener( this );
            curButton.cleanup();
            curButton = null;
        }

        this.buttons.trimToSize();

        //help out the gc
        curButton = null;
    }

}
