// $Header:

/**
* This panel acts like a canvas where drawables pass thier associated
* property panel to be displayed allowing alterations of respective 
* attributes.
* 
* @author Christian Karolos
* @date 30/9/99
*/

package au.edu.swin.synergy.plugin.diagram;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import au.edu.swin.synergy.shapes.BPropPanel;
import au.edu.swin.synergy.plugin.mod13.event.BCloseWindowCE;
import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;
import au.edu.swin.synergy.plugin.diagram.BViewport;

import au.edu.swin.synergy.kernel.BPlugin;

public class BGlobalPropPanel extends JPanel
{
    protected BPropPanel propPanel;
    private BPlugin parent;
    private BCloseWindowCE closeEvent;
	BViewport activeView;

    protected JButton okButton = new JButton();
    protected JButton cancelButton = new JButton();

    public BGlobalPropPanel ( BPlugin par, BViewport view, BPropPanel prop )
    {
        propPanel = prop;
        parent = par;
		activeView = view;
        
        closeEvent = new BCloseWindowCE ( this, this );
        
        setLayout ( new GridBagLayout () );
        GridBagConstraints c = new GridBagConstraints ();

        okButton.setText( BDialect.getString ( "ok" ) );
        okButton.addActionListener ( new onOk () );
        cancelButton.setText( BDialect.getString ( "cancel" ) );
        cancelButton.addActionListener ( new onCancel () );

        setConst ( c, 0, 0, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH, 1 );
        add ( prop, c );

        setConst ( c, 0, 1, 1, 1, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, 0 );
        add ( okButton, c );

        setConst ( c, 1, 1, 1, 1, GridBagConstraints.WEST, GridBagConstraints.VERTICAL, 0 );
        add ( cancelButton, c );
    }

	/**
	* Custom made function for plotting labels and combo boxes on a panel.
	* Panel shouldn't be expandable, so concerning attribs are fixed.
	*/
    public static void setConst( GridBagConstraints gbc, int x, int y, int w, int h, int anc, int fill, int weight )
    {
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.gridwidth = w;
        gbc.gridheight = h;
        gbc.fill = fill;
        gbc.anchor = anc;
        gbc.insets = new Insets ( 5, 5, 5, 5 );

		// Only a light weight so things don't expand too much.
        gbc.weightx = weight;
        gbc.weighty = weight;
        gbc.ipady = 5;
        gbc.ipadx = 10;
    }

    public void doDialect ()
    {
        BSwingUtilities.setJButtonText ( okButton, BDialect.getString ( "ok" ) );
        BSwingUtilities.setJButtonText ( cancelButton, BDialect.getString ( "cancel" ) );
    }

    private class onOk implements ActionListener
    {
        public void actionPerformed ( ActionEvent e )
        {
            if ( propPanel.okPressed () )
            {
                parent.dispatchEvent ( closeEvent );
				activeView.repaint ();
            }
        }
    }

    private class onCancel implements ActionListener
    {
        public void actionPerformed ( ActionEvent e )
        {
            if ( propPanel.cancelPressed () )
            {
                parent.dispatchEvent ( closeEvent );
            }
        }
    }
}