
/**
 * Grid class for the BViewport..
 *
 * @author Christian Karolos
 * @author Imron Alston
 * @copyright Team Synergy 1999
 */


package au.edu.swin.synergy.plugin.diagram;

import au.edu.swin.synergy.math.BScalarRect;
import au.edu.swin.synergy.math.BScalar;
import au.edu.swin.synergy.plugin.diagram.BView;
import au.edu.swin.synergy.debug.Debug;

import java.awt.*;

public class BGrid
{
	private BView view = null;// the view that the grid is on..

	private BScalar initialX = null;

	private BScalar xPos = null; // the cur x position
	private BScalar xInc = null; // the amount to increment x by..
	private BScalar xEndPos = null; // the end of the x grid..

	private BScalar yPos = null; // the cur y position
	private BScalar yInc = null; // the amount to increment y by..
	private BScalar yEndPos = null; // the end of the y grid..


	private boolean dots = true;

	private Color gridColour = Color.black;

	public BGrid( BView view )
	{
		this.view = view;

		this.initialX = new BScalar(0);
		this.xPos = new BScalar(0);
		this.xInc = new BScalar(0);
		this.xEndPos = new BScalar(0);
		this.yPos = new BScalar(0);
		this.yInc = new BScalar(0);
		this.yEndPos = new BScalar(0);
	}
	
	
	public void setGridColour( Color newColour )
	{
		this.gridColour = newColour;
	}

	public void setDots( boolean dots )
	{
		this.dots = dots;
	}

	public boolean getDots()
	{
		return this.dots;
	}

	public Color getGridColour()
	{
		return this.gridColour;
	}


	/**
	 * Renders the grid onto a graphics context..
	 *
	 */
	public void render( Graphics2D g2, int minSpacing, int majSpacing )
	{
		BScalarRect viewBounds = this.view.getViewBounds();
		int width, height; // the length of the line
		int xInt, yInt; // the integer value of the current x and y positions
		int gridSpacing;

		if ( dots )
		{
			gridSpacing = minSpacing;
		}
		else
		{
			gridSpacing = majSpacing * minSpacing;
		}

		if ( gridSpacing == 0 ) return;

		g2.setColor( this.gridColour );

		width = this.view.getWidth();
		height = this.view.getHeight();

		//calculate the values for x
		this.xPos.setValueX( viewBounds );
		this.xPos.mod( gridSpacing );
		this.xPos.neg();
		this.xPos.mult( this.view.getOOZ() );
		this.xPos.add( gridSpacing );
		//calculate the increment...
		// gridspacing * ooz
		this.xInc.setValue( gridSpacing );
		this.xInc.mult( this.view.getOOZ() );
		this.xEndPos.setValue( width );


		//calculate the values for y
		this.yPos.setValueY( viewBounds );
		this.yPos.mod( gridSpacing );
		this.yPos.neg();
		this.yPos.mult( this.view.getOOZ() );
		this.yPos.add( gridSpacing );
		//calculate the increment...
		// gridspacing * ooz
		this.yInc.setValue( gridSpacing );
		this.yInc.mult( this.view.getOOZ() );
		this.yEndPos.setValue( height );


		if ( !dots ) // if we aren't rendering dots...
		{
			//render the x lines..
			//for ( ; pos < endPos; pos += inc )
			for ( ; this.xPos.lessThan( this.xEndPos ); this.xPos.add( this.xInc ) )
			{
				xInt = this.xPos.getValue();
				g2.drawLine( xInt, 0, xInt, height );
			}

			//render the y lines....
			for ( ; this.yPos.lessThan( this.yEndPos ); this.yPos.add( this.yInc ) )
			{
				yInt = this.yPos.getValue();
				g2.drawLine( 0, yInt, width, yInt );
			}
		}
		else //we are rendering dots..
		{
			this.initialX.setValue( this.xPos );

			for ( ; this.yPos.lessThan( this.yEndPos ); this.yPos.add( this.yInc ) )
			{
				this.xPos.setValue( this.initialX );
				for ( ; this.xPos.lessThan( this.xEndPos ); this.xPos.add( this.xInc ) )
				{
					xInt = this.xPos.getValue();
					yInt = this.yPos.getValue();
					g2.drawLine( xInt, yInt, xInt, yInt);
				}
			}
		}
	}

}