/**
* Ruler class for the BViewport
*
* @author Christian Karolos
* @copyright Team Synergy 1999
*/

package au.edu.swin.synergy.plugin.diagram;

import au.edu.swin.synergy.math.BScalarRect;
import au.edu.swin.synergy.math.BScalar;
import au.edu.swin.synergy.plugin.diagram.BView;
import au.edu.swin.synergy.debug.Debug;

import java.awt.*;

public class BRuler
{
	private static final int kMinorLineLen = 4;
	private static final int kMajorLineLen = 12;

	private BView view = null;

	private BScalar initialX = null;

	private BScalar xPos = null; // the cur x position
	private BScalar xInc = null; // the amount to increment x by..
	private BScalar xEndPos = null; // the end of the x grid..

	private BScalar yPos = null; // the cur y position
	private BScalar yInc = null; // the amount to increment y by..
	private BScalar yEndPos = null; // the end of the y grid..


	/**
	* Stuff needed when rendering rulers.
	*/
	private FontMetrics fm;
	private Color rulerColour = Color.black;
	private BasicStroke rulerStroke = new BasicStroke ( 1.0f );
	private Font rulerFont;
	private float majDist;
	private int initFontSize;
	private int textXInterval;
	private int textYInterval;
	private int majGap;
	private int avgStrLen;
	private int avgStrHeight;


	public BRuler ( BView view )
	{
		this.view = view;

		this.initialX = new BScalar(0);
		this.xPos = new BScalar(0);
		this.xInc = new BScalar(0);
		this.xEndPos = new BScalar(0);
		this.yPos = new BScalar(0);
		this.yInc = new BScalar(0);
		this.yEndPos = new BScalar(0);	
	}

	public void setRulerColour ( Color c )
	{
		this.rulerColour = c;
	}

	public void setRulerStroke ( BasicStroke s )
	{
		this.rulerStroke = s;
	}

	public void render ( Graphics2D g2, int minSpacing, int majSpacing )
	{
		BScalarRect viewBounds = this.view.getViewBounds ();

		int width = this.view.getWidth ();
		int height = this.view.getHeight ();
		int xVal;
		int yVal;
		int xInt;
		int yInt;

		Color oldColour = g2.getColor ();
		BasicStroke oldStroke = ( BasicStroke ) g2.getStroke ();		

		g2.setColor ( rulerColour );
		g2.setStroke ( rulerStroke );

		rulerFont = g2.getFont ();		
		
		majGap = minSpacing * majSpacing;
		fm = g2.getFontMetrics ( rulerFont );
		avgStrLen = fm.stringWidth ( "-0000" );
		avgStrHeight = fm.getHeight ();

		majDist = ( float ) majSpacing * this.view.getOOZ().getFloatValue ();

        // we only want to render ruler identifiers every...
        float tmpX = ( ( float ) avgStrLen / majDist ) + 0.5f;
        float tmpY = ( ( float ) avgStrHeight / majDist ) + 0.5f;

        textXInterval = Math.round ( tmpX );
        textYInterval = Math.round ( tmpY );
        // major line.  Add 0.5 so we always round up.

		//calculate the values for x
		this.xPos.setValueX( viewBounds );
		this.xPos.mod( minSpacing );
		this.xPos.neg();
		this.xPos.mult( this.view.getOOZ() );
		this.xPos.add( minSpacing );
		//calculate the increment...
		// gridspacing * ooz
		this.xInc.setValue( minSpacing );
		this.xInc.mult( this.view.getOOZ() );
		this.xEndPos.setValue( width );


		//calculate the values for y
		this.yPos.setValueY( viewBounds );
		this.yPos.mod( minSpacing );
		this.yPos.neg();
		this.yPos.mult( this.view.getOOZ() );
		this.yPos.add( minSpacing );
		//calculate the increment...
		// gridspacing * ooz
		this.yInc.setValue( minSpacing );
		this.yInc.mult( this.view.getOOZ() );
		this.yEndPos.setValue( height );


		//render the x lines..
		//for ( ; pos < endPos; pos += inc )
		for ( ; this.xPos.lessThan( this.xEndPos ); this.xPos.add( this.xInc ) )
		{
			xInt = this.xPos.getValue();
/*			if () // Check if on major intersection.
			{
				g2.drawLine ( xInt, 0, xInt, kMajorLineLen );
				g2.drawString ( xVal, xInt?, kMajorLineLen + kMinorLineLen );
			}
			else
*/
			{
				g2.drawLine( xInt, 0, xInt, kMinorLineLen );
			}
		}

		//render the y lines....
		for ( ; this.yPos.lessThan( this.yEndPos ); this.yPos.add( this.yInc ) )
		{
			yInt = this.yPos.getValue();
/*			if () // Check if on major intersection.
			{
				g2.drawLine ( 0, yInt, kMajorLineLen, yInt );
				g2.drawString ( yVal, kMinorLineLen, yInt );
			}
			else
*/
			{
				g2.drawLine( 0, yInt, kMinorLineLen, yInt );
			}
		}

		g2.setColor ( oldColour );
		g2.setStroke ( oldStroke );
	}
}