//$Header

/**
* Handles and displays preference options for the package plugin.
* 
* @author Christian Karolos
* @date 20/9/99
*/
package au.edu.swin.synergy.plugin.diagram;

import au.edu.swin.synergy.lib.BColorCombo;
import au.edu.swin.synergy.plugin.dialect.BString;
import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.lib.BGridBagConstraints;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;

import au.edu.swin.synergy.debug.Debug;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.lang.Integer;

public class BViewportPrefs extends JPanel 
{
	public static final int kDotted = 0;
	public static final int kLines = 1;

	JLabel rulerLab;
	JLabel gridLab;
	JLabel rulerColour;
	JLabel gridColour;
	JLabel minRulerLab;
	JLabel majRulerLab;
    JLabel backColourLab;
	JLabel dotLineLab;
	TitledBorder rulerBorder;
	TitledBorder gridBorder;
	TitledBorder backgroundBorder;
	JCheckBox rulerCheck;
	JCheckBox gridCheck;
	JCheckBox snapCheck;
	JTextField minVal = new JTextField ( 4 );
	JTextField majVal = new JTextField ( 4 );
	BColorCombo gridColourBox = new BColorCombo ();
	BColorCombo rulerColourBox = new BColorCombo ();
    BColorCombo backColourBox = new BColorCombo ();
	JComboBox gridTypeCombo;

	String[] gridType;

	rulerPrefs rulerPanel;
	gridPrefs gridPanel;
	backgroundPrefs bgPanel;
	
	public BViewportPrefs ()
	{
		rulerPanel = new rulerPrefs ();
		gridPanel = new gridPrefs ();
		bgPanel = new backgroundPrefs ();

		setLayout ( new GridLayout ( 3, 1 ) );

		add ( rulerPanel );
		add ( gridPanel );
		add ( bgPanel );
	}				

	public void doDialect ()
	{
		BSwingUtilities.setJLabelText ( rulerColour, BDialect.getString ( this, "rulercolour" ) );
		BSwingUtilities.setJLabelText ( gridColour, BDialect.getString ( this, "gridcolour" ) );
		BSwingUtilities.setJLabelText ( minRulerLab, BDialect.getString ( this, "minruler" ) );
		BSwingUtilities.setJLabelText ( majRulerLab, BDialect.getString ( this, "majruler" ) );
        BSwingUtilities.setJLabelText ( backColourLab, BDialect.getString ( this, "backColour" ) );
		BSwingUtilities.setJButtonText ( rulerCheck, BDialect.getString ( this, "showruler" ) );
		BSwingUtilities.setJButtonText ( gridCheck, BDialect.getString ( this, "showgrid" ) );
	}

	public void setRulerCheck ( boolean checked )
	{
		rulerCheck.setSelected ( checked );
	}

	public void setGridCheck ( boolean checked )
	{
		gridCheck.setSelected ( checked );
	}

	public void setMinVal ( int val )
	{
		minVal.setText ( Integer.toString ( val ) );
	}

	public void setMajVal ( int val )
	{
		majVal.setText ( Integer.toString ( val ) );
	}

	public void setGridColour ( Color c )
	{
		gridColourBox.setSelectedItem ( c );
	}

	public void setRulerColour ( Color c )
	{
		rulerColourBox.setSelectedItem ( c );
	}

    public void setBackColour ( Color c )
    {
        backColourBox.setSelectedItem ( c );
    }

	public boolean isRulerEnabled ()
	{
		return rulerCheck.isSelected ();
	}

	public boolean isGridEnabled ()
	{
		return gridCheck.isSelected ();
	}

	public Color getGridColour ()
	{
		return gridColourBox.getSelectedColor ();
	}

	public Color getRulerColour ()
	{
		return rulerColourBox.getSelectedColor ();
	}

    public Color getBackColour ()
    {
        return backColourBox.getSelectedColor ();
    }

	public void setSnap ( boolean check )
	{
		snapCheck.setSelected ( check );
	}

	public boolean isSnapEnabled ()
	{
		return snapCheck.isSelected ();
	}

	public void setGridType ( int type )
	{
		gridTypeCombo.setSelectedIndex ( type );
	}

	public int getGridType ()
	{
		return gridTypeCombo.getSelectedIndex ();
	}

	public int getMinVal ()
	{
		int space = 100; // if something goes wrong, make this the default.
		try
		{
			space = Integer.parseInt ( minVal.getText () );
		}
		catch ( NumberFormatException e )
		{
			Debug.doPrintln ( "Min ruler spacing should contain an int." );
		}

		return space;
	}

	public int getMajVal ()
	{
		int space = 5; // Making this the default if things go wrong.
		try
		{
			space = Integer.parseInt ( majVal.getText () );
		}
		catch ( NumberFormatException e )
		{
			Debug.doPrintln ( "Maj ruler spacing should contain an int." );
		}

		return space;
	}

	public class rulerPrefs extends JPanel
	{
		public rulerPrefs ()
		{
			rulerBorder = new TitledBorder ( new EtchedBorder ( EtchedBorder.LOWERED ), BDialect.getString ( this, "ruler" ) );
			setBorder ( rulerBorder );

			setLayout ( new GridBagLayout () );
			GridBagConstraints c = new GridBagConstraints ();		

			rulerColour = new JLabel ( BDialect.getString ( this, "rulercolour" ) );
			minRulerLab = new JLabel ( BDialect.getString ( this, "minruler" ) );
			majRulerLab = new JLabel ( BDialect.getString ( this, "majruler" ) );
			rulerCheck = new JCheckBox ( BDialect.getString ( this, "showruler" ) );

			rulerCheck.addChangeListener ( new comboBoxChange () );
			c = new BGridBagConstraints(0, 0, 2, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0);
			add ( rulerCheck, c );

			c = new BGridBagConstraints(2, 0, 1, 1, 0, 1, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0);
			add ( rulerColour, c );

			c = new BGridBagConstraints(3, 0, 1, 1, 0, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0);
			add ( rulerColourBox, c );

			c = new BGridBagConstraints(0, 1, 1, 1, 0, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0);
			add ( minRulerLab, c );

			c = new BGridBagConstraints(1, 1, 1, 1, 0, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0);
			add ( minVal, c );			

			c = new BGridBagConstraints(2, 1, 1, 1, 0, 1, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0); 
			add ( majRulerLab, c );

			c = new BGridBagConstraints(3, 1, 1, 1, 0, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0);
			add ( majVal, c);
		}
	}

	public class backgroundPrefs extends JPanel
	{
		public backgroundPrefs ()
		{
			backgroundBorder = new TitledBorder ( new EtchedBorder ( EtchedBorder.LOWERED ), BDialect.getString ( this, "background" ) );		
			setBorder ( backgroundBorder );

			setLayout ( new GridBagLayout () );
			GridBagConstraints c = new GridBagConstraints ();

	        backColourLab = new JLabel ( BDialect.getString ( this, "backcolour" ) );
			
			setConst ( c, 0, 0, GridBagConstraints.EAST );
			add ( backColourLab, c );

			setConst ( c, 1, 0, GridBagConstraints.WEST );
			add ( backColourBox, c );
		}
	}

	public class gridPrefs extends JPanel
	{
		public gridPrefs ()
		{
			gridBorder = new TitledBorder ( new EtchedBorder ( EtchedBorder.LOWERED ), BDialect.getString ( this, "grid" ) );
			setBorder ( gridBorder );

			setLayout ( new GridBagLayout () );
			GridBagConstraints c = new GridBagConstraints ();

			gridColour = new JLabel ( BDialect.getString ( this, "gridcolour" ) );
			dotLineLab = new JLabel ( BDialect.getString ( this, "dotline" ) );
			gridCheck = new JCheckBox ( BDialect.getString ( this, "showgrid" ) );
			snapCheck = new JCheckBox ( BDialect.getString ( this, "snaptogrid" ) );

			gridType = new String[2];
			gridType[0] = BDialect.getString ( this, "dotted" );
			gridType[1] = BDialect.getString ( this, "lines" );
			gridTypeCombo = new JComboBox ( gridType );

			gridCheck.addChangeListener ( new comboBoxChange () );
			setConst ( c, 0, 0, GridBagConstraints.CENTER );
			add ( gridCheck, c );

			setConst ( c, 2, 0, GridBagConstraints.EAST );
			add ( gridColour, c );

			setConst ( c, 3, 0, GridBagConstraints.CENTER );
			add ( gridColourBox, c );

			setConst ( c, 0, 1, GridBagConstraints.CENTER );
			add ( snapCheck, c );

			setConst ( c, 2, 1, GridBagConstraints.EAST );
			add ( dotLineLab, c );

			setConst ( c, 3, 1, GridBagConstraints.CENTER );
			add ( gridTypeCombo, c );
		}
	}

	public class comboBoxChange implements ChangeListener
	{
		public void stateChanged ( ChangeEvent e )
		{
			boolean state;
			if ( e.getSource () == gridCheck )
			{
				state = gridCheck.isSelected ();
				gridColourBox.setEnabled ( state );
				snapCheck.setEnabled ( state );
				gridTypeCombo.setEnabled ( state );
			}
			else if ( e.getSource () == rulerCheck )
			{
				state = rulerCheck.isSelected ();
				rulerColourBox.setEnabled ( state );
			}

			minVal.setEnabled ( rulerCheck.isSelected () || gridCheck.isSelected () );
			majVal.setEnabled ( rulerCheck.isSelected () || gridCheck.isSelected () );
		}
	}

	/**
	* Custom made function for plotting labels and combo boxes on a panel.
	* Panel shouldn't be expandable, so concerning attribs are fixed.
	*/
	public void setConst( GridBagConstraints gbc, int x, int y, int anc )
	{
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.fill = GridBagConstraints.NONE;
		gbc.anchor = anc;
		gbc.insets = new Insets ( 5, 5, 5, 5 );

		// Only a light weight so things don't expand too much.
		gbc.weightx = 1;
		gbc.weighty = 1;
	}   
}
