
//Title:        BDialect
//Version:
//Copyright:    Copyright (c) 1998
//Author:       Tristan Austin
//Company:      Synergy
//Description:  Allows other classes to load language resources at run time

package au.edu.swin.synergy.plugin.dialect;

import au.edu.swin.synergy.lib.BGridBagConstraints;
import au.edu.swin.synergy.lib.BListModel;
import au.edu.swin.synergy.lib.io.BTextdb;
import au.edu.swin.synergy.lib.io.BTextdbRecord;

import au.edu.swin.synergy.plugin.mod13.bwt.*;

import java.util.Locale;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * The dialog panel allows the user to change languages on the fly
 * at any time. When the change takes place, a BDialectChangeCE will
 * by fired and all plugins will need to re-load all their text
 * for the screen for re-displaying.
 *
 * @author Tristan Austin
 */
public class BDialectPreferences extends JPanel
{
    JScrollPane dialectScrollPane = new JScrollPane();
    JPanel samplePanel = new JPanel();
    JScrollPane sampleScrollPane = new JScrollPane();
    BorderLayout borderLayout1 = new BorderLayout();
    JButton applyButton = new JButton();
    JLabel dialectLabel = new JLabel();
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    JList dialectList = new JList();
    JTextPane sampleTextArea = new JTextPane();

    /**
     * This is the dialect to tell when the user clicks apply
     */
    BDialect dialect;

    /**
     * This is the locale the users last selected
     */
    Locale currentLocale = Locale.getDefault();

    /**
     * The list model we're using
     */
    BListModel listModel =  new BListModel();

    /**
     * This will handle all the events for this panel
     */
    BEventListener listener = new BEventListener();

    /**
     * Creates a new preferences panel to be added to
     * the preferences plugin. This will initialise the
     * interface with the required text.
     */
    public BDialectPreferences(BDialect dialect)
    {
        this.dialect = dialect;

        samplePanel.setLayout(borderLayout1);
        this.setLayout(gridBagLayout1);
        this.add(dialectScrollPane, new BGridBagConstraints(0, 1, 1, 2, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
        dialectScrollPane.getViewport().add(dialectList, null);
        this.add(samplePanel, new BGridBagConstraints(1, 1, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
        samplePanel.add(sampleScrollPane, BorderLayout.CENTER);
        sampleScrollPane.getViewport().add(sampleTextArea, null);
        this.add(applyButton, new BGridBagConstraints(1, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
        this.add(dialectLabel, new BGridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

        //Set our model
        this.dialectList.setModel(this.listModel);
        this.dialectList.addListSelectionListener(listener);
        this.applyButton.addActionListener(listener);

        //Read in the available locales
        this.doLocaleInitialisation();

        try
        {
        //Set the screen to the appropriate language
        this.doDialect();
        this.doSampleDialect();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * This will load all of the available locales from
     * the dialect.conf file which should exist in the same
     * directory as the BDialectPreferneces class.
     */
    void doLocaleInitialisation()
    {
        BTextdb configFile = new BTextdb();
        BTextdbRecord record;
        BLocale locale;
        boolean keepGoing = true;
        configFile.openFileForRead(getClass().getResourceAsStream("/au/edu/swin/synergy/plugin/dialect/dialect.conf"));

        //Read all the records
        while(keepGoing)
        {
            record = configFile.getRecord();
            keepGoing = ( record != null );

            //Only keep going if we have a valid record
            if (keepGoing)
            {
                locale = new BLocale(record.getElement(0), record.getElement(1), record.getElement(2));
                this.listModel.addElement(locale);
            }
        }

        //Close the file
        configFile.close();
    }

    /**
     * This method is called whenever the text on the user
     * interface needs to be updated. It will use the
     * BDialect plugin to read the appropriate values
     * for the current Locale.
     */
    protected void doDialect()
    {
        BSwingUtilities.setBorder(samplePanel, new TitledBorder(BDialect.getString(this, "sampletitle")));
        BSwingUtilities.setJButtonText(applyButton, BDialect.getString(this, "apply"));
        BSwingUtilities.setJLabelText(dialectLabel, BDialect.getString(this, "dialect"));
        BSwingUtilities.doBListModelRefresh(this.listModel);
    }

    /**
     * This will return the new locale as selected
     * by the user.
     *
     * @return The locale selected by the user
     */
    protected Locale getNewLocale()
    {
        return this.currentLocale;
    }

    /**
     * This method will set the sample text to the currently selected
     * dialect. This is separate from the doDialect because it is
     * called when the user changes their selection, not necessarily
     * when the entire system is updated.
     */
    protected void doSampleDialect()
    {
        BLocale tempLocale = (BLocale)dialectList.getSelectedValue();

        //Make sure we have a locale
        if (tempLocale == null)
        {
            currentLocale = Locale.getDefault();
        }
        else
        {
            currentLocale = tempLocale.getLocale();
        }

        //Get the sample text for this locale
        sampleTextArea.setText(BDialect.getString(this, "sample", currentLocale));
    }

    /**
     * This will listen for changes to the dialect list and change
     * the sample text as required.
     *
     * @autohr Tristan Austin
     */
    public class BEventListener implements ListSelectionListener, ActionListener
    {
        /**
         * When the list is changed, this method is called
         *
         * @param   selectionEvent  The selection event fired
         */
        public void valueChanged(ListSelectionEvent selectionEvent)
        {
            BDialectPreferences.this.doSampleDialect();
        }

        /**
         * This is only for the apply button to send out the
         * dialect update event to inform all the plugins
         * to update their text.
         *
         * @param   actionEvent The event fired
         */
        public void actionPerformed(ActionEvent actionEvent)
        {
            //Tell the plugin to inform the other plugins
            BDialectPreferences.this.dialect.doDialectChange();
        }
    }
}


