

package au.edu.swin.synergy.plugin.dialect;

import java.util.*;

/**
 * This version of Locale simply allows us to
 * the toString() method to return the display name
 * rather than the codes.
 *
 * @author Tristan Austin
 */
public class BLocale
{
    /**
     * This is the locale we represent
     */
    private Locale internalLocale;

    /**
     * Creates a BLocale object as per the java.util.Locale
     * constructor.
     *
     * @param   language    The language of the locale
     * @param   country The country of the locale
     * @param   variant Any variant required
     * @see java.util.Locale
     */
    public BLocale(String language, String country, String variant)
    {
        this.internalLocale = new Locale(language, country, variant);
    }

    /**
     * This will use the display name as the
     * value to show the user.
     *
     * @return The display name for the user
     */
    public String toString()
    {
        Locale currentLocale = Locale.getDefault();

        //Need to get the display name in the appropriate locale
        Locale.setDefault(this.internalLocale);
        String displayName = this.internalLocale.getDisplayName();

        //Set the locale back to what it was
        Locale.setDefault(currentLocale);

        return displayName;
    }

    /**
     * Gets the locale this object represents
     *
     * @return The actual locale object
     */
    public Locale getLocale()
    {
        return this.internalLocale;
    }
}