

package au.edu.swin.synergy.plugin.dialect;

/**
 * The <code>BString</code> is a locale sensitive version of the normal
 * Java <code>String</code>. By using this instead of the normal String,
 * changes to the Locale can be automatically retrieved from their
 * appropriate places depending on the Locale without the source
 * of the String having to do anything.
 *
 * @author Tristan Austin
 */
public class BString
{
    /**
     * This is the object who's package directory contains the files required
     */
    Object source = null;

    /**
     * Indicates whether to looko in the sources package directory
     */
    boolean useSource = false;

    /**
     * These are the arguments to the compound message provided
     */
    Object compoundArgs[] = null;

    /**
     * Indicates whether to create a compound message using the compound args or not
     */
    boolean isCompound = false;

    /**
     * This is the key to lookup the value for in the appropriate file
     */
    String key = null;

    /**
     * This will create a new BString object that represents the string
     * indicated by the key paramater in the globaltext<>.properties file
     * in the package directory of the BDialect class.
     *
     * @param   key The key to be used for the string in the appropriate location
     */
    public BString (String key)
    {
        this.key = key;
    }

    /**
     * This will create a new BString object that represents the string
     * indicated by the key paramater in the screentext<>.properties file
     * in the package directory of the source object. If the key is not found
     * in the screentext<>.properties file, the globaltext<>.properties file
     * will be searched.
     *
     * @param source The object who's package indicates the location of the resource file
     * @param   key The key to be used for the string in the appropriate location
     */
    public BString(Object source, String key)
    {
        this(key);
        this.source = source;
        this.useSource = true;
    }

    /**
     * This will create a new BString object that represents the string
     * indicated by the key paramater in the screentext<>.properties file
     * in the package directory of the source object. If the key is not found
     * in the screentext<>.properties file, the globaltext<>.properties file
     * will be searched.
     * <p>
     * This version allows the user to specify a compound message providing the
     * arguments for the message. This allows the same message to be adjusted
     * correctly for different Locales
     *
     * @param source The object who's package indicates the location of the resource file
     * @param   key The key to be used for the string in the appropriate location
     */
    public BString(Object source, String key, Object compoundArgs[])
    {
        this(source, key);
        this.compoundArgs = compoundArgs;
        this.isCompound = true;
    }

    /**
     * This version of toString() will get the value from
     * the BDialect class in order to ensure that it is correct
     * for the current locale no matter where it is called from.
     *
     * @return The value of this string in respect to the current Locale
     */
    public String toString()
    {
        String currentValue = null;

        //Determine which method to use
        if (this.useSource)
        {
            //Has a source, specify it
            currentValue = BDialect.getString(this.source, this.key);
        }
        else if (this.isCompound)
        {
            //Is a compound message, construct it
            currentValue = BDialect.formatText(BDialect.getString(this.source, this.key), this.compoundArgs);
        }
        else
        {
            //Nothing specified, use global
            currentValue = BDialect.getString(this.key);
        }

        return currentValue;
    }
}
