// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/dialectedit/BDialectEditor.java,v 1.8 1999/10/18 01:57:02 tp079210 Exp $

package au.edu.swin.synergy.plugin.dialectedit;

import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BBlockingComplete;
import au.edu.swin.synergy.kernel.event.BBootCE;

import au.edu.swin.synergy.plugin.dialect.BString;
import au.edu.swin.synergy.plugin.dialect.BDialect;

import au.edu.swin.synergy.debug.Debug;

import au.edu.swin.synergy.plugin.command.BOnCommand;
import au.edu.swin.synergy.plugin.command.event.BRegisterCmdCE;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import au.edu.swin.synergy.lib.BStringList;
import java.util.Vector;
import au.edu.swin.synergy.plugin.mod13.event.BAddWindowCE;
import au.edu.swin.synergy.plugin.mod13.event.BGetFilesCE;
import au.edu.swin.synergy.lib.BGridBagConstraints;
import au.edu.swin.synergy.lib.io.BFileFilter;


import javax.swing.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.text.*;
import java.util.Properties;
import java.util.Enumeration;

/**
 * Dialect Editor
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 *
 */

public class BDialectEditor extends BPlugin
{
	private static final char blockStarts[] = {
	    '\u0000',
	    '\u0080',
	    '\u0100',
	    '\u0180',
	    '\u0250',
	    '\u02B0',
	    '\u0300',
	    '\u0370',
	    '\u0400',
	    '\u0500', // unassigned
	    '\u0530',
	    '\u0590',
	    '\u0600',
	    '\u0700', // unassigned
	    '\u0900',
	    '\u0980',
	    '\u0A00',
	    '\u0A80',
	    '\u0B00',
	    '\u0B80',
	    '\u0C00',
	    '\u0C80',
	    '\u0D00',
	    '\u0D80', // unassigned
	    '\u0E00',
	    '\u0E80',
	    '\u0F00',
	    '\u0FC0', // unassigned
	    '\u10A0',
	    '\u1100',
	    '\u1200', // unassigned
	    '\u1E00',
	    '\u1F00',
	    '\u2000',
	    '\u2070',
	    '\u20A0',
	    '\u20D0',
	    '\u2100',
	    '\u2150',
	    '\u2190',
	    '\u2200',
	    '\u2300',
	    '\u2400',
	    '\u2440',
	    '\u2460',
	    '\u2500',
	    '\u2580',
	    '\u25A0',
	    '\u2600',
	    '\u2700',
	    '\u27C0', // unassigned
	    '\u3000',
	    '\u3040',
	    '\u30A0',
	    '\u3100',
	    '\u3130',
	    '\u3190',
	    '\u3200',
	    '\u3300',
	    '\u3400', // unassigned
	    '\u4E00',
		'\u5300', // split up...
		'\u5700', // split up...
		'\u5B00', // split up...
		'\u5F00', // split up...
		'\u6300', // split up...
		'\u6700', // split up...
		'\u6B00', // split up...
		'\u6F00', // split up...
		'\u7300', // split up...
		'\u7700', // split up...
		'\u7B00', // split up...
		'\u7F00', // split up...
		'\u8300', // split up...
		'\u8700', // split up...
		'\u8B00', // split up...
		'\u8F00', // split up...
		'\u9300', // split up...
		'\u9700', // split up...
		'\u9B00', // split up...
		'\u9F00', // split up...
	    '\uA000', // unassigned
	    '\uAC00',
		'\uB000', // split up...
		'\uB400', // split up...
		'\uB800', // split up...
		'\uBC00', // split up...
		'\uC000', // split up...
		'\uC400', // split up...
		'\uC800', // split up...
		'\uCC00', // split up...
		'\uD000', // split up...
		'\uD400', // split up...
	    '\uD7A4', // unassigned
	    '\uD800',
	    '\uE000',
	    '\uF900',
	    '\uFB00',
	    '\uFB50',
	    '\uFE00', // unassigned
	    '\uFE20',
	    '\uFE30',
	    '\uFE50',
	    '\uFE70',
	    '\uFEFF', // special
	    '\uFF00',
	    '\uFFF0',
		'\uFFFF' // Can't access the last char, but who cares?
	};

	private static final String blockNames[] = {
	    "BASIC_LATIN",
	    "LATIN_1_SUPPLEMENT",
	    "LATIN_EXTENDED_A",
	    "LATIN_EXTENDED_B",
	    "IPA_EXTENSIONS",
	    "SPACING_MODIFIER_LETTERS",
	    "COMBINING_DIACRITICAL_MARKS",
	    "GREEK",
	    "CYRILLIC",
	    null,
	    "ARMENIAN",
	    "HEBREW",
	    "ARABIC",
	    null,
	    "DEVANAGARI",
	    "BENGALI",
	    "GURMUKHI",
	    "GUJARATI",
	    "ORIYA",
	    "TAMIL",
	    "TELUGU",
	    "KANNADA",
	    "MALAYALAM",
	    null,
	    "THAI",
	    "LAO",
	    "TIBETAN",
	    null,
	    "GEORGIAN",
	    "HANGUL_JAMO",
	    null,
	    "LATIN_EXTENDED_ADDITIONAL",
	    "GREEK_EXTENDED",
	    "GENERAL_PUNCTUATION",
	    "SUPERSCRIPTS_AND_SUBSCRIPTS",
	    "CURRENCY_SYMBOLS",
	    "COMBINING_MARKS_FOR_SYMBOLS",
	    "LETTERLIKE_SYMBOLS",
	    "NUMBER_FORMS",
	    "ARROWS",
	    "MATHEMATICAL_OPERATORS",
	    "MISCELLANEOUS_TECHNICAL",
	    "CONTROL_PICTURES",
	    "OPTICAL_CHARACTER_RECOGNITION",
	    "ENCLOSED_ALPHANUMERICS",
	    "BOX_DRAWING",
	    "BLOCK_ELEMENTS",
	    "GEOMETRIC_SHAPES",
	    "MISCELLANEOUS_SYMBOLS",
	    "DINGBATS",
	    null,
	    "CJK_SYMBOLS_AND_PUNCTUATION",
	    "HIRAGANA",
	    "KATAKANA",
	    "BOPOMOFO",
	    "HANGUL_COMPATIBILITY_JAMO",
	    "KANBUN",
	    "ENCLOSED_CJK_LETTERS_AND_MONTHS",
	    "CJK_COMPATIBILITY",
	    null,
	    "CJK_UNIFIED_IDEOGRAPHS (1)",
	    "CJK_UNIFIED_IDEOGRAPHS (2)",
	    "CJK_UNIFIED_IDEOGRAPHS (3)",
	    "CJK_UNIFIED_IDEOGRAPHS (4)",
	    "CJK_UNIFIED_IDEOGRAPHS (5)",
	    "CJK_UNIFIED_IDEOGRAPHS (6)",
	    "CJK_UNIFIED_IDEOGRAPHS (7)",
	    "CJK_UNIFIED_IDEOGRAPHS (8)",
	    "CJK_UNIFIED_IDEOGRAPHS (9)",
	    "CJK_UNIFIED_IDEOGRAPHS (10)",
	    "CJK_UNIFIED_IDEOGRAPHS (11)",
	    "CJK_UNIFIED_IDEOGRAPHS (12)",
	    "CJK_UNIFIED_IDEOGRAPHS (13)",
	    "CJK_UNIFIED_IDEOGRAPHS (14)",
	    "CJK_UNIFIED_IDEOGRAPHS (15)",
	    "CJK_UNIFIED_IDEOGRAPHS (16)",
	    "CJK_UNIFIED_IDEOGRAPHS (17)",
	    "CJK_UNIFIED_IDEOGRAPHS (18)",
	    "CJK_UNIFIED_IDEOGRAPHS (19)",
	    "CJK_UNIFIED_IDEOGRAPHS (20)",
	    "CJK_UNIFIED_IDEOGRAPHS (21)",
	    null,
	    "HANGUL_SYLLABLES (1)",
	    "HANGUL_SYLLABLES (2)",
	    "HANGUL_SYLLABLES (3)",
	    "HANGUL_SYLLABLES (4)",
	    "HANGUL_SYLLABLES (5)",
	    "HANGUL_SYLLABLES (6)",
	    "HANGUL_SYLLABLES (7)",
	    "HANGUL_SYLLABLES (8)",
	    "HANGUL_SYLLABLES (9)",
	    "HANGUL_SYLLABLES (10)",
	    "HANGUL_SYLLABLES (11)",
	    null,
	    "SURROGATES_AREA",
	    null, //"PRIVATE_USE_AREA",
	    "CJK_COMPATIBILITY_IDEOGRAPHS",
	    "ALPHABETIC_PRESENTATION_FORMS",
	    "ARABIC_PRESENTATION_FORMS_A",
	    null,
	    "COMBINING_HALF_MARKS",
	    "CJK_COMPATIBILITY_FORMS",
	    "SMALL_FORM_VARIANTS",
	    "ARABIC_PRESENTATION_FORMS_B",
	    "SPECIALS",
	    "HALFWIDTH_AND_FULLWIDTH_FORMS",
	    "SPECIALS"
	};
	
	JPanel panel = new JPanel();
	JScrollPane scrollPane = new JScrollPane();
	JPanel keyPanel = new JPanel();
	ActionListener actionListener = new OnClick();

	private Font font = new Font("dialog", Font.PLAIN, 16);

	private JTextComponent destination = null;



    /**
     * Return a list of plugin dependancies
	 *
     * @return a list of plugins such as "au.edu.swin.synergy.plugin.mod13.BWindowManager"
	 * or null if there are no dependancies
     */

    public BStringList getDependancies()
	{
		BStringList result = new BStringList();
		result.add("au.edu.swin.synergy.plugin.mod13.BWindowManager");
		return result;
	}

    public void setup(BStringList params)
    {
		// build a list of UnicodeSets...
		Vector sets = new Vector();

		for(int i=0; i < blockNames.length; i++)
		{
			if(blockNames[i] != null)
			{
				UnicodeSet s = new UnicodeSet(blockNames[i], blockStarts[i], (char)((int)blockStarts[i + 1] - 1));
				sets.add(s);
			}
		}

		// add lots of characters to the panel

		panel.setLayout(new BorderLayout());
		JPanel selectPanel = new JPanel();
		selectPanel.setLayout(new FlowLayout());

		JComboBox unicodeSelect = new JComboBox( sets );
		selectPanel.add(unicodeSelect);
		unicodeSelect.addActionListener(new OnUnicodeSelect());

		panel.add(selectPanel, BorderLayout.NORTH);

		keyPanel.setLayout(new GridBagLayout());
		panel.add(scrollPane, BorderLayout.CENTER);

		scrollPane.getViewport().add(keyPanel);

        BAddWindowCE addWindowEvent = new BAddWindowCE( this, panel, new BString(this, "keyboard") );
        dispatchEvent( addWindowEvent );

		
    }

	private void populatePanel(UnicodeSet set)
	{
		SwingUtilities.invokeLater(new DoPopulate(set));
	}

	public void setDestination(JTextComponent fComponent)
	{
		destination = fComponent;
	}
    
    public void onBCommandEvent( Object source, BCommandEvent event )
    {
        if (event instanceof BBootCE)
        {
            // register command line commands        
    		dispatchEvent( new BRegisterCmdCE( this, "edit", new OnEdit() ) );
        }
    }

	class DoPopulate implements Runnable
	{
		UnicodeSet set;

		DoPopulate(UnicodeSet fSet)
		{
			set = fSet;
		}

		public void run()
		{
			keyPanel.removeAll();
			keyPanel.setLayout(new GridBagLayout());
			
			int gridx = 0;
			int gridy = 0;
	
			for(char i=set.getStart(); i <= set.getEnd(); i++)
			{
				JButton label = new JButton("" + i);
				label.setFont(font);
				label.addActionListener(actionListener);
				keyPanel.add(label, new BGridBagConstraints(gridx, gridy, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, 0, 0 ));
				gridx++;
				if(gridx >= 10)
				{
					gridx = 0;
					gridy++;
				}
			}
			keyPanel.invalidate();
			panel.validate();

		}
	}

	class OnClick implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			Debug.doPrintln("Mouse clicked");

			JButton source = (JButton)e.getSource();

			char c = source.getText().charAt(0);

			// post this key to the event queue
			if(destination != null)
			{
				destination.dispatchEvent(new KeyEvent(source, KeyEvent.KEY_TYPED, System.currentTimeMillis(), 0, KeyEvent.VK_UNDEFINED, c));
			}
		}
	}

	class OnUnicodeSelect implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			Debug.doPrintln("In actionListener");
			JComboBox comboBox = (JComboBox) e.getSource();

			Object item = comboBox.getSelectedItem();

			if(item != null)
			{
				Debug.doPrintln("Selected " + item);
				UnicodeSet selection = (UnicodeSet) item;
				Debug.doPrintln("" + ((int)selection.getStart()) + " -> " + ((int)selection.getEnd()));

				populatePanel(selection);
			}
		}

	}


    class OnEdit implements BOnCommand
    {
        public int onCommand( BStringList params, InputStream in, PrintStream out )
        {
            Object compoundArgs[];
			BFileFilter fileFilter = new BFileFilter("properties", "Dialect resource files");

/*			if(params.getCount() != 2)
			{
				out.println("Required parameter missing");
				return 1;
			}//*/
			// load properties from the selected file
			Properties properties = new Properties();

//			String fileName = params.get(1);//"au/edu/swin/synergy/plugin/dialect/globaltext.properties";
			BGetFilesCE getFile = new BGetFilesCE(".");
			getFile.setTitle("Dialect file");
			getFile.setFileFilter(fileFilter);
			dispatchEventAndWait(getFile);

			File fileName = getFile.getSelectedFile();


			try
			{
				FileInputStream input = new FileInputStream(fileName);

				properties.load(input);
				input.close();
			}
			catch(Exception e)
			{
				Debug.doPrintln("Loading properties", e);
				return 1;
			}

			// create a window...
			JPanel editWindow = new JPanel();
            editWindow.setLayout(new GridBagLayout());
            BGridBagConstraints constraints;
			editWindow.setName(fileName.getAbsolutePath());


			OnFocus onFocus = new OnFocus();
			// dump the properties
			Enumeration keys = properties.keys();
            int i = 0;
			while(keys.hasMoreElements())
			{
				String key = (String) keys.nextElement();
				String value = (String) properties.get(key);
				Debug.doPrintln(key + " = " + value);

				// add the text fields
				JLabel l = new JLabel(key);
                constraints = new BGridBagConstraints(0, i, 1, 1, 1, 0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0);
				editWindow.add(l, constraints);

				JTextField field = new JTextField(value);
				field.addFocusListener(onFocus);
                constraints = new BGridBagConstraints(1, i, 1, 1, 1, 0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0);
				editWindow.add(field, constraints);
                i++;
			}

			JButton save = new JButton("save");
			save.addActionListener(new OnSave());

            constraints = new BGridBagConstraints(0, i, 1, 1, 1, 0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0);
			editWindow.add(save, constraints);

			// show the window
            compoundArgs = new Object[]{fileName};

            JScrollPane scrollPane = new JScrollPane();
            scrollPane.getViewport().add(editWindow);
            scrollPane.getHorizontalScrollBar().setUnitIncrement(10);
	        BAddWindowCE addWindowEvent = new BAddWindowCE( this, scrollPane, new BString(this, "editTitle", compoundArgs));
    	    dispatchEvent( addWindowEvent );



            return 0;
        }
        
        public String getHelp( BStringList params )
        {
            return "Start the dialect editor\n" +
					"Usage: edit filename";
        }
    }

	class OnFocus extends FocusAdapter
	{
		public void focusGained(FocusEvent e)
		{
			setDestination((JTextComponent)e.getSource());	
		}
	}

	class OnSave implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			Debug.doPrintln("Saving properties");
			JButton button = (JButton) e.getSource();
			JPanel panel = (JPanel) button.getParent();
			Debug.doPrintln("Saving to " + panel.getName());

			// there is a magic order to the panel
			Component components[] = panel.getComponents();
			int propertyCount = (components.length - 1) / 2;

			Properties properties = new Properties();
			
			for(int i=0; i<propertyCount; i++)
			{
				JLabel label = (JLabel)components[i * 2];
				JTextField field = (JTextField)components[(i * 2) + 1];
				Debug.doPrintln(label.getText() + " = " + field.getText());
				properties.put(label.getText(), field.getText());
			}

			try
			{
				FileOutputStream output = new FileOutputStream(panel.getName());
				properties.store(output, null);
				output.close();
			}
			catch(Exception E)
			{
				Debug.doPrintln("Trying to save properties", E);
			}
		}
	}

	private class UnicodeSet
	{
		String name = "";
		char start = 0;
		char end = 0;

		public UnicodeSet(String fName, char fStart, char fEnd)
		{
			name = fName;
			start = fStart;
			end = fEnd;
		}

		public String getName()
		{
			return name;
		}

		public String toString()
		{
			return name;
		}

		public char getStart()
		{
			return start;
		}

		public char getEnd()
		{
			return end;
		}
	}
}


