package au.edu.swin.synergy.plugin.editmanager;

import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.DataFlavor;

import java.util.Vector;

/**
 * This will allow any type of data at all to be copied to the clipboard
 * and pasted to another location. Specified types can be associated with
 * an object if required.
 *
 * @author Tristan Austin
 */
public class BTransferable implements Transferable
{
	/**
	 * The data being copied
	 */
	Object data;

	/**
	 * This contains a list of supported data flavors in order of detail
	 */
	Vector dataFlavors = new Vector();	

	/**
	 * A simple object to be copied
	 *
	 * @param	data	The data to be copied
	 */
	public BTransferable(Object data)
	{
		Class theClass = data.getClass();
		DataFlavor flavour = new DataFlavor(theClass, "Data");

		this.data = data;
		this.dataFlavors.addElement(flavour);
	}

	/**
	 * This will get the data this transferable object
	 * contains or represents in a format dependent on the
	 * data flavor requested.
	 *
	 * @param	dataFlavor	The type of data required
	 */
	public Object getTransferData(DataFlavor dataFlavor)
	{
		//The first index will always contain the class type
		if (dataFlavor.equals(this.dataFlavors.elementAt(0)))
		{
			return this.data;
		}
		else
			return null;
	}

	/**
	 * This will enable the caller to determine what DataFlavors are
	 * supported by getting a list of them.
	 *
	 * @return The data flavors supported
	 */
	public DataFlavor[] getTransferDataFlavors()
	{
		DataFlavor flavours[] = (DataFlavor[])this.dataFlavors.toArray();

		return flavours;
	}

	/**
	 * Indicates whether the given data flavor is supported
	 *
	 * @param	dataFlavor	The data flavor to check
	 */
	public boolean isDataFlavorSupported(DataFlavor dataFlavor)
	{
		return (this.dataFlavors.contains(dataFlavor));
	}
}
