package au.edu.swin.synergy.plugin.editmanager;

import javax.swing.undo.UndoManager;
import javax.swing.undo.*;
import javax.swing.event.*;

/**
 * This will be the undo listener which needs to be informed
 * of all undo and redoable events. It will inform the edit
 * manager automatically when an event occurs so that the menu
 * is updated as required.
 *
 * @author Tristan Austin
 */
public class BUndoManager extends UndoManager
{
	/**
	 * The edit manager to inform of undo events
	 */
	BEditManager editManager;

	/**
	 * Creates a new undo manager with the given edt manager
	 *
	 * @param	editManager	The manager
	 */
	public BUndoManager(BEditManager editManager)
	{
		this.editManager = editManager;
	}

	/**
	 * Undoable edit events indicate when an edit occurs
	 * and contain the edit object which is placed in the history list.
	 * <p>
	 * This will inform the edit manager of the event so that it can update the
	 * interface as required.
	 *
	 * @param	undoEvent	The undo event
	 */
	public void undoableEditHappened(UndoableEditEvent undoEvent)
	{
		super.undoableEditHappened(undoEvent);
		this.editManager.editOccured(this);
	}//*/

	/**
	 * Calling this method will make this undo manager the
	 * current one in the edit manager so that any undo
	 * calls by the user will occur in this manager.
	 */
	public void setUndoManager()
	{
		this.editManager.setUndoManager(this);
	}

    /**
     * This will remove the current undo manager if it is this one
     */
    public void relinquishUndoManager()
    {
        this.editManager.relinquishUndoManager(this);
    }

    /**
     * This will unregister this undo manager with the edit manager
     */
    public void unregisterUndoManager(Object source)
    {
        this.editManager.unregisterUndoManager(source);
    }

}
