package au.edu.swin.synergy.plugin.editmanager.event;

import au.edu.swin.synergy.plugin.editmanager.BUndoManager;

import javax.swing.undo.UndoableEditSupport;

import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;

/**
 * Registers the given Undo support object with the edit manager
 * so that it may be used in combination with the
 */
public class BRegisterUndoCE extends BCommandEvent
{
	/**
	 * The listener for the source to use
	 */
	UndoableEditSupport undoSupport;

    BUndoManager undoManager;

	/**
	 * This will simply create the event with the given source
	 *
	 * @param	source	The source of the event
	 */
	public BRegisterUndoCE(Object source)
	{
		super(source);
	}

	/**
	 * Allows the edit manager to set the edit listener for the
	 * source to add to whatever it wants.
	 *
	 * @param	listener	The listener for the source to use
	 */
	public void setUndoSupport(UndoableEditSupport undoSupport)
	{
		this.undoSupport = undoSupport;
	}

    /**
     * Gets the undo manager to be used in places that
     * already have an undo support or other type of class to dispatch
     * edit events.
     *
     * @return The undo manager that listens for events
     */
    public BUndoManager getUndoListener()
    {
        return this.undoManager;
    }

    /**
     * Sets the undo manager to be used in places that
     * already have an undo support or other type of class to dispatch
     * edit events.
     *
     * @param   undoManager The undo manager that listens for events
     */
    public void setUndoListener(BUndoManager undoManager)
    {
        this.undoManager = undoManager;
    }

	/**
	 * Gets the undo listener to add
	 *
	 * @return The undo listener
	 */
	public UndoableEditSupport getUndoSupport()
	{
		return this.undoSupport;
	}
}
