package au.edu.swin.synergy.plugin.help;

import au.edu.swin.synergy.plugin.help.BHelpUtil;
import au.edu.swin.synergy.plugin.help.BHelpManager;

import javax.swing.*;
import javax.swing.text.html.FormView;
import java.util.Vector;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.net.URL;
import javax.swing.event.*;
import au.edu.swin.synergy.debug.Debug;

import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;

public class BHelpViewpane extends JPanel
{
	BHelpManager helpManager;

    Vector historyList;
    int historyListIndex;

    JButton homeButton;
    JButton backButton;
    JButton forwardButton;

    JEditorPane viewpane;
    JScrollPane scrollpane;

    public BHelpViewpane ( BHelpManager hm )
    {
        setPreferredSize ( new Dimension ( 400, 500 ) );
        setLayout ( new GridBagLayout () );
		GridBagConstraints c = new GridBagConstraints ();

        homeButton = new JButton ( BDialect.getString ( this, "homebutton" ) );
        homeButton.addActionListener ( new onHomeButton () );
        BHelpUtil.setConst ( c, 0, 0, 1, 1, BHelpUtil.kNone, BHelpUtil.kCenter, 0 );
        add ( homeButton, c );

        backButton = new JButton ( BDialect.getString ( this, "backbutton" ) );
        backButton.addActionListener ( new onBackButton () );
        BHelpUtil.setConst ( c, 1, 0, 1, 1, BHelpUtil.kNone, BHelpUtil.kCenter, 0 );
		add ( backButton, c );
        backButton.setEnabled ( false );

        forwardButton = new JButton ( BDialect.getString ( this, "forwardbutton" ) );
        forwardButton.addActionListener ( new onForwardButton () );
        BHelpUtil.setConst ( c, 2, 0, 1, 1, BHelpUtil.kNone, BHelpUtil.kCenter, 0 );
        add ( forwardButton, c );
        forwardButton.setEnabled ( false );

		viewpane = new JEditorPane ();
		viewpane.setEditable ( false );
        viewpane.addHyperlinkListener ( new onHyperlink () );
		scrollpane = new JScrollPane ( viewpane );
        BHelpUtil.setConst ( c, 0, 1, 3, 1, BHelpUtil.kBoth, BHelpUtil.kCenter, 1 );
        add ( scrollpane, c );

        historyListIndex = -1;
        historyList = new Vector ();
		helpManager = hm;

        setHelpPage ( hm.getHome () );
    }

	public void doDialect ()
	{
		BSwingUtilities.setJButtonText ( homeButton, BDialect.getString ( this, "homebutton" ) );
		BSwingUtilities.setJButtonText ( backButton, BDialect.getString ( this, "backbutton" ) );
		BSwingUtilities.setJButtonText ( forwardButton, BDialect.getString ( this, "forwardbutton" ) );
	}

    private void validateBackForward ()
    {
        // If at start of list...
        if ( historyListIndex > 0 )
        {
            backButton.setEnabled ( true );
        }
        else
        {
            backButton.setEnabled ( false );
        }

        // If at end of list... ( NOTE: This could be out by one! )
        if ( historyListIndex < historyList.size () - 1 )
        {
            forwardButton.setEnabled ( true );
        }
        else
        {
            forwardButton.setEnabled ( false );
        }
    }

    public void setHelpPage ( URL url )
    {
        try
        {
            viewpane.setPage ( url );			
			historyListIndex++;
			Debug.doPrintln ( "Current index " + historyListIndex );
			if ( historyListIndex == historyList.size () )
			{
				Debug.doPrintln ( "Adding url to end of list" );				
				historyList.add ( url );
			}
			else
			{
				historyList.insertElementAt ( url, historyListIndex );
				Debug.doPrintln ( "Inserting at " + historyListIndex );
				removeRange ( historyListIndex + 1, historyList.size () - 1 );
			}
            validateBackForward ();
        }
        catch ( Exception ex )
        {
            Debug.doPrintln ( ex.getMessage () );
			// Display error html page.
        }
    }
	
	/** Removes a range of items from the history list **/
	private void removeRange ( int from, int to )
	{
		int amount = to - from;

		// For some reason, this stuffs up if you start the initial value
		// to from, and the terminating statement to "< to".  It works like
		// this so I'm happy.
		Debug.doPrintln ( "Removing " + from + "-" + to + " from history." );
		for ( int i = 0; i < amount; i++ )
		{
			historyList.removeElementAt ( i );
		}
	}

    public class onBackButton implements ActionListener
    {
        public void actionPerformed ( ActionEvent e )
        {
			try
			{
				historyListIndex--;
				URL url = ( URL ) historyList.elementAt ( historyListIndex );
				Debug.doPrintln ( "back index now = " + historyListIndex );

				// We do this instead of calling "setHelpPage" so the url doesn't
				// get reinserted to the history list.
				viewpane.setPage ( url );
				validateBackForward ();
			}
			catch ( Exception exception )
			{
				Debug.doPrintln ( exception.getMessage () );
			}
        }
    }

    public class onForwardButton implements ActionListener
    {
        public void actionPerformed ( ActionEvent e )
        {
			try
			{
				historyListIndex++;
				URL url = ( URL ) historyList.elementAt ( historyListIndex );
				Debug.doPrintln ( "For Index now = " + historyListIndex );

				// We do this instead of calling "setHelpPage" so the url doesn't
				// get reinserted to the history list.
				viewpane.setPage ( url );
				validateBackForward ();
			}
			catch ( Exception exception )
			{
				Debug.doPrintln ( exception.getMessage () );
			}
        }
    }

    public class onHomeButton implements ActionListener
    {
        public void actionPerformed ( ActionEvent e )
        {
			setHelpPage ( helpManager.getHome () );
        }
    }

    public class onHyperlink implements HyperlinkListener
    {
        public void hyperlinkUpdate ( HyperlinkEvent e )
        {
			HyperlinkEvent.EventType evtType = e.getEventType ();

			if ( evtType == HyperlinkEvent.EventType.ENTERED )
			{
				Debug.doPrintln ( "Over link..." );
				viewpane.setCursor ( Cursor.getPredefinedCursor ( Cursor.HAND_CURSOR ) );
			}
			else if ( evtType == HyperlinkEvent.EventType.EXITED )
			{
				Debug.doPrintln ( "Left link..." );
				viewpane.setCursor ( Cursor.getDefaultCursor () );
			}
			else
			{
				Debug.doPrintln ( "URL selected..." );
				URL nextUrl = e.getURL ();
				setHelpPage ( nextUrl );
			}
        }
    }
}
