// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/mailclient/BMailGui.java,v 1.37 1999/10/21 22:18:59 tp079210 Exp $
//

package au.edu.swin.synergy.plugin.mailclient;

import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.event.BBootCE;
import au.edu.swin.synergy.plugin.command.event.BRegisterCmdCE;

import au.edu.swin.synergy.lib.BTextArea;
import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.lib.BGridBagConstraints;
import au.edu.swin.synergy.plugin.smtpsvr.event.BMailCE;
import au.edu.swin.synergy.plugin.directsvr.event.BDirectMailCE;
import au.edu.swin.synergy.plugin.addressbook.event.BAddressDisplayCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddWindowCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddMenuItemCE;
import au.edu.swin.synergy.plugin.mod13.event.BCloseWindowCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddStatusCE;
import au.edu.swin.synergy.plugin.mod13.event.BRemoveStatusCE;
import au.edu.swin.synergy.plugin.prefs.event.BAddPrefsCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddToolBarCE;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.plugin.command.BOnCommand;
import au.edu.swin.synergy.plugin.dialect.BString;
import au.edu.swin.synergy.plugin.dialect.event.BDialectChangeCE;
import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;

import java.awt.event.*;
import java.awt.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.text.JTextComponent;
import javax.swing.table.AbstractTableModel;
import java.io.*;
import java.net.URL;
import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.plugin.dialect.BString;

import au.edu.swin.synergy.plugin.help.BHelpItem;
import au.edu.swin.synergy.plugin.help.BHelpUtil;
import au.edu.swin.synergy.plugin.help.BHelpTreeModel;
import au.edu.swin.synergy.plugin.help.event.BRegisterHelpCE;


/**
 * Provides the user interface for sending and receiving mail events
 *
 * @author Matthew Evans
 * @copyright Team Synergy, 1999
 */
public class BMailGui extends BPlugin
{

   /**
    * The collection of messages that have been received during a Cohecion session
    */
   Vector receivedMessages = new Vector();


   BMailPreferences mailPrefs = new BMailPreferences();
   
   /**
    * Identifies whehter a mail window will open when a message is received with
    * the message contens displayed appropriataely
    */
   boolean openOnReceive = false;

   /**
    * Identifies whether a sound will be played when a message is received
    */
   boolean soundOnReceive = true;

   /**
    * Identifies whether the status bar currently displays the mail status component
    */
   boolean statusAttended = false;

   ImageIcon noMailIcon = null;
   ImageIcon newMailIcon = null; 

   /**
    * The label containing the mail status image
    */
   JLabel statusLabel = new JLabel();

   /**
    * the panel to be added to the status bar of the application
    */
   JPanel statusPanel = new JPanel();

    /**
     * Initialses the BMailGui plugin, by registering itself on the client, and
     * adding menu items to the client gui
     */
    public void setup(BStringList params)
    {
      BRegisterCmdCE event = new BRegisterCmdCE( this, "mail", new BMailCmd() );
      dispatchEvent( event );

      //Initialise the dialect code
      this.doDialect();

      //Set the mail icons
      this.setupMailStatus();

      addMessagingMenu();
	  addMailToolBar();

      BAddPrefsCE prefEvent = new BAddPrefsCE( this, mailPrefs, mailPrefs.getPrefTabString (), mailPrefs.getPrefToolTip () );
      this.dispatchEvent( prefEvent );
    }

    /**
     * Return a list of plugin dependancies
	 *
     * @return a list of plugins such as "au.edu.swin.synergy.plugin.mod13.BWindowManager"
	 * or null if there are no dependancies
     */

    public BStringList getDependancies()
	{
		BStringList result = new BStringList();
		result.add("au.edu.swin.synergy.plugin.mod13.BWindowManager");
		return result;
	}

    /**
     * When the dialect changes, this method is called to update everything
     */
    protected void doDialect()
    {
        noMailIcon = BSwingUtilities.getImage(this, BDialect.getString(this, "noMailIcon"));
        newMailIcon = BSwingUtilities.getImage(this, BDialect.getString(this, "newMailIcon"));

        //Set the prefs dialect
        mailPrefs.doDialect();
    }

    /**
     * Catches any relevant events for the BMailGUI plugin
     * Please complete the missing tags for onBCommandEvent
     * @param source the instigator of the event caught by this method
     * @param event the command evend in question
     */
    public void onBCommandEvent( Object source, BCommandEvent event )
    {
		if ( event instanceof BBootCE )
		{
			registerHelp ();
		}
        else if( event instanceof BDirectMailCE )
        {
            Debug.doPrintln( "\nBMailGui: onCE: received direct message" );
            BDirectMailCE directMailEvent = ( BDirectMailCE ) event;
            BMessage messageReceived = directMailEvent.getMessage();

            //add the message to the list received so far
            this.receivedMessages.addElement(messageReceived);

			if (! statusAttended)
			{
				this.addMailStatus();
			}

            if (this.mailPrefs.isDisplay())
            {
              //create a new BMailWindow with the contents of 'messageReceived'
              //in the relevant fields
              addMailWindow(messageReceived);
            }

            if (this.mailPrefs.isSoundEnabled())
            {
              //insert this line in onCommandEvent
              JPanel toolkitPanel = new JPanel();
              toolkitPanel.getToolkit().beep();
            }
        }
        else if (event instanceof BDialectChangeCE)
        {
            //Make sure we have the right text
            this.doDialect();
        }
    }

   private void addMailToolBar()
   {
	  String path = new String ( "/au/edu/swin/synergy/plugin/mailclient/images/" );

   	  JButton sendMailButton = new JButton(BSwingUtilities.getImage(this, path + "messagesend.gif"));
   	  JButton viewMailButton = new JButton(BSwingUtilities.getImage(this, path + "messageview.gif"));
	  sendMailButton.setToolTipText((new BString(this, "composemessage")).toString());
	  viewMailButton.setToolTipText((new BString(this, "receivedmessages")).toString());
	  sendMailButton.addActionListener(new onSendMessage());
	  viewMailButton.addActionListener(new onViewMessages());

	  BAddToolBarCE addToolbar = new BAddToolBarCE(this, 85);
	  addToolbar.addButton(sendMailButton);
	  addToolbar.addButton(viewMailButton);
	  dispatchEvent(addToolbar);
   }

	private void registerHelp ()
	{
		try
		{
			BHelpTreeModel tree = new BHelpTreeModel ( "Messaging" );
			String path = "/au/edu/swin/synergy/plugin/mailclient/help/";

			BHelpItem dm = new BHelpItem ( "Deleting Instant Messages", path + "deleteMessages.html" );
			BHelpItem m = new BHelpItem ( "Messaging", path + "messaging.html" );
			BHelpItem sm = new BHelpItem ( "Sending an Instant or Email Message", path + "sendMessage.html" );
			BHelpItem vm = new BHelpItem ( "Viewing Instant Messages", path + "viewMessages.html" );


			tree.addHelpItem ( null, m );
			tree.addHelpItem ( null, sm );
			tree.addHelpItem ( null, vm );
			tree.addHelpItem ( null, dm );

			BRegisterHelpCE event = new BRegisterHelpCE ( this, tree );
			dispatchEvent ( event );
		}
		catch ( Exception e )
		{
			e.printStackTrace ();
		}
	}

	private void setupMailStatus()
	{
        statusPanel.setSize(30,25);
        statusPanel.setLayout( new GridLayout( 1, 0 ) );
        statusLabel.setVerticalAlignment( SwingConstants.CENTER );
        statusLabel.setHorizontalAlignment( SwingConstants.CENTER );
        setStatusNormal();
        statusPanel.add(statusLabel);
        statusLabel.addMouseListener(new onMailStatusAction());
        statusLabel.setIcon(newMailIcon);
        statusPanel.setToolTipText(BDialect.getString(this, "newMailTip"));
	}

    /**
     * Adds a panel to the status bar on the main Cohesion application frame.
     * The Mail status panel contains an image which is modified when a message
     * is received, and changed back to its initial status when the messages
     * have been read
     */
    private void addMailStatus()
    {
        BAddStatusCE addStatus = new BAddStatusCE( this, statusPanel );
        dispatchEvent( addStatus );
		statusAttended = true;
    }

	private void removeMailStatus()
	{
		BRemoveStatusCE removeStatus = new BRemoveStatusCE(this, statusPanel);
        dispatchEvent( removeStatus );
	}

    class onMailStatusAction implements MouseListener
    {
       public void mouseClicked(MouseEvent event)
       {
          //addViewMessagesWindow();
       }

       public void mousePressed(MouseEvent event)
       {
          //addViewMessagesWindow();
       }

       public void mouseReleased(MouseEvent event)
       {
          if (event.getClickCount() == 2)
             addViewMessagesWindow();
       }

       public void mouseEntered(MouseEvent event)
       {
          //addViewMessagesWindow();
       }

       public void mouseExited(MouseEvent event)
       {
          //addViewMessagesWindow();
       }
    }

    /**
     * sets the icon on the status bar to the newMailIcon, to notify the user
     * that there is mail waiting to be read
     */
    private void setStatusNew()
    {
        statusLabel.setIcon(newMailIcon);
        statusPanel.setToolTipText(BDialect.getString(this, "newMailTip"));
    }


    /**
     * sets the icon on the status bar to the noMailIcon, to notify the user
     * that there is no new mail waiting to be read
     */
    private void setStatusNormal()
    {
       statusLabel.setIcon(noMailIcon);
       Object args[] = new Object[]{new Integer(receivedMessages.size())};
       statusPanel.setToolTipText(BDialect.formatText(BDialect.getString(this, "receivedMailTip"), args));
    }

    /**
     * Returns the receivers, as a BStringList, from the 'receivers' textArea.
     *
     * @param groupedReceivers a string containing the contents of the 'receviers' field.
     * Please complete the missing tags for getReceivers
     * @return a BStringList containing a collection of receivers of the message
     */
    private BStringList getReceivers( String groupedReceivers )
    {
        BStringList receivers = new BStringList();

        if( groupedReceivers.length() > 0 )
        {
            String tempReceiver = new String();

            for( int i = 0; i < groupedReceivers.length(); i++ )
            {
                if( groupedReceivers.substring( i, i + 1 ).equals( BDialect.getString(this, "separator") ) )
                {
                    receivers.add( tempReceiver );
                    tempReceiver = new String();
                }
                else
                {
                  tempReceiver += groupedReceivers.charAt( i );
                }
            }

            if( tempReceiver.length() > 0 )
            receivers.add( tempReceiver );

            return receivers;
        }
        else
        {
          return null;
        }
    }

    /**
     * displays a message indicating an error of some kind in a JOptionPane
     *
     * @param message the message to dislay
     */
    void displayErrorMessage(String message)
    {
       BStandardOutCE output = new BStandardOutCE(this, message);
       dispatchEvent(output);
    }

    /**
     * Dispatches an add menu event with a messaging menu containing a 'send message'
     * menu item.
     */
    private void addMessagingMenu()
    {
        JMenuItem sendMenuItem = new JMenuItem();
        JMenuItem viewMenuItem = new JMenuItem();
        BAddMenuItemCE menuEvent = new BAddMenuItemCE( this, new BString(this, "menuName") );
        menuEvent.addMenuItem(sendMenuItem, new BString(this, "send"));
        menuEvent.addMenuItem(viewMenuItem, new BString(this, "view"));
		menuEvent.setPreferredMenuMnemonic(new BString( this, "menuKey" ));

        //Set up the menu
        sendMenuItem.addActionListener( new onSendMessage() );
        viewMenuItem.addActionListener( new onViewMessages() );
        dispatchEvent( menuEvent );

        Debug.doPrintln( "\nBMailGui: addMessagingMenu: dispatched add menu event" );
    }
    
    /**
     * Dispatches an add window event, contianing the mail interface.  This
     * interface is created with empty fields and 'Send' and 'Cancel' buttons for
     * creating new messages, or with non-empty fields and an 'OK' button for
     * a window displaying a recieved direct message
     *
     * @param directMessage a BMessage containing the data of a received
     *      direct message
     * Please complete the missing tags for addMailWindow
     */
    private void addMailWindow( BMessage directMessage )
    {
        BMailWindow mailWindow;

        mailWindow = new BMailWindow( directMessage, this );

        BAddWindowCE addWindow = new BAddWindowCE( this, mailWindow, new BString(this, "windowtitle") );
        addWindow.setDialog( true );

        Debug.doPrintln( "\nBMailGui: addMailWindow: Adding a mail window" );

        dispatchEvent( addWindow );
    }

    /**
     * Dispatches an event to add the View mesages window
     */
    private void addViewMessagesWindow()
    {
        BViewMessagesWindow viewWindow;

        viewWindow = new BViewMessagesWindow( this.receivedMessages, this );

        BAddWindowCE addWindow = new BAddWindowCE( this, viewWindow, new BString(this, "ReceivedMessages") );
        addWindow.setDialog( true );

        Debug.doPrintln( "\nBMailGui: addViewMessagesWindow: Adding a view window" );

        dispatchEvent( addWindow );
    }
    
    /**
     * The dialog that displays direct mail messages as they are received, and
     * allows users to enter a new message to send (directly or indirectly).
     *
     * @author Matthew Evans
     */
    public class BMailWindow extends JPanel
    {
        JButton toButton = new JButton();
        JLabel fromLabel = new JLabel();
        JScrollPane receiversScrollPane = new JScrollPane( JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        BTextArea toAndFromField = new BTextArea();
        JLabel subjectLabel = new JLabel();
        JLabel msgLabel = new JLabel();
        JTextField subjectField = new JTextField();
        JScrollPane messageScrollPane = new JScrollPane( JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        BTextArea messageArea = new BTextArea();
        JButton sendButton = new JButton();
        JButton cancel_deleteButton = new JButton();
        JButton okButton = new JButton();
        GridBagLayout gridBagLayout = new GridBagLayout();
        BMessage directMessage;

        BMailGui parent;

        /**
         * Constructs the mail window, and sets interchangeable buttons and labels,
         * such as toButton  and fromlabel, and send & cancel Button's with okButton.
         *
         * @param receiving is this window being created to display a received direct
         * mail message, or to send a new mail message
         * @param parent the parent frame of this dialog.
         */
        public BMailWindow( BMessage directMessage, BMailGui parent )
        {
            this.directMessage = directMessage;
            this.parent = parent;
            
            this.setLayout( gridBagLayout );
            setSize( 492, 328 );
            
            this.add( receiversScrollPane, new BGridBagConstraints( 1, 0, 3, 1, 1.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets( 5, 5, 5, 5 ), 0, 7 ) );

            toAndFromField.setLineWrap( true );
            receiversScrollPane.getViewport().add( toAndFromField, null );

            subjectLabel.setForeground( java.awt.Color.black );
            this.add( subjectLabel, new BGridBagConstraints( 0, 1, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );

            this.add( subjectField, new BGridBagConstraints( 1, 1, 3, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets( 5, 5, 5, 5 ), 0, 5 ) );

            msgLabel.setForeground( java.awt.Color.black );
            this.add( msgLabel, new BGridBagConstraints( 0, 2, 1, 1, 0.0, 1.0
            , GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );

            this.add( messageScrollPane, new BGridBagConstraints( 1, 2, 3, 1, 1.0, 1.0
            , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );

            messageArea.setFont( new Font( "Dialog", Font.PLAIN, 12 ) );
            messageArea.setLineWrap( true );
            messageScrollPane.getViewport().add( messageArea, null );

            if( directMessage != null )
            {
                add( fromLabel, new BGridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 10, 0 ) );

                //!!**##
                add( okButton, new BGridBagConstraints( 1, 3, 1, 1, 0.0, 0.0
                , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets( 5, 5, 5, 5 ), 10, 2 ) );

                //set the text fields with the contents of the message
                BStringList receivers = directMessage.getReceivers();
                String receiversString = new String();

                //get all receivers into a single string
                for (int i = 0; i < receivers.getCount(); i++)
                {
                    receiversString += receivers.get(i) + BDialect.getString(this, "separator");
                }

                toAndFromField.setText(receiversString);
                toAndFromField.setEnabled( false );
                subjectField.setText( directMessage.getSubject() );
                subjectField.setEnabled( false );
                messageArea.setText( directMessage.getMessage() );
                messageArea.setEnabled( false );
            }
            else
            {
                this.add( toButton, new BGridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 10, 0 ) );
                this.add( sendButton, new BGridBagConstraints( 1, 3, 1, 1, 0.0, 0.0
                , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets( 5, 5, 5, 5 ), 15, 2 ) );
                this.add( cancel_deleteButton, new BGridBagConstraints( 2, 3, 1, 1, 0.0, 0.0
                , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 15, 2 ) );

            }

            //add listeners
            MailGuiAction mailGuiAction = new MailGuiAction();
            toButton.addActionListener( mailGuiAction );
            sendButton.addActionListener( mailGuiAction );
            cancel_deleteButton.addActionListener( mailGuiAction );
            okButton.addActionListener( mailGuiAction );

            //Do dialect
            this.doDialect();
        }

        /**
         * This will set the text on the interface to suit the current Locale
         */
        protected void doDialect()
        {
            subjectLabel.setText(BDialect.getString(this, "subject"));
            msgLabel.setText( BDialect.getString(this, "message"));

            if (directMessage != null)
            {
                fromLabel.setText(BDialect.getString(this, "from"));
                okButton.setText( BDialect.getString("ok") );
                toAndFromField.setText(toAndFromField.getText().replace(';', BDialect.getString(this, "separator").charAt(0)));
            }
            else
            {
                toButton.setText( BDialect.getString(this, "to") );
                sendButton.setText( BDialect.getString(this, "send") );
                cancel_deleteButton.setText( BDialect.getString("cancel") );
            }
        }

        /**
         * appends a string of receiver names to the toAndFromField
         *
         * @param receivers a string of receivers to be added to the toAndFromField
         */
        public void appendReceivers(String receivers)
        {
            if ( toAndFromField.getText().trim().length() > 0 )
               toAndFromField.setText(toAndFromField.getText().trim() + BDialect.getString(this, "separator") + receivers);
            else
               toAndFromField.setText(receivers);
        }
        
        /**
         * Dispatches an event to display the address book gui
         */
        void toButton_ActionPerformed()
        {
            BAddressDisplayCE displayAddressBook = new BAddressDisplayCE(this, new BAddressDisplayCC());
            parent.dispatchEvent(displayAddressBook);
        }
        
        //!! needs to be modifiedto get the sender of the message (currently hardcoded as Snaggs...)
        /**
         * Creates a new BMessage object and dispatches it with the details of
         * the message (sender, receivers, subject, message) in a BMailCE
         * for distribution by the client kernel.
         */
        void sendButton_ActionPerformed()
        {
            BStringList receivers = getReceivers( toAndFromField.getText() );
            
            if( receivers != null )
            {
                BMessage messageData = new BMessage( "Snaggs <me@it.swin.edu.au>", receivers );
                messageData.setSubject( subjectField.getText() );
                messageData.setMessage( messageArea.getText() );
                
                if (mailPrefs.isSignatureAttached())
                {
                   String textMessage = messageArea.getText();
                   textMessage += "\n\n" + getAttachment();
                   messageData.setMessage(textMessage);
                }
                else
                {
                   messageData.setMessage( messageArea.getText() );
                }

                BMailCE messageEvent = new BMailCE(parent, messageData);
				messageEvent.setCompletionObject(new SendBMailComplete());
                parent.dispatchEventServer( messageEvent );
                closeMailWindow();
                Debug.doPrintln( "\nBMailGui: MailWindow: sendBtn: dispatched new BmailCE" );
            }
            else
            {
                Debug.doPrintln( "\nBMailGui: MailWindow:  sendBtn: Not sending... no receivers listed" );
            }
        }
        

        String getAttachment()
        {
           String attachment = new String();
           BufferedReader attachmentStream = null;
           
           try
           {
              attachmentStream = new BufferedReader(new FileReader(mailPrefs.getAttachmentName()));

              String line = attachmentStream.readLine();
              
              while (line != null) 
              {
                 attachment += "\n" + line;
                 line = attachmentStream.readLine();
              }
           }
           catch (FileNotFoundException missingFileException)
           {
              Debug.doPrintln("The attachment file was not found");
           }
           catch (IOException ioException)
           {
              Debug.doPrintln("Error opening attachment file");
           }
           
           return attachment;
        }

        /**
         * Dispatches an event to the serverto close an instantiation of BMailWindow
         */
        void closeMailWindow()
        {
            BCloseWindowCE closeWindow = new BCloseWindowCE( parent, this );
            parent.dispatchEvent( closeWindow );
        }

        /**
         *  Action listener for the BMailWindow
         */
        class MailGuiAction implements ActionListener
        {
            /**
             * handles action events on the BMailWindow
             *
             * @param event the ActionEvent raised on the BMailWindow
             */
            public void actionPerformed( java.awt.event.ActionEvent event )
            {
                Object object = event.getSource();

                if( object == toButton )
                  toButton_ActionPerformed();
                else if( object == sendButton )
                   sendButton_ActionPerformed();
                else if( object == cancel_deleteButton )
                   closeMailWindow();
                else if( object == okButton )
                   closeMailWindow();
            }
        }


        /**
         * The CommandComplete event to attach to a BAddressDisplayCE
         *
         * @author Matthew Evans
         */
        class BAddressDisplayCC extends BCommandComplete
        {
           /**
            * When the users have been retrieved this method is called
            *
            * @param   event   The event that was originally fired
            * @param   status  The success status of the operation
            */
           public void doComplete( BCommandEvent event, int status )
           {
               Debug.doPrintln("BMailGui: BAddressDisplayCC: caught completed event=> " + event.toString());

               if (event.getCompletionStatus() == BCommandComplete.kNetError)
                  displayErrorMessage(BDialect.getString(this, "networkError"));
               else if (event.getCompletionStatus() == BCommandComplete.kError)
                  displayErrorMessage(BDialect.getString(this, "generalError"));
               else
               {
                  BAddressDisplayCE addressInfo = (BAddressDisplayCE) event;
                  String selectedReceivers = addressInfo.getReceivers();

                  if (selectedReceivers != null)
                     appendReceivers(selectedReceivers);
               }
           }

         }        
    }

	class SendBMailComplete extends BCommandComplete
	{
  		public void doComplete( BCommandEvent event, int status )
  		{
			Debug.doPrintln("Mail Sent status = " + status);
			if(status == BCommandComplete.kSuccess)
			{
				// show a message
				dispatchEvent(new BStandardOutCE(this , "Message successfuly sent"));
			}
			else if(status == BCommandComplete.kNetError)
			{
				// show a message
				dispatchEvent(new BStandardOutCE(this , "Mail can only be sent when you are logged in"));
			}
			else if(status == BCommandComplete.kDefault)
			{
				dispatchEvent(new BStandardOutCE(this , "No plugins are available to process this message"));
			}
		}

	}
	
    /**
     * action listener for the the sendMenuItem on the mail gui
     *
     * @author Matthew Evans
     */
    class onSendMessage implements java.awt.event.ActionListener
    {
        /**
         * handles the action event from the sendMenuItem
         *
         * @param event The ActionEvent raised from the sendMenuItem
         */
        public void actionPerformed( ActionEvent event )
        {
           addMailWindow( null );
        }
    }


    /**
     * Action listener for the the viewMessagesMenuItem on the mail gui
     *
     * @author Matthew Evans
     */
    class onViewMessages implements java.awt.event.ActionListener
    {
        /**
         * Handles the action event from the sendMenuItem
         *
         * @param event The ActionEvent raised from the viewMessagesMenuItem
         */
        public void actionPerformed( ActionEvent e )
        {
          addViewMessagesWindow();
        }
    }


    /**
     * This class defines the command line interfacing for sending messages
     *
     * @author Matthew Evans
     */
    class BMailCmd implements BOnCommand
    {
        /**
         * Handles a console command to send mail
         *
         * @params the collection of parameters entered for the mail command
         */
        public int onCommand( BStringList params, InputStream in, PrintStream out )
        {
            if( params.getCount() > 1 )
            {
                BStringList receivers = getReceivers( params.get( 1 ) );
                //receivers.add("ta@it.swin.edu.au");
                BMessage messageData = new BMessage( "Snaggs <me@it.swin.edu.au>", receivers );

                if( params.getCount() > 2 )
                {
                    messageData.setSubject( params.get( 2 ) );
                }

                if( params.getCount() > 3 )
                {
                    String messageText = new String();

                    for( int i = 3; i < params.getCount(); i++ )
                    {
                        messageText = messageText + " " + params.get( i );
                    }

                    messageData.setMessage( messageText );
                }
                
                BMailCE messageEvent = new BMailCE (this, messageData);
                dispatchEvent( messageEvent );
            }
            else
            {
                System.out.println( getHelp( new BStringList() ) );
            }
            
            return 0;
        }
        
        /**
         * Returns help info for sending mail form the command line
         *
         * @param params words entered after "help mail" was typed.
         */
        public String getHelp( BStringList params )
        {
            return "mail receiver1;receiver2;..;receiverN subject message";
        }

    }

    /**
     * The window containing direct messages received during a session
     *
     * @author Matthew Evans
     */
    public class BViewMessagesWindow extends JPanel
    {
      JScrollPane receviedScrollPane = new JScrollPane();
      JTable receivedTable = new JTable();
      JButton viewMessageButton = new JButton();
      JButton closeButton = new JButton();
      JButton deleteButton = new JButton();
      JPanel buttonPanel = new JPanel();
      GridBagLayout gridBagLayout1 = new GridBagLayout();
      MessageTableModel tableModel;
      BMailGui parent;

      /**
       * Constructs the view messages window
       * @param data a collection of messages that have been recieved during by
       * a client during a session
       * @param parent the BMailGui object that created this instantiation of the
       * viewMessages window
       */
      public BViewMessagesWindow(Vector data, BMailGui parent )
      {
        this.parent = parent;
        tableModel = new MessageTableModel(data);
        this.setLayout(gridBagLayout1);

        this.add(receviedScrollPane, new BGridBagConstraints(0, 0, 2, 3, 1.0, 1.0
                ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(10, 10, 10, 10), 0, 0));
        receivedTable.setModel(tableModel);
        receivedTable.setRowSelectionAllowed(true);
        receivedTable.setCellSelectionEnabled(false);
        receivedTable.setColumnSelectionAllowed(false);
        //only one row can be selected at a time
        receivedTable.setSelectionMode(0);
        receviedScrollPane.getViewport().add(receivedTable, null);

        this.add(buttonPanel, new BGridBagConstraints(0, 4, 2, 1, 0.0, 0.0
                ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
        buttonPanel.add(viewMessageButton);
        buttonPanel.add(deleteButton);
        buttonPanel.add(closeButton);

        //add listeners
        ReceivedMailActionListener actionListener = new ReceivedMailActionListener();
        viewMessageButton.addActionListener(actionListener);
        closeButton.addActionListener(actionListener);
        deleteButton.addActionListener(actionListener);

        //Do the dialect stuff
        this.doDialect();
      }

      /**
       * Sets the correct text for the current dialect
       */
      protected void doDialect()
      {
        viewMessageButton.setText(BDialect.getString(this, "viewMessage"));
        closeButton.setText(BDialect.getString("close"));
        deleteButton.setText(BDialect.getString(this, "deleteMessage"));
      }

        /**
         * Closes the view messages window by sending a close event to the
         * BWindowManager
         */
        void closeViewWindow()
        {
            BCloseWindowCE closeWindow = new BCloseWindowCE( parent, this );
            parent.dispatchEvent( closeWindow );
            //setStatusNormal();
			removeMailStatus();
			statusAttended = false;
        }

        /**
         * Open the selceted message in the view message window
         */
        private void openMessage()
        {
            int selectedRow = receivedTable.getSelectedRow();

            if (selectedRow != -1)
            {
              BMessage receivedMessage = (BMessage) tableModel.getRowObject(selectedRow);
              addMailWindow(receivedMessage);
              
              if (mailPrefs.isDelete())
                 deleteMessage();
            }
        }
        
        private void deleteMessage()
        {
           int selectedRow = receivedTable.getSelectedRow();
           
            if (selectedRow != -1)
            {
               receivedMessages.removeElementAt(selectedRow);
               tableModel.fireTableRowsDeleted(selectedRow, selectedRow);
            }
        }

       /**
        * The table model for the received messages table
        *
        * @author Matthew Evans
        */
       class MessageTableModel extends AbstractTableModel
       {
          final String [] columnNames = {BDialect.getString(this, "sender"), BDialect.getString(this, "subject")};
          Vector data;

          /**
           * Constructs the table model with therelevant message data
           *
           * @param messageData the collection of direct messages that have
           * been recieved by a client during a session
           */
          public MessageTableModel(Vector messageData)
          {
            data = messageData;
          }

          /**
           * returns the number of columns in the table
           *
           * @return th enumber of columns defined in this model
           */
          public int getColumnCount()
          {
             return columnNames.length;
          }

          /**
           * returns the number of rows iin the table model
           *
           * @return the number of rows (messages) currently existing in this table model
           */
          public int getRowCount()
          {
             return data.size();
          }

          /**
           * returns the name of the specified column from the global array of
           * column names
           *
           * @param col the column position to return the corresponding name
           * @return the title of the specified column
           */
          public String getColumnName(int col)
          {
             return columnNames[col];
          }

          /**
           * returns the object in the collection of messages at position 'row'
           *
           * @param row the position of the BMessage object to retrieve
           * @return a BMessage object, actually
           */
          public Object getRowObject(int row)
          {
            return data.elementAt(row);
          }

          /**
           * returns the object in the position specified by row and col
           *
           * @param row the row in which the cell containing the object exists
           * @param column the column in which the cell containing the object exists
           * @return the BMessage object at position 'row'
           */
          public Object getValueAt(int row, int col)
          {
             BMessage aMessage = (BMessage) data.elementAt(row);

             switch (col)
             {
               case 0: return aMessage.getSender();
               case 1: return aMessage.getSubject();
               default: return new String(BDialect.getString(this, "unexpectedColumn"));
             }

          }

          /**
           * returns the class representation of all objects at contained in
           * the specified column
           *
           * @param column the object to geta class resentation of
           */
          public Class getColumnClass(int column)
          {
             return getValueAt(0,column).getClass();
          }
       }

       /**
        * The listener for the received messages dialog
        *
        * @author Matthew Evans
        */
       class ReceivedMailActionListener implements ActionListener
       {
          /**
           * Called when an action is performed on an object with anaction listener
           * registered
           *
           * @param event the event that was spawned
           */
          public void actionPerformed( ActionEvent event )
          {
              if (event.getSource() == viewMessageButton)
              {
                  openMessage();
              }
              else if (event.getSource() == closeButton)
              {
                closeViewWindow();
              }
              else if (event.getSource() == deleteButton)
              {
                 deleteMessage();
              }
          }
       }
    }
    
    class BMailPreferences extends JPanel
    {
		BString prefTabName;
		BString prefToolTip;
       JLabel sendLabel = new JLabel("Send Options-:");
       JLabel receiveLabel = new JLabel("Receive Options-:");
       JCheckBox soundCheck = new JCheckBox("Beep on arrival");
       JCheckBox displayCheck = new JCheckBox("Display on arrival");
       JCheckBox deleteCheck = new JCheckBox("Delete messages after reading");
       JCheckBox signatureCheck = new JCheckBox("Attach Signature");
       JTextField signatureField = new JTextField();
       JButton browseButton = new JButton("Browse");


       public BMailPreferences()
       {
	      this.prefTabName = new BString ( this, "mailclient" );
		  this.prefToolTip = new BString ( this, "tooltip" );
         this.setLayout(new GridBagLayout());
         signatureField.setEnabled(false);

         this.add(sendLabel, new BGridBagConstraints(0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));


         JPanel signaturePanel = new JPanel();
         signaturePanel.setLayout(new GridBagLayout());
         signaturePanel.add(signatureCheck, new BGridBagConstraints(0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

         signaturePanel.add(signatureField, new BGridBagConstraints(0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

         signaturePanel.add(browseButton, new BGridBagConstraints(0, 2, 1, 1, 0.0, 0.0
                ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

         this.add(signaturePanel, new BGridBagConstraints(0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));

        this.add(receiveLabel, new BGridBagConstraints(1, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

         this.add(soundCheck, new BGridBagConstraints(1, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

         this.add(displayCheck, new BGridBagConstraints(1, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

         this.add(deleteCheck, new BGridBagConstraints(1, 3, 1, 1, 1.0, 1.0
                ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

        //SET DEFAULTS
        signatureField.setEnabled(false);
        browseButton.setEnabled(false);
        displayCheck.setSelected(true);
        soundCheck.setSelected(true);
        deleteCheck.setSelected(false);

        //add listeners
        MailPrefsAction actionListener = new MailPrefsAction();
        signatureCheck.addActionListener(actionListener);
        browseButton.addActionListener(actionListener);
       }

       /**
        * Set the text for the interface for the current locale
        */
       protected void doDialect()
       {
           sendLabel.setText(BDialect.getString(this, "sendOptions"));
           receiveLabel.setText(BDialect.getString(this, "receiveOptions"));
           soundCheck.setText(BDialect.getString(this, "beep"));
           displayCheck .setText(BDialect.getString(this, "displayOnArrival"));
           deleteCheck.setText(BDialect.getString(this, "deleteMessages"));
           signatureCheck.setText(BDialect.getString(this, "attachSignature"));
       }

       public BString getPrefTabString ()
	   {
	      return prefTabName;
	   }

	   public BString getPrefToolTip ()
	   {
		  return prefToolTip;
	   }

       public void setSignatureEnable()
       {
          if (signatureCheck.isSelected())
          {
             browseButton.setEnabled(true);
             signatureField.setEnabled(true);
          }
          else
          {
             browseButton.setEnabled(false);
             signatureField.setEnabled(false);
          }
       }

       public void setSignatureAttachment()
       {
          JFileChooser fileBrowser = new JFileChooser(".");

           if (fileBrowser.showDialog(this, BDialect.getString(this, "add")) == JFileChooser.APPROVE_OPTION)
           {
              File attachmentFile = fileBrowser.getSelectedFile();
              signatureField.setText(attachmentFile.getPath());
           }
          
       }
       
      public boolean isDisplay()
      {
         return displayCheck.isSelected();
      }

       public boolean isDelete()
       {
          return deleteCheck.isSelected();
       }
       
      public boolean isSoundEnabled()
      {
         return soundCheck.isSelected();
      }

      public boolean isSignatureAttached()
      {
         return signatureCheck.isSelected();
      }
      
      public String getAttachmentName()
      {
         return signatureField.getText();
      }


       class MailPrefsAction implements ActionListener
       {
          public void actionPerformed(ActionEvent event)
          {
             if (event.getSource() == signatureCheck)
             {
                setSignatureEnable();
             }
             else if (event.getSource() == browseButton)
             {
                setSignatureAttachment();
             }
          }
       }
       
    }
}
