//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/metamodeller/BMetaElement.java,v 1.18 1999/10/12 16:05:32 tp079479 Exp $

   package au.edu.swin.synergy.plugin.metamodeller;

   import java.util.Vector;

   import java.io.Serializable;

   import au.edu.swin.synergy.lib.io.BSaveable;
   import au.edu.swin.synergy.lib.io.BDataFile;
   import au.edu.swin.synergy.debug.Debug;


/**
 *
 * class BMetaElement
 * @author Imron Alston
 * @date 21/7/99
 *
 * This class describes a metaelement (such as a node or an arc)
 * from a conceptual point-of-view. That is, an identifying name
 * (such as class, package, inheritance etc), and a list of properties that
 * belong to the element.
 * Please complete these missing tags
 * @copyright
 */
   public class BMetaElement implements Serializable, BSaveable
   {
    /**
     * the name/identifier of this metaelement *
     */
      protected String name = null;
   
    /**
     * the list of properties (string ids only) belonging to this meta-element *
     */
      protected Vector properties = null;

  	// for BSaveable
	protected static final String kKeyName = "name";
	protected static final String kDefaultName = "untitled";

	protected static final String kKeyProperty = "property";

   
    /**
     * Construct the metaelement..
     * Please complete the missing tags for BMetaElement
     * @param
     * @return
     * @pre
     * @post
     */
      public BMetaElement()
      {
         this.properties = new Vector();
      }
   
    /**
     * Sets the name of the meta-element
     * Please complete the missing tags for setName
     * @param
     * @return
     * @pre
     * @post
     */
      public void setName( String theName )
      {
         this.name = theName;
      }
   
      /**
       * Gets the name of the meta-element
       *
       * @return   The name of this meta element
       */
      public String getName()
      {
         return this.name;
      }
	  public String toString()
	  {
		return this.name;
	  }
   
      /**
       * Removes the property from the elements list of properties.
       *
       * @param The property to remove
       * @pre
       * @post
       */
      public void addProperty( String theProperty )
      {
         this.properties.addElement( theProperty );
      }
   	
   	/**
   	 * Resets the Meta Element.  It clears  all the properties 
     */
   	
   	public void resetProperties()
   	{
   		properties.clear();
   	}
   
      /**
       * Inserts the given property in the list at the index specified.
       *
       * @param theProperty The property to remove
       * @param index   The index to insert the property at
       * @pre
       * @post
       */
      public void insertPropertyAt( String theProperty, int index)
      {
         this.properties.insertElementAt( theProperty, index );
      }
   
      /**
       * Removes the property from the elements list of properties at the
       * given index.
       *
       * @param propertyIndex The index of the property to remove
       * @pre
       * @post
       */
      public void removePropertyAt( int propertyIndex )
      {
         this.properties.removeElementAt( propertyIndex );
      }
   
      /**
       * Removes the property from the elements list of properties.
       *
       * @param theProperty The property to remove
       * @pre
       * @post
       */
      public void removeProperty( String theProperty )
      {
		 int index = this.properties.indexOf( theProperty );

		 if ( index >= 0 )
		 {
			this.properties.remove( theProperty );
		 }

      }
   
      /**
       * This will provide access to the properties of
       * this meta element
       *
       * @return Vector of <b><i>string ids???</b></i>
       */
      public Vector getProperties()
      {
         return this.properties;
      }

   
      /**
       * This will determine whether the given BMetaElement
       * is the same as this meta element. This based on the
       * name of the meta element as well as the properties it
       * contains.
       * <p>
       * This is not limited to referential comparisons, that is, one
       * meta element may be created based on one file and another from
       * a different file, (ie. they are two separate instances in memory),
       * yet will still be treated as equal if their members match.
       *
       * @param target  The target element to compare
       */
      public boolean equals(Object target)
      {
         BMetaElement targetElement = (BMetaElement)target;
         boolean match = false;
      
        //Compare the names of the element
         match = this.name.equals(targetElement.name);
      
        //Compare the properties
         match &= this.properties.equals(targetElement.properties);
      
        //Return the result
         return match;
      }
   


      public boolean load(BDataFile file)
      {
         Debug.doPrintln("BMetaElement.load");
         name = file.getData(kKeyName, kDefaultName);
         Debug.doPrintln("name = " + name);
      
      // read in the properties...
         while(true)
         {
            String property = file.getData(kKeyProperty, null);
            if(property == null)
            {
               break;
            }
            Debug.doPrintln("Property " + property);
            properties.add(property);
         }
         return true;
      }
   
      public boolean save(BDataFile file)
      {
         Debug.doPrintln("BMetaElement.save");
         file.addData(kKeyName, name, kDefaultName);
      
         for(int i=0; i<properties.size(); i++)
         {
            file.addData(kKeyProperty, (String) properties.get(i));
         }
      
         return true;
      }
   
   }
