// $Header:
/**
* Just providing a simple tree cell renderer, without it getting mixed
* up with all the application code of other classes.
*
* @author Christian Karolos
* @date 12/10/99
*/
package au.edu.swin.synergy.plugin.metamodeller;

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;

import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;
import au.edu.swin.synergy.plugin.mod13.BWindowManager;

import au.edu.swin.synergy.plugin.metamodeller.BMetaNode;
import au.edu.swin.synergy.plugin.metamodeller.BMetaArc;
import au.edu.swin.synergy.plugin.metamodeller.BMetaRule;

import javax.swing.tree.DefaultMutableTreeNode;

public class BMetaModelCellRenderer extends JPanel implements TreeCellRenderer
{
	public static final int kFontSize = 12;
	public static final int kIconGap = 1;

	JLabel label;
	Color back;
	Color highlight;

	ImageIcon metaModel;
	ImageIcon property;

	ImageIcon mRuleGroup;
	ImageIcon metaRule;

	ImageIcon mNodeGroup;
	ImageIcon metaNode;

	ImageIcon mArcGroup;
	ImageIcon metaArc;

	public BMetaModelCellRenderer ()
	{
		label = new JLabel ();
		label.setFont ( new Font ( "system", Font.PLAIN, kFontSize ) );
		label.setIconTextGap ( kIconGap );
		String path = new String ( "/au/edu/swin/synergy/plugin/metamodeller/icons/" );

		// White just so happens to be the background of the package / notation tree at
		// the moment.
		back = Color.white;
		highlight = BWindowManager.getHighlightColor ();

        metaModel = BSwingUtilities.getImage(this, path + "packagemodel.gif" ) ;

        mRuleGroup = BSwingUtilities.getImage(this, path + "terminatorroot.gif" ) ;
        metaRule = BSwingUtilities.getImage(this, path + "terminator.gif" ) ;


        mNodeGroup = BSwingUtilities.getImage(this, path + "NODE.GIF" ) ;
        metaNode = BSwingUtilities.getImage(this, path + "NODE2.GIF" ) ;


        mArcGroup = BSwingUtilities.getImage(this, path + "ARC.GIF" ) ;
        metaArc = BSwingUtilities.getImage(this, path + "ARC2.GIF" ) ;

		add ( label );
	}

	
	public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, 
		boolean expanded, boolean leaf, int row, boolean hasFocus)
	{
        DefaultMutableTreeNode treeNode = null;
        String nodeName = null;
		label.setText ( value.toString () );


		if ( selected )
		{
			setBackground (BWindowManager.getHighlightColor());
		}
		else
		{
			setBackground ( this.back );
		}

		if ( value instanceof DefaultMutableTreeNode )
		{
            treeNode = ( DefaultMutableTreeNode ) value;
			Object userObject = treeNode.getUserObject();
			Debug.doPrintln("Rendering a " + userObject.getClass().getName(), Debug.kWarningLevel);

			if ( userObject instanceof BMetaNode )
			{
				label.setIcon ( metaNode );
			}
			else if ( userObject instanceof BMetaArc )
			{
				label.setIcon ( metaArc );
			}
			else if ( userObject instanceof BMetaRule )
			{
                label.setIcon ( mRuleGroup );
//				label.setIcon ( metaRule );
			}
			else
			{

			
	            nodeName = treeNode.toString ();
				Debug.doPrintln("nodeName = " + nodeName, Debug.kWarningLevel);
	            
	            if ( treeNode.isRoot () )
	            {
	                label.setIcon ( metaModel );
	            } 
	            else if ( nodeName.compareTo ( "Nodes" ) == 0 )
	            {
	    			label.setIcon ( mNodeGroup );
	            }
	            else if ( nodeName.compareTo ( "Arcs" ) == 0 )
    	        {
	                label.setIcon ( mArcGroup );
	            }
	            else if ( nodeName.compareTo ( "Rules" ) == 0 )
	            {
	                label.setIcon ( mRuleGroup );
	            }
			}
		}

		return this;
	}

}

