package au.edu.swin.synergy.plugin.mod13;

import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import au.edu.swin.synergy.debug.Debug;
import javax.swing.plaf.basic.*;

/**
 * Class which allows for the display of images on the cohesion desktop
 *
 * @author Shane Fitzgibbon
 * @copyright Synergy Software 1999
 */

public class BDesktopPaneUI extends BasicDesktopPaneUI
{
	Image img;
	Rectangle rect;
	String filename = "au\\edu\\swin\\synergy\\plugin\\mod13\\images\\cohesion.jpg";
	boolean loaded = false;
	BImageObserver imageObserver;


	/**
	 * Constructor for the BDesktopPaneUI
	 */
	public BDesktopPaneUI()
	{
		img = Toolkit.getDefaultToolkit().getImage(filename);
		imageObserver = new BImageObserver();
	}


	/**
	 * Contructor for the BDesktopPaneUI
	 *
	 * @param name String that contains the path of the iamge to be displayed
	 */
	public BDesktopPaneUI( String name )
	{
		this.filename = name;
		img = Toolkit.getDefaultToolkit().getImage(filename);
		imageObserver = new BImageObserver();
	}

	/**
	 * Paints the desktop
	 */
	public void paint(Graphics g, JComponent c)
	{
		rect = this.desktop.getBounds( null );
		if ( loaded )
		{
			g.drawImage( img, ( rect.width/2 - img.getWidth( null )/2 ), ( rect.height/2 - img.getHeight( null )/2 ), null );
		}
		else
		{
			g.drawImage( img, ( rect.width/2 - img.getWidth( null )/2 ), ( rect.height/2 - img.getHeight( null )/2 ), imageObserver );
		}
	}

	/**
	 * Cuases the desktop to be revalidated
	 */
	public void revalidate()
	{
		this.desktop.repaint();
	}


	/**
	 * Sets the image to be displayed
	 *
	 * @param imageName String that contains the path of the image to be displayed
	 */
	public void setImage( String imageName )
	{
		this.filename = imageName;
		this.loaded = false;
	}


	class BImageObserver implements ImageObserver
	{

		public boolean imageUpdate( Image img, int infoflags, int x, int y, int width, int height)
		{
			if ( ( infoflags | ImageObserver.ALLBITS ) == ImageObserver.ALLBITS )
			{
				Debug.doPrintln( "Image Loaded" );
				BDesktopPaneUI.this.loaded = true;
				BDesktopPaneUI.this.revalidate();
				return false;
			}
			return true;
		}
	}

}
