

package au.edu.swin.synergy.plugin.mod13;

import au.edu.swin.synergy.plugin.mod13.event.BAddMenuItemCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddMenuCE;
import au.edu.swin.synergy.plugin.mod13.event.BRemoveMenuItemCE;

import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BPlugin;

import au.edu.swin.synergy.plugin.command.BOnCommand;

import au.edu.swin.synergy.debug.Debug;

import au.edu.swin.synergy.lib.BStringList;

import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.dialect.BString;

import java.awt.*;
import java.awt.event.*;

import java.io.InputStream;
import java.io.PrintStream;
import java.io.IOException;

import javax.swing.*;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;

/**
 * This will manage all the functionality related
 * to the menus in the window manager. Everything is
 * encapsulated and should require no outside intervention
 * with anything to do with dynamic menu management.
 *
 * @author Tristan Austin
 */
class BMenuAgent
{
    /**
     * The menu bar all the menu reside on
     */
    JMenuBar menuBar = null;

    /**
     * This maps menus to their location policies
     */
    Hashtable menuPolicyTable = new Hashtable();

    /**
     * This is the existing list of mnemonics
     */
    Hashtable existingMnemonics = new Hashtable();

    /**
     * This is the mnemonic BString values mapped to the menus
     */
    Hashtable menuMnemonics = new Hashtable();

    /**
     * This keeps track of all the locale sensitive menu names
     */
    Hashtable menuNames = new Hashtable();

    /**
     * This keeps track of all the locale sensitive menu item names
     */
    Hashtable menuItemNames = new Hashtable();

    /**
     * This maps menu items to their preferred mnemonic
     */
    Hashtable menuItemMnemonics = new Hashtable();

    /**
     * The command line for windows
     */
    BMenuCL commandLine = new BMenuCL();

    /**
     * Creates a new BMEnuManager with the given
     * menu bar to manage.
     *
     * @param   menuBar The menu bar to manage
     */
    public BMenuAgent()
    {
        this.menuBar = new JMenuBar();
    }

    /**
     * This method will update all the text of the menus and menu
     * related interface components for the current locale.
     */
    protected void doDialect()
    {
        //For each menu, set it's name and it's mnemonic
        MenuElement currentMenus[] = this.menuBar.getSubElements();
        MenuElement currentMenuItems[];
        JMenu currentMenu = null;
        MenuElement currentMenuItem = null;
        Object menuMnemonic = null;
        Object menuName = null;

        //Run through the existing menus and see if the required menu already exists
        for (int i = 0; i < currentMenus.length; i++)
        {
            currentMenu = (JMenu)currentMenus[i];
            menuMnemonic = this.menuMnemonics.get(currentMenu);
            menuName = this.menuNames.get(currentMenu);

            //Make sure a mnemonic was specified at all
            if (menuMnemonic != null)
            {
                //currentMenu.setMnemonic(menuMnemonic.toString().charAt(0));
            }

            currentMenu.setText(menuName.toString());

            //Need to run through the menu items now
            currentMenuItems = currentMenu.getSubElements();

            //Process them all
            for (int j = 0; j < currentMenu.getItemCount(); j++)
            {
                currentMenuItem = currentMenu.getItem(j);

                //Make sure it's not a separator
                if (currentMenuItem instanceof JMenuItem)
                {
                    menuMnemonic = this.menuItemMnemonics.get(currentMenuItem);
                    menuName = this.menuItemNames.get(currentMenuItem);

                    //Make sure a mnemonic was specified at all
                    if (menuMnemonic != null)
                    {
                        ((JMenuItem)currentMenuItem).setMnemonic(menuMnemonic.toString().charAt(0));
                    }

                    //The window agent is responsible for window menu items
                    if (menuName != null)
                    {
                        ((JMenuItem)currentMenuItem).setText(menuName.toString());
                    }
                }
            }
        }
    }

    /**
     * Takes a menu and the preferred mnemonic for that menu
     * and either assigns it to the menu or makes a new one
     * if the given one is already taken.
     *
     * @param   menu    The menu to assign a mnemonic
     * @param   preferredMnemonic   The preferred mnemonic to assign
     */
    private void assignMnemonic(JMenu menu, BString preferredMnemonic)
    {
        Integer mnemonic = new Integer((int)preferredMnemonic.toString().charAt(0));
        String menuName = menu.getText();
        boolean alreadyExists = false;
        int i = 0;

        //Find the mnemonic
        while (i < menuName.length() && (alreadyExists = this.existingMnemonics.containsKey(mnemonic)))
        {
            mnemonic = new Integer(menuName.charAt(i++));
        }

        //Only set the mnemonic if we found one
        if (!alreadyExists)
        {
            //menu.setMnemonic(mnemonic.intValue());
            this.existingMnemonics.put(mnemonic, mnemonic);
            this.menuMnemonics.put(menu, preferredMnemonic);
        }
    }

    /**
     * Gets the command line to be added to the command line module
     *
     * @returns The command line object for this agent
     */
    public BOnCommand getCommandLine()
    {
        return this.commandLine;
    }

    /**
     * This will insure that new menus are added in the
     * correct location relative to menus that already
     * exist. Menus with the rightMost alignment should
     * always exist on the right most side of the
     * menu bar.
     *
     * @param   newMenu The menu to add
     * @param   locationPolicy  Where to place the menu item
     */
    protected JMenu addMenu(JMenu newMenu, int locationPolicy)
    {
        return this.addMenu(newMenu, locationPolicy, null, null);
    }

    /**
     * This will insure that new menus are added in the
     * correct location relative to menus that already
     * exist. Menus with the rightMost alignment should
     * always exist on the right most side of the
     * menu bar.
     *
     * @param   newMenu The menu to add
     * @param   locationPolicy  Where to place the menu item
     * @param   preferredMnemonic The preferred mnemonic for the menu
     */
    protected JMenu addMenu(JMenu newMenu, int locationPolicy, Object menuName, BString preferredMnemonic)
    {
        MenuElement currentMenus[] = null;
        JMenu currentMenu = null;
        int currentMenuPolicy = -1;
        boolean done = false;

        //If always right aligned, add to the end of the menu
        if (locationPolicy == BAddMenuCE.kAlwaysRightAligned || this.menuBar.getMenuCount() == 0)
        {
            menuBar.add(newMenu);
        }
        else
        {
            currentMenus = this.menuBar.getSubElements();

            //Run through the existing menus and see if the required menu already exists
            for (int i = (currentMenus.length -1); i >= 0; i--)
            {
                currentMenu = (JMenu)currentMenus[i];
                currentMenuPolicy = ((Integer)(this.menuPolicyTable.get(currentMenu))).intValue();

                //If we find the menu already there, return it
                if (currentMenuPolicy == BAddMenuCE.kNormalPolicy)
                {
                    menuBar.add( newMenu, i+1 );
                    break;
                }
            }
        }

        //Make sure it's valid
        if (preferredMnemonic != null)
        {
            //Assign the mnemonic to the menu
            this.assignMnemonic(newMenu, preferredMnemonic);
        }

        this.menuPolicyTable.put(newMenu, new Integer(locationPolicy));
        this.menuNames.put(newMenu, menuName);

        //Set it's name
        newMenu.setText(menuName.toString());

        //Return the menu added
        return newMenu;
    }

    /**
     * This will find the menu with the given name and return it
     *
     * @param   menuName    The name of the menu to find
     * @return The menu with that name
     */
    private JMenu getMenu(String menuName)
    {
        return this.getMenu(menuName, false);
    }

    /**
     * This will find the menu with the given name and return it or
     * the menu that starts with the given name if the justStart flag is set to true
     *
     * @param   menuName    The name of the menu to find
     * @param   justStart   This will return the menu if the start of the menuname matches
     * @return The menu with that name
     */
    private JMenu getMenu(String menuName, boolean justStart)
    {
        MenuElement currentMenus[] = this.menuBar.getSubElements();
        String cMenuName;
        menuName = menuName.toLowerCase();
        JMenu currentMenu = null;

        //Run through the existing menus and see if the required menu already exists
        for (int i = 0; i < currentMenus.length; i++)
        {
            currentMenu = (JMenu)currentMenus[i];
            cMenuName = currentMenu.getText().toLowerCase();

            //If we find the menu already there, return it
            if ((justStart && cMenuName.startsWith(menuName.toLowerCase())) || cMenuName.equals(menuName))
            {
                return currentMenu;
            }
        }

        return null;
    }

    /**
     * This method will provide the JMenu specified by
     * the given menu name. If the menu doesn't already
     * exist, it will create a new one, add it to the menu
     * bar and return it to be worked on.
     *
     * @param   menuName    The name of the menu to provide
     */
    private JMenu getMenu(Object menuName, int locationPolicy, BString preferredMnemonic)
    {
        MenuElement currentMenus[] = this.menuBar.getSubElements();
        JMenu currentMenu = null;

        try
        {
            currentMenu = this.getMenu(menuName.toString());

            //See if the menu was found
            if (currentMenu == null)
            {
                //If we get to here, the menu wasn't found, we need to create and insert it
                currentMenu = new JMenu(menuName.toString());
                currentMenu = this.addMenu(currentMenu, locationPolicy, menuName, preferredMnemonic);
            }
        }
        catch(NullPointerException npException)
        {
            npException.printStackTrace();
        }

        //This will return the menu we've either added or found
        return currentMenu;
    }

    /**
     * Takes the add menu item event and places the menu item
     * in the appropriate menu as specified.
     *
     * @param   event The event
     * Please complete the missing tags for addMenuItem
     * @return
     * @pre
     * @post
     */
    protected void addMenuItem( BAddMenuItemCE event )
    {
        Vector menus = event.getMenuItems();
        JMenu targetMenu = this.getMenu(event.getMenuName(), event.getLocationPolicy(), event.getPreferredMenuMnemonic());
        Object currentItem = null;
        BString name;

        //See if we have any menu items to run through
        if( menus != null && menus.size() > 0 )
        {
            //Only add the separator if there's something in there already
            if( targetMenu.getMenuComponentCount() > 0 )
            {
                //Add the separator
                targetMenu.insertSeparator( 0 );
            }

            //Add all the menu items
            for( int i = (menus.size() -1); i>= 0; i-- )
            {
                currentItem = menus.elementAt(i);

                //Test to see if it is actually a separator
                if (currentItem instanceof JSeparator)
                {
                    targetMenu.insertSeparator(0);
                }
                else
                {
                    targetMenu.insert( (JMenuItem)currentItem, 0 );

                    //Get the locale sensitive name, if it has one
                    name = event.getNameFor((JMenuItem)currentItem);

                    //Make sure a name has been specified
                    //If it hasn't it is assumed the item has it set internally
                    if ( name != null)
                    {
                        this.menuItemNames.put(currentItem, name);
                        ((JMenuItem)currentItem).setText(name.toString());
                    }
                }
            }
        }

        //Indicate the event is done
        event.doComplete(BCommandComplete.kSuccess);
    }
    /**
     * Takes the remove menu item event and places the menu item
     * in the appropriate menu as specified.
     *
     * @param   event The event
     * Please complete the missing tags for addMenuItem
     * @return
     * @pre
     * @post
     */
    protected void removeMenuItem( BRemoveMenuItemCE event )
    {
        Vector items = event.getMenuItems();
        JMenu targetMenu = this.getMenu(event.getMenuName().toString());

        //Remove the menu items from the menu
        if (targetMenu != null)
        {
            //Run through and remove them all
            for (int i = 0; i < items.size(); i++)
            {
                targetMenu.remove((JMenuItem)items.elementAt(i));
            }

            //Need to remove the menu or the separator depending on how many are left
            if (targetMenu.getMenuComponentCount() > 0)
            {
                targetMenu.remove(0);
            }
            else
            {
                this.menuBar.remove(targetMenu);
            }
        }
    }

    /**
     * Gets the menu bar this manager is taking care of
     *
     * @return The menu bar
     */
    public JMenuBar getMenuBar()
    {
        return this.menuBar;
    }

    /**
     * This will allow the user to access the menus
     * via the command line.
     *
     * @author Tristan Austin
     */
    public class BMenuCL implements BOnCommand
    {
        /**
         * When the command mapped to this command line
         * object is called, the paramaters and streams
         * are passed in here.
         *
         * @param   params  The paramaters to the command
         * @param   in  The input stream to read from
         * @param   out The output stream to write to
         * @return An -1 on failure
         */
        public int onCommand( BStringList params, InputStream in, PrintStream out )
        {
            String menu;
            String nextItem;
            JMenu selectedMenu;
            JMenuItem selectedMenuItem;
            int count = params.getCount();

            //Make sure there are enough to deal with
            if (count >= 2)
            {
                menu = params.get(1);

                //Get the menu typed in
                selectedMenu = BMenuAgent.this.getMenu(menu, true);

                //Make sure it's a valid menu
                if (selectedMenu != null)
                {
                    selectedMenu.doClick();

                    //Recursively select more menu bits
                    for (int i = 2; i < count; i++)
                    {
                        nextItem = params.get(i);

                        //Get the menu item
                        selectedMenuItem = this.getMenuItem(selectedMenu, nextItem);

                        //Make sure it exists
                        if (selectedMenuItem != null)
                        {
                            selectedMenuItem.doClick();
                        }
                        else
                        {
                            //Indicate menu item not found
                            out.println("Menu item \"" + nextItem + "\" not found");
                        }
                    }

                    //Only hide it if they have specified an item
                    if (count > 2)
                    {
                        selectedMenu.setPopupMenuVisible(false);
                        selectedMenu.setSelected(false);
                    }
                }
                else
                {
                    //Indicate wrong menu
                    out.println("The menu \"" + menu + "\" was not found");
                }
            }
            else
            {
                //Indicate wrong usage
                out.println(this.getHelp(null));
            }

            return 0;
        }

        /**
         * When the user types "help" at the command prompt, the kernel
         * will call the getHelp method on all the registered onCommand
         * objects. This will indicate how to use this command line option.
         *
         * @param   params  The paramatesr to the help call (eg. help nadduser)
         */
         public String getHelp( BStringList params )
         {
            return "Insert help here";
         }

         /**
          * This will locate the menu item with the given
          * name and in the given menu and return it.
          *
          * @param  menu    The menu to look in
          * @param  itemName    The menu items name to look for
          * @return The menu matching the name given, null if not found
          */
         private JMenuItem getMenuItem(JMenu menu, String itemName)
         {
            JMenuItem menuItem;
            String menuName;
            itemName = itemName.toLowerCase();
            Component[] menuItems = menu.getMenuComponents();

            //Run through all menu items
            for (int i=0; i < menuItems.length; i++)
            {
                //Make sure it's not separator or something else
                if (menuItems[i] instanceof JMenuItem)
                {
                    menuItem = (JMenuItem)menuItems[i];
                    menuName = menuItem.getText().toLowerCase();

                    //See if they match
                    if (menuName.startsWith(itemName))
                    {
                        return menuItem;
                    }
                }
            }

            //Didn't find anything
            return null;
         }
    }
}
