// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/mod13/BPaneManager.java,v 1.12 1999/10/16 23:59:53 tp079210 Exp $
package au.edu.swin.synergy.plugin.mod13;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import au.edu.swin.synergy.debug.Debug;
import java.util.Hashtable;

/**
 * The BPaneManager allows the addition of many components to
 * the single component and divides up the screen space using
 * tabbed panes to separate each component.
 *
 * @author Tristan Austin
 * Please complete these missing tags
 * @copyright
 */
public class BPaneManager extends JPanel
{
    /**
     * The tabs go at the top
     */
    public static final int kTabTop = JTabbedPane.TOP;

    /**
     * The tabs go at the bottom
     */
    public static final int kTabBottom = JTabbedPane.BOTTOM;

    /**
     * The tabs go at the left
     */
    public static final int kTabLeft = JTabbedPane.LEFT;

    /**
     * The tabs go at the right
     */
    public static final int kTabRight = JTabbedPane.RIGHT;

    /**
     * The layout to use
     */
    private BorderLayout borderLayout1 = new BorderLayout();

    /**
     * The tab placement
     */
    private int tabPlacement = kTabTop;

    /**
     * The tabbed pane all added components reside on
     */
    private JTabbedPane tabbedPane = null;

    /**
     * This maps the tabbed components on the tabbed
     * pane to their locale specific titles
     */
    private Hashtable tabsToTitles = new Hashtable();

    /**
     * This is the default constructor. The BPaneManager defaults to
     * having a horizontal split it also sets the layout to the borderlayout.
     * Please complete the missing tags for BPaneManager
     * @param
     * @return
     * @pre
     * @post
     */
    public BPaneManager()
    {
        this.setLayout( this.borderLayout1 );
        this.setName( "BPaneManager" );
    }

    /**
     * The tab placement for the tabbed pane is specified by tabPlacement.
     *
     * @param   tabPlacement    Acceptable values include: JTabbedPane.TOP, JTabbedPane.BOTTOM, JTabbedPane.LEFT, JTabbedPane.RIGHT
     * Please complete the missing tags for BPaneManager
     * @return
     * @pre
     * @post
     */
    public BPaneManager( int tabPlacement )
    {
        this();
        this.tabPlacement = tabPlacement;
    }

    /**
     * This will run through all the tabs on the tabbed pane
     * and conver their titles to the value required for the current
     * locale.
     */
    protected void doDialect()
    {
        Component currentTab = null;
        Object title;

        //Run through all the scroll panels to find it
        for( int i = 0; i < tabbedPane.getComponentCount(); i++ )
        {
            currentTab = tabbedPane.getComponentAt( i );

            //Get the title and set it
            title = this.tabsToTitles.get(currentTab);
            tabbedPane.setTitleAt(i, title.toString());
        }
    }

    /**
     * This will remove the given component from the tabbed pane
     *
     * @param   targetComponent The component to remove
     * Please complete the missing tags for removeComponent
     * @return
     * @pre
     * @post
     */
    public void removeComponent( Component targetComponent )
    {
        int targetLocation = this.findComponent( targetComponent );
        
        //Make sure it was found
        if( targetLocation >= 0 )
        {
            this.tabsToTitles.remove(this.tabbedPane.getComponentAt(targetLocation));
            this.tabbedPane.removeTabAt( targetLocation );
        }
        
        Debug.doPrintln( "Panel removed (if found)" );
    }

    /**
     * This will return the tab with the given title
     *
     * @param   tabTitle    The title of the tab to find
     * @return The component with the tab title as supplied, null if not found
     */
    public boolean selectComponentCalled(String tabTitle)
    {
        Component tab = null;
        String currentTab;
        int index = -1;

        tabTitle = tabTitle.toLowerCase();

        Debug.doPrintln("Searching for tab: " + tabTitle, Debug.kWarningLevel);

        //Run through checking the titles
        for (int i = 0; i < this.tabbedPane.getTabCount(); i++)
        {
            currentTab = this.tabbedPane.getTitleAt(i).toLowerCase();
            Debug.doPrintln(currentTab, Debug.kWarningLevel);

            if (currentTab.startsWith(tabTitle))
            {
                this.tabbedPane.setSelectedIndex(i);
                return true;
            }
        }

        //Indicate failure
        return false;
    }

    /**
     * Gets the component with the given name
     *
     * @param tabTitle  The name of the tabbed panel to get
     */
    public JComponent getComponentCalled(String tabTitle)
    {
        Component tab = null;
        String currentTab;
        int index = -1;

        tabTitle = tabTitle.toLowerCase();

        Debug.doPrintln("Searching for tab: " + tabTitle, Debug.kWarningLevel);

        //Run through checking the titles
        for (int i = 0; i < this.tabbedPane.getTabCount(); i++)
        {
            currentTab = this.tabbedPane.getTitleAt(i).toLowerCase();
            Debug.doPrintln(currentTab, Debug.kWarningLevel);

            if (currentTab.startsWith(tabTitle))
            {
                return (JComponent)this.tabbedPane.getComponentAt(i);
            }
        }

        //Nothing found
        return null;
    }


    /**
     * This will select the given component on the tabbed pane
     *
     * @param   targetComponent The component to select
     */
    public void selectComponent( Component targetComponent )
    {
        int targetLocation = this.findComponent( targetComponent );

        //Make sure it was found
        if( targetLocation >= 0 )
        {
            this.tabbedPane.setSelectedIndex( targetLocation );
        }
    }
    
    /**
     * This will take whatever components are added and add them to the
     * tabbed pane as a new tab. The constraints provided should contain
     * the name of the tab.
     * <p>
     * This will always place the component on a scroll pane.
     *
     * @param   comp    The new item to put on the tab
     * @param   constraints The name of the tab
     * @param   index   The index to put it at
     */
    protected void addImpl( Component comp, Object constraints, int index )
    {
        this.add(comp, constraints, true);
    }

    /**
     * This version of add will allow the caller to specify whether they
     * want their component placed on a scroll pane or not.
     *
     * @param   component   The component to add
     * @param   constraints The tab title of the component
     * @param   useScrollPane   True to place the component on a scroll pane, false to not
     */
    public void add(Component component, Object constraints, boolean useScrollPane)
    {
        JScrollPane scrollPane;
        Component targetComponent = component;

        //If the tabbed pane hasn't yet been created, create it
        if( this.tabbedPane == null )
        {
            tabbedPane = new JTabbedPane( this.tabPlacement );
            super.addImpl( this.tabbedPane, BorderLayout.CENTER, -1 );
        }

        //If they want a scroll pane, we'll give them one!
        if (useScrollPane)
        {
            scrollPane = new JScrollPane( JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
            scrollPane.getViewport().add( component );
            targetComponent = scrollPane;
        }

        synchronized(this.tabbedPane)
        {
            this.tabbedPane.addTab( constraints.toString(), targetComponent );
            this.tabsToTitles.put(targetComponent, constraints);
        }
    }

    /**
     * Locates the given component as one of the panes on the
     * tabbed pane. If it can't locate it it returns -1.
     *
     * @param   targetComponent The component to find
     * @return The location of the target, -1 if not found
     * Please complete the missing tags for findComponent
     * @pre
     * @post
     */
    private int findComponent( Component targetComponent )
    {
        JScrollPane currentScrollPane = null;
        JViewport currentViewport = null;
        Component component;

        //Run through all the scroll panels to find it
        for( int i = 0; i < tabbedPane.getComponentCount(); i++ )
        {
            component = tabbedPane.getComponentAt( i );

            if (component instanceof JScrollPane)
            {
                currentScrollPane = ( JScrollPane ) ( component );

                currentViewport = currentScrollPane.getViewport();

                //See if it is the target component
                if( currentViewport.getView() == targetComponent )
                {
                    return i;
                }
            }
            else
            {
                if (component == targetComponent)
                {
                    return i;
                }
            }
        }

        return - 1;
    }

    /**
     * This will return the number of components currently
     * being displayed on this dynamic pane.
     *
     * @return The number of components
     */
    public int getDynamicCount()
    {
        int count = 0;

        //Check to see that the tabbed pane exists
        if (this.tabbedPane != null)
        {
            count = this.tabbedPane.getComponentCount();
        }

        //Return the count
        return count;
    }
}



