package au.edu.swin.synergy.plugin.mod13;

import au.edu.swin.synergy.plugin.mod13.event.BAddStatusCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddPanelCE;
import au.edu.swin.synergy.plugin.mod13.event.BRemovePanelCE;
import au.edu.swin.synergy.plugin.mod13.event.BSelectPanelCE;
import au.edu.swin.synergy.plugin.mod13.event.BRemovePanelCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddToolBarCE;
import au.edu.swin.synergy.plugin.mod13.event.BRemoveToolBarCE;

import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.lib.BDynamicPane;

import au.edu.swin.synergy.plugin.command.BOnCommand;

import au.edu.swin.synergy.lib.BStringList;


import java.awt.*;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.IOException;

import javax.swing.*;

import javax.swing.border.BevelBorder;

import java.util.Hashtable;
import java.util.Vector;

/**
 * This will manage all the functionality related
 * to the menus in the window manager. Everything is
 * encapsulated and should require no outside intervention
 * with anything to do with dynamic menu management.
 *
 * @author Tristan Austin
 */
class BPanelAgent
{
    /**
     * This splits up the screen space between the leftPM and the desktop
     */
    private BPaneManager leftPM = null;

    /**
     * This contains both the status bar and the bottom dynamic pane
     */
    private JPanel bottomPane = new JPanel();

    /**
     * This is the pane manager for the bottom of the screen
     */
    private BPaneManager bottomPM = null;

    /**
     * This is the pane for the right pane manager
     */
    private BPaneManager rightPM = null;

    /**
     * This is the pane manager for the top of the screen
     */
    private BPaneManager topPM = null;

    /**
     * The tool bar
     */
    private JToolBar theToolBar = null;

    /**
     * This is the split pane for the top pane manager
     */
    BDynamicPane topDynamicPane = null;

    /**
     * The bottom split pane containing the bottom pane manager
     */
    BDynamicPane bottomDynamicPane = null;

    /**
     * The left split pane containing the bottom pane manager
     */
    BDynamicPane leftDynamicPane = null;

    /**
     * The right split pane containing the right pane manager
     */
    BDynamicPane rightDynamicPane = null;

    /**
     * The command line for windows
     */
    BPanelCL commandLine = new BPanelCL();

    /**
     * The main frame to place the dynamic panes
     */
    JFrame mainFrame = null;

    /**
     * The limit for the dividers
     */
    Point maxBottomRight = new Point(5000, 5000);

    /**
     * The limit for the dividers
     */
    Point maxTopLeft = new Point(0, 0);

    /**
     * The status bar is where any status components are placed
     */
    JPanel statusBar;

    /**
     * This contains all the tool bars in their prioritised order
     */
    Vector toolBars = new Vector();

    /**
     * Contains all the status bars in their prioritised order
     */
    Vector statusItems = new Vector();

    /**
     * Creates a new BMEnuManager with the given
     * menu bar to manage.
     *
     * @param   menuBar The menu bar to manage
     */
    public BPanelAgent(JFrame mainFrame)
    {
        this.mainFrame = mainFrame;

        topDynamicPane = new BDynamicPane( this.mainFrame, BorderLayout.SOUTH );
        topDynamicPane.setMaxBottomRight(this.maxBottomRight);
        topDynamicPane.setMaxTopLeft(this.maxTopLeft);

        bottomDynamicPane = new BDynamicPane( this.mainFrame, BorderLayout.NORTH );
        bottomDynamicPane.setMaxBottomRight(this.maxBottomRight);
        bottomDynamicPane.setMaxTopLeft(this.maxTopLeft);

        leftDynamicPane = new BDynamicPane( this.mainFrame, BorderLayout.EAST );
        leftDynamicPane.setMaxBottomRight(this.maxBottomRight);
        leftDynamicPane.setMaxTopLeft(this.maxTopLeft);

        rightDynamicPane = new BDynamicPane( this.mainFrame, BorderLayout.WEST );
        rightDynamicPane.setMaxBottomRight(this.maxBottomRight);
        rightDynamicPane.setMaxTopLeft(this.maxTopLeft);

        //Set up the bottom pane
        this.bottomPane.setLayout( new BorderLayout() );

        //Add the bottom pane straight away
        this.mainFrame.getContentPane().add( this.bottomPane, BorderLayout.SOUTH );
    }

    /**
     * This will initialise the limits for the dividers
     */
    protected void doInitialisation()
    {
        Point mainPoint = this.mainFrame.getLocationOnScreen();

        this.maxBottomRight.x = mainPoint.x + this.mainFrame.getWidth();
        this.maxBottomRight.y = mainPoint.x + this.mainFrame.getHeight();

        this.maxTopLeft.x = mainPoint.x;
        this.maxTopLeft.y = mainPoint.x;
    }

    /**
     * This method will update all the text of the menus and menu
     * related interface components for the current locale.
     */
    public void doDialect()
    {
        //Make sure it exists first
        if (this.bottomPM != null)
        {
            this.bottomPM.doDialect();
        }

        //Make sure it exists first
        if (this.leftPM != null)
        {
            this.leftPM.doDialect();
        }

        //Make sure it exists first
        if (this.rightPM != null)
        {
            this.rightPM.doDialect();
        }

        //Make sure it exists first
        if (this.topPM != null)
        {
            this.topPM.doDialect();
        }
    }

    /**
     * Gets the command line to be added to the command line module
     *
     * @returns The command line object for this agent
     */
    public BOnCommand getCommandLine()
    {
        return this.commandLine;
    }

    /**
     * Because adding a toolbar is simply a convenience
     * event for adding a JComponent to the top dynamic pane, we
     * simply map the event to a remove panel event.
     *
     * @param   event   The event fired
     */
    protected void removeToolBar( BAddToolBarCE event )
    {
        Vector buttons = event.getButtons();
        JComponent currentComponent;
        int removeIndex;

        if (theToolBar != null)
        {
            //Run through and add our stuff to it
            for (int i = 0; i < buttons.size(); i++)
            {
                currentComponent = (JComponent)buttons.elementAt(i);

                removeIndex = theToolBar.getComponentIndex(currentComponent);

                //See if we've left a separator
                if (removeIndex > 0 && theToolBar.getComponentAtIndex(removeIndex -1) instanceof JSeparator)
                {
                    theToolBar.remove(--removeIndex);
                }

                //Remove the actual component
                theToolBar.remove(removeIndex);
            }

            //If there's nothing left on the tool bar, remove it
            if (theToolBar.getComponentCount() <= 0)
            {
                this.mainFrame.getContentPane().remove(theToolBar);
                theToolBar = null;
            }
        }

        //Indicate success
        event.doComplete(BCommandComplete.kSuccess);
    }

    /**
     * Because we use the north, south, east and west sides
     * of the main frame for our dynamic panes, we can't
     * put a tool bar in there at this stage.
     * <p>
     * Our work around for the time being is simply to place
     * the tool bar given on the top dynamic pane.
     *
     * @param   event   The event fired
     */
    protected void addToolBar( BAddToolBarCE event )
    {
        Vector buttons = event.getButtons();
        BAddToolBarCE currentToolBar;
        JComponent currentComponent;
        int priority = 0;
        int insertIndex = -1;

        if (theToolBar == null)
        {
            theToolBar = new JToolBar(JToolBar.HORIZONTAL);
            theToolBar.setFloatable(false);
            this.mainFrame.getContentPane().add( theToolBar, BorderLayout.NORTH );
            this.mainFrame.getContentPane().doLayout();
        }

        //Need to put the buttons in the priority list
        for (int i = 0; i < toolBars.size(); i++)
        {
            currentToolBar = (BAddToolBarCE)toolBars.elementAt(i);

            if (event.getPriority() <= currentToolBar.getPriority())
            {
                insertIndex = i;
            }
        }

        //Put it into the vector in the right place
        if (insertIndex < 0)
        {
            toolBars.addElement(event);
        }
        else
        {
            //This will insert the tool bar components in the vector
            toolBars.insertElementAt(event, insertIndex);
        }

        //Remove everything in the tool bar
        theToolBar.removeAll();

        //Run through all of the tool bars and add their contents
        for (int i = toolBars.size() -1; i >= 0; i--)
        {
            currentToolBar = (BAddToolBarCE)toolBars.elementAt(i);
            buttons = currentToolBar.getButtons();

            //Make sure we separate the tool bars
            if (theToolBar.getComponentCount() > 0)
            {
                theToolBar.addSeparator();
            }

            //Run through and add our stuff to it
            for (int j = 0; j < buttons.size(); j++)
            {
                currentComponent = (JComponent)buttons.elementAt(j);

                if (currentComponent instanceof JSeparator)
                {
                    theToolBar.addSeparator();
                }
                else
                {
                    theToolBar.add(currentComponent);
                }
            }
        }

        //Indicate success
        event.doComplete(BCommandComplete.kSuccess);
    }

    /**
     * When the BWindowManager receives a remove status event
     * this method will remove it from the status bar. If the status
     * bar contains no more items, it will be removed.
     *
     * @param   addStatusEvent  The event received
     * Please complete the missing tags for addStatus
     * @return
     * @pre
     * @post
     */
    protected void removeStatus( BAddStatusCE removeStatusEvent )
    {
        JPanel panel = removeStatusEvent.getPanel();

        //See if we already have a status bar
        if( this.statusBar != null )
        {
            //Remove the panel from the status bar
            this.statusBar.remove( panel );

            //If there are no components left, remove the status bar
            if (this.statusBar.getComponentCount() <= 0)
            {
                //removes status bar
                this.bottomPane.remove( this.statusBar);
                this.statusBar = null;

                //Make sure the parent is relayed out
                this.bottomPane.getParent().doLayout();
            }
            else
            {
                //Refresh the status bar
                this.statusBar.repaint();
                this.statusBar.doLayout();
            }
        }

        //Indicate the event is done
        removeStatusEvent.doComplete(BCommandComplete.kSuccess);
    }

    /**
     * When the BWindowManager receives an add status event
     * this method will place it on the status bar, creating it
     * if it doesn't already exist.
     *
     * @param   addStatusEvent  The event received
     * Please complete the missing tags for addStatus
     * @return
     * @pre
     * @post
     */
    protected void addStatus( BAddStatusCE addStatusEvent )
    {
        JPanel panel;
        BAddStatusCE currentStatus;
        JComponent components;
        int insertIndex = -1;

        //See if we already have a status bar
        if( this.statusBar == null )
        {
            //create status bar
            this.statusBar = new JPanel();
            this.statusBar.setSize( 100, 40 );
            this.statusBar.setLayout( new FlowLayout( FlowLayout.LEFT ) );
            this.statusBar.setVisible( true );
            this.statusBar.setBorder( new BevelBorder( BevelBorder.LOWERED ) );
            this.bottomPane.add( this.statusBar, BorderLayout.SOUTH );
        }

        //Need to put the buttons in the priority list
        for (int i = 0; i < statusItems.size(); i++)
        {
            currentStatus = (BAddStatusCE)statusItems.elementAt(i);

            if (addStatusEvent.getPriority() <= currentStatus.getPriority())
            {
                insertIndex = i;
            }
        }

        //Put it into the vector in the right place
        if (insertIndex < 0)
        {
            statusItems.addElement(addStatusEvent);
        }
        else
        {
            //This will insert the tool bar components in the vector
            statusItems.insertElementAt(addStatusEvent, insertIndex);
        }

        //Remove everything in the tool bar
        statusBar.removeAll();

        //Run through all of the tool bars and add their contents
        for (int i = statusItems.size() -1; i >= 0; i--)
        {
            currentStatus = (BAddStatusCE)statusItems.elementAt(i);
            panel = currentStatus.getPanel();
            panel.setBorder( new BevelBorder( BevelBorder.LOWERED ) );
            statusBar.add( panel );
            panel.setFont( new Font( panel.getFont().getName(), Font.PLAIN, 9 ) );
            panel.setVisible( true );
        }

        //Indicate the event is done
        addStatusEvent.doComplete(BCommandComplete.kSuccess);
    }
         /**
     * Handles program flow when a BAddPanelCE is received
     *
     * @param   event   The event fired
     * Please complete the missing tags for addPanel
     * @return
     * @pre
     * @post
     */
    protected void addPanel( BAddPanelCE panelEvent )
    {
        Dimension size = panelEvent.getPanel().getSize();
        int rightLocation;
        int leftLocation;
        int dividerSize;
        int offset;

        //Determine where it should go
        if( panelEvent.getLocation() == BAddPanelCE.kLeft )
        {
            //Make sure the left pane manager exists
            if( this.leftPM == null)
            {
                //Create and add it
                this.leftPM = new BPaneManager( BPaneManager.kTabBottom );
                this.leftPM.setName( "Left PM" );
                leftDynamicPane.add( leftPM );
            }

            //Make sure the split pane is initialised the first time it is used
            if( ! this.leftDynamicPane.isInitialised() || !this.leftDynamicPane.isShowing())
            {
                size = panelEvent.getPanel().getSize();
                if (size == null || (size.width == 0) && (size.height == 0))
                {
                    this.setDynamicPaneSize( this.mainFrame.getSize(), leftDynamicPane );
                }
                else
                {
                    leftDynamicPane.setSize(size);
                    leftDynamicPane.setPreferredSize(size);
                }

                leftDynamicPane.setStepSize(panelEvent.getStepSize());
                leftDynamicPane.setInitialised( true );
                this.mainFrame.getContentPane().add( this.leftDynamicPane, BorderLayout.WEST );
            }

            //Add to the left pane manager
            this.leftPM.add( panelEvent.getPanel(), panelEvent.getName(), panelEvent.getScrollBarPolicy() );
            //*/
        }
        else if( panelEvent.getLocation() == BAddPanelCE.kBottom )
        {
            //Make sure the left pane manager exists
            if( this.bottomPM == null )
            {
                //Create and add it
                this.bottomPM = new BPaneManager( BPaneManager.kTabBottom );
                this.bottomPM.setName( "Bottom PM" );
                bottomDynamicPane.add( bottomPM );
            }

            //Make sure the split pane is initialised the first time it is used
            if( ! bottomDynamicPane.isInitialised() || !bottomDynamicPane.isShowing())
            {
                if (size == null || (size.width == 0) && (size.height == 0))
                {
                    this.setDynamicPaneSize( this.mainFrame.getSize(), bottomDynamicPane );
                }
                else
                {
                    bottomDynamicPane.setSize(size);
                    bottomDynamicPane.setPreferredSize(size);
                }

                bottomDynamicPane.setStepSize(panelEvent.getStepSize());
                bottomDynamicPane.setInitialised( true );
                this.bottomPane.add( this.bottomDynamicPane, BorderLayout.CENTER );
                this.bottomDynamicPane.setTargetParent( this.mainFrame );
            }

            //Add to the right pane manager
            this.bottomPM.add( panelEvent.getPanel(), panelEvent.getName(), panelEvent.getScrollBarPolicy() );
            //*/
        }
        else if( panelEvent.getLocation() == BAddPanelCE.kTop )
        {
            //Make sure the left pane manager exists
            if( this.topPM == null )
            {
                //Create and add it
                this.topPM = new BPaneManager( BPaneManager.kTabBottom );
                this.topPM.setName( "Top PM" );
                topDynamicPane.add( topPM );
            }

            //Make sure the split pane is initialised the first time it is used
            if( ! topDynamicPane.isInitialised() || !this.topDynamicPane.isShowing())
            {
                if (size == null || (size.width == 0) && (size.height == 0))
                {
                    this.setDynamicPaneSize( this.mainFrame.getSize(), topDynamicPane );
                }
                else
                {
                    topDynamicPane.setSize(size);
                    topDynamicPane.setPreferredSize(size);
                }

                topDynamicPane.setStepSize(panelEvent.getStepSize());
                topDynamicPane.setInitialised( true );
                this.mainFrame.getContentPane().add( this.topDynamicPane, BorderLayout.NORTH );
            }

            //Add to the right pane manager
            this.topPM.add( panelEvent.getPanel(), panelEvent.getName(), panelEvent.getScrollBarPolicy() );
            //*/
        }
        else if( panelEvent.getLocation() == BAddPanelCE.kRight )
        {
            //Make sure the left pane manager exists
            if( this.rightPM == null )
            {
                //Create and add it
                this.rightPM = new BPaneManager( BPaneManager.kTabBottom );
                this.rightDynamicPane.add( this.rightPM );
            }

            //Make sure the split pane is initialised the first time it is used
            if( ! rightDynamicPane.isInitialised() || !this.rightDynamicPane.isShowing())
            {
                if (size == null || (size.width == 0) && (size.height == 0))
                {
                    this.setDynamicPaneSize( this.mainFrame.getSize(), rightDynamicPane );
                }
                else
                {
                    rightDynamicPane.setSize(size);
                    rightDynamicPane.setPreferredSize(size);
                }

                rightDynamicPane.setStepSize(panelEvent.getStepSize());
                rightDynamicPane.setInitialised( true );
                this.mainFrame.getContentPane().add( this.rightDynamicPane, BorderLayout.EAST );
            }

            //Add to the right pane manager
            this.rightPM.add( panelEvent.getPanel(), panelEvent.getName(), panelEvent.getScrollBarPolicy() );
            //*/
        }

        //Indicate the event is done
        panelEvent.doComplete(BCommandComplete.kSuccess);
    }

    /**
     * Finds the BPaneManager with the target panel on it
     * and removes that panel
     *
     * @param   event   The event fired
     * Please complete the missing tags for removePanel
     * @return
     * @pre
     * @post                                                                                              
     */
    protected void removePanel( BRemovePanelCE event )
    {
        BPaneManager targetManager = this.locatePM( event.getLocation() );

        //Make sure there is a manager to search
        if( targetManager != null )
        {
            Debug.doPrintln( "Removing the component..." );
            targetManager.removeComponent( event.getTargetPanel() );

            //See if there's any other panels on the pane
            if (targetManager.getDynamicCount() <= 0)
            {
                this.removeDynamicPane(event.getLocation());
            }
        }

        //Indicate the event is done
        event.doComplete(BCommandComplete.kSuccess);
    }

    /**
     * Given a border layout location, this will remove
     * the dynamic pane at that position, regardless of
     * whether there is any components on it or not.
     *
     * @param   location    The location of the dynamic pane to remove
     */
    private void removeDynamicPane(int location)
    {
        //Determine which dynamic pane to remove
        if (location == BAddPanelCE.kBottom)
        {
            //Remove the bottom dynamic pane
            this.mainFrame.getContentPane().remove( this.bottomDynamicPane);
        }
        else if (location == BAddPanelCE.kLeft)
        {
            //Remove the left dynamic pane
            this.mainFrame.getContentPane().remove( this.leftDynamicPane);
        }
        else if (location == BAddPanelCE.kRight)
        {
            //Remove the right dynamic pane
            this.mainFrame.getContentPane().remove( this.rightDynamicPane);
        }
        else if (location == BAddPanelCE.kTop)
        {
            //Remove the top dynamic pane
            this.mainFrame.getContentPane().remove( this.topDynamicPane);
        }
    }

    /**
     * This will locate the pane manager associated
     * with the given screen location and return it
     *
     * @param   location    kLeft, kRight, kTop, kBottom
     * @return The BPaneManager responsible for that location
     * Please complete the missing tags for locatePM
     * @pre
     * @post
     */
    private BPaneManager locatePM( int location )
    {
        BPaneManager targetManager = null;
        
        //Determine which pane manager to use
        if( location == BAddPanelCE.kBottom )
        {
            targetManager = this.bottomPM;
        }
        else if( location == BAddPanelCE.kLeft )
        {
            targetManager = this.leftPM;
        }
        else if( location == BAddPanelCE.kRight )
        {
            targetManager = this.rightPM;
        }
        else if( location == BAddPanelCE.kTop )
        {
            targetManager = this.topPM;
        }
        
        //Return the located manager if any
        return targetManager;
    }

    /**
     * This method will set the dividers to their appropriate
     * locations given the size of the frame they are contained
     * within.
     *
     * @param       frameSize       The size of the frame
     * Please complete the missing tags for setDynamicPaneSize
     * @return
     * @pre
     * @post
     */
    private void setDynamicPaneSize( Dimension frameSize, BDynamicPane targetPane )
    {
        //Do top one first
        int height = frameSize.height;
        int width = frameSize.width;
        int componentHeight = ( ( int ) ( height * 0.2 ) );
        // - dividerSize;
        int componentWidth = ( ( int ) ( width * 0.2 ) );
        // - dividerSize;
        //Set the divider locations to be used when they become visible
        targetPane.setSize( componentWidth, componentHeight );
        targetPane.setPreferredSize( targetPane.getSize() );
    }
    
    /**
     * Finds the BPaneManager with the target panel on it
     * and attempts to make that panel the currently selected panel
     *
     * @param   event   The event fired
     * Please complete the missing tags for selectPanel
     * @return
     * @pre
     * @post
     */
    protected void selectPanel( BSelectPanelCE event )
    {
        BPaneManager targetManager = this.locatePM( event.getLocation() );
        
        //Make sure there is a manager to search
        if( targetManager != null )
        {
            targetManager.selectComponent( event.getTargetPanel() );
        }

        //Indicate the event is done
        event.doComplete(BCommandComplete.kSuccess);
    }

    /**
     * This will allow the user to access the menus
     * via the command line.
     *
     * @author Tristan Austin
     */
    public class BPanelCL implements BOnCommand
    {
        /**
         * When the command mapped to this command line
         * object is called, the paramaters and streams
         * are passed in here.
         *
         * @param   params  The paramaters to the command
         * @param   in  The input stream to read from
         * @param   out The output stream to write to
         * @return An -1 on failure
         */
        public int onCommand( BStringList params, InputStream in, PrintStream out )
        {
            String tabbedPane;
            String location = null;
            BPaneManager paneManager = BPanelAgent.this.leftPM;
            int tabIndex = 0;
            int count = params.getCount();

            //Make sure we have enough paramaters
            if (count == 2 || count == 3)
            {
                //Determine the locatin of the paramaters
                if (count == 2)
                {
                    tabIndex = 1;
                }
                else if (count == 3)
                {
                    location = params.get(1);
                    tabIndex = 2;
                }

                //Get the command and the
                tabbedPane = params.get(tabIndex);

                //Go with default pane manager if not specified
                if (location != null)
                {
                    location = location.toLowerCase();

                    //Determine which pane manager to search (defaults to left)
                    if ("left".startsWith(location))
                    {
                        paneManager = BPanelAgent.this.leftPM;
                    }
                    else if ("right".startsWith(location))
                    {
                        paneManager = BPanelAgent.this.rightPM;
                    }
                    else if ("bottom".startsWith(location))
                    {
                        paneManager = BPanelAgent.this.bottomPM;
                    }
                    else if ("top".startsWith(location))
                    {
                        paneManager = BPanelAgent.this.topPM;
                    }
                }

                //Get the tabbed pane mapping to the given name
                if (!paneManager.selectComponentCalled(tabbedPane))
                {
                    out.println("The panel \"" + tabbedPane + "\" not found on the " + location + " panel");
                }
            }
            else
            {
                //Indicate wrong usage
                out.println(this.getHelp(params));
            }

            return 0;
        }

        /**
         * When the user types "help" at the command prompt, the kernel
         * will call the getHelp method on all the registered onCommand
         * objects. This will indicate how to use this command line option.
         *
         * @param   params  The paramatesr to the help call (eg. help nadduser)
         */
         public String getHelp( BStringList params )
         {
            String command = params.get(0);
            StringBuffer help = new StringBuffer(744);

            //Put the help together
            help.append(command + " usage:\n");
            help.append(command + " panel <left|right|top|bottom> <panelname>   Will select the panel on the given side with the given panel name\n");
            help.append(command + " panel <panelname>                           Will select the panel with the given panel name on the left side\n");

            return help.toString();
         }
    }
}
