

package au.edu.swin.synergy.plugin.mod13;

import au.edu.swin.synergy.plugin.mod13.event.BAddWindowCE;
import au.edu.swin.synergy.plugin.mod13.event.BCloseWindowCE;
import au.edu.swin.synergy.plugin.mod13.event.BSelectWindowCE;
import au.edu.swin.synergy.plugin.mod13.event.BGetFilesCE;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowAdapter;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowEvent;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowBinder;
import au.edu.swin.synergy.plugin.mod13.bwt.BFocusManager;
import au.edu.swin.synergy.plugin.mod13.bwt.BTabListener;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;

import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;

import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.dialect.BString;

import au.edu.swin.synergy.plugin.command.BOnCommand;

import au.edu.swin.synergy.lib.BStringList;

import au.edu.swin.synergy.debug.Debug;

import java.awt.*;
import java.awt.event.*;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.IOException;
import java.io.File;

import javax.swing.*;
import javax.swing.filechooser.*;
import java.beans.PropertyVetoException;

import java.net.URL;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

/**
 * Handles all the functionality and management of the
 * internal and normal frames on the Window manager.
 *
 * @author Tristan Austin
 */
class BWindowAgent
{
    /**
     * This is the desktop on which all internal frames
     * will be placed
     */
    JDesktopPane desktop = new JDesktopPane();

    /**
     * The darker of the two synergy blues
     */
    Color synergyDarkBlue =  new Color(0x3A, 0x51, 0x95);

    /**
     * The darker of the two synergy blues
     */
    Color synergyLightBlue =  new Color(0xB8, 0xCC, 0xE8);

	/**
	 * The name of the image to be displayed on the desktop
	 */
	String imageName = null;

	/**
	 * The UI for the Desktop
	 */
	 BDesktopPaneUI desktopUI = null;

	/**
	 * The desktop manager
	 */
	 DesktopManager desktopManager;

    /**
     * The hashtable mapping content to windows
     */
    private Hashtable contentToWindows = new Hashtable();

    /**
     * This is a list of all the menus currently in the menu
     */
    private Hashtable windowsToMenuItems = new Hashtable();

    /**
     * This is a list of all the menus currently in the menu
     */
    private Hashtable menuItemsToWindows = new Hashtable();

    /**
     * This maps the window content to their locale sensitive names
     */
    private Hashtable windowsToTitles = new Hashtable();

    /**
     * Handles program flow when a frame is closed
     */
    private InternalWindowListener frameListener = new InternalWindowListener();

    /**
     * The default window width for window with no specified size
     */
    public static final int kDefaultWindowWidth = 400;

    /**
     * The default window height for window with no specified size
     */
    public static final int kDefaultWindowHeight = 300;

    /**
     * This is the main
     */
    JFrame mainFrame = null;

    /**
     * The window menu which reflects all the currently open windows
     */
    JMenu windowMenu;

    /**
     * The command line for windows
     */
    BWindowCL commandLine = new BWindowCL();

    /**
     * Tiles the windows horizontally
     */
    JMenuItem tileHMenuItem = new JMenuItem();

    /**
     * Tiles the windows vertically
     */
    JMenuItem tileVMenuItem = new JMenuItem();

    /**
     * Tiles the windows in a grid
     */
    JMenuItem tileMenuItem = new JMenuItem();

    /**
     * Lays out all the windows in a cascade
     */
    JMenuItem cascadeMenuItem = new JMenuItem();

    /**
     * This is responsible for rearranging the window layout when selected
     */
    BWindowToolListener windowToolListener = new BWindowToolListener();

    /**
     * Creates a new window agent with the container
     * the desktop is to be added to. This should
     * usually be the content pane of the main frame
     *
     * @param   mainContainer   The container of the desktop
     */
    public BWindowAgent(JFrame mainFrame, JMenu windowMenu, String iName)
    {
		BFocusManager focusManager;
        this.mainFrame = mainFrame;
        this.windowMenu = windowMenu;
		this.imageName = iName;

        //Add the pane managers
        this.desktop = new JDesktopPane();
        this.desktop.setBackground(this.synergyDarkBlue);
        this.desktopManager = this.desktop.getDesktopManager();
		focusManager = new BFocusManager(new BTabListener(this.desktop));
		javax.swing.FocusManager.setCurrentManager(focusManager);
		if ( this.imageName == null)
		{
			this.desktopUI =  new BDesktopPaneUI();
		}
		else
		{
			this.desktopUI= new BDesktopPaneUI( imageName );
		}
		this.desktop.setUI( this.desktopUI );

        //Need to add the action listeners to the window tool menu items
        tileHMenuItem.addActionListener(windowToolListener);
        tileVMenuItem.addActionListener(windowToolListener);
        tileMenuItem.addActionListener(windowToolListener);
        cascadeMenuItem.addActionListener(windowToolListener);

        //Add the window menu items
        this.windowMenu.add(tileMenuItem);
        this.windowMenu.add(cascadeMenuItem);
        this.windowMenu.add(tileVMenuItem);
        this.windowMenu.add(tileHMenuItem);

        //Disable the window items
        this.enableWindowTools(false);

        //Add the desktop to the main frame
        mainFrame.getContentPane().add( this.desktop, BorderLayout.CENTER );
		mainFrame.addKeyListener(focusManager);
		desktop.resetKeyboardActions();
    }

    /**
     * This will enable or disable the window menu items
     * as specified by the enabled value given.
     *
     * @param   enabled True to enable, false to disable
     */
    protected void enableWindowTools(boolean enabled)
    {
        //Set the tiled menu item options
        this.tileHMenuItem.setEnabled(enabled);
        this.tileVMenuItem.setEnabled(enabled);
        this.tileMenuItem.setEnabled(enabled);
        this.cascadeMenuItem.setEnabled(enabled);
    }

    /**
     * This method will update all the text of the menus and menu
     * related interface components for the current locale.
     */
    protected void doDialect()
    {
        Enumeration windows = this.windowsToTitles.keys();
        Container window;
        BString title;

        //Set the tiled menu item options
        this.tileHMenuItem.setText(BDialect.getString(this, "tileH"));
        this.tileVMenuItem.setText(BDialect.getString(this, "tileV"));
        this.tileMenuItem.setText(BDialect.getString(this, "tile"));
        this.cascadeMenuItem.setText(BDialect.getString(this, "cascade"));

        //Run through all the windows and find it
        while(windows.hasMoreElements())
        {
            window = (Container)windows.nextElement();
            title = (BString)this.windowsToTitles.get(window);

            //Make sure there is actually a title
            if (title != null)
            {
                //See if we have a match
                if (window instanceof JInternalFrame)
                {
                    ((JInternalFrame)window).setTitle(title.toString());
                }
                else if (window instanceof JDialog)
                {
                    ((JDialog)window).setTitle(title.toString());
                }
            }
        }
    }

    /**
     * Gets the command line to be added to the command line module
     *
     * @returns The command line object for this agent
     */
    protected BOnCommand getCommandLine()
    {
        return this.commandLine;
    }

    /**
     * When the window agent receives a standard out
     * command event, it will show a dialog with the
     * given message, set the response type to that
     * supplied and let the source know what the users
     * response is.
     *
     * @param   standardOutCE   The standard out event
     */
    protected void doStandardOut(BStandardOutCE standardOutCE)
    {
        JOptionPane messageDialog = null;
        String message = standardOutCE.getMessage();
        boolean requiresResponse = standardOutCE.isResponseRequired();
        int messageType = standardOutCE.getMessageType();
        int responseOption = standardOutCE.getUserResponseOption();
        int userResponse = -1;
        Object responses[] = new Object[]{"Yes", "No"};

        //Map the message type to a JOption pane message type
        if (messageType == BStandardOutCE.kError)
        {
            messageType = JOptionPane.ERROR_MESSAGE;
        }
        else if (messageType == BStandardOutCE.kInfo)
        {
            messageType = JOptionPane.INFORMATION_MESSAGE;
        }
        else if (messageType == BStandardOutCE.kWarning)
        {
            messageType = JOptionPane.WARNING_MESSAGE;
        }

        //Map the user response option to JOption pane type
        if (responseOption == BStandardOutCE.kDefaultOption)
        {
            responseOption = JOptionPane.DEFAULT_OPTION;

        }
        else if (responseOption == BStandardOutCE.kOkCancelOption)
        {
            responseOption = JOptionPane.OK_CANCEL_OPTION;
            responses = new Object[]{BDialect.getString("ok"), BDialect.getString("cancel")};
        }
        else if (responseOption == BStandardOutCE.kYesNoCancelOption)
        {
            responseOption = JOptionPane.YES_NO_CANCEL_OPTION;
            responses = new Object[]{BDialect.getString("yes"), BDialect.getString("no"), BDialect.getString("cancel")};
        }
        else if (responseOption == BStandardOutCE.kYesNoOption)
        {
            responseOption = JOptionPane.YES_NO_OPTION;
            responses = new Object[]{BDialect.getString("yes"), BDialect.getString("no")};
        }

        //See if we need to set response labels
        if (requiresResponse)
        {
            //Show the dialog and get the users response
            userResponse = JOptionPane.showOptionDialog(this.mainFrame, message, BDialect.getString(this, "dialogtitle"), responseOption, messageType, null, responses, responses[0]);

            //Determine the response of the user
            switch(userResponse)
            {
                //Any of these should be indicated as a negative response
                case JOptionPane.CANCEL_OPTION:
                case JOptionPane.CLOSED_OPTION:
                {
                    //Indicate the user cancelled
                    standardOutCE.setUserResponse(BStandardOutCE.kCancel);
                    break;
                }
                case JOptionPane.NO_OPTION:
                {
                    //Set the response to false
                    standardOutCE.setUserResponse(BStandardOutCE.kNo);
                    break;
                }
                case JOptionPane.OK_OPTION:
                {
                    //Set the response to true
                    standardOutCE.setUserResponse(BStandardOutCE.kYes);
                    break;
                }
                default:
                {
                    //If we get here then then we haven't hadn't a response from the option pane we should have
                    Debug.doPrintln("The JOption result is not a recognised response: " + userResponse, Debug.kErrorLevel);
                    break;
                }
            }
        }
        else
        {
            //Simply show the dialog
            JOptionPane.showConfirmDialog(this.mainFrame, message, BDialect.getString(this, "dialogtitle"), responseOption, messageType);
        }

        //Indicate completion
        standardOutCE.doComplete(BCommandComplete.kSuccess);
    }

	/**
	 * This will display a JFileChooser and send the results of the
	 * display back along with the event to the caller.
	 *
	 * @param	event	The get files event
	 */
	protected void doGetFile(BGetFilesCE event)
	{
		String startingDirectory = event.getStartingDir();
		FileFilter fileFilter = event.getFileFilter();
		String title = event.getTitle();
		String accept = "Save";
		boolean isLoading = event.isLoading();
        boolean fileMustExist = event.getFileMustExist();
		int response;
		JFileChooser fileChooser = new JFileChooser(startingDirectory);

		//Set the paramaters
		fileChooser.setDialogTitle(title);
		fileChooser.setFileFilter(fileFilter);
		fileChooser.setSelectedFile(event.getSelectedFile());

		if (isLoading && fileMustExist)	
		{
			accept = "Open";
		}
		else if (isLoading && !fileMustExist)	
		{
			accept = "New";
		}

		Debug.doPrintln("Displaying browser...");

		//Display the dialog and get the users response
		while(true)
		{
			response = fileChooser.showDialog(this.mainFrame, accept);
	
			if (response == JFileChooser.APPROVE_OPTION)
			{
				File selectedFile = fileChooser.getSelectedFile();
				if(isLoading && fileMustExist && (!selectedFile.exists()))
				{
            		JOptionPane.showMessageDialog(this.mainFrame, BDialect.getString(this, "filemustexist"), BDialect.getString(this, "dialogtitle"), JOptionPane.WARNING_MESSAGE);
				}
				else
				{
					event.setSelectedFile(fileChooser.getSelectedFile());
					break;
				}
			}
			else
			{
				event.setSelectedFile(null);
				break;
			}
		}

		//Indicate that we're done
		event.doComplete(BCommandComplete.kSuccess);
	}

    /**
     * Handles program flow when an add window event is received.
     * This method will place the panel in the event on a JInternalFrame
     * and make it available.
     *
     * @param   windowEvent The event fired
     * Please complete the missing tags for addWindow
     * @return
     * @pre
     * @post
     */
    protected void addWindow( BAddWindowCE windowEvent )
    {
        JComponent panel = windowEvent.getWindowContent();
        Dimension screenSize = null;
        JDialog newDialog = null;
        JComponent parentPanel = null;
        JInternalFrame frame = null;
		Insets insets;
        int windowWidth = 0;
		int windowHeight = 0; 

        //If the panel size hasn't already been set, set it to the default size
        if( panel.getSize() == null || panel.getWidth() == 0 || panel.getHeight() == 0 )
        {
            windowWidth = kDefaultWindowWidth;
			windowHeight = kDefaultWindowHeight;
        }
		else
		{
			windowWidth = panel.getWidth();
			windowHeight = panel.getHeight();
		}


        //See if it is a normal window or a dialog
        if( windowEvent.isDialog() )
        {
            //Create and display the dialog
            screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            parentPanel = windowEvent.getParentDialog();
            
            newDialog = new JDialog( this.mainFrame, windowEvent.getName().toString(), windowEvent.isModal() );

            newDialog.setSize( windowWidth, windowHeight );
            newDialog.setLocation( ( screenSize.width - newDialog.getSize().width ) / 2, ( screenSize.height - newDialog.getSize().height ) / 2 );
			newDialog.pack();
			insets = newDialog.getInsets();
			newDialog.setSize(windowWidth + insets.left + insets.right, windowHeight + insets.top + insets.bottom);
            newDialog.getContentPane().add( panel );
            newDialog.addWindowListener( new BWindowBinder( this.frameListener, panel ) );

            //Make sure only one thread accesses the hashtable at once
            synchronized( this.contentToWindows )
            {
                this.contentToWindows.put( panel, newDialog );
                this.windowsToTitles.put(newDialog, windowEvent.getName());
            }

            //Add the window listeners
            windowEvent.addListenersToWindow( newDialog );

            newDialog.setDefaultCloseOperation(windowEvent.getCloseAction());
			newDialog.setResizable(windowEvent.getResizable());
            newDialog.setVisible(true);
        }
        else
        {
            frame = new JInternalFrame( windowEvent.getName().toString() );
            frame.setResizable( windowEvent.getResizable() );
            frame.setClosable( windowEvent.getClosable() );
            frame.setMaximizable( windowEvent.getMaximizable() );
            frame.setIconifiable( windowEvent.getIconifiable() );

            frame.getContentPane().add( panel );

            //Determine the size for the window
            frame.setSize( windowWidth, windowHeight);
			frame.pack();
			insets = frame.getInsets();
			frame.setSize(windowWidth + insets.left + insets.right, windowHeight + insets.top + insets.bottom);
            frame.addInternalFrameListener( new BWindowBinder( this.frameListener, panel ) );
            frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE/*windowEvent.getCloseAction()*/);
            desktop.add( frame );

            //Add the window to the menu
            this.windowsToTitles.put(frame, windowEvent.getName());
            this.addWindowToWindowList(frame, panel);

            //Add the window listeners
            windowEvent.addListenersToWindow( frame );

            frame.setFrameIcon(this.getSmallWindowIcon());

            //Can only call show if it is showing
            if (this.desktop.isShowing())
            {
                //This will display it if it's not already
                //And bring it to the front regardless
                frame.show();
                frame.requestFocus();
            }
        }

        //Indicate success of add window, the event source will receive this when the dialog closes
        //Or when the window becomes visible
        windowEvent.doComplete( BCommandComplete.kSuccess );
		desktop.resetKeyboardActions();
    }

    /**
     * This will add the given window to the appropriate menus
     * and to the appropriate tables. It also makes sure the name of
     * the component isn't already used, if it is, it will append
     * number suffix to each of the same named windows.
     *
     * @param   newWindow   The window to add
     * @param   content     The content of the window beging added
     */
    private void addWindowToWindowList(Container newWindow, JComponent content)
    {
        JMenuItem newWindowItem;
        String title;

        //Get the title of the window
        if (newWindow instanceof JInternalFrame)
        {
            title = ((JInternalFrame)newWindow).getTitle();

            //Make sure only one thread accesses the hashtable at once
            synchronized( this.contentToWindows )
            {
                this.contentToWindows.put( content, newWindow );
            }

            //Add a menu item for this item to the window menu
            synchronized(this.windowMenu)
            {
                newWindowItem = new JMenuItem( title );
                newWindowItem.addActionListener( new windowMenuItem( (JInternalFrame)newWindow ) );

                //This will put in a separator if required
                if (this.windowMenu.getItemCount() == 4)
                {
                    this.windowMenu.add(new JSeparator());
                    this.enableWindowTools(true);
                }

                windowMenu.add( newWindowItem );

                //Map the window to the
                synchronized(this.windowsToMenuItems)
                {
                    this.windowsToMenuItems.put( newWindow, newWindowItem );
                    this.menuItemsToWindows.put(newWindowItem, newWindow);
                }

                //Make sure we don't have any duplicates
                this.updateNames(newWindowItem);
            }
        }
    }

    /**
     * This will run through all the windows and make
     * sure there aren't any with the same name. If it finds
     * duplicates, it will append a number to the end of
     * the window title.
     *
     * firstItem    The item to insert
     */
    private void updateNames(JMenuItem newMenuItem)
    {
        Object compoundArgs[] = new Object[2];
        Integer currentInteger = null;
        Component windows[];
        char numSeparator = BDialect.getString(this, "numseparator").toString().charAt(0);
        JMenuItem secondItem = null;
        String firstName = null;
        String secondName = null;
        JInternalFrame currentFrame = null;
        boolean initialise = false;

        //Make sure we have synchronization on the window items
        windows = this.windowMenu.getMenuComponents();
        firstName = newMenuItem.getText();
        compoundArgs[0] = firstName;

        //And see if it exists elsewhere in the array
        for (int i = (windows.length -1); i >= 0; i--)
        {
            //Ignore the separators
            if (windows[i] instanceof JMenuItem)
            {
                secondItem = (JMenuItem)windows[i];
                secondName = secondItem.getText();


                if (newMenuItem != secondItem)
                {
                    //See if this menu item is the same name as the previous one
                    //Excluding the suffix if there is one
                    if (secondName.startsWith(firstName))
                    {
                        if ((firstName.length() == secondName.length()))
                        {
                            initialise = true;
                        }
                        else
                        {
                            int lastSpace = secondName.lastIndexOf(numSeparator);

                            //Make sure it is a number
                            if (lastSpace >= 0)
                            {
                                try
                                {
                                    currentInteger = new Integer(secondName.substring(lastSpace +1).trim());

                                    //If we get here, the value is a number
                                    currentFrame = (JInternalFrame)this.menuItemsToWindows.get(newMenuItem);
                                    compoundArgs[1] = new Integer((currentInteger.intValue() + 1));
                                    newMenuItem.setText(BDialect.formatText(BDialect.getString(this, "windowname"), compoundArgs));
                                    currentFrame.setTitle(newMenuItem.getText());

                                    //Because we started at the bottom of the list, this will
                                    //be the largest number so now we finish
                                    break;
                                }
                                catch(NumberFormatException numFormatException)
                                {
                                    //The last part of the name isn't a number, need to start a new one
                                    initialise = true;
                                }
                            }
                        }

                        //If we need to initialise the numbering system, do it here
                        if (initialise)
                        {
                            //There is no suffix, just append 1 and 2 respectively
                            currentFrame = (JInternalFrame)this.menuItemsToWindows.get(newMenuItem);
                            compoundArgs[1] = new Integer(2);
                            newMenuItem.setText(BDialect.formatText(BDialect.getString(this, "windowname"), compoundArgs));
                            currentFrame.setTitle(newMenuItem.getText());

                            //Set the new name of the existing window
                            currentFrame = (JInternalFrame)this.menuItemsToWindows.get(secondItem);
                            compoundArgs[1] = new Integer(1);
                            secondItem.setText(BDialect.formatText(BDialect.getString(this, "windowname"), compoundArgs));
                            currentFrame.setTitle(secondItem.getText());

                            //Because we started at the bottom of the list, this will
                            //be the largest number so now we finish
                            break;
                        }
                    }
                }
            }
        }
    }

    /**
     * This will select the window containing the
     * component contained in this event by de-iconifying
     * it if necessary and bringing it to the front.
     *
     * @param   event   The select window event
     */
    protected void selectWindow( BSelectWindowCE event )
    {
        JComponent windowContent = event.getWindowContent();
        Container window = ( Container ) this.contentToWindows.get( windowContent );

        //Hide the window
        if( window instanceof Window )
        {
            ( ( Window ) window ).show();
        }
        else if (window instanceof JInternalFrame)
        {
            JInternalFrame internalFrame = ((JInternalFrame)window);
            internalFrame.show();
        }
        else
        {
            Debug.doPrintln("BWindow Agent got a non-window", Debug.kErrorLevel);
        }

        //Indicate the event is done
        event.doComplete(BCommandComplete.kSuccess);
    }

    /**
     * This will locate the window associated with the panel
     * supplied in the close window event and close the dialog
     * or window as required.
     *
     * @param   event   The close window event fired
     * Please complete the missing tags for closeWindow
     * @return
     * @pre
     * @post
     */
    protected void closeWindow( BCloseWindowCE event )
    {
        JComponent windowContent = event.getWindowContent();
        Container window = ( Container ) this.contentToWindows.get( windowContent );

        //Hide the window
        if( window instanceof Window )
        {
            ( ( Window ) window ).dispose();
        }
        else if (window instanceof JInternalFrame)
        {
            try
            {
                ((JInternalFrame)window).setClosed( true );
            }
            catch(PropertyVetoException vetoException)
            {
                Debug.doPrintln("Internal Frame Close Exception: " + vetoException);
                vetoException.printStackTrace();
            }
        }
        else
        {
            Debug.doPrintln("BWindow Agent got a non-window", Debug.kErrorLevel);
        }

        //Indicate the event is done
        event.doComplete(BCommandComplete.kSuccess);
    }

    /**
     * Gets the file path to the small window icon
     *
     * @return The relative file name and path as a String
     */
    public ImageIcon getSmallWindowIcon()
    {
        return BSwingUtilities.getImage(this, BDialect.getString(this, "smallwindowicon"));
    }

    /**
     * Gets the file path to the large window icon
     *
     * @return The relative file name and path as a String
     */
    public ImageIcon getLargeWindowIcon()
    {
        return BSwingUtilities.getImage(this, BDialect.getString(this, "largewindowicon"));
    }

    /**
     * When one of the internal frames is closed, this method is called
     * so that the WindowManager can remove it from the Window menu
     *
     * @param   internalFrame   The window that was closed
     * Please complete the missing tags for doWindowClose
     * @return
     * @pre
     * @post
     */
    private void doWindowClose( JInternalFrame internalFrame )
    {
        JMenuItem frameMenuItem = ( JMenuItem ) this.windowsToMenuItems.get( internalFrame );

        this.windowMenu.remove( frameMenuItem );

        //This will put in a separator if required
        if (this.windowMenu.getItemCount() == 5)
        {
            this.windowMenu.remove(4);
            this.enableWindowTools(false);
        }

        this.windowsToMenuItems.remove( internalFrame );
        this.menuItemsToWindows.remove(frameMenuItem);
    }

	/**
	 * Sets the name of the image to be displayed on the desktop
	 */
	public void setImageName( String name )
	{
		if ( name.endsWith(BDialect.getString(this, "jpegextension")) || name.endsWith(BDialect.getString(this, "gifextension")) )
		{
			this.imageName = name;
			desktopUI.setImage( this.imageName );
		}	
	}

    /**
     * This is responsible for laying out the windows appropriately
     * given the selected menu item.
     *
     * @author Tristan Austin
     */
    class BWindowToolListener implements ActionListener
    {
        /**
         * Tile vertically
         */
        final int kVertical = 0;

        /**
         * Tile horizonally
         */
        final int kHorizontal = 1;

        /**
         * Tile in grid
         */
        final int kGrid = 2;

        /**
         * Called when the user selects the menu items
         *
         * @param   actionEvent The event fired
         */
        public void actionPerformed(ActionEvent actionEvent)
        {
            Object source = actionEvent.getSource();
            JInternalFrame allFrames[] = desktop.getAllFrames();
            int size = allFrames.length;

             try
            {
                //We can't discriminate between iconified and main windows
                //(not when we're just changing the layout anyway)
                for (int i = 0; i < allFrames.length; i++)
                {
                    allFrames[i].setMaximum(false);
                    allFrames[i].setIcon(false);
                }
            }
            catch(PropertyVetoException vetoException)
            {
                vetoException.printStackTrace();
            }

            //Determine what to do
            if (source == tileHMenuItem)
            {
                this.tileInline(kHorizontal);
            }
            else if (source == tileVMenuItem)
            {
                this.tileInline(kVertical);
            }
            else if(source == tileMenuItem)
            {
                this.tileInline(kGrid);
            }
            else if(source == cascadeMenuItem)
            {
                this.cascade();
            }
        }

        /**
         * Tiles the windows horizontally
         */
        protected void tileInline(int type)
        {
            JInternalFrame allFrames[] = desktop.getAllFrames();
            int size = allFrames.length;

            //Make sure there are windows to tile
            if (size > 0)
            {
                LayoutManager currentLayout = desktop.getLayout();

                if (type == kHorizontal)
                {
                    desktop.setLayout(new GridLayout(0, size));
                }
                else if (type == kVertical)
                {
                    desktop.setLayout(new GridLayout(size, 0));
                }
                else if (type == kGrid)
                {
                    int gridWidth = (int)Math.sqrt(size);
                    desktop.setLayout(new GridLayout(gridWidth, gridWidth));
                }

                //Redo the layout
                desktop.doLayout();
                desktop.repaint();
                desktop.setLayout(currentLayout);
            }
        }

        /**
         * This will layout the windows in a cascading layout
         */
        protected void cascade()
        {
            int x = 0;
            int y = 0;
            int nextOffset = 10;
            int offset;
            int bottomRight = desktop.getWidth() - 500;
            JInternalFrame allFrames[] = desktop.getAllFrames();
            int size = allFrames.length;

            //Make sure there are windows to tile
            if (size > 0)
            {
                offset = allFrames[0].getHeight() - allFrames[0].getContentPane().getHeight() - (allFrames[0].getInsets().top * 2);
                try
                {
                    //Run through and move everything
                    for (int i = 0; i < size; i++)
                    {
                        allFrames[i].setMaximum(false);
                        allFrames[i].setLocation(x, y);
                        allFrames[i].show();

                        if (x <= bottomRight)
                        {
                            x += offset;
                            y += offset;
                        }
                        else
                        {
                            x = 0;
                            y = nextOffset;
                            nextOffset += offset;
                        }
                    }
                }
                catch(PropertyVetoException vetoException)
                {
                    vetoException.printStackTrace();
                }
            }
        }
    }

    /**
     * This handles the user interaction with the window menu items
     */
    class windowMenuItem implements java.awt.event.ActionListener
    {
        JInternalFrame frame;

        public windowMenuItem( JInternalFrame fFrame )
        {
            frame = fFrame;
        }

        /**
         * When a window menu item is selected, this method
         * is called to show the window
         *
         * @param   actionEvent The event fired
         */
        public void actionPerformed( ActionEvent actionEvent )
        {
            desktopManager.deiconifyFrame( this.frame );
            this.frame.show();
            this.frame.requestFocus();
        }
    }
    
    /**
     * This listens to all the windows and dialogs that are
     * added to the window manager. When they are closed it
     * removes them from the window menu, if they are in it,
     * and from the window hashtable.
     *
     * @author Tristan Austin
     */
    class InternalWindowListener extends BWindowAdapter
    {
        /**
         * Invoked when a window has been closed as the result of calling dispose on the window.
         */
        public void windowClosed( BWindowEvent windowEvent )
        {
            Object source = windowEvent.getSource();
            JComponent windowContent = windowEvent.getWindowContent();
            Object window = BWindowAgent.this.contentToWindows.get(windowContent);

            //Remove the window from the list
            BWindowAgent.this.windowsToTitles.remove(window);
            BWindowAgent.this.windowsToTitles.remove(window);
            BWindowAgent.this.contentToWindows.remove( windowContent );

            //If this is an internal frame, remove it from the window menu
            if( source instanceof JInternalFrame )
            {
                BWindowAgent.this.doWindowClose( ( JInternalFrame ) source );
            }
        }
    }

    /**
     * This will allow the user to access the menus
     * via the command line.
     *
     * @author Tristan Austin
     */
    public class BWindowCL implements BOnCommand
    {
        /**
         * When the command mapped to this command line
         * object is called, the paramaters and streams
         * are passed in here.
         *
         * @param   params  The paramaters to the command
         * @param   in  The input stream to read from
         * @param   out The output stream to write to
         * @return An -1 on failure
         */
        public int onCommand( BStringList params, InputStream in, PrintStream out )
        {
            String window;
            String command;
            JInternalFrame frame;
            int count = params.getCount();

            //Make sure there are enough to deal with
            if (count == 3)
            {
                command = params.get(1).toLowerCase();
                window = params.get(2);

                //Get the window mapping to the given name
                frame = this.getInternalFrame(window);

                try
                {
                    //Make sure it exists
                    if (frame != null)
                    {
                        //Handle each of the commands
                        if ("close".startsWith(command))
                        {
                            //Only close it if it is closeable
                            if (frame.isClosable())
                            {
                                frame.setClosed(true);
                            }
                            else
                            {
                                out.println("The window \"" + window + "\" is not closeable");
                            }
                        }
                        else if (BDialect.getString(this, "minimise").startsWith(command))
                        {
                            //Minimise the frame
                            frame.setIcon(true);
                        }
                        else if (BDialect.getString(this, "maximise").startsWith(command))
                        {
                            //Maximise the frame
                            frame.setMaximum(true);
                        }
                        else if ("restore".startsWith(command))
                        {
                            //Restore the frame
                            frame.setMaximum(false);
                            BWindowAgent.this.desktop.repaint();
                        }
                        else if ("tofront".startsWith(command))
                        {
                            //Move the frame to the front
                            frame.toFront();
                        }
                        else if ("toback".startsWith(command))
                        {
                            //Move the frame to the back
                            frame.toBack();
                        }
                        else if ("hide".startsWith(command))
                        {
                            //Hide the frame
                            frame.setVisible(false);
                        }
                        out.println(this.getHelp(params));
                    }
                    else
                    {
                        out.println("The window \"" + window + "\" wasn't found");
                    }
                }
                catch(PropertyVetoException vetoException)
                {
                    Debug.doPrintln("Property veto exception", vetoException);
                }
            }
            else
            {
                //Indicate wrong usage
                out.println(this.getHelp(null));
            }

            return 0;
        }

        /**
         * This will locate the internal frame with the given
         * name. If it can't find anything it will return null.
         *
         * @param   windowName  The name of the window to find
         * @return The internal frame matching the name, null if nothing found
         */
        private JInternalFrame getInternalFrame(String windowName)
        {
            Enumeration windows = BWindowAgent.this.contentToWindows.elements();
            JInternalFrame window;

            //Run through all the windows and find it
            while(windows.hasMoreElements())
            {
                window = (JInternalFrame)windows.nextElement();

                //See if we have a match
                if (window.getTitle().toLowerCase().startsWith(windowName.toLowerCase()))
                {
                    return window;
                }
            }

            //Didn't find a match
            return null;
        }

        /**
         * When the user types "help" at the command prompt, the kernel
         * will call the getHelp method on all the registered onCommand
         * objects. This will indicate how to use this command line option.
         *
         * @param   params  The paramatesr to the help call (eg. help nadduser)
         */
         public String getHelp( BStringList params )
         {
            String command = params.get(0);
            StringBuffer help = new StringBuffer(744);

            //Put the help together
            help.append(command + " usage:\n");
            help.append(command + " <windowname> close          Closes the window called <windowname>\n");
            help.append(command + " <windowname> hide           Hides the window called <windowname>\n");
            help.append(command + " <windowname> minimise       Minimises the window called <windowname>\n");
            help.append(command + " <windowname> maximise       Maximises the window called <windowname>\n");
            help.append(command + " <windowname> restore        Restores the window called <windowname> to its original size\n");
            help.append(command + " <windowname> tofront        Moves the window called <windowname> to the front\n");
            help.append(command + " <windowname> toback         Moves the window called <windowname> to the back\n");

            return help.toString();
         }
    }
}
