package au.edu.swin.synergy.plugin.mod13.bwt;

import java.awt.*;
import java.awt.event.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * The focus manager is going to handle the cycling through of
 * windows using control tab (and shift) as well as the default use
 * of cycling through components with normal tab.
 *
 * @author Tristan Austin
 */
public class BFocusManager extends javax.swing.DefaultFocusManager implements KeyListener
{
	/**
	 * The tab listener
	 */
	BTabListener tabListener;

	/**
	 * Indicates whether the control key is being held down
	 */
	boolean isControlDown = false;

	/**
	 * Creates a new focus manager that is going to handle cycling through windows
	 * on the given desktop pane.
	 *
	 * @param	desktopPane	The desktop pane to cycle the internal windows of
	 */
	public BFocusManager(BTabListener tabListener)
	{
		this.tabListener = tabListener;
	}

	/**
	 * This is the method that is called when the user presses any key on
	 * the swing interface. We can filter out the control tab keys and
	 * redirect them to the window agent. 
	 */
	public void processKeyEvent(Component component, KeyEvent keyEvent)	
	{
		//We only care if it was a tab
		if(keyEvent.getKeyCode() == KeyEvent.VK_TAB || keyEvent.getKeyChar() == '\t')
		{
			//Don't let the default focus handler do anything with control tabs
			if (component != null && (keyEvent.getModifiers() & ActionEvent.CTRL_MASK) != ActionEvent.CTRL_MASK)
			{	
				//Let the default process handle it
				super.processKeyEvent(component, keyEvent);
			}	
			else
			{
				//We only care about the pressed type
				if (keyEvent.getID() == KeyEvent.KEY_PRESSED)
				{
					if ((keyEvent.getModifiers() & ActionEvent.SHIFT_MASK) == ActionEvent.SHIFT_MASK)
					{
						this.tabListener.cycleBackward();
					}
					else
					{
						this.tabListener.cycleForward();
					}

					//Consume the event
					keyEvent.consume();
				}
			}
		}
		else if(keyEvent.getKeyCode() == KeyEvent.VK_CONTROL)
		{
			//When control is pressed, need to start stuff
			if (keyEvent.getID() == KeyEvent.KEY_PRESSED)
			{
				this.tabListener.setControl(true);
			}
			else if (keyEvent.getID() == KeyEvent.KEY_RELEASED) 
			{
				this.tabListener.setControl(false);
			}
		}
	}

	/**
	 * Redirects the key to the process key method
	 */
	public void keyPressed(KeyEvent keyEvent)
	{
		this.processKeyEvent(null, keyEvent);
	}

	/**
	 * Redirects the key to the process key method
	 */
	public void keyReleased(KeyEvent keyEvent)
	{
		this.processKeyEvent(null, keyEvent);
	}

	/**
	 * Redirects the key to the process key method
	 */
	public void keyTyped(KeyEvent keyEvent)
	{/*Ignore*/}
}
