// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/mod13/bwt/BSwingUtilities.java,v 1.16 1999/10/17 15:57:29 tp079210 Exp $

/**
 * Base class for modal dialogs.
 * @see
 * @concurrency
 * @copyright
 * @rref
 * Please complete these missing tags
 * @author
 */

package au.edu.swin.synergy.plugin.mod13.bwt;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.text.*;
import java.awt.*;
import java.lang.reflect.InvocationTargetException;

import java.util.Vector;
import java.net.URL;

import au.edu.swin.synergy.lib.BListModel;

import au.edu.swin.synergy.debug.Debug;
import javax.swing.tree.*;


/**
 * Because the current version of swing is not thread safe, all methods
 * on swing components need to be called from the awt event thread. Due to
 * the fact that Cohesion is so incredibly thread centric, this requires
 * a central means for performing these tasks.
 * <p>
 * This class needs to provide a means of setting all the required methods
 * with their given paramaters as needed. Obviously it won't cover everything
 * and so needs to be updated as new requirements come to hand.
 * <p>
 * The alternative is to write a your own inner class for each
 * method you call and call <code>SwingUtilities.invokeLater()</code> yourself
 *
 * @author Team Synergy
 */
public class BSwingUtilities
{
    /**
     * Sets the text in the given JLabel to the value provided
     *
     * @param   fLabel  The label to set the text of
     * @param   fText   The text to set it to
     */
	public static void setJLabelText(JLabel fLabel, String fText)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetJLabelText(fLabel, fText));
	}

    /**
     * Sets the user object in the given default mutable tree node
     *
     * @param   node The node to set the user object in
     * @param   value   The value to set
     */
	public static void setUserObject(DefaultMutableTreeNode node, Object value)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetUserObject(node, value));
	}

    /**
     * Sets the value at the specified row and column to the given value
     *
     * @param   tableModel  The table model to set the value in
     * @param   value   The value to set
     * @param   row The row to set the value in
     * @param   column  The column to set the value in
     */
	public static void setTableModelValueAt(DefaultTableModel tableModel, Object value, int row, int column)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetValueAt(tableModel, value, row, column));
	}

    /**
     * Sets the text in the given textComponent to the value provided
     *
     * @param   fLabel  The label to set the text of
     * @param   fText   The text to set it to
     */
	public static void setJButtonText(AbstractButton button, String fText)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetJButtonText(button, fText));
	}

    /**
     * Sets the tooltip text on the given JComponent to the value provided
     *
     * @param   component   The component to set the tool tip on
     * @param   fText   The text to set it to
     */
	public static void setToolTip(JComponent component, String fText)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetToolTip(component, fText));
	}

    /**
     * Sets the size or prefered size of the component
     *
     * @param   component   The component to set the size of
     * @param   size    The size to set it to
     * @param   preferred   True to set the preferred size, false to set the size
     */
	public static void setSize(Component component, Dimension size, boolean preferred)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetPreferedSize(component, size, preferred));
	}

    /**
     * Sets the title for the given component on the given table to the given value!
     *
     * @param   component   The component to set the title for
     * @param   tabbedPane  The tabbed pane to set the title on
     * @param   String The new title
     */
	public static void setTitleFor(JTabbedPane tabbedPane, JComponent component, String title)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetTitleFor(tabbedPane, component, title));
	}

    /**
     * Sets the title for the given component on the given table to the given value!
     *
     * @param   frame The frame to set the title of
     * @param   String The new title
     */
	public static void setFrameTitle(Frame frame, String title)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetFrameTitle(frame, title));
	}

    /**
     * Sets the frame icon for the given frame
     *
     * @param   frame The frame to set the title of
     * @param   icon    The icon to set on the frame
     */
	public static void setFrameIcon(Frame frame, Image icon)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetFrameIcon(frame, icon));
	}



    /**
     * This will call the doRefresh method in list model provided
     *
     * @param   listModel   The list model to call refresh on
     */
    public static void doBListModelRefresh(BListModel listModel)
    {
        SwingUtilities.invokeLater(new BSwingUtilities().new BListModelRefresh(listModel));
    }

    /**
     * Gets an image icon using the give objects class as the class loader
     * and the given location as the path to the icon.
     * <p>
     * This should be used in the following manner:
     * <p>
     * <blockquote>
     * ImageIcon icon = BSwingUtilities.getImage(this, "/au/edu/swin/synergy/plugin/mod13/images/logo.gif")
     * </blockquote>
     * <p>
     * This will allow the images to be stored in a jar along with the classes. Make sure you
     * have the slash on the front.
     *
     * @param   loader  The object to be used as the resource loader
     * @param   location    The relative path to the icon
     */
    public static ImageIcon getImage(Object loader, String location)
    {
        try
        {
            URL iconURL = loader.getClass().getResource(location);
            return new ImageIcon(iconURL);
        }
        catch(Exception catchAll)
        {
            Debug.doPrintln("Image load error: ", catchAll);
        }

        //Return an empty image icon
        return new ImageIcon();
    }
    
	/**
     * This will call the addItem() method of the given combo box with the given object
     *
     * @param   comboBox    The combo to add to
     * @param   item    The item to add
     */
    public static void doJComboSetItem(JComboBox comboBox, Object item)
    {
        SwingUtilities.invokeLater(new BSwingUtilities().new BSetJComboItem(comboBox, item));
    }

    /**
     * This will call the addItem() method of the given combo box with the given object
     *
     * @param   comboBox    The combo to add to
     * @param   item    The item to add
     */
    public static void doJComboAddItem(JComboBox comboBox, Object item)
    {
        SwingUtilities.invokeLater(new BSwingUtilities().new BAddJComboItem(comboBox, item));
    }

    /**
     * This will call the removeAllItems() method of the given combo box
     *
     * @param   comboBox    The combo box to remove all the items from
     */
    public static void doJComboRemoveAllItems(JComboBox comboBox)
    {
        SwingUtilities.invokeLater(new BSwingUtilities().new BRemoveAllJComboItems(comboBox));
    }

    /**
     * Sets the text in the given JMenuItem to the value provided
     *
     * @param   menuItem  The menu item to set the text of
     * @param   fText   The text to set it to
     */
	public static void setJMenuItemText(JMenuItem menuItem, String fText)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetJMenuItemText(menuItem, fText));
	}

    /**
     * Sets the accererator for the given menu item
     *
     * @param   menuItem  The menu item to set accererator for
     * @param   keyStroke   The keystroke to set as the accelerator
     */
	public static void setMenuAccererator(JMenuItem menuItem, KeyStroke keyStroke)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetAccelerator(menuItem, keyStroke));
	}

    /**
     * Adds the given component to the given JTable as a tab
     *
     * @param   table   The table add the component to
     * @param   component   The component to add as a tab to the given table
     */
	public static void addTab(JTabbedPane tabbedPane, String title, JComponent component)
	{
        SwingUtilities.invokeLater(new BSwingUtilities().new BAddTab(tabbedPane, title, component));
	}

    /**
     * Sets the mnemonic for the given abstract button
     *
     * @param   button  The button item to set mnemonic for
     * @param   mnemonic   The mnemonic to set as the mnemonic
     */
	public static void setMnemonic(AbstractButton button, int mnemonic)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetMnemonic(button, mnemonic));
	}

    /**
     * Sets the mnemonic for the given abstract button
     *
     * @param   button  The button item to set mnemonic for
     * @param   mnemonic   The mnemonic to set as the mnemonic
     */
	public static void setMnemonic(AbstractButton button, char mnemonic)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetMnemonic(button, mnemonic));
	}

    /**
     * Sets the title in the given title border
     *
     * @param   border  The border to set the title of
     * @param   title   The title to set
     */
	public static void setBorderTitle(TitledBorder border, String title)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetBorderTitle(border, title));
	}

    /**
     * Sets the text in the given textComponent to the value provided
     *
     * @param   fLabel  The label to set the text of
     * @param   fText   The text to set it to
     */
	public static void setJText(JTextComponent textComponent, String fText)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetJText(textComponent, fText));
	}

    /**
     * This accepts any JComponent and sets its enabled state to the given value
     *
     * @param   fComponent  The component to set the border of
     * @param   fEnabled    The new enabled state of the given component
     */
	public static void setBorder(JComponent fComponent, Border border)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetJBorder(fComponent, border));
	}

    /**
     * This accepts any JComponent and sets its enabled state to the given value
     *
     * @param   fComponent  The component to set
     * @param   fEnabled    The new enabled state of the given component
     */
	public static void setJComponentEnabled(JComponent fComponent, boolean fEnabled)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BSetJComponentEnabled(fComponent, fEnabled));
	}

    /**
     * This accepts a BListModel and a Vector of contents for that model and
     * performs the operation in the awt thread.
     *
     * @param   listModel   The list model to set the contents of
     * @param   contents    The contents of the list model to set
     */
    public static void setBListModelContents(BListModel listModel, Vector contents)
	{
		SwingUtilities.invokeLater(new BSwingUtilities().new BListModelSetContents(listModel, contents));
	}

    /**
     * Handles the set text on a JTextComponents
     */
	public class BSetJText implements Runnable
	{
		JTextComponent textComponent;
		String text;

		public BSetJText(JTextComponent textComponent, String fText)
		{
			this.textComponent = textComponent;
			text = fText;
		}

		public void run()
		{
			textComponent.setText(text);
		}
	}

    /**
     * Handles the set text on a JLabel
     */
	public class BSetJLabelText implements Runnable
	{
		JLabel label;
		String text;

		public BSetJLabelText(JLabel fLabel, String fText)
		{
			label = fLabel;
			text = fText;
		}

		public void run()
		{
            if (label == null)
                Debug.doPrintln("Null pointer for: " + text, Debug.kErrorLevel);
			label.setText(text);
		}
	}

    /**
     * Handles the set text on a JLabel
     */
	public class BSetJMenuItemText implements Runnable
	{
		JMenuItem menuItem;
		String text;

		public BSetJMenuItemText(JMenuItem menuItem, String fText)
		{
			this.menuItem = menuItem;
			text = fText;
		}

		public void run()
		{
			this.menuItem.setText(text);
		}
	}

    /**
     * Handles the set enabled on a JComponent
     */
	public class BSetJBorder implements Runnable
	{
		JComponent component;
		Border border;

		public BSetJBorder(JComponent fComponent, Border fborder)
		{
			component = fComponent;
			border = fborder;
		}

		public void run()
		{
			component.setBorder(border);
		}
	}

    /**
     * Handles the set enabled on a JComponent
     */
	public class BSetJComponentEnabled implements Runnable
	{
		JComponent component;
		boolean enabled;

		public BSetJComponentEnabled(JComponent fComponent, boolean fEnabled)
		{
			component = fComponent;
			enabled = fEnabled;
		}

		public void run()
		{
			component.setEnabled(enabled);
		}
	}

    /**
     * Handles the set enabled on a JComponent
     */
	public class BSetJButtonText implements Runnable
	{
		AbstractButton button;
		String text;

		public BSetJButtonText(AbstractButton button, String text)
		{
			this.button = button;
			this.text = text;
		}

		public void run()
		{
            if (button == null)
                Debug.doPrintln("Null button passed to BSwingUtilities: " + text, Debug.kErrorLevel);
			button.setText(this.text);
		}
	}

    /**
     * Handles the removeAllItems method of the combo
     */
	public class BRemoveAllJComboItems implements Runnable
	{
		JComboBox comboBox;

		public BRemoveAllJComboItems(JComboBox comboBox)
		{
			this.comboBox = comboBox;
		}

		public void run()
		{
			synchronized(comboBox)
			{
				comboBox.removeAllItems();
			}
		}
	}
    /**
     * Handles the add item method in the combo
     */
	public class BSetJComboItem implements Runnable
	{
		JComboBox comboBox;
		Object item;

		public BSetJComboItem(JComboBox comboBox, Object item)
		{
			this.comboBox = comboBox;
			this.item = item;
		}

		public void run()
		{
			comboBox.setSelectedItem(this.item);
		}
	}


    /**
     * Handles the add item method in the combo
     */
	public class BAddJComboItem implements Runnable
	{
		JComboBox comboBox;
		Object item;

		public BAddJComboItem(JComboBox comboBox, Object item)
		{
			this.comboBox = comboBox;
			this.item = item;
		}

		public void run()
		{
			comboBox.addItem(this.item);
		}
	}

    /**
     * Handles the add item method in the combo
     */
	public class BSetValueAt implements Runnable
	{
		DefaultTableModel tableModel;
		Object value;
        int row;
        int column;

		public BSetValueAt(DefaultTableModel tableModel, Object value, int row, int column)
		{
			this.tableModel = tableModel;
			this.value = value;
            this.row = row;
            this.column = column;
		}

		public void run()
		{
			tableModel.setValueAt(value, row, column);
		}
	}

    /**
     * This provides a means of safely setting the contents
     * of a BListModel from within the awt thread.
     * <p>
     * Simply instantiate this class with the contents to
     * set and the model to set them in and send it off
     * to the <code>SwingUtilities.invokeLater()</code> method.
     *
     * @author Tristan Austin
     */
    public class BListModelRefresh implements Runnable
    {
        /**
         * The list model to set the contents of
         */
        private BListModel listModel;

        /**
         * The contents to set in the list model
         */
        private Vector contents;

        /**
         * Instantiates the class with the given contents and model
         * to combine.
         *
         * @param   listModel   The model to set the contents of
         * @param   contents    The contents to set in the model
         */
        public BListModelRefresh(BListModel listModel)
        {
            this.listModel = listModel;
        }

        /**
         * Called when at the head of the awt event queue. This
         * will set notify all the listeners that the contents have changed.
         */
        public void run()
        {
            this.listModel.doRefresh();
        }
    }

    /**
     * This provides a means of safely setting the contents
     * of a BListModel from within the awt thread.
     * <p>
     * Simply instantiate this class with the contents to
     * set and the model to set them in and send it off
     * to the <code>SwingUtilities.invokeLater()</code> method.
     *
     * @author Tristan Austin
     */
    public class BListModelSetContents implements Runnable
    {
        /**
         * The list model to set the contents of
         */
        private BListModel listModel;

        /**
         * The contents to set in the list model
         */
        private Vector contents;

        /**
         * Instantiates the class with the given contents and model
         * to combine.
         *
         * @param   listModel   The model to set the contents of
         * @param   contents    The contents to set in the model
         */
        public BListModelSetContents(BListModel listModel, Vector contents)
        {
            this.listModel = listModel;
            this.contents = contents;
        }

        /**
         * Called when at the head of the awt event queue. This
         * will set the contents in the list model.
         */
        public void run()
        {
            this.listModel.setContents(this.contents);
        }
    }

   /**
     * Sets the user object in the given tree node
     */
	public class BSetUserObject implements Runnable
	{
		DefaultMutableTreeNode node;
		Object value;

		public BSetUserObject(DefaultMutableTreeNode node, Object value)
		{
			this.node = node;
			this.value = value;
		}

		public void run()
		{
			node.setUserObject(value);
		}
	}

    /**
     * Sets the user object in the given tree node
     */
	public class BSetBorderTitle implements Runnable
	{
		TitledBorder border;
		String title;

		public BSetBorderTitle(TitledBorder border, String title)
		{
			this.border = border;
            this.title = title;
		}

		public void run()
		{
			border.setTitle(title);
		}
	}


    /**
     * Sets the user object in the given tree node
     */
	public class BSetToolTip implements Runnable
	{
		JComponent component;
		String toolTip;

		public BSetToolTip(JComponent component, String toolTip)
		{
			this.component = component;
            this.toolTip = toolTip;
		}

		public void run()
		{
			component.setToolTipText(toolTip);
		}
	}

    /**
     * Sets the user object in the given tree node
     */
	public class BSetAccelerator implements Runnable
	{
		JMenuItem menuItem;
		KeyStroke keyStroke;

		public BSetAccelerator(JMenuItem menuItem, KeyStroke keyStroke)
		{
			this.menuItem = menuItem;
            this.keyStroke = keyStroke;
		}

		public void run()
		{
			menuItem.setAccelerator(keyStroke);
		}
	}

    /**
     * Sets the user object in the given tree node
     */
	public class BSetMnemonic implements Runnable
	{
		AbstractButton button;
		int characterInt;
        char characterChar;
        boolean character = false;

		public BSetMnemonic(AbstractButton button, int characterInt)
		{
			this.button = button;
            this.characterInt = characterInt;
		}

        public BSetMnemonic(AbstractButton button, char characterChar)
		{
			this.button = button;
            this.characterChar = characterChar;
            this.character = true;
		}

		public void run()
		{
            if (character)
            {
    			button.setMnemonic(characterChar);
            }
            else
            {
                button.setMnemonic(characterInt);
            }
		}
	}

    /**
     * Adds the given component to the table as a tab
     */
	public class BAddTab implements Runnable
	{
		JTabbedPane tabbedPane;
		JComponent component;
        String title;

		public BAddTab(JTabbedPane tabbedPane, String title, JComponent component)
		{
			this.tabbedPane = tabbedPane;
            this.title = title;
            this.component = component;
		}

		public void run()
		{
            if (tabbedPane == null)
                Debug.doPrintln("Null tabbedPane passed to BSwingUtilities: " + title, Debug.kErrorLevel);

            tabbedPane.addTab(title, component);
		}
	}

    /**
     * Adds the given component to the table as a tab
     */
	public class BSetPreferedSize implements Runnable
	{
		Component component;
        Dimension size;
        boolean prefered = false;

		public BSetPreferedSize(Component component, Dimension size, boolean prefered)
		{
            this.component = component;
            this.size = size;
            this.prefered = prefered;
		}

		public void run()
		{
            if (prefered)
            {
                ((JComponent)component).setPreferredSize(size);
            }
            else
            {
                component.setSize(size);
            }
		}
	}

    /**
     * Adds the given component to the table as a tab
     */
	public class BSetTitleFor implements Runnable
	{
		JTabbedPane tabbedPane;
		JComponent component;
        String title;

		public BSetTitleFor(JTabbedPane tabbedPane, JComponent component, String title)
		{
			this.tabbedPane = tabbedPane;
            this.title = title;
            this.component = component;
		}

		public void run()
		{
            tabbedPane.setTitleAt(tabbedPane.indexOfComponent ( component ), title );
		}
	}

    /**
     * Adds the given component to the table as a tab
     */
	public class BSetFrameTitle implements Runnable
	{
		Frame frame;
        String title;

		public BSetFrameTitle(Frame frame, String title)
		{
			this.frame = frame;
            this.title = title;
		}

		public void run()
		{
            frame.setTitle(title);
		}
	}

    /**
     * Adds the given component to the table as a tab
     */
	public class BSetFrameIcon implements Runnable
	{
		Frame frame;
        Image icon;

	public BSetFrameIcon(Frame frame, Image icon)
		{
			this.frame = frame;
            this.icon = icon;
		}

		public void run()
		{
            frame.setIconImage(icon);
		}
	}
}

