package au.edu.swin.synergy.plugin.mod13.bwt;

import javax.swing.*;
import java.beans.*;

/**
 * This is going to be informed when the user presses the control
 * key in combination with the tab key. It will then transfer the
 * control of cycling through the windows.
 *
 * @author Tristan Austin
 */
public class BTabListener
{
	/**
	 * Moved backwards
	 */
	static final int kBackwards = -1;

	/**
	 * Moved forwards 
	 */
	static final int kForwards = 1;

	/**
	 * Moved nowhere 
	 */
	static final int kNeutral = 0;

	/**
	 *  Indicates which way we last went 
	 */
	int lastDirection = kNeutral;

	/**
	 * The number of times tab was pressed while control is down
	 */
	int tabCount = 0;

	/**
	 * The desktop manager
	 */
	DesktopManager desktopManager;

	/**
	 * The desktop 
	 */
	JDesktopPane desktop;

	/**
	 * Creates a new tab listener for the given desktop
	 */
	public BTabListener(JDesktopPane desktop)
	{
		this.desktop = desktop;
		this.desktopManager = desktop.getDesktopManager();
	}

	/**
	 * Set the state of control
	 */
	public void setControl(boolean down)
	{
		if (!down)
		{
			tabCount = 0;
			lastDirection = kNeutral;
		}
	}


	/**
	 * This will cycle forwards through the windows, skipping
	 * windows that have been minimised.
	 */
	public void cycleForward()
	{
		JInternalFrame allFrames[] = desktop.getAllFrames();
		int i = allFrames.length;

		//Need to make sure we're going the right way
		if (lastDirection != kForwards)
		{
			lastDirection = kForwards;
			tabCount = 0;
		}

		//Make sure there are frames to deal with
		if (i > 0)
		{
			if (tabCount++ == 0)
			{
				i = 1;	

			}
			else if (i > 1)
			{
				i--;
			}

			//Make sure the index is valid (for only one window)
			if (allFrames.length == 1)
			{
				i= 0;
			}

			//Skip the minimised windows
			while(allFrames[i].isIcon())
			{
				i--;
				if (i <= 0)
				{
					break;
				}
			}
		
			//Activate the frame we got to
			desktopManager.activateFrame(allFrames[i]);
			try
			{
				allFrames[i].setSelected(true);
			}
			catch(PropertyVetoException vetoException)
			{/*Ignore*/}
		}
	}

	/**
	 * This will bring the last window to the front
	 */
	public void cycleBackward()
	{
		JInternalFrame allFrames[] = desktop.getAllFrames();
		int size = allFrames.length;

		//Need to make sure we're going the right way
		if (lastDirection != kBackwards)
		{
			lastDirection = kBackwards;
			tabCount = 0;
		}

		//Make sure there are some non minimised frames
		if (size > 1) 
		{
			//Find the next one
			if (++tabCount >= size || allFrames[tabCount].isIcon())
				tabCount--;

			desktopManager.activateFrame(allFrames[tabCount]);
			try
			{
				allFrames[tabCount].setSelected(true);
			}
			catch(PropertyVetoException vetoException)
			{/*Ignore*/}
		}
	}
}
