/**
 * Please complete these missing tags
 * @see
 * @concurrency
 * @copyright
 * @rref
 * @author
 */
// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/mod13/event/BAddMenuCE.java,v 1.12 1999/10/14 08:35:05 tp079519 Exp $

package au.edu.swin.synergy.plugin.mod13.event;

import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.plugin.dialect.BString;
import javax.swing.JMenu;

/**
 * Add menu allows you to add an entire JMenu of components to
 * the menu bar. It is recommended that you use BAddMenuItem instead
 * and simply specify the name of the menu you want it to go in since
 * adding an entire JMenu results in taking everything out of it and adding
 * it to a BAddMenuItemCE event anyway.
 * <p>
 * The reason for this is because if two Menus exist, it needs to be merged
 * into one in which case the specified location will be ignored and the
 * existing location will be used and custom menu items may be lost.
 * <p>
 * For similar reasons there is no associated BRemoveMenuCE. Because the names
 * of the menus can coincide with other plugins naming conventions, we can't
 * assume that the plugin that initially created the menu is the only
 * one that has items in it. Therefore, to remove a menu, the plugin must
 * place all of the menu items in a BRemoveMenuItemCE and send that to
 * the kernel.
 * <p>
 * This way if it is the only plugin with items in the menu, the menu will be
 * removed, if not, the other plugins menu items will stay
 * <p>
 * If the menu doesn't exist it will simply be added as normal in the specified location.
 */
public class BAddMenuCE extends BWindowCE
{
    /**
     * Using this policy will ensure that the menu is
     * right aligned. Any right aligned menus added after
     * this will become the right most menu followed by
     * this one and so on...
     */
    public static final int kAlwaysRightAligned = 0;

    /**
     * Using this location policy, the menu added will
     * go after all the existing menus, but before the
     * menus with the kAlwaysRightAligned policy.
     */
    public static final int kNormalPolicy = 1;

    /**
     * This is the location policy to use for this menu
     */
    private int locationPolicy = kNormalPolicy;

    /**
     * The preferred mnemonic for the menu
     */
    private BString mnemonic = null;

    /**
     * This is the locale sensitive name of the menu
     */
    private BString name = null;

    // add data elements in here....
    JMenu menu;

    /**
     * Creates a new add menu ce with the default
     * location policy.
     *
     * @param   source  The source of the event
     * @param   fMenu   The menu to add to the menu bart
     */
    public BAddMenuCE( Object source, BString menuName, JMenu fMenu )
    {
        this(source, menuName, fMenu, kNormalPolicy);
    }

    /**
     * Creates a new add menu event with the specified
     * location policy.
     *
     * @post
     * @pre
     * @param   source  The source of the event
     * @param   menuName    The locale sensitive name of the menu
     * @param   fMenu   The menu to add to the menu bart
     * @param   locationPolicy  The location policy for the menu kAlwaysRightAligned, kNormalPolicy
     */
    public BAddMenuCE( Object source, BString menuName, JMenu fMenu, int locationPolicy )
    {
        super( source );
        // other stuff...
        this.name = menuName;
        this.menu = fMenu;
        this.locationPolicy = locationPolicy;
    }

    /**
     *  Gets the locale sensitive name of this menu
     *
     * @return The locale sensntive name of the menu
     */
    public BString getMenuName()
    {
        return this.name;
    }
    /**
     * Please complete the missing tags for getMenu
     * @post
     * @pre
     * @throws
     * @return
     * @param
     */

    public JMenu getMenu()
    {
        return menu;
    }

    /**
     * This will return the requested location
     * policy for this menu.
     *
     * @return The location policy for this new menu
     */
    public int getLocationPolicy()
    {
        return this.locationPolicy;
    }

    /**
     * You can set the mnemonic you prefer for this
     * menu here. It should be noted that if another
     * menu already has the mnemonic specified it will
     * be changed to the next available mnemonic.
     *
     * @param   preferredMnemonic   The preferred mnemonic for this menu
     */
    public void setPreferredMnemonic(BString preferredMnemonic)
    {
        this.mnemonic = preferredMnemonic;
    }

    /**
     * This will return the preferred mnemonic for this menu.
     *
     * @return The preferred mnemonic
     */
    public BString getPreferredMnemonic()
    {
        return this.mnemonic;
    }
}
