package au.edu.swin.synergy.plugin.mod13.event;

import au.edu.swin.synergy.kernel.BCommandEvent;
import javax.swing.filechooser.*;
import java.io.File;

public class BGetFilesCE extends BWindowCE
{
	/**
	 * Indicates whether we're opening or saving a file
	 */
	boolean loadFile = true;

	/**
 	 * The file filter for the chooser
	 */
	FileFilter fileFilter = null;

	/**
 	 * This is the directory the browser will start in 
	 */
	String startingDirectory = ".";	

	/**
	 * This is the title of the browser dialog
	 */
	String title = "Browse";	

	/**
	 * This is the file that was selected by the user
	 */
	File selectedFile = null;

    /**
     * Indicates whether the file must exist
     */
    boolean fileMustExist = true;

	/**
	 * The default constructor
	 */
	public BGetFilesCE(Object source)
	{
		super(source);
	}

    /**
	 * Creates the event with the given starting directory. The browser
	 * will default to loading and the title to "Browse"
	 *
	 * @param	startingDir	The directory to start browsing in.
	 */
	public BGetFilesCE(Object source, String startingDir)
    {
        this(source, startingDir, true);
    }

	/**
	 * Creates the event with the given starting directory. The browser
	 * will default to loading and the title to "Browse"
	 *
	 * @param	startingDir	The directory to start browsing in.
	 */
	public BGetFilesCE(Object source, String startingDir, boolean mustExist)
	{
		this(source);
		this.setStartingDir(startingDir);
        this.fileMustExist = mustExist;

		if (!mustExist)
		{
			//
		}
	}

	/**
	 * Sets the title of the browsing window to the value given
	 *
	 * @param	windowTitle	The title of the window
	 */
	public void setTitle(String windowTitle)
	{
		this.title = windowTitle;
	}

	/**
	 * This will get the title of the window
	 *
	 * @return The title as a String
	 */
	public String getTitle()
	{
		return this.title;
	}

	/**
	 * Sets the file filter to use in the browser
	 *
	 * @param	fileFilter	The file filter
	 */
	public void setFileFilter(FileFilter fileFilter)
	{
		this.fileFilter = fileFilter;
	}

	/**
	 * Gets the file filter to use in the browser
	 *
	 * @return The file filter
	 */
	public FileFilter getFileFilter()
	{
		return this.fileFilter;
	}

	/**
	 * Sets the starting directory to the given value. The
	 * browser will begin looking in this directory.
	 *
	 * @param	startingDir	The directory to start in ("." by default)
	 */
	public void setStartingDir(String startingDir)
	{
		this.startingDirectory = startingDir;
	}

	/**
	 * Gets the starting directory for the browser.
	 *
	 * @return Gets the directory to start browsing in.
	 */
	public String getStartingDir()
	{
		return this.startingDirectory;
	}

	/**
	 * Indicates whether the user is going to be loading or saving a file.
	 *
	 * @param	load	True to load a file, false to save
	 */
	public void setLoad(boolean load)
	{
		this.loadFile = load;
	}

	/**
	 * Indicates whether the user is wants to load a file or save one.
	 *
	 * @return True to load, false to save
	 */
	public boolean isLoading()
	{
		return this.loadFile;
	}

	/**
	 * Sets the selected file for the source to retrieve
	 * 
	 * @param	selectedFile	The selected file
	 */
	public void setSelectedFile(File selectedFile)
	{
		this.selectedFile = selectedFile;
	}

	/**
	 * Will return the selected file. If no file was selected, this will
	 * return null. This will be the case if the user selects cancel or closes the
	 * dialog using something other than the buttons.
	 *
	 * @return The selected file or null if nothing selected
	 */
	public File getSelectedFile()
	{
		return this.selectedFile;
	}

    /**
     * Indicates whether the file must exist and if so
     * it will prompt the user until they select a real file.
     *
     * @return True if it does, false otherwise
     */
    public boolean getFileMustExist()
    {
        return this.fileMustExist;
    }
}

