//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/modeller/BModel.java,v 1.51 1999/10/21 13:32:51 tp079519 Exp $
package au.edu.swin.synergy.plugin.modeller;

import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.lib.BTreeNode;
import au.edu.swin.synergy.plugin.diagram.BDiagram;
import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.plugin.metamodeller.BMetaModel;
import au.edu.swin.synergy.plugin.packager.BPackageModel;
import au.edu.swin.synergy.plugin.packager.BPackageNode;
import au.edu.swin.synergy.plugin.packager.BPackageArc;
import au.edu.swin.synergy.plugin.workspace.BProject;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.debug.Assertion;
import au.edu.swin.synergy.math.BScalarRect;
import au.edu.swin.synergy.math.BScalarPoint;
import au.edu.swin.synergy.plugin.workspace.BWorkspaceTree;
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import au.edu.swin.synergy.shapes.BTextArea;

/**
 * Contains a collection of nodes, arcs, and terminators. Can be added
 * to a project, and saved to file.
 *
 * @author Matthew Evans, Imron Alston
 */
public class BModel implements BSaveable, BTreeNode, BDiagram
{
  public static final String kFileExtension = "cmo";

   	// for BSaveable
	protected static final String kKeyName = "name";
	protected static final String kDefaultName = "untitled";
	
	/**
	 * This is the next unique model element ID.  This needs to be unique for all distributed versions
	 * of a single model
	 */
	long IDCounter = 1;
	Object IDlock = new Object();

   String name;
   String filePath;
   Vector modelElements = null;
   BProject parent = null;
   BPackageModel packageModel = null;
   Vector placeable = null;
   BDrawable curAdd = null;
   boolean hasChanged = true;



   public BModel (BProject parent, String name, String path)
   {
      this.name = name;
      this.filePath = path;
      this.parent = parent;
      modelElements = new Vector();
	  this.placeable = new Vector();
	}


	public BModel (BProject parent, String name, String path, BPackageModel packageModel)
   {
	   this(parent, name, path);

		if ( packageModel != null )
		{
			this.setPackageModel( packageModel );
		}
   }

/*	public BModel( BPackageModel packageModel )
	{
		modelElements = new Vector();
		this.placeable = new Vector();

		if ( packageModel != null )
		{
			this.setPackageModel( packageModel );
		}

		this.setupPopup();
	}*/


	/**
	 * Default constructor for BSaveable
	 */
	public BModel()
	{
		this(null, null, null);
	}

	public long getNewUniqueID()
	{
		synchronized(IDlock)
		{
			IDCounter += (int)(Math.random() * 1024); 
			return IDCounter;
		}
	}

	public void setParent(BProject parent)
	{
		this.parent = parent;
	}
	
	/**
	 * This method is called by a model element after is has loaded it's self
	 * It makes sure that duplicate ID's are not given out
	 */
	public void loadedID(long ID)
	{
		synchronized(IDlock)
		{
			if(IDCounter < ID)
			{
				IDCounter = ID + 1;
			}
		}
	}

   public void setPackageModel( BPackageModel packageModel )
   {
		int i, size;
		BDrawable cur = null;

		this.packageModel = packageModel;
		this.placeable.clear();
		this.packageModel.addBGroupsToVector( BPackageModel.kNodeBranch, this.placeable );
		this.packageModel.addBGroupsToVector( BPackageModel.kArcBranch, this.placeable );

		size = this.modelElements.size();
		for ( i = 0; i < size; i++ )
		{
			cur = (BDrawable)this.modelElements.elementAt( i );
//			cur.setPackageNode( packageModel.get
		}
   }

   public BPackageModel getPackageModel()
   {
   	return packageModel;
   }

   public String getPath()
   {
      return this.filePath;
   }

   public String getName()
   {
      return this.name;
   }

   public void setName(String name)
   {
      this.name = name;
   }


   public boolean isLeaf()
   {
      return this.modelElements.size() <= 0;
   }

   public Object getChildAt( int index )
   {
      return this.modelElements.elementAt( index );
   }

   public int getChildCount()
   {
      return this.modelElements.size();
   }

   public Object getParent()
   {
      return this.parent;
   }

   public int getIndexOf( Object node )
   {
      return this.modelElements.indexOf( node );
   }

   public void setParent( Object parent )
   {
      this.parent = (BProject)parent;
   }

   public void removeAllChildren()
   {
      Debug.doPrintln("about to clear the children of " + this.name );
      this.modelElements.clear();
   }

  public String toString()
  {
    return this.name;
  }


   public boolean load(BDataFile file)
   {
      Debug.doPrintln("BModel.load");
      this.name = file.getData(kKeyName, kDefaultName);
	  filePath = file.getFileName();
      Debug.doPrintln("Name is : " + name);

	  BPackageModel newPackageModel = (BPackageModel) file.autoLoad();

      while(true)
      {
         Object object = (Object) file.autoLoad();
         Debug.doPrintln("Got Object form autoload " + object);
         if(object == null)
         {
            Debug.doPrintln("Load Finished");
            break;
         }
         else if (object instanceof BModelElement)
         {
		 	BModelElement me = (BModelElement) object;
            Debug.doPrintln("Adding Drawable element");
			me.setParent(this);
            //addDrawable((BDrawable)object);
			modelElements.add(me);
			//this.parent.nodeWasInserted(this, modelElements.size() - 1, (BTreeNode) me);
			me.updateBounds();
         }
		 else
		 {
		 	Debug.doPrintln("Can't add a " + object.getClass().getName() + " to a BModel");
		 }
      }

	  if(newPackageModel != null)
	  {
		  setPackageModel(newPackageModel);
	  }
	  for(int i=0; i<modelElements.size(); i++)
	  {
	  	Debug.doPrintln("element is " + modelElements.get(i).getClass().getName());
		((BModelElement)modelElements.get(i)).resolveLoadedObject(this);
	  }

      return true;
   }

   public boolean save(BDataFile file)
   {
      Debug.doPrintln("BModel.save");
      file.addData(kKeyName, this.name, kDefaultName);

	  file.save(packageModel);

	  Debug.doPrintln("Saving " + modelElements.size() + " elements");

		// check the ID's
      for(int i = 0; i < modelElements.size(); i++)
	  {
	  	 ((BModelElement)modelElements.elementAt(i)).checkID(this);
	  }
	  // then save everything.
      for(int i = 0; i < modelElements.size(); i++)
      {
         file.save((BSaveable)modelElements.elementAt(i));
      }

	  Debug.doPrintln("done");

      hasChanged = false;

      return true;
   }

	/**
	 * Get an element by ID.
	 * The ID should be unique across distributed versions of a model
	 * If the element is not found, it returns null
	 * (and yes I know, single entry, single exit.... high <i>Cohesion</i> etc....)
	 */ 
   public BModelElement getElementByID(long ID)
   {
   	Debug.doPrintln("Finding element ID = " + ID);
   	for(int i=0; i<modelElements.size(); i++)
	{
		BModelElement element = (BModelElement)modelElements.elementAt(i);
		if(element.getID() == ID)
		{
			Debug.doPrintln("Found");
			return element;
		}
	}
	Debug.doPrintln("Not Found");
	return null;
   }



   public static final int kChangePackageOK = 0;
   public static final int kChangePackageNotCompatible = 1;
   
	/**
	 * Change the package used by this model.
	 * This method is called before doChangePackage
	 * @param newPackageModel the packagemodel to change to
	 * @return any of the kChangePackage constants
	 */

   public int doChangePackage(BPackageModel newPackageModel)
   {
   	// see if the package is compatible...
	if(getPackageModel().getMetaModel().getName().equals(newPackageModel.getMetaModel().getName()))
	{
   	  // each model element needs to be resolved into the new package....
	  if(newPackageModel != null)
	  {
		  setPackageModel(newPackageModel);
	  }
	  for(int i=0; i<modelElements.size(); i++)
	  {
	  	Debug.doPrintln("element is " + modelElements.get(i).getClass().getName());
		((BModelElement)modelElements.get(i)).doSaveText();
		((BModelElement)modelElements.get(i)).resolveLoadedObject(this);
	  }
	  return kChangePackageOK;
	 }
	 else
	 {
	 	return kChangePackageNotCompatible;
	 }
   }

////////BDiagram stuff


	public void addVisibleDrawablesToList( Vector visibleList, BScalarRect bounds )
	{
		BDrawable cur = null;
		int size, i, j;
        Vector props;
        BTextArea ta;

		size = this.modelElements.size();
		for ( i = 0; i < size; i++ )
		{
			cur = (BDrawable)this.modelElements.elementAt( i );

            // We will have to add the arc anyway!
            if ( cur.intersects( bounds ) )
			{
				visibleList.add( cur );
			}
/*
            if ( cur instanceof BModelArc )
            {
                props = ( ( BModelArc ) cur ).getProperties ();
                for ( j = 0; j < props.size (); j++ )
                {
                    ta = ( BTextArea ) props.elementAt ( j );
                    if ( ta.intersects ( bounds ) )
                    {
                        visibleList.add ( ta );
                    }
                }
            }
/**/
            
		}
	}

    public Vector getPlaceableDrawables()
	{
		return this.placeable;
	}

    public void addDrawable( BDrawable drawable )
	{
		BModelElement me = null;
		if ( drawable instanceof BModelNode || drawable instanceof BModelArc )
		{
			me = (BModelElement)drawable;
			this.modelElements.add( me );
			this.parent.nodeWasInserted(this, this.modelElements.size() - 1, (BTreeNode) me );
			Debug.doPrintln("Adding a new me to model - " + me.getBounds() );
		}
		else
		{
			Debug.doPrintln("Adding a non modelelement");
		}
	}

	public void addElement(BModelElement fElement)
	{
		modelElements.add(fElement);
	}

   

	public void createDrawable( String drawable, BScalarPoint pos )
	{
	}

	/**
	 *
	 * starts adding a drawable to the diagram...
	 *
	 * @pre the model must have a valid packageModel,
	 * @pre the package model must have a valid metamodel..
	 *
	 */
	public BDrawable startAdd( BDrawable drawable, BScalarPoint pos, BDrawable under )
	{
		BModelElement me = null;
		BModelArc ma = null;
		BModelNode mn = null;
		BMetaModel metaModel = null;
		BDrawable d = drawable;
		BDrawable result = null;
		boolean keepGoing = true;
		int i;

		Assertion.doAssert( this.packageModel != null, "Have to have a package model to add to the diagram!!" );
		Assertion.doAssert( this.packageModel.getMetaModel() != null, "Have to have a meta model to add to the diagram!!" );

        //Need to get stuffout if it's a model node
        if (drawable instanceof BModelNode)
        {
            d = (BPackageNode)((BModelNode)drawable).getPackageNode();
        }
        else if (drawable instanceof BModelArc)
        {
            d = (BPackageArc)((BModelArc)drawable).getPackageArc();
        }

        //Set up the drawable
        d = d.newInstance();
        d.setPos( pos );
		d.setHighlighted( true );

		if ( d instanceof BPackageNode )
		{
			me = new BModelNode( (BPackageNode)d );
			result = me;
			me.setParent(this);
			this.modelElements.add( me );
			this.parent.nodeWasInserted(this, modelElements.size() - 1, (BTreeNode) me);
		}
		else if ( d instanceof BPackageArc )
		{
			me = new BModelArc( (BPackageArc)d );
			result = me;
			me.setParent(this);
			this.modelElements.add( me );
			this.parent.nodeWasInserted(this, modelElements.size() - 1, (BTreeNode) me);
			if ( under instanceof BModelNode )
			{
				ma = (BModelArc)me;
				metaModel = this.packageModel.getMetaModel();
				mn = (BModelNode)under;

				ma.setRuleStatus( BMetaModel.kNoRuleViolation );

				keepGoing = true;
				for ( i = BMetaModel.kStartRuleTypes; i <= BMetaModel.kMaxRuleTypes && keepGoing; i++ )
				{
					switch( i )
					{
						case BMetaModel.kMaxOutRuleViolation:
							keepGoing &= checkMaxOutViolation( metaModel, ma, mn );
							break;
						case BMetaModel.kConnectionRuleViolation:
							keepGoing &= checkStartConnectionViolation( metaModel, ma, mn );
							break;
					}

				}

				if ( keepGoing )
				{
					Debug.doPrintln("About to set the start Node");
					((BModelArc)me).setStartNode( (BModelNode)under );
				}

			}
			else if ( under == null )
			{
				Debug.doPrintln("About to set the start Node");
				((BModelArc)me).setStartNode( (BModelNode)under );
			}
			else
			{
				Debug.doPrintln("under isn't a model node!!" + under );
			}


		}
		else
		{
			Debug.doPrintln("BModel::startAdd - Adding an unknown model element!!", Debug.kWarningLevel );
			result = null;
		}

		if ( result != null )
		{
			this.curAdd = me;
			me.startAdd();
		}

		return result;
	}

	public boolean continueAdd( BScalarPoint pos, BDrawable under )
	{
		return true;
	}

	public void dragAdd( BScalarPoint deltaXY, BDrawable under )
	{
		if ( this.curAdd != null )
		{
			this.curAdd.dragAdd( deltaXY );
		}
	}

	/**
	 * stops adding a drawable to the diagram...
	 *
	 * @pre the packageModel cannot be null
	 * @pre the packageModel's metamodel cannot be null..
	 */
	public boolean stopAdd( BScalarPoint pos, BDrawable under, int modifiers )
	{
		Assertion.doAssert( this.packageModel != null, "Package Model cannot be null");
		Assertion.doAssert( this.packageModel.getMetaModel() != null, "Package Model must have a metamodel");


		BMetaModel metaModel = this.packageModel.getMetaModel();
		BModelArc ma = null;
		BModelNode mn = null;
		boolean keepGoing = true;
		int i;


		if ( this.curAdd != null )
		{
			if ( this.curAdd instanceof BModelArc )
			{
				if ( under instanceof BModelNode )
				{
					mn = (BModelNode)under;
					ma = (BModelArc)this.curAdd;

					Debug.doPrintln("Lah-di dah");
					keepGoing = true;
					for ( i = BMetaModel.kStartRuleTypes; i <= BMetaModel.kMaxRuleTypes && keepGoing; i++ )
					{
						switch( i )
						{
							case BMetaModel.kMaxInRuleViolation:
								keepGoing &= checkMaxInViolation( metaModel, ma, mn );
								break;
							case BMetaModel.kConnectionRuleViolation:
								keepGoing &= checkConnectionViolation( metaModel, ma, mn );
								break;
							case BMetaModel.kCircRefRuleViolation:
								((BModelArc)this.curAdd).setEndNode( (BModelNode)under );
								keepGoing &= checkCircRefViolation( metaModel, ma, mn );
								break;
						}

					}
					
				}
				else Debug.doPrintln("The under isn't a model node its' a " + under );
			}else Debug.doPrintln("The curAdd isn't a model node its' a " + this.curAdd );

		}else Debug.doPrintln("The curAdd is null!!");


		this.curAdd.stopAdd();
		this.curAdd = null;
		return true;
	}

	public void delete( BDrawable drawable )
	{
		int index = this.modelElements.indexOf( drawable );		

		if ( index >= 0 )
		{
			this.modelElements.removeElementAt( index );

			drawable.delete();
			this.parent.nodeWasRemoved( this, index, (BTreeNode) drawable );
		}
	}

	/*
	 * this is a really inefficient way to do this
	 * however the selection vector is only small, 
	 * and i'm in a hurry.
	 */
	public void bringSelectionToFront( Vector selection )
	{
		int i, size;
		size = selection.size();
		for ( i = 0; i < size; i++ )
		{
			int index = this.modelElements.indexOf(selection.elementAt(i));
			this.modelElements.removeElementAt(index);
			this.parent.nodeWasRemoved(this, index, (BTreeNode) selection.elementAt(i));
		}

		for ( i = size-1; i >= 0; i-- )
		{
			this.modelElements.add(selection.elementAt(i) );
			this.parent.nodeWasInserted(this, modelElements.size() - 1 , (BTreeNode) selection.elementAt(i));
		}

	}

	/*
	 * this is a really inefficient way to do this
	 * however the selection vector is only small,
	 * and i'm in a hurry.
	 */
	public void sendSelectionToBack( Vector selection )
	{
		int i, size;
		size = selection.size();

		for ( i = 0; i < size; i++ )
		{
			int index = this.modelElements.indexOf(selection.elementAt(i));
			this.modelElements.removeElementAt(index);
			this.parent.nodeWasRemoved(this, index, (BTreeNode) selection.elementAt(i));
		}

		for ( i = size-1; i >= 0; i-- )
		{
			this.modelElements.add( 0, selection.elementAt( i ) );
			this.parent.nodeWasInserted(this, 0, (BTreeNode) selection.elementAt(i));
		}

	}


	/**
	 *@return true if rule evaluation/arc adding is to continue....
	 * needs to perform all handling of the violation if one occurs.
	 */
	protected boolean checkConnectionViolation( BMetaModel metaModel, BModelArc arc, BModelNode node )
	{
		boolean result = true;

		if ( !metaModel.evalConnection( arc.getMetaElement(), arc.getStartNode().getMetaElement(), node.getMetaElement() ) )
		{
			Debug.doPrintln("Arc violates the connection rule");
			arc.setRuleStatus( BMetaModel.kConnectionRuleViolation );
		}


		return result;
	}

	protected boolean checkStartConnectionViolation( BMetaModel metaModel, BModelArc arc, BModelNode node )
	{
		boolean result = true;

		if ( !metaModel.evalStartConnection( arc.getMetaElement(), node.getMetaElement() ) )
		{
			Debug.doPrintln("Arc will violate the connection rule");
			arc.setRuleStatus( BMetaModel.kConnectionRuleViolation );
		}


		return result;
	}

	protected boolean checkMaxInViolation( BMetaModel metaModel, BModelArc arc, BModelNode node )
	{
		boolean result = true;
		if ( !metaModel.evalMaxIn( node.getMetaElement(), arc.getMetaElement(), node.getNumInArcs() ) )
		{
			Debug.doPrintln("Arc violates the max in rule");
			arc.setRuleStatus( BMetaModel.kMaxInRuleViolation );
		}

		return result;
	}

	protected boolean checkMaxOutViolation( BMetaModel metaModel, BModelArc arc, BModelNode node )
	{
		boolean result = true;
		if ( !metaModel.evalMaxOut( node.getMetaElement(), arc.getMetaElement(), node.getNumInArcs() ) )
		{
			Debug.doPrintln("Arc violates the max out rule");
			arc.setRuleStatus( BMetaModel.kMaxOutRuleViolation );
		}

		return result;
	}

	protected boolean checkCircRefViolation( BMetaModel metaModel, BModelArc arc, BModelNode node )
	{
		boolean result = true;
		if ( metaModel.evalCirc( arc.getMetaElement() ) ) // have to eval circ ref
		{
			if ( node.circRefExists( arc.getName() ) )
			{
				Debug.doPrintln("The new arc violates the circ ref rule");
				arc.setRuleStatus( BMetaModel.kCircRefRuleViolation );
				

			}
		}

		return result;
	}



    /**
     * This identifies whether the model has been changed since it has
     * been opened
     *
     * @return if the model has been modified
     */
    public boolean hasBeenModified()
    {
        return hasChanged;
    }

    public void setModified(boolean modified)
    {
        this.hasChanged = modified;
    }






	class onSave implements ActionListener
	{
		public void actionPerformed(ActionEvent event)
		{
			 //save the model to its own file too
			 BDataFile modelData = new BDataFile(BModel.this.filePath);
			 modelData.openFileForWrite(false);
			 modelData.save(BModel.this);
			 modelData.close();
		}
	}

	class onRemove implements ActionListener
	{
		public void actionPerformed(ActionEvent event)
		{
			if (parent != null)
			{
				parent.removeModel(BModel.this);
			}
		}
	}
}

