
package au.edu.swin.synergy.plugin.modeller;

import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BCommandEvent;

import au.edu.swin.synergy.plugin.diagram.BViewWindow;
import au.edu.swin.synergy.plugin.diagram.BViewport;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowListener;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowEvent;

import au.edu.swin.synergy.plugin.editmanager.event.BRegisterUndoCE;

import au.edu.swin.synergy.debug.Debug;

import javax.swing.*;
import java.awt.*;

public class BModelEditor extends JPanel implements BWindowListener
{
	private BViewWindow viewwindow;
	private BViewport viewport;

	public BModelEditor( BViewport viewport)
	{
		this.viewwindow = new BViewWindow( viewport );
		this.viewport = viewport;

		this.setLayout( new BorderLayout() );
		this.add( viewwindow, BorderLayout.CENTER );
	}

	/**
	 * Returns the viewport being dispalyed on the editor
	 */
	public BViewport getViewport()
	{
		return viewport;
	}
	
	/**
     * Invoked the first time a window is made visible.
     */
    public void windowOpened( BWindowEvent windowEvent )
	{
	}
    
    /**
     * Invoked when the user attempts to close the window from the window's system menu. If the program does not explicitly hide or
     * dispose the window while processing this event, the window close operation will be cancelled.
     */
    public void windowClosing( BWindowEvent windowEvent )
	{
	}
    
    /**
     * Invoked when a window has been closed as the result of calling dispose on the window.
     */
    public void windowClosed( BWindowEvent windowEvent )
	{
        this.viewport.unregisterUndoManager(this.viewport);
	}
    
    /**
     * Invoked when a window is changed from a normal to a minimized state. For many platforms, a minimized window is displayed as
     * the icon specified in the window's iconImage property.
     */
    public void windowIconified( BWindowEvent windowEvent )
	{

	}
    
    /**
     * Invoked when a window is changed from a minimized to a normal state.
     */
    public void windowDeiconified( BWindowEvent windowEvent )
	{
		Debug.doPrintln("Window has been deinconified");
	}
    
    /**
     * Invoked when the window is set to be the user's active window, which means the window (or one of its subcomponents) will
     * receive keyboard events.
     */
    public void windowActivated( BWindowEvent windowEvent )
	{
		Debug.doPrintln("Window has been activated", Debug.kWarningLevel);
        this.viewport.requestFocus();
		this.viewport.refreshPalette();
        this.viewport.setCurrentEditSupport(this.viewport);
	}
    
    /**
     * Invoked when a window is no longer the user's active window, which means that keyboard events will no longer be delivered to the
     * window or its subcomponents.
     */
    public void windowDeactivated( BWindowEvent windowEvent )
	{
		this.viewport.disablePalette();
        this.viewport.relinquishUndoManager();
	}

    /**
     * Returns an register undo command complete object to be used
     * with the dispatch event method.
     *
     * @return A command complete object
     */
    public BRegisterUndoCC getRegisterUndoCC()
    {
        return new BRegisterUndoCC();
    }

    /**
     * This will be used when the register undo is complete for this model
     */
    public class BRegisterUndoCC extends BCommandComplete
    {
        /**
         * this method is invoked when a pluing calls doCompletion on an event
         * @post all handling of the event has been completed
         * @pre the event has been dispatched
         * @throws none
         * @return void
         * @param The event object
         * @param the return status
         */
         public void doComplete( BCommandEvent event, int status )
        {
            BRegisterUndoCE registerUndoCE = (BRegisterUndoCE)event;
            viewport.setEditSupport(viewport, registerUndoCE.getUndoSupport(), registerUndoCE.getUndoListener());
        }
    }
}
