//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/modeller/BModelElement.java,v 1.33 1999/10/21 13:45:08 tp079519 Exp $


package au.edu.swin.synergy.plugin.modeller;

import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.lib.BTreeNode;
import au.edu.swin.synergy.plugin.metamodeller.BMetaElement;
import au.edu.swin.synergy.plugin.metamodeller.BMetaModel;
import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.plugin.diagram.BView;
import au.edu.swin.synergy.math.BScalarPoint;
import au.edu.swin.synergy.math.BScalarRect;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.plugin.packager.BPackageModel;
import au.edu.swin.synergy.plugin.diagram.BDiagram;
import au.edu.swin.synergy.shapes.BTextArea;
import au.edu.swin.synergy.plugin.conflict.BConflict;
import au.edu.swin.synergy.plugin.workspace.propertybag.BPropertyBagItem;
import au.edu.swin.synergy.plugin.workspace.propertybag.BPropertyBag;

import java.awt.*;
import java.util.Vector;

public abstract class BModelElement extends BDrawable implements BTreeNode, BSaveable, BPropertyBagItem
{
	private BModel parent = null;
	protected BMetaElement metaElement = null;

	/**
	 * This is a unique ID which is used to identify a paticular object when loading models
	 * and detecting conflicts
	 */
	public static final int kUnassignedID = -1;
	protected long uniqueID = kUnassignedID;

	/**
	 * Used to load text areas
	 */
	 private Vector newProperties = null;


	protected int ruleStatus = BMetaModel.kNoRuleViolation;

	private int conflictState = BConflict.kNoConflict;


	public BModelElement( BMetaElement element )
	{
		this.metaElement = element;
		if(element != null)
		{
			this.name = element.getName();
		}
	}

   abstract public Vector getProperties();

	public long getUniqueID()
	{
		return uniqueID;
	}

	public void setRuleStatus( int ruleStatus )
	{
		this.ruleStatus = ruleStatus;
	}

	public int getRuleStatus()
	{
		return this.ruleStatus;
	}

	public void updateCommon( Vector common )
	{
		int i, j;
		int commonSize;
		int index;
		Vector props = this.metaElement.getProperties();


		commonSize = common.size() - 1;
		for ( i = 0; i < commonSize; i++ )
		{
		}


	}

	public BMetaElement getMetaElement()
	{
		return this.metaElement;
	}

	public void setMetaElement(BMetaElement element)
	{
		this.metaElement = element;
		if(element != null)
		{
			this.name = element.getName();
		}
	}

	public long getID()
	{
		return uniqueID;
	}

//////////BDrawable stuff

	/**
	 * Transforms the coordinates of the drawable object
	 * to a new coordinate space, as defined by the
	 * viewport and parent pos.
	 * Subclasses should follow the following equation for
	 * tranforming relevant points etc.
	 *
	 * xformPt.x = ((localPt.x + parentPos.x + pos.x - viewport.x) * viewport.zoom ) + viewport.halfx;
	 *
	 * @param BView viewport - the viewport that this object is going to be drawn in
	 * @param Point parentPos - coordinates of any parent object
	 */
	public abstract void transform( BView viewport, BScalarPoint parentPos, int frameCounter );


	/**
	 * Renders the transformed coordinates of the drawable
	 * object to a Graphics2D
	 *
	 * @param Graphics2D g2 - the place to render the object to.
	 */
	public abstract void render( Graphics2D g2 );


	/**
	 * Creates and returns a new instance of the BDrawable object
	 *
	 * @return a new instance of this BDrawable object.
	 */
	public abstract BDrawable newInstance();


	/**
	 * Loads the BDrawable from a file
	 * @param BDataFile file - the file to load from
	 * @return boolean - true on success, false on failure.
	 */
	public boolean load(BDataFile file)
	{
		Debug.doPrintln( "BModelElement - load" );

		super.load( file );
		uniqueID = Long.parseLong(file.getData("ID", "0"));


		this.ruleStatus = Integer.parseInt(file.getData("rule", "0"));
		//this.metaElement = (BMetaElement) file.autoLoad();

		int propertyCount = Integer.parseInt(file.getData("properties", "0"));

		newProperties = new Vector();

		while(propertyCount > 0)
		{
			BSaveTextArea saveArea = (BSaveTextArea) file.autoLoad();
			if(saveArea != null)
			{
				newProperties.add(saveArea);
				propertyCount --;
			}
			else
			{
				break;
			}
		}
		Debug.doPrintln("Loaded the text!!");

		


		return true;
	}

	/**
	 * This method is where pointers to required objects are set up
	 * Because the file is saved flat, with (eventualy) no duplication
	 * of data, they have to be set after the object is loaded.
	 *
	 * This is done after to load to allow for circular refrences. (or forward dependancies)
	 */
	public void resolveLoadedObject(BModel model)
	{
		model.loadedID(uniqueID);
		parent = model;

	}

	public void resolveText()
	{
		// find the properties objects and give then the correct text...
		Vector properties = getProperties();

		if(properties != null)
		{

			for(int i=0; i<properties.size(); i++)
			{
				BTextArea textArea = (BTextArea) properties.get(i);
				if(textArea != null)
				{
					String name = textArea.getName();
					
					BSaveTextArea saveArea = findNewProperty(name);
					if(saveArea != null)
					{
						textArea.setMultipleStrings(saveArea.getStrings());
					}
				}
			}
		}
	}

	private BSaveTextArea findNewProperty(String fName)
	{
		if(newProperties != null)
		{
			for(int i=0; i<newProperties.size(); i++)
			{
				BSaveTextArea saveArea = (BSaveTextArea) newProperties.get(i);
				if(saveArea.getName().equals(fName))
				{
					return saveArea;
				}
			}
		}
		return null;
	}

	public void checkID(BModel model)
	{
		if(uniqueID == kUnassignedID)
		{
			uniqueID = model.getNewUniqueID();
		}
	}

	/**
	 * Saves the BDrawable to a file
	 * @param BDataFile file - the file to save to
	 * @return boolean - true on success, false on failure.
	 */
	public boolean save(BDataFile file)
	{
		Debug.doPrintln("BModelElement - save");

		super.save( file );
		// save the properties
		
		file.addData("ID", "" + uniqueID);
		file.addData("rule", "" + this.ruleStatus );

		//file.save(metaElement);
		
		// save textarea stuff
		int savedProperties = 0;
		Vector properties = getProperties();
		if(properties != null)
		{
			file.addData("properties", "" + properties.size());
			for(int i=0; i<properties.size(); i++)
			{
				BTextArea textArea = (BTextArea) properties.get(i);
				if(textArea != null)
				{
					BSaveTextArea saveArea = new BSaveTextArea();
					saveArea.setName(textArea.getName());
					saveArea.setStrings(textArea.getStrings());
					file.save(saveArea);
					savedProperties++;
				}
			}
		}

		return true;
	}

	/**
	 * Create a copy of text in the package properties so it is save to change the package
	 */
	public void doSaveText()
	{
		newProperties = new Vector();
		Vector properties = getProperties();
		if(properties != null)
		{
			for(int i=0; i<properties.size(); i++)
			{
				BTextArea textArea = (BTextArea) properties.get(i);
				if(textArea != null)
				{
					BSaveTextArea saveArea = new BSaveTextArea();
					saveArea.setName(textArea.getName());
					saveArea.setStrings(textArea.getStrings());
					newProperties.add(saveArea);
				}
			}
		}

		Debug.doPrintln("Saved the text!!");

	}

	/**
	 * updates the bounding region of the drawable
	 * Called when something requires the bounds of
	 * the drawable to be updated.  Subclasses should
	 * determine a suitable way to update themselves.
	 */
	public abstract void updateBounds();



///////////BTreeNode stuff
	public boolean isLeaf()
	{
		return true;
	}

	public Object getChildAt( int index )
	{
		return null;
	}

	public int getChildCount()
	{
		return 0;
	}

	public Object getParent()
	{
		return this.parent;
	}

	public int getIndexOf( Object node )
	{
		return -1;
	}

	public void setParent( Object parent )
	{
		if ( parent instanceof BModel )
		{
			this.parent = (BModel)parent;
		}
		else
		{
			Debug.doPrintln("Error! ModelElement's parent must be a BModel");
		}
	}

	public void removeAllChildren()
	{
	}

	public String getName()
	{
		return this.name;
	}

	/**
	 * Called when a drawable is pasted onto a diagram.
	 * When things like model elements are pasted
	 * they need to change their package
	 */
	public void doPasted(BDiagram diagram)
	{
		if(diagram instanceof BModel)
		{
			doSaveText();
			resolveLoadedObject((BModel) diagram);
		}
	}

	public void setConflict(int fConflict)
	{
		conflictState = fConflict;
	}

	public int getConflict()
	{
		return conflictState;
	}


	/**
	 * Detect changes in versions of this object.
	 * If a change is detected, any information from oldElement which the user might want to know
	 * should be stored in this.
	 */
	public abstract int findChanges(BModelElement oldElement);

// ADDED BY PHIL

	public Object getPropertyValue (String name)
	{
		if (name.equals(BScalarPoint.kKeyX))
		{
			return new Float(pos.getXFloat());
		}
		else// (name.equals(BScalarPoint.kKeyY))
		{
			return new Float(pos.getYFloat());
		}
	}

	public boolean isChangeAllowed (String name, Object oldValue, Object newValue)
	{
		if (name.equals(BScalarPoint.kKeyX))
		{
			String val = newValue.toString(); 
			try
			{
				float test =  Float.parseFloat(val);
				Debug.doPrintln("New value is " + test);
				pos.setX(test);
				return true;
			}
			catch (Exception e)
			{
			}
		}
		else
		{
			String val = newValue.toString();
			try
			{
				float test =  Float.parseFloat(val);
				Debug.doPrintln("New value is " + test);
				pos.setY(test);
				return true;
			}
			catch (Exception e)
			{
			}
		}
		return false;
	}

	public void addProperties (BPropertyBag bag)
	{
		bag.addStringProperty(BScalarPoint.kKeyX, this);
		bag.addStringProperty(BScalarPoint.kKeyY, this);
	}
// END ADDED BY PHIL
}
