//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/modeller/BModelNode.java,v 1.27 1999/10/17 01:14:43 tp079511 Exp $


package au.edu.swin.synergy.plugin.modeller;

import au.edu.swin.synergy.lib.BTreeNode;

import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.plugin.metamodeller.BMetaElement;
import au.edu.swin.synergy.plugin.modeller.BModelElement;
import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.plugin.diagram.BView;
import au.edu.swin.synergy.plugin.packager.BPackageNode;
import au.edu.swin.synergy.plugin.packager.BPackageModel;
import au.edu.swin.synergy.plugin.dialect.BString;
import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.conflict.BConflict;

import au.edu.swin.synergy.shapes.BPropPanel;
import au.edu.swin.synergy.debug.Debug;

import au.edu.swin.synergy.math.BScalarPoint;
import au.edu.swin.synergy.math.BScalarRect;


import java.util.Vector;
import java.awt.*;


public class BModelNode extends BModelElement implements BSaveable
{
	private Vector inArcs = null;
	private Vector outArcs = null;
	private BPackageNode pNode = null;


	public BModelNode( BPackageNode pNode )
	{
		super( pNode.getMetaElement() );
		this.pNode = pNode;
		this.pos.setLocation( pNode.getPos() );

		//set the references..so actions on mn affect the pn..
		this.pos = pNode.getPos();
		this.bounds = this.pNode.getBounds();

		this.inArcs = new Vector();
		this.outArcs = new Vector();
	}

	public BModelNode( BModelNode modelNode )
	{
		this( modelNode.pNode );
		this.copy( modelNode );
	}

	/**
	 * Default constructor for BSaveable
	 */
	public BModelNode()
	{
		super(null);
		this.inArcs = new Vector();
		this.outArcs = new Vector();
	}


    public BPropPanel getPropertyPanel ()
    {
        return pNode.getPropertyPanel ();
    }

	/**
	 * @return true if this drawable is able to be deleted at this point in time
	 * @return false otherwise
	 *
	 */
	public String isDeletable()
	{
		String result = null;
		if ( inArcs.size() > 0  || outArcs.size() > 0 )
		{
			result = BDialect.getString ( this, "cannotdeletenode" );
		}
		return result;
	}

	

   public Vector getProperties()
   {
      return this.pNode.getProperties();
   }


	public void addInArc( BModelArc arc )
	{
		int index = inArcs.indexOf( arc );

		if ( index < 0 )
		{
			this.inArcs.add( arc );
		}
	}

	public void removeInArc( BModelArc arc )
	{
		int index = inArcs.indexOf( arc );

		if ( index >= 0 )
		{
			this.inArcs.removeElementAt( index );
		}
	}

    public BDrawable getPackageNode()
    {
        return this.pNode;
    }

	public void addOutArc( BModelArc arc )
	{
		 int index = outArcs.indexOf( arc );

		if ( index < 0 )
		{
			this.outArcs.add( arc );
		}
	}

	public void removeOutArc( BModelArc arc )
	{
		int index = outArcs.indexOf( arc );

		if ( index >= 0 )
		{
			this.outArcs.removeElementAt( index );
		}
	}

	public int getNumInArcs()
	{
		return inArcs.size();
	}

	public int getNumOutArcs()
	{
		return outArcs.size();
	}



	public void copy( BModelNode src )
	{
		int i, size;

		super.copy( src );


	}

	public boolean circRefExists( String arcType )
	{
		boolean result = false;
		int size, i;
		BModelArc cur;

		size = this.outArcs.size();
		for ( i = 0; i < size && !result; i++ )
		{
			cur = (BModelArc)this.outArcs.elementAt( i );
			if ( cur.getName().equals( arcType) && cur.getEndNode() != null )
			{
				result |= cur.getEndNode().circularRef( this, arcType );
			}
		}

		return result;
	}

	/**
	 * checks to see if there is a circular reference with this
	 * node, for a given arc type.
	 */
	protected boolean circularRef( BModelNode startNode, String arcType )
	{
		int i;
		int size;
		boolean result = false;
		BModelArc cur = null;

		if ( this == startNode ) //circular ref found...
		{
			result = true;
		}
		else // not found yet, check all out arcs...
		{
			size = this.outArcs.size();
			for ( i = 0; i < size && !result; i++ )
			{
				cur = (BModelArc)this.outArcs.elementAt( i );
				if ( cur.getName().equals( arcType) )
				{
					result |= cur.getEndNode().circularRef( startNode, arcType );
				}
			}
		}

		return result;
	}


	public final Rectangle getXFormedBounds()
	{
		return this.pNode.getXFormedBounds();
	}


/////////////BDrawable stuff

	/**
	 * Transforms the coordinates of the drawable object
	 * to a new coordinate space, as defined by the
	 * viewport and parent pos.
	 * Subclasses should follow the following equation for
	 * tranforming relevant points etc.
	 *
	 * xformPt.x = ((localPt.x + parentPos.x + pos.x - viewport.x) * viewport.zoom ) + viewport.halfx;
	 *
	 * @param BView viewport - the viewport that this object is going to be drawn in
	 * @param Point parentPos - coordinates of any parent object
	 */
	public void transform( BView viewport, BScalarPoint parentPos, int frameCounter )
	{
		if ( this.lastFrame != frameCounter )
		{
			this.pNode.transform( viewport, parentPos, frameCounter );

			this.lastFrame = frameCounter;
		}
	}


	/**
	 * Renders the transformed coordinates of the drawable
	 * object to a Graphics2D
	 *
	 * @param Graphics2D g2 - the place to render the object to.
	 */
	public void render( Graphics2D g2 )
	{
		this.pNode.render( g2 );
	}


	/**
	 * Creates and returns a new instance of the BDrawable object
	 *
	 * @return a new instance of this BDrawable object.
	 */
	public BDrawable newInstance()
	{
		return new BModelNode( this );
	}

	public void setFlag( int flag, boolean on )
	{
		if ( on )
		{
			this.statusFlag |= flag;
		}
		else
		{
			this.statusFlag &= ~flag;
		}

		this.pNode.setFlag( flag, on );
	}


	/**
	 * Checks to see if a point has hit the transformed bounds
	 * of this object
	 * @param Point pt - the point to check the hit against..
	 * @return kNotHit - not hit
	 * @return kHitBounds - hit the main bounds
	 */
	public int checkXFormBounds( Point pt )
	{
		return this.pNode.checkXFormBounds( pt );
	}


	/**
	 * Checks to see if a sizer has been hit by a point
	 * Works on xformed values.
	 * Subclasses should determine appropriate places for
	 * the sizer..
	 *
	 * @param Point pt - the point to check the hit against
	 * @return kNotHit - not hit
	 * @return kHitSizer + XX hit the XX sizer where XX is any number.
	 */
	public int checkSizerHit( Point pt )
	{
		return this.pNode.checkXFormBounds( pt );
	}


	/**
	 * Loads the BDrawable from a file
	 * @param BDataFile file - the file to load from
	 * @return boolean - true on success, false on failure.
	 */
	public boolean load(BDataFile file)
	{
		Debug.doPrintln( "BModelNode - load" );

		super.load(file);

		// all the work is done by the resolveLoadedObject function

		return true;
	}

	public void resolveLoadedObject(BModel model)
	{
		super.resolveLoadedObject(model);

		Debug.doPrintln("Resolving the package node..." + name);
		// find the pNode with name name
		BPackageModel packageModel = model.getPackageModel();
		pNode = (BPackageNode) (packageModel.getPackageItemByName(name));

		// we need to make out own copy of the package node
		pNode = (BPackageNode) pNode.newInstance();
		pNode.getPos().setLocation(pos);

		setMetaElement(pNode.getMetaElement());
		
		this.pos = pNode.getPos();
		this.bounds = this.pNode.getBounds();
		updateBounds();
		resolveText();
	}

	/**
	 * Saves the BDrawable to a file
	 * @param BDataFile file - the file to save to
	 * @return boolean - true on success, false on failure.
	 */
	public boolean save(BDataFile file)
	{
		Debug.doPrintln( "BModelNode - save" );

		super.save(file);

		return true;
	}

	/**
	 * updates the bounding region of the drawable
	 * Called when something requires the bounds of
	 * the drawable to be updated.  Subclasses should
	 * determine a suitable way to update themselves.
	 */
	public void updateBounds()
	{
		if(pNode != null)
		{
			this.pNode.updateBounds();
		}
	}

	// conflict detection
	public int findChanges(BModelElement oldNode)
	{
		int result = BConflict.kNoConflict;
		
		Debug.doPrintln("Looking for changes");
		if(!pos.equals(oldNode.getPos ()))
		{
			result |= BConflict.kConflictCosmetic;
		}

		// now look at the hard stuff.
		// properties.....
		// result |= kConflict
		return result;
	}

	/**
	 * When the node is deleted, it needs to tell the connected arcs that they are no longer connected...
	 */
	public void delete()
	{
		for(int i=0; i<inArcs.size(); i++)
		{
			BModelArc arc = (BModelArc) inArcs.get(i);
			arc.setEndNode(null);
		}
		//inArcs.clear();
		
		for(int i=0; i<outArcs.size(); i++)
		{
			BModelArc arc = (BModelArc) outArcs.get(i);
			arc.setEndNode(null);
		}
		//outArcs.clear();
	}

}
