
//Title:        Packager
//Version:
//Copyright:    Copyright (c) 1998
//Author:       Tristan Austin
//Company:      Synergy
//Description:  Allows user to define the look of a meta model

   package au.edu.swin.synergy.plugin.packager;

   import au.edu.swin.synergy.lib.BGridBagConstraints;
   import au.edu.swin.synergy.kernel.BPlugin;
   import au.edu.swin.synergy.kernel.BCommandEvent;
   import au.edu.swin.synergy.kernel.BCommandComplete;
   import au.edu.swin.synergy.kernel.BBlockingComplete;

   import au.edu.swin.synergy.plugin.mod13.event.BCloseWindowCE;
   import au.edu.swin.synergy.plugin.mod13.event.BAddWindowCE;
   import au.edu.swin.synergy.plugin.mod13.event.BAddPanelCE;
	import au.edu.swin.synergy.plugin.metamodeller.BMetaModel;
	import au.edu.swin.synergy.lib.io.BFileFilter;
import au.edu.swin.synergy.plugin.metamodeller.event.BOpenMetamodelCE;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;


	import au.edu.swin.synergy.plugin.dialect.BDialect;

   import au.edu.swin.synergy.debug.Debug;

   import java.awt.*;
   import java.awt.event.*;
   import javax.swing.*;
	
	import java.io.FileInputStream;
	import java.io.File;
	import java.io.ObjectInputStream;
	import java.io.IOException;


/**
 * This will allow the user to specify the details
 * of the package they are about to create. They can
 * give it a name, specify the meta model it is based on
 * and varios other options.
 *
 * @author Tristan Austin
 */
   public class BNewPackageDialog extends JPanel
   {
      JLabel nameLabel;
      JTextField nameField = new JTextField ();
      JLabel modelLabel;
      JTextField modelField = new JTextField ();
      JButton browseButton = new JButton();
      JButton okButton = new JButton();
      JButton cancelButton = new JButton();
      GridBagLayout gridBagLayout1 = new GridBagLayout();	  
   
    /**
     * The dispatcher
     */
      private BPackagePlugin packagePlugin = null;
   
    /**
     * The action listener for internal components
     */
      private BActionListener actionListener = new BActionListener();
   
    /**
     * The file browser
     */
      private JFileChooser fileChooser = null;
   	
   	/**
   	 * This is the meta model file that the user selected
   	 */
	File metaModelFile = null;
   	
   
   
    /**
     * Creates a new package dialog with the given
     * bplugin to dispatch events via.
     *
     * @param   dispatcher  The object which can dispatch events via the kernel
     */
      public BNewPackageDialog(BPackagePlugin packagePlugin)
      {
         this.packagePlugin = packagePlugin;
         try
         {
            jbInit();
         }
            catch (Exception ex)
            {
               ex.printStackTrace();
            }
      }

      void jbInit() throws Exception
      {
         nameLabel = new JLabel ( BDialect.getString ( this, "name" ) );
         modelLabel = new JLabel ( BDialect.getString ( this, "mm" ) );
         browseButton.setText( BDialect.getString ( this, "browse" ) );
         okButton.setText( BDialect.getString ( "ok" ) );
         cancelButton.setText( BDialect.getString ( "cancel" ) );
         this.setLayout(gridBagLayout1);
         this.add(nameLabel, new BGridBagConstraints(0, 0, 1, 1, 0.0, 0.0
                                   ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
         this.add(nameField, new BGridBagConstraints(1, 0, 2, 1, 1.0, 0.0
                                   ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
         this.add(modelLabel, new BGridBagConstraints(0, 1, 1, 1, 0.0, 0.0
                                   ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
         this.add(modelField, new BGridBagConstraints(1, 1, 1, 1, 1.0, 0.0
                                   ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
         this.add(browseButton, new BGridBagConstraints(2, 1, 1, 1, 0.0, 0.0
                                   ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
         this.add(okButton, new BGridBagConstraints(2, 2, 1, 1, 0.0, 0.0
                                   ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
         this.add(cancelButton, new BGridBagConstraints(1, 2, 1, 1, 1.0, 0.0
                                   ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
      
        //Add action listeners
         this.okButton.addActionListener(this.actionListener);
         this.cancelButton.addActionListener(this.actionListener);
         this.browseButton.addActionListener(this.actionListener);
      }
   
    /**
     * This will be called when the user clicks on the
     * ok buton. It will check the fields have been
     * filled in correctly.
     */
      private void doOK()
      {
         BCloseWindowCE event = null;
         String packageName = this.nameField.getText().trim();
        //Need to send off some event to open the package editor and close the window
		 String metaModelName = null;

		 if ( this.metaModelFile != null )
		 {
			metaModelName = this.metaModelFile.getPath();
		 }
		 else
		 {
			metaModelName = this.modelField.getText().trim();
		 }
      
        //Make sure they entered a valid name
         if (packageName.length() > 0 && metaModelName.length() > 0)
         {

			this.packagePlugin.doCreateNewPackage( packageName, metaModelName );

            //Close the new dialog
            event = new BCloseWindowCE(this, BNewPackageDialog.this);
         
            //Send the event to the window manager
            this.packagePlugin.dispatchEvent(event);
         
            //Display the package plugin
            this.packagePlugin.doEditPackage( packageName + "." + BDialect.getString ( this, "fileextension" ) );
         }
      }
   
    /**
     * This will listen for action events on the buttons
     *
     * @author Tristan Austin
     */
      private class BActionListener implements ActionListener
      {
        /**
         * This method will be called whent he user selects
         * a button.
         *
         * @param   actionEvent The event fired
         */
         public void actionPerformed(ActionEvent actionEvent)
         {
            Object source = actionEvent.getSource();
            BCommandEvent event = null;
         
            //Determine where the action event came from
            if (source == BNewPackageDialog.this.cancelButton)
            {
                //Simply close the window
               event = new BCloseWindowCE(this, BNewPackageDialog.this);
            }
            else if (source == BNewPackageDialog.this.okButton)
            {
               BNewPackageDialog.this.doOK();
            }
            else if (source == BNewPackageDialog.this.browseButton)
            {
            	Debug.doPrintln("Browse Button Clicked");
            
                //Make sure the file chooser has been created
/*               if (BNewPackageDialog.this.fileChooser == null)
               {					
					BFileFilter filter = new BFileFilter( BDialect.getString ( this, "mtafileextension" ), BDialect.getString( this, "mtafilterdesc") );
                  BNewPackageDialog.this.fileChooser = new JFileChooser(".");
				  BNewPackageDialog.this.fileChooser.setFileFilter( filter );
               }

               int returnValue = BNewPackageDialog.this.fileChooser.showOpenDialog(BNewPackageDialog.this);

               if (returnValue == JFileChooser.APPROVE_OPTION)
               {
                    BNewPackageDialog.this.metaModelFile = BNewPackageDialog.this.fileChooser.getSelectedFile();
               		BNewPackageDialog.this.modelField.setText(metaModelFile.getName());               		
               	}
               	else if (returnValue == JFileChooser.CANCEL_OPTION)
               	{
					
               	}*/
				event = new BOpenMetamodelCE(this, new OnGetModelComplete());
               /*event = new BAddWindowCE(this, BNewPackageDialog.this.fileChooser, "File Chooser");
               ((BAddWindowCE)(event)).setDialog(true);//*/
            }

            //Make sure an event was created before we try and send it
            if (event != null)
            {
                //Send the event via the kernel
               BNewPackageDialog.this.packagePlugin.dispatchEvent(event);
            }
         }
      }

  	class OnGetModelComplete extends BCommandComplete
	{
	   public void doComplete( BCommandEvent event, int status )
		{
			if(status == BCommandComplete.kSuccess)
			{
				BOpenMetamodelCE openEvent = (BOpenMetamodelCE) event;
				if(openEvent.getFileName() != null)
				{
				   BSwingUtilities.setJText(modelField, openEvent.getFileName());
				}
			}
			else
			{
				Debug.doPrintln("Error opening file");
			}

		}
	}

   }

