// $Header:

/**
*
* @author Christian Karolos
* @date 11/10/99
*/

package au.edu.swin.synergy.plugin.packager;

import javax.swing.*;
import java.awt.*;

import au.edu.swin.synergy.shapes.BPropPanel;
import au.edu.swin.synergy.shapes.BLineEditorPanel;
import au.edu.swin.synergy.plugin.packager.BPackageArc;
import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.debug.Debug;

public class BPackageArcProperties extends BPropPanel
{
	BLineEditorPanel arcEditor;
	BPackageArc packageArc;

	String[] arcTypes = new String[ 3 ];
	JComboBox typeBox;
	JLabel arcType;

	public BPackageArcProperties ( BPackageArc arc )
	{
        Debug.doPrintln ( "INIT package arc properties" );
		setLayout ( new GridBagLayout () );		
        GridBagConstraints c = new GridBagConstraints ();

        this.packageArc = arc;

		arcTypes[BPackageArc.kStraightType] = BDialect.getString ( this, "straight" );
		arcTypes[BPackageArc.kSteppedType] = BDialect.getString ( this, "stepped" );
		arcTypes[BPackageArc.kCurvedType] = BDialect.getString ( this, "curved" );

		arcType = new JLabel ( BDialect.getString ( this, "arcstyle" ) );
		typeBox = new JComboBox ( arcTypes );

		arcEditor = new BLineEditorPanel ( arc.getStroke (), arc.getColour (), arc.getStyle () );

        setConst ( c, 0, 0, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH, 1 );
		add ( arcEditor, c );

        setConst ( c, 0, 1, 1, 1, GridBagConstraints.EAST, GridBagConstraints.NONE, 1 );
		add ( arcType, c );

        setConst ( c, 1, 1, 1, 1, GridBagConstraints.WEST, GridBagConstraints.NONE, 1 );
		add ( typeBox, c );

		this.init ();
	}

	/**
	* Custom made function for plotting labels and combo boxes on a panel.
	* Panel shouldn't be expandable, so concerning attribs are fixed.
	*/
    public static void setConst( GridBagConstraints gbc, int x, int y, int w, int h, int anc, int fill, int weight )
    {
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.gridwidth = w;
        gbc.gridheight = h;
        gbc.fill = fill;
        gbc.anchor = anc;
        gbc.insets = new Insets ( 10, 10, 10, 10 );

		// Only a light weight so things don't expand too much.
        gbc.weightx = weight;
        gbc.weighty = weight;
        gbc.ipady = 10;
        gbc.ipadx = 10;
    }

	private void init ()
	{
		arcEditor.setColour ( packageArc.getColour () );
		arcEditor.setLineStroke ( packageArc.getStroke () );
		arcEditor.setStyle ( packageArc.getStyle () );
		typeBox.setSelectedIndex ( packageArc.getArcType () );
	}

	public boolean okPressed ()
	{
		packageArc.setStyle ( arcEditor.getStyle () );
		packageArc.setColour ( arcEditor.getColour () );
		packageArc.setStroke ( arcEditor.getLineStroke () );
		packageArc.setArcType ( typeBox.getSelectedIndex () );

		// Plus terminator changes made, if we allow this at
		// this stage.
		
		return true;
	}
	
	public boolean cancelPressed ()
	{
		Debug.doPrintln ( "Changes should have been aborted." );

		return true;
	}
}