// $Header:
/**
* Just providing a simple tree cell renderer, without it getting mixed
* up with all the application code of other classes.
*
* @author Christian Karolos
* @date 12/10/99
*/
package au.edu.swin.synergy.plugin.packager;

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;

import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.plugin.packager.BPackageModel;
import au.edu.swin.synergy.plugin.packager.BPackageNode;
import au.edu.swin.synergy.plugin.packager.BPackageArc;
import au.edu.swin.synergy.plugin.packager.BPackageTerminator;
import au.edu.swin.synergy.plugin.packager.BPackageModelBranch;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;

import au.edu.swin.synergy.plugin.mod13.BWindowManager;
import au.edu.swin.synergy.shapes.BTextArea;

import au.edu.swin.synergy.lib.BTreeNode;

public class BPackageCellRenderer extends JPanel implements TreeCellRenderer
{
	public static final int kFontSize = 12;
	public static final int kIconGap = 1;

	JLabel label;
	BPackageTree pTree;
	Color back;
	Color highlight;

	ImageIcon pac;
	ImageIcon property;

	ImageIcon terminatorPackage;
	ImageIcon terminator;

	ImageIcon nodePackage;
	ImageIcon node;

	ImageIcon arcPackage;
	ImageIcon arc;

	public BPackageCellRenderer ( BPackageTree t )
	{
		pTree = t;
		label = new JLabel ();
		label.setFont ( new Font ( "system", Font.PLAIN, kFontSize ) );
		label.setIconTextGap ( kIconGap );
		String path = new String ( "/au/edu/swin/synergy/plugin/packager/icons/" );

		// White just so happens to be the background of the package / notation tree at
		// the moment.
		back = Color.white;
		highlight = BWindowManager.getHighlightColor ();

        pac = BSwingUtilities.getImage(this, path + "packagemodel.gif" ) ;
        Debug.doPrintln("pac: " + pac, Debug.kWarningLevel);

        property = BSwingUtilities.getImage(this, path + "property.gif" ) ;
        Debug.doPrintln("Property: " + property, Debug.kWarningLevel);


        terminatorPackage = BSwingUtilities.getImage(this, path + "terminatorroot.gif" ) ;
        Debug.doPrintln("terminatorPackage: " + terminatorPackage, Debug.kWarningLevel);

        terminator = BSwingUtilities.getImage(this, path + "terminator.gif" ) ;
        Debug.doPrintln("terminator: " + terminator, Debug.kWarningLevel);


        nodePackage = BSwingUtilities.getImage(this, path + "node.gif" ) ;
        Debug.doPrintln("nodePackage: " + nodePackage, Debug.kWarningLevel);

        node = BSwingUtilities.getImage(this, path + "node2.gif" ) ;
        Debug.doPrintln("node: " + node, Debug.kWarningLevel);


        arcPackage = BSwingUtilities.getImage(this, path + "arc.gif" ) ;
        Debug.doPrintln("arcPackage: " + arcPackage, Debug.kWarningLevel);

        arc = BSwingUtilities.getImage(this, path + "arc2.gif" ) ;
        Debug.doPrintln("arc: " + arc, Debug.kWarningLevel);

		add ( label );
	}

	
	public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, 
		boolean expanded, boolean leaf, int row, boolean hasFocus)
	{
		BTreeNode treeNode = ( BTreeNode ) value;

		label.setText ( value.toString () );

		if ( selected )
		{
			setBackground (BWindowManager.getHighlightColor());
		}
		else
		{
			setBackground ( this.back );
		}

		if ( treeNode instanceof BPackageModel )
		{
			label.setIcon ( pac );
		}
		else if ( treeNode instanceof BPackageNode )
		{
			label.setIcon ( nodePackage );
		}
		else if ( treeNode instanceof BPackageArc )
		{
			label.setIcon ( arcPackage );
		}
		else if ( treeNode instanceof BPackageTerminator )
		{
			label.setIcon ( terminatorPackage );
		}
		else if ( treeNode instanceof BTextArea )
		{
			label.setIcon ( property );
		}
		else if ( treeNode instanceof BPackageModelBranch )
		{
			switch( pTree.getModel ().getIndexOfChild( treeNode.getParent (), treeNode ) )
			{
				case BPackageModel.kNodeBranch:
					label.setIcon ( node );
					break;
				case BPackageModel.kArcBranch:
					label.setIcon ( arc );
					break;
				case BPackageModel.kTerminatorBranch:
					label.setIcon ( terminator );
					break;
			}
		}

		return this;
	}

}

