
//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/packager/BPackageModel.java,v 1.35 1999/10/05 19:11:10 tp082177 Exp $
   package au.edu.swin.synergy.plugin.packager;

   import au.edu.swin.synergy.plugin.diagram.BDiagram;

   import au.edu.swin.synergy.plugin.metamodeller.BMetaModel;
   import au.edu.swin.synergy.plugin.metamodeller.BMetaElement;
   import au.edu.swin.synergy.lib.io.BSaveable;
   import au.edu.swin.synergy.lib.io.BDataFile;
   import au.edu.swin.synergy.lib.BTreeUtil;

   import au.edu.swin.synergy.lib.BTreeNode;

   import au.edu.swin.synergy.debug.Debug;

	import au.edu.swin.synergy.plugin.dialect.BDialect;

	import au.edu.swin.synergy.lib.BTreeModelInterface;

   import java.util.Vector;
   import java.util.Enumeration;

   import java.awt.Point;

   import java.io.File;
   import java.io.FileInputStream;
   import java.io.FileOutputStream;
   import java.io.ObjectInputStream;
   import java.io.ObjectOutputStream;
   import java.io.IOException;
   import java.io.Serializable;

   import javax.swing.event.TreeModelListener;
   import javax.swing.event.TreeModelEvent;
   import javax.swing.tree.TreeModel;
   import javax.swing.tree.TreePath;
import au.edu.swin.synergy.lib.io.BDataFile;


/**
 * Data structure for the package model.
 * Keeps track of the p-nodes, p-arcs, and p-terminators
 * that make up the package.  Also has a meta-model.
 *
 * Implements the TreeModel interface, so that the data
 * can be viewed from inside a JTree.
 *
 * @author Imron Alston
 * @date 12/8/99
 *
 * Please complete these missing tags
 * @copyright
 */
   public class BPackageModel implements BTreeModelInterface, BTreeNode, Serializable, BSaveable
   {

    /**
     * This is the list of nodes in this package
     */
      private BPackageModelBranch nodeBranch = null;

    /**
     * This is the list of arcs in this package
     */
      private BPackageModelBranch arcBranch = null;

    /**
     * This is the list of terminators in this package
     */
      private BPackageModelBranch terminatorBranch = null;


  	// for BSaveable
	protected static final String kKeyName = "name";
	protected static final String kDefaultName = "untitled";



    /**
     * This is hte list of listeners that need to be notified
     * of the changes to the tree
     */
      transient private Vector treeListeners = null;

      public static final int kNumBranches = 3;
      public static final int kNodeBranch = 0;
      public static final int kArcBranch = 1;
      public static final int kTerminatorBranch = 2;


    /**
     * This is the meta model that this package represents
     */
      private BMetaModel metaModel = null;

    /**
     * This is the name of the package
     */
      private String name = null;

    /**
     * When you create a package model, you must specify the
     * meta model on which it is based. You cannot have a package
     * without a meta model.
     *
     * @param metaModel The model this package is based on
     * @pre The meta model already exists
     */
      public BPackageModel(BMetaModel metaModel, String packageName)
      {
		this.treeListeners= new Vector();
         this.nodeBranch = new BPackageModelBranch( this, BDialect.getString ( this, "nodes" ) );
         this.arcBranch = new BPackageModelBranch( this, BDialect.getString ( this, "arcs" ) );
         this.terminatorBranch = new BPackageModelBranch( this, BDialect.getString ( this, "terminators" ) );


        //Set the meta model
         this.setMetaModel(metaModel);

		 this.name = packageName;

      }

      public BPackageModel()
      {

		 this.treeListeners= new Vector();
         this.nodeBranch = new BPackageModelBranch( this, BDialect.getString ( this, "nodes" ) );
         this.arcBranch = new BPackageModelBranch( this, BDialect.getString ( this, "arcs" ) );
         this.terminatorBranch = new BPackageModelBranch( this, BDialect.getString ( this, "terminators" ) );

      }

    /**
     * This will set the meta model this tree represents
     * to the given meta model. It will also re-initialise
     * the nodes and so forth that represent the meta elements
     * on screen. This should be considered a resetting of the
     * package editor and therefore anything already in this
     * package will be replaced. That is to say, this is to be
     * used for new Packages only.
     *
     * @param   metaModel   The new meta model
     */
      public void setMetaModel(BMetaModel metaModel)
      {
         this.metaModel = metaModel;
         this.doInitialisation(this.metaModel);
      }


	  public BMetaModel getMetaModel()
	  {
		return this.metaModel;
	  }

    /**
     * This method will run through the meta model and add the
     * nodes, arcs and terminators to the appropriate tree nodes
     * in the model. This will populate the tree as required.
     *
     * @param   metaModel   The meta model to use
     */
      private void doInitialisation(BMetaModel metaModel)
      {
         Vector metaNodes = metaModel.getMetaNodes();
         Vector metaArcs = metaModel.getMetaArcs();
		 BMetaElement curElement = null;
         BTreeNode currentNode = null;

         Debug.doPrintln("Doing initialisation of new meta model in the package model", Debug.kWarningLevel);

         this.nodeBranch.removeAllChildren();
         this.arcBranch.removeAllChildren();
         this.terminatorBranch.removeAllChildren();


        //Make sure there are meta nodes to add
         if (metaNodes != null)
         {
            //Go through and create the tree nodes for each meta node
            for (int i = 0; i < metaNodes.size(); i++)
            {
				curElement = (BMetaElement)metaNodes.elementAt( i );
               currentNode = new BPackageNode( curElement );
               currentNode.setParent( this.nodeBranch );
               this.nodeBranch.addNode( currentNode );
            }

         }



        //Make sure there are meta arcs to add
        if (metaArcs != null)
        {
            //Go through and create the tree nodes for each meta node
            for (int i = 0; i < metaArcs.size(); i++)
            {
				curElement = (BMetaElement)metaArcs.elementAt( i );
      			currentNode = new BPackageArc( curElement );
      			currentNode.setParent( this.arcBranch );
                this.arcBranch.addNode( currentNode );
            }

        }


         this.fireStructureChanged();

      }

    /**
     * Sets the name of the package to the value
     * specified. This will be reflected on the screen straight
     * away.
     *
     * @param   newPackageName  The name of the package
     */
      public void setName(String newPackageName)
      {
         this.name = newPackageName;

         Debug.doPrintln("Setting name of package tree - provide a better event firer", Debug.kWarningLevel);
         this.fireStructureChanged();
      }

   /**
    * This will add all BGroups in a branch of the model to the vector
    * provided.  Used to build up a list of all groups that can
    * be added to a tool-palette.  It is the job of the caller to
    * clear the vector.  That way the groups from a number of
    * different branches can be added to the one list.
    *
    */

      public void addBGroupsToVector( int branch, Vector groupList )
      {

         switch ( branch )
         {
            case BPackageModel.kNodeBranch:
               this.nodeBranch.addBGroupsToVector( groupList );
               break;
            case BPackageModel.kArcBranch:
               this.arcBranch.addBGroupsToVector( groupList );
               break;
            case BPackageModel.kTerminatorBranch:
               this.terminatorBranch.addBGroupsToVector( groupList );
               break;
         }

      }


	/**
	 * Gets a package element from the package model..
	 * @param String name - the name of the package item..
	 * @return the first item that matches name..
	 * @return null if no item is found..
	 *
	 */
	public Object getPackageItemByName( String name )
	{
		Object result = null;

		result = this.nodeBranch.getNodeByName( name );

		if ( result == null )
		{
			result = this.arcBranch.getNodeByName( name );

			if ( result == null )
			{
				result = this.terminatorBranch.getNodeByName( name );
			}
		}

		return result;
	}

	/**
	 * Gets a package node from the package model..
	 * @param String name - the name of the package node..
	 * @return the first item that matches name..
	 * @return null if no item is found..
	 *
	 */
	public Object getPackageNodeByName( String name )
	{
		return this.nodeBranch.getNodeByName( name );
	}

	/**
	 * Gets a package arc from the package model..
	 * @param String name - the name of the package arc..
	 * @return the first item that matches name..
	 * @return null if no item is found..
	 *
	 */
	public Object getPackageArcByName( String name )
	{
		return this.arcBranch.getNodeByName( name );
	}

	/**
	 * Gets a package terminator from the package model..
	 * @param String name - the name of the package terminator..
	 * @return the first item that matches name..
	 * @return null if no item is found..
	 *
	 */
	public Object getPackageTerminatorByName( String name )
	{
		return this.terminatorBranch.getNodeByName( name );
	}



	/**
	 * Adds a package node to the package model..
	 *
	 */
	public void addPackageNode( BTreeNode node )
	{
		node.setParent( this );
		this.nodeBranch.addNode( node );
	}

	/**
	 * Adds a package arc to the package model..
	 *
	 */
	public void addPackageArc( BTreeNode node )
	{
		node.setParent( this );
		this.arcBranch.addNode( node );
	}

	/**
	 * Adds a package terminator to the package model..
	 *
	 */
	public void addPackageTerminator( BTreeNode node )
	{
		node.setParent( this );
		this.terminatorBranch.addNode( node );

		Object [] added = null;
		int [] indices = null;
		indices = new int[1];
		indices[0] = this.terminatorBranch.getChildCount() -  1;
		added = new Object[1];
		added[0] = node;

		BTreeNode [] path = BTreeUtil.getPathToRoot( this.terminatorBranch, 0 );

		Debug.doPrintln("Path to root-->");
		for ( int i = 0; i < path.length; i++ )
		{
			Debug.doPrintln("\t" + i + ": " + path[i] );
		}

		TreeModelEvent me = new TreeModelEvent( this.terminatorBranch, path, indices, added );

		BTreeUtil.fireNodesInserted( this, me );
	}



   /**
    * loads a package from a file..
    * @param String fileName - the file to load
    * @return the loaded package, or null on error..
    *
    */
/*      public static BPackageModel loadFromFile( String fileName )
      {
         BPackageModel result = null;

         try
         {

            File packageModelFile = new File( fileName );

            FileInputStream inputStream = new FileInputStream(packageModelFile);
            ObjectInputStream objectIn = new ObjectInputStream(inputStream);

            result = (BPackageModel)objectIn.readObject();
            objectIn.close();

			result.treeListeners= new Vector();

            result.fireStructureChanged();

         }
            catch(IOException ioException)
            {
               Debug.doPrintln("ioException " + ioException, Debug.kWarningLevel );
            //            ioException.printStackTrace();

            }
            catch(ClassNotFoundException cnfException)
            {
               Debug.doPrintln("cnfException " + cnfException, Debug.kWarningLevel );
            //            cnfException.printStackTrace();
            }
            catch( Exception e )
            {
               Debug.doPrintln("***other exception "+ e, Debug.kWarningLevel );
               e.printStackTrace();
            }

         return result;
      }*/
      public static BPackageModel loadFromFile( String fileName )
      {
         BPackageModel result = null;

         try
         {

	   		BDataFile dataFile = new BDataFile( fileName );

			dataFile.openFileForRead();
			try
			{
				result = (BPackageModel)dataFile.autoLoad();
			}
			finally
			{
				dataFile.close();
			}

			result.treeListeners= new Vector();

            result.fireStructureChanged();

         }
            catch( Exception e )
            {
               Debug.doPrintln("Error loading BPackageModel ", e);
            }

         return result;
	}

   /**
    * saves this package to a file..
    */
/*      public void saveToFile( String fileName )
      {
         File packageFile = new File(fileName);
         try
         {
            FileOutputStream outputStream = new FileOutputStream(packageFile);
            ObjectOutputStream objectOut = new ObjectOutputStream(outputStream);

            objectOut.writeObject(this);
            objectOut.close();
         }
            catch(IOException ioException)
            {
               Debug.doPrintln("***ioBBAAException " + ioException );
            			ioException.printStackTrace();
            }
            catch(Exception e )
            {
               Debug.doPrintln("***other exception " + e );
            			e.printStackTrace();
            }
      }*/

      public void saveToFile( String fileName )
      {
         try
         {
			BDataFile packageFile = new BDataFile( fileName );
			try
			{
				packageFile.openFileForWrite(false);
				packageFile.save(this);
			}
			finally
			{
				packageFile.close();
			}
         }
         catch(Exception e )
         {
             Debug.doPrintln("Error saving BPackageModel " , e );
         }
      }
   //////////////Tree Model stuff

      public void addTreeModelListener(TreeModelListener l)
      {
         this.treeListeners.add( l );
      }

      public Object getChild(Object parent, int index)
      {
         BTreeNode branch = (BTreeNode)parent;
         return branch.getChildAt( index );
      }

      public int getChildCount(Object parent)
      {
         BTreeNode branch = (BTreeNode)parent;
         return branch.getChildCount();
      }

      public int getIndexOfChild(Object parent, Object child)
      {
         BTreeNode branch = (BTreeNode)parent;
         return branch.getIndexOf( child );
      }

      public Object getRoot()
      {
         return this;
      }

      public boolean isLeaf(Object node)
      {
         BTreeNode branch = (BTreeNode)node;
         return branch.isLeaf();
      }

      public void removeTreeModelListener(TreeModelListener l)
      {
         this.treeListeners.remove( l );
      }

      public void valueForPathChanged(TreePath path, Object newValue)
      {
         Debug.doPrintln( "Value for Path Changed", Debug.kWarningLevel );
      }

     public void fireInserted( TreeModelEvent e )
     {
      TreeModelListener l = null;
      int size, i;
      size = this.treeListeners.size();
      for ( i = 0; i < size; i++ )
      {
         l = (TreeModelListener)this.treeListeners.elementAt(i);
         l.treeNodesInserted( e );
      }
     }

     public void fireRemoved( TreeModelEvent e )
     {
      TreeModelListener l = null;
      int size, i;
      size = this.treeListeners.size();
      for ( i = 0; i < size; i++ )
      {
         l = (TreeModelListener)this.treeListeners.elementAt(i);
         l.treeNodesRemoved( e );
      }
     }

      protected void fireStructureChanged()
      {

         TreePath tp = new TreePath( this );
         TreeModelListener l = null;
         int i, size;

         TreeModelEvent me = new TreeModelEvent( this, tp.getPath() );


         size = this.treeListeners.size();

         for ( i = 0; i < size; i++ )
         {

            l = (TreeModelListener)this.treeListeners.elementAt( i );
            l.treeStructureChanged( me );
         }



      }

	public String toString()
	{
		return this.name;
	}

	public String getName()
	{
		return this.name;
	}

   ///////////BTreeNodeStuff

      public boolean isLeaf()
      {
         return false;
      }

      public Object getChildAt( int index )
      {
         Object result = null;

         switch ( index )
         {
            case BPackageModel.kNodeBranch:
               result = this.nodeBranch;
               break;
            case BPackageModel.kArcBranch:
               result = this.arcBranch;
               break;
            case BPackageModel.kTerminatorBranch:
               result = this.terminatorBranch;
               break;

         }

         return result;
      }

      public int getChildCount()
      {
         return BPackageModel.kNumBranches;
      }

      public Object getParent()
      {
         return null;
      }

      public int getIndexOf( Object node )
      {
         int result = -1;
         if ( node == this.nodeBranch )
         {
            result = BPackageModel.kNodeBranch;
         }
         else if ( node == this.arcBranch )
         {
            result = BPackageModel.kArcBranch;
         }
         else if ( node == this.terminatorBranch )
         {
            result = BPackageModel.kTerminatorBranch;
         }

         return result;
      }

      public void setParent( Object parent )
      {

      }

      public void removeAllChildren()
      {
      }


      public boolean load( BDataFile file )
      {
      	Debug.doPrintln( "BPackageModel - load" );

		name = file.getData( kKeyName, kDefaultName );
		nodeBranch = (BPackageModelBranch)file.autoLoad();
		nodeBranch.setParent( this );
		arcBranch = (BPackageModelBranch)file.autoLoad();
		arcBranch.setParent( this );
		terminatorBranch = (BPackageModelBranch)file.autoLoad();
		terminatorBranch.setParent( this );
		metaModel = (BMetaModel) file.autoLoad();

		return true;
      }

      public boolean save( BDataFile file )
      {
		Debug.doPrintln( "BPackageModel - save" );
		file.addData(kKeyName, name, kDefaultName);
		file.save( (BSaveable)nodeBranch );
		file.save( (BSaveable)arcBranch );
		file.save( (BSaveable)terminatorBranch );
		file.save( (BSaveable)metaModel );

      	return true;
      }


   }