
package au.edu.swin.synergy.plugin.packager;

import au.edu.swin.synergy.lib.BTreeNode;
import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.lib.io.BDataFile;

import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.plugin.packager.BPackageModel;
import au.edu.swin.synergy.plugin.metamodeller.BMetaElement;

import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.lib.BTreeUtil;

import java.util.Vector;
import java.io.Serializable;

public class BPackageModelBranch implements BTreeNode, Serializable, BSaveable
{
	static private BDataFile.ClassMap hack = new BDataFile.ClassMap("au.edu.swin.synergy.plugin.packager.BPackageModelBranch", "packageBranch");

	private Vector list = null;
	private BPackageModel parent = null;
	private String name = null;

	public BPackageModelBranch( BPackageModel parent, String name )
	{
		this.list = new Vector();
		this.parent = parent;
		this.name = name;
	}
	public BPackageModelBranch( )
	{
		this.list = new Vector();
	}

	public void addNode( BTreeNode item )
	{

		if ( item instanceof BDrawable )
		{
			item.setParent( this );
			this.list.add( item );


			Debug.doPrintln("Adding a new BDrawable!", Debug.kWarningLevel );

		}
		else
		{
			Debug.doPrintln("Trying to add a package item, that isn't a BDrawable!", Debug.kWarningLevel );
		}
	}

	public void addBGroupsToVector( Vector addList )
	{
		int i, size;
		BTreeNode curItem = null;

		size = this.list.size();
		for ( i = 0; i < size; i++ )
		{
			curItem = (BTreeNode)this.list.elementAt(i);
			addList.add( curItem );
			Debug.doPrintln("Adding to the list - " + curItem.getClass().getName(), Debug.kWarningLevel );
		}
	}

	/**
	 * Gets a node from the branch.
	 * @param String name - the name of the node..
	 * @return the first item that matches name..
	 * @return null if no item is found..
	 *
	 */
	public Object getNodeByName( String name )
	{
		Object result = null;

		int i, size;
		BTreeNode curItem = null;

		size = this.list.size();
		for ( i = 0; i < size && result == null; i++ )
		{
			curItem = (BTreeNode)this.list.elementAt(i);
			if ( curItem.getName().equals( name ) )
			{
				result = curItem;
			}

		}


		return result;
	}

	public String toString()
	{
		return this.name;
	}


///////BTreeNode stuff..
	public boolean isLeaf()
	{
		return this.list.size() <= 0;
	}

	public Object getChildAt( int index )
	{
		return this.list.elementAt( index );
	}

	public int getChildCount()
	{
		return this.list.size();
	}

	public Object getParent()
	{
		return this.parent;
	}

	public int getIndexOf( Object node )
	{
		return this.list.indexOf( node );
	}

	public void setParent( Object parent )
	{
		this.parent = (BPackageModel)parent;
	}

	public void removeAllChildren()
	{
		Debug.doPrintln("about to clear the children of " + this.name );
		this.list.clear();
	}

	public String getName()
	{
		return this.name;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	// for BSaveable
	protected static final String kKeyName = "name";
	protected static final String kDefaultName = "untitled";
	
	public boolean load(BDataFile file)
	{
		Debug.doPrintln("BPackageModelBranch load");

		this.name = file.getData( kKeyName, kDefaultName );
		while(true)
		{
			Object object = file.autoLoad();
			if ( object == null )
			{
				break;

			}
			else if ( object instanceof BDrawable )
			{

				list.add( (BDrawable) object );
				if ( object instanceof BTreeNode )
				{
					((BTreeNode)object).setParent( this );
				}
			}
		}
		return true;
	}

	public boolean save(BDataFile file)
	{
		Debug.doPrintln("BPackageModelBranch save");
		file.addData(kKeyName, name, kDefaultName);
		for( int i = 0; i < list.size(); i++ )
		{
			file.save( (BSaveable) list.get(i) );
		}

		return true;
	}

}
