//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/packager/BProperty.java,v 1.20 1999/10/15 05:06:02 tp079479 Exp $

package au.edu.swin.synergy.plugin.packager;

import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.plugin.diagram.BView;
import au.edu.swin.synergy.debug.Debug;

import au.edu.swin.synergy.lib.BTreeNode;
import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.lib.io.BSaveable;

import au.edu.swin.synergy.math.BScalarPoint;
import au.edu.swin.synergy.math.BScalarRect;
import au.edu.swin.synergy.math.BScalar;

import au.edu.swin.synergy.shapes.BTextArea;

import au.edu.swin.synergy.plugin.workspace.propertybag.BPropertyBagItem;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;


public class BProperty extends BDrawable implements BTreeNode, BSaveable, BPropertyBagItem
{
	private Object parent = null; //the parent in the tree...
	private Vector items = new Vector();
	private String tempValue;
	private BTextArea textArea = null;
	private BScalarPoint tmpPoint = null;

   private static int propertyNum = 1;

	public BProperty()
	{
		this( (String)null );
	}


	public BProperty( String name )
	{
		super();
		this.name = name;
		this.items.addElement(new String(name + propertyNum));
		this.tmpPoint = new BScalarPoint();
	}


	protected BProperty( BProperty prop )
	{
		this( prop.name );
		this.copy( prop );
	}

	
	public void addString(String item)
	{
		Debug.doPrintln ( "adding string to property." );
		items.addElement(item);
	}

	public void removeString(String item)
	{
		Debug.doPrintln ( "removing string from property." );
		items.removeElement(item);
	}

	public Object getPropertyValue(String name)
	{
		return items.elementAt(0);
	}

	public boolean isChangeAllowed (String name, Object oldValue, Object newValue)
	{
        String updatedValue = (String) newValue;

        if (updatedValue.length() > 0)
        {
			items.setElementAt(updatedValue, 0);
			Debug.doPrintln("changed value");
			if ( textArea != null )
			{
				textArea.setString ( updatedValue );
				Debug.doPrintln ( "updated correctly" );
			}
			else
			{
				Debug.doPrintln ( "WHY ARE WE STILL NULL" );
			}
          return true;
        }
        else
        {
          Debug.doPrintln("change not allowed");
          return false;
        }
	}


	/**
	 * Transforms the coordinates of the drawable object
	 * to a new coordinate space, as defined by the
	 * viewport and parent pos.
	 *
	 * @param BViewport viewport - the viewport that this object is going to be drawn in
	 * @param Point parentPos - coordinates of any parent object
	 */
	public void transform( BView viewport, BScalarPoint parentPos, int frameCounter )
	{

		if ( this.lastFrame != frameCounter )
		{
			this.tmpPoint.setLocation( this.pos );
			this.tmpPoint.incXY( parentPos );

			if ( this.textArea != null )
			{
				this.textArea.transform( viewport, tmpPoint, frameCounter );
			}

			this.lastFrame = frameCounter;
		}

	}

	/**
	 * Renders the transformed coordinates of the drawable
	 * object to a Graphics2D
	 *
	 * @param Graphics2D g2 - the place to render the object to.
	 */
	public void render( Graphics2D g2 )
	{		
		if ( this.textArea != null )
		{
			Debug.doPrintln ( "Rendering text area" );
			this.textArea.render( g2 );
		}
	}

	/**
	 * Creates and returns a new instance of the BDrawable object
	 *
	 * @return a new instance of this BDrawable object.
	 */
	public BDrawable newInstance()
	{
		return new BProperty( this );
	}


	/**
	 * Loads the BDrawable from a file
	 * @param BDataFile file - the file to load from
	 * @return boolean - true on success, false on failure.
	 */
	public boolean load(BDataFile file)
	{
		Debug.doPrintln("BProperty load");
		super.load(file);
		textArea = ( BTextArea ) file.autoLoad ();
		if ( textArea == null )
		{
			Debug.doPrintln ( "TEXT AREA IS NULL" );
		}
		else
		{
			Debug.doPrintln ( "NOT NULL, NOT NULL...WE HAVE SUCCEEDED for " + name );
		}
		return true;
	}

	protected static final String kKeyItem = "item";

	/**
	 * Saves the BDrawable to a file
	 * @param BDataFile file - the file to save to
	 * @return boolean - true on success, false on failure.
	 */
	public boolean save(BDataFile file)
	{
		super.save( file );
		Debug.doPrintln("BProperty save");
		file.save ( textArea );
		for(int i=0; i<items.size(); i++)
		{
			file.addData(kKeyItem, (String)items.get(i));
		}
		return true;
	}

	/**
	 * updates the bounding region of the drawable
	 * Called when something requires the bounds of
	 * the drawable to be updated.  Subclasses should
	 * determine a suitable way to update themselves.
	 */
	public void updateBounds()
	{
		textArea.updateBounds ();
	}

	public BScalarRect getBounds ()
	{
		return textArea.getBounds ();
	}

    /**
     * Please complete the missing tags for toString
     * @param
     * @return
     * @pre
     * @post
     */

    public String toString()
    {
        return this.name;
    }

	public void setColour( Color colour )
	{
	}

	public void copy( BProperty src )
	{
		int i, size;

		super.copy( src );

		this.parent = src.parent; //the parent in the tree...

		size = src.items.size ();
		for ( i = 0; i < size; i++ )
		{
			this.items.add ( new String ( ( String ) src.items.elementAt ( i ) ) );
		}

		if ( src.textArea != null )
		{
			this.textArea = new BTextArea ( src.textArea );
			if ( textArea == null )
			{
				Debug.doPrintln ( "after copy AREA is null" );
			}
			else
			{
				Debug.doPrintln ( "Everything seemed to go to plan during COPY" );
			}
		}
		else
		{
			Debug.doPrintln ( "SRC.TEXTAREA is NULL!" );
		}
	}

	public void setTextArea( BTextArea text )
	{
		Debug.doPrintln("setting the value of text to " + text );
		this.textArea = text;
		this.items = this.textArea.getRefToStrings();

	}

	public void addDrawable( BDrawable drawable )
	{
	}

///////////tree node stuff...

	public boolean isLeaf()
	{
		return true;
	}

	public Object getChildAt( int index )
	{
		return null;
	}

	public int getChildCount()
	{
		return 0;
	}

	public Object getParent()
	{
		return this.parent;
	}

	public int getIndexOf( Object node )
	{
		return -1;
	}

	public void setParent( Object parent )
	{
		Debug.doPrintln("Setting the parent of the property to " + parent );
		this.parent = parent;
	}

	public void removeAllChildren()
	{
	}

	public String getName()
	{
		return this.name;
	}

	/**
	 * Overridden from our super class for it's not the property which is
	 * drawable rather than the drawables contained to represent the data we
	 * hold.
	 *
	 * At the moment will only cater for text area until better hacks can be thought
	 * of.
	 */
	public boolean intersects( BScalarRect bounds )
	{
		boolean result = false;

		if ( textArea != null )
		{
			result = textArea.intersects ( bounds );	
		}

		return result;
	}

	public void setHighlighted( boolean on )
	{
		if ( textArea != null )
		{
			textArea.setHighlighted ( on );
		}
	}

	public void setSelected( boolean on )
	{
		if ( textArea != null )
		{
			textArea.setSelected ( on );
		}
	}

	/**
	 * sets a flag on or off
	 *
	 */
	public void setFlag( int flag, boolean on )
	{
		this.textArea.setFlag ( flag, on );
	}

	public boolean isStatusFlag( int flag )
	{
		return this.textArea.isStatusFlag ( flag );
	}

	public boolean isHighlighted()
	{
		return this.textArea.isHighlighted ();
	}

	public boolean isSelected()
	{		
		return this.textArea.isSelected ();
	}


	public int checkSizerHit( Point pt )
	{
		return this.textArea.checkSizerHit ( pt );
	}

	/**
	 * Checks to see if a point has hit the transformed bounds
	 * of this object
	 * @param Point pt - the point to check the hit against..
	 * @return kNotHit - not hit
	 * @return kHitBounds - hit the main bounds
	 */
	public int checkXFormBounds( Point pt )
	{
		int result = BDrawable.kNotHit;
		if ( textArea != null )
		{
			result = this.textArea.checkXFormBounds ( pt );
		}

		return result;
	}
// asdf

	/**
	 * called when the drawable is about to be moved..
	 *
	 */
	public void startMove()
	{
		textArea.startMove ();
	}

	/**
	 * Called as the drawable is being moved
	 *
	 */
	public void dragMove( BScalarPoint deltaXY )
	{
		this.pos.incXY ( deltaXY );
	}

	/**
	 * called when the drawable has finished being moved..
	 *
	 */
	public void stopMove()
	{
		this.textArea.updateBounds();
	}

	/**
	 * called when the drawable is about to be resized..
	 *
	 * @param int sizerNum - a sizerNum returned from checkSizerHit
	 */
	public void startResize( int sizerNum )
	{
		this.textArea.startResize ( sizerNum );
	}

	/**
	 * Called as the drawable is being resized
	 *
	 */
	public void dragResize( BScalarPoint deltaXY )
	{
		this.textArea.dragResize ( deltaXY );
	}

	/**
	 * called when the drawable has finished being resized..
	 *
	 */
	public void stopResize()
	{
		this.textArea.stopResize ();
	}



}
