// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/repository/BACL.java,v 1.9 1999/10/17 22:48:07 tp079541 Exp $

package au.edu.swin.synergy.plugin.repository;

import java.lang.String;
import java.lang.Integer;

import au.edu.swin.synergy.debug.Debug;

import java.io.Serializable;
import java.util.Vector;
import au.edu.swin.synergy.plugin.userdb.BUser;



/**
 * File Repository Security Descriptor
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 *
 * The descriptor information is saved in the meta data database
 */

public class BACL implements Serializable
{
	static final long serialVersionUID = -1289270489892600342L;

	int owner = 0;
	Vector acl = new Vector();


	public int getOwner()
	{
		return owner;
	}

	public void setOwner(int fOwner)
	{
		owner = fOwner;
	}

	public boolean addDescriptor(String descriptor)
	{
		// see if this is a valid descriptor
		// [ kUser | kGroup ] <id> '=' kAccess
		int realm;
		int id;
		int access;

		Debug.doPrintln("Parsing descriptor '" + descriptor + "'");

		if(descriptor.startsWith(BDescriptor.kUserString))
		{
			Debug.doPrintln("kUser");
			realm = BDescriptor.kUser;

			descriptor = descriptor.substring(BDescriptor.kUserString.length());
		}
		else if(descriptor.startsWith(BDescriptor.kGroupString))
		{
			Debug.doPrintln("kGroup");
			realm = BDescriptor.kGroup;

			descriptor = descriptor.substring(BDescriptor.kUserString.length());
		}
		else
		{
			Debug.doPrintln("Invalid security descriptor");
			return false;
		}

		// the next bit is the ID, up to the '='
		int p = descriptor.indexOf('=');
		if(p < 0 )
		{
			Debug.doPrintln("Invalid security descriptor.  (No '=')");
			return false;
		}
		else
		{
			id = Integer.parseInt(descriptor.substring(0, p));
			descriptor = descriptor.substring(p + 1); // skip over the '='
			Debug.doPrintln("ID = " + id);
		}
		Debug.doPrintln(descriptor);

		if(descriptor.equals(BDescriptor.kReadString))
		{
			access = BDescriptor.kRead;
		}
		else if(descriptor.equals(BDescriptor.kReadWriteString))
		{
			access = BDescriptor.kReadWrite;
		}
		else
		{
			Debug.doPrintln("Invalid security descriptor. (No access specifier)");
			return false;
		}

		// lets add a BDescriptor to the ACL
		BDescriptor desc = new BDescriptor(id, realm, access);
		acl.add(desc);

		return true;
	}

	public boolean removeDescriptor(String fDescriptor)
	{
		for(int i=0; i<getCount(); i++)
		{
			BDescriptor descriptor = getDescriptor(i);

			if(descriptor.toString().equals(fDescriptor))
			{
				acl.remove(i);
				return true;
			}
		}
		return false;
	}

	int getCount()
	{
		return acl.size();
	}

	BDescriptor getDescriptor(int index)
	{
		return (BDescriptor)acl.get(index);
	}

	public void clear()
	{
		acl.clear();
	}

	public boolean checkAccess(BUser user, int access)
	{
		Debug.doPrintln("Checking " + access + " access for " + user.getName());

		// the owner always has read/write access
		if(user.getUniqueID().intValue() == getOwner() && getOwner() != -1)
		{
			Debug.doPrintln(user.getName() + " is the owner");
			return true;
		}

		// check for the required permission in the acl
		Vector groups = user.getGroups();

		// check for acl permissions
		for(int i=0; i<getCount(); i++)
		{
			BDescriptor descriptor = getDescriptor(i);

			// is this a user descriptor
			if(descriptor.getRealm() == BDescriptor.kUser)
			{
				// is it for the specified user
				if(descriptor.getId() == user.getUniqueID().intValue())
				{
					// does it provide enough access
					if(descriptor.checkAccess(access))
					{
						// we allow it
						Debug.doPrintln("Access granted by user acl");
						return true;
					}
				}
			}
			else if(descriptor.getRealm() == BDescriptor.kGroup)
			{
				// is the user in this specified group?
				if(user.isInGroup(descriptor.getId()))
				{
					// does it provide enough access
					if(descriptor.checkAccess(access))
					{
						// we allow it
						Debug.doPrintln("Access granted by user groups");
						return true;
					}
					Debug.doPrintln("User is not a member of this group");
				}
			}
		}

		Debug.doPrintln("Access denied");

		return false;

	}

	public String[] getDescriptorsStrings()
	{
		String[] result = new String[ getCount() ];
		Debug.doPrintln( "Got " + getCount() + " Descriptors" );
		for( int i = 0; i < getCount(); i++)
		{
			result[i] = ((BDescriptor)acl.get(i)).toString();
			Debug.doPrintln( result[i] );
		}
		return result;
	}

	public Vector getDescriptors()
	{
		return acl;
	}

}
