// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/repository/BDescriptor.java,v 1.7 1999/10/17 01:16:03 tp079541 Exp $

package au.edu.swin.synergy.plugin.repository;

import java.lang.String;
import java.lang.Integer;

import au.edu.swin.synergy.debug.Debug;
import java.io.File;
import au.edu.swin.synergy.lib.BFile;


import au.edu.swin.synergy.lib.io.BTextdb;
import au.edu.swin.synergy.lib.io.BTextdbRecord;
import java.io.Serializable;


/**
 * File Repository Security Descriptor
 * the acl is a string like
 * u10=r which means user 10 has read access
 * g3=rw which means anyone in group 3 has read/write access
 * Assume no access, unless specificly granted
 *
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 *
 * The descriptor information is saved in the meta data database
 */

public class BDescriptor implements Serializable
{
	static final long serialVersionUID = 7621087892428540484L;

	/**
	 * Realm of users
	 */
	public static final int kUser = 1;

	/**
	 * Realm of groups
	 */
	public static final int kGroup = 2;

	/**
	 * Realm of users string used when saving to file
	 */
	public static final String kUserString = "u";

	/**
	 * Realm of groups string used when saving to file
	 */
	public static final String kGroupString = "g";

	/**
	 * Access denied
	 */
	public static final int kNone = 0;

	/**
	 * Read only access
	 */
	public static final int kRead = 1;

	/**
	 * Read/Write access
	 */
	public static final int kReadWrite = 2;

	/**
	 * Read only access string used when saving to file
	 */
	public static final String kReadString = "r";

	/**
	 * Read/Write access string used when saving to file
	 */
	public static final String kReadWriteString = "rw";

	private int id;
	private int realm;
	private int access;

	public BDescriptor(int fId, int fRealm, int fAccess)
	{
		id = fId;
		realm = fRealm;
		access = fAccess;
	}

	public int getRealm()
	{
		return realm;
	}

	public int getId()
	{
		return id;
	}

	public int getAccess()
	{
		return access;
	}

	public boolean checkAccess(int fAccess)
	{
		// see if the acl grants the requested access or higher
		if(fAccess >= access)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	public boolean checkPerm( int fAccess )
	{
		// see if the ACL has the requseted access set
		if(fAccess == access)
		{
			return true;
		}
		else
		{
			return false;
		}
	}


	public String toString()
	{
		String result = "";
		if(realm == kUser)
		{
			result = result + kUserString;
		}
		else if(realm == kGroup)
		{
			result = result + kGroupString;
		}
		else
		{
			return null;
		}
		result = result + id + "=";
		if(access == kRead)
		{
			result = result + kReadString;
		}
		else if(access == kReadWrite)
		{
			result = result + kReadWriteString;
		}
		else
		{
			return null;
		}
		return result;
	}
}
