// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/repository/BMetaData.java,v 1.10 1999/10/15 10:20:38 tp079541 Exp $

package au.edu.swin.synergy.plugin.repository;

import java.lang.String;
import java.lang.Integer;

import au.edu.swin.synergy.debug.Debug;
import java.io.File;
import au.edu.swin.synergy.lib.BFile;


import au.edu.swin.synergy.lib.io.BTextdb;
import au.edu.swin.synergy.lib.io.BTextdbRecord;
import java.io.Serializable;
import au.edu.swin.synergy.plugin.userdb.BUser;
import java.util.Vector;


/**
 * File Repository MetaData
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 *
 * Files are stored in a directory, under data
 * This is meta-data for each directory stored in the file meta.data
 * This is a BTextDB file, which stores filename to file info
 */

public class BMetaData implements Serializable
{
	public static final long serialVersionUID = 3818502838335410236L;

	String fileName = null;
	int version = 1;
	String type = null;
	BACL acl = new BACL();
	long date = 0;
	boolean server = true;

	public void setClientSide()
	{
		server = false;
	}

	public void setServerSide()
	{
		server = true;
	}

	public void setRecord(BTextdbRecord record)
	{
		fileName = record.getElement(0);
		version = Integer.parseInt(record.getElement(1));
		type = record.getElement(2);
		//acl = new BACL();
		acl.clear();

		acl.setOwner(Integer.parseInt(record.getElement(3)));

		if(server)
		{
			Debug.doPrintln("In BMetaData:setRecord:ServerSide");
			for(int i=4; i<record.getCount(); i++)
			{
				acl.addDescriptor(record.getElement(i));
			}
		}
		else
		{
			Debug.doPrintln("InBMetaData::getRecord:ClientSide");
			date = Long.parseLong(record.getElement(4));
		}
	}

	public BTextdbRecord getRecord()
	{
		BTextdbRecord record = new BTextdbRecord();
		record.addElement(fileName);
		record.addElement("" + version);
		record.addElement(type);
		record.addElement("" + acl.getOwner());

		if(server)
		{
			Debug.doPrintln("In BMetaData:getRecord:ServerSide");
			for(int i=0; i<acl.getCount(); i++)
			{
				record.addElement(acl.getDescriptor(i).toString());
			}
		}
		else
		{
			Debug.doPrintln("InBMetaData::getRecord:ClientSide");
			record.addElement("" + date);
		}

		return record;
	}

	public String getFileName()
	{
		return fileName;
	}

	public void setFileName(String fFileName)
	{
		fileName = fFileName;
	}

	public int getOwner()
	{
		return acl.getOwner();
	}

	public void setOwner(int fOwner)
	{
		acl.setOwner(fOwner);
	}

	public long getDate()
	{
		return date;
	}

	public void setDate(long fDate)
	{
		date = fDate;
	}

	public int getVersion()
	{
		return version;
	}

	/*public void setVersion(int fVersion)
	{
		version = fVersion;
	}*/

	public void incrementVersion()
	{
		version++;
	}

	public void setType(String fType)
	{
		type = fType;
	}

	public String getType()
	{
		return type;
	}

	public boolean checkAccess(BUser user, int access)
	{
		return acl.checkAccess(user, access);
	}

	public static boolean matchesFile(BTextdbRecord record, String fileName)
	{
		if(record.getElement(0).equals(fileName))
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	public BACL getACL()
	{
		return acl;
	}

	public void setACL(BACL fACL)
	{
		acl = fACL;
	}


}
