// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/repository/BRepository.java,v 1.34 1999/10/17 21:18:47 tp079541 Exp $

package au.edu.swin.synergy.plugin.repository;

import java.lang.String;
import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BBlockingComplete;
import au.edu.swin.synergy.kernel.event.BBootCE;

import au.edu.swin.synergy.debug.Debug;
import java.io.File;
import au.edu.swin.synergy.lib.BFile;

import au.edu.swin.synergy.plugin.repository.event.BRepositoryCE;
import au.edu.swin.synergy.plugin.repository.event.BFileXferCE;
import au.edu.swin.synergy.plugin.repository.event.BGetFileCE;
import au.edu.swin.synergy.plugin.repository.event.BGetFilesCE;
import au.edu.swin.synergy.plugin.repository.event.BPutFileCE;
import au.edu.swin.synergy.plugin.repository.event.BStatFileCE;
import au.edu.swin.synergy.plugin.repository.event.BStatDirCE;
import au.edu.swin.synergy.plugin.repository.event.BSetSecurityCE;
import au.edu.swin.synergy.plugin.repository.event.BRegisterTypeCE;

import au.edu.swin.synergy.plugin.userdb.event.BGetUsersCE;
import au.edu.swin.synergy.plugin.userdb.event.BGetGroupsCE;

import au.edu.swin.synergy.lib.io.BTextdb;
import au.edu.swin.synergy.lib.io.BTextdbRecord;
import au.edu.swin.synergy.lib.BStringList;
import java.util.Vector;
import java.util.Hashtable;

import au.edu.swin.synergy.plugin.server.event.BGetUserCE;
import au.edu.swin.synergy.plugin.userdb.BUser;
import au.edu.swin.synergy.plugin.userdb.BPrivileged;



/**
 * Server Side File Repository
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 *
 * Files are stored in a directory, under data
 * This is meta-data for each directory stored in the file meta.data
 * This is a BTextDB file, which stores filename to file info
 */

public class BRepository extends BPlugin
{
	File repositoryRoot = new File("data");

	private static final String kMetaFileName = "meta.data";

	private Hashtable registeredTypes = new Hashtable();

    public void setup(BStringList params)
    {
		if(!repositoryRoot.exists())
		{
			repositoryRoot.mkdir();
		}
    }

    public void onBCommandEvent( Object source, BCommandEvent event )
    {
		// do we need security...?
		boolean secure = true;
		BUser user = null;

		if(event instanceof BRepositoryCE)
		{
			// get the user ID, and any group that they belong to...
			Debug.doPrintln("netowrk status = " + event.getNetworkStatus());
			Debug.doPrintln("connection ID  = " + event.getConnectionID());
			if(event.getNetworkStatus() == BCommandEvent.kServer && event.getConnectionID() != -1)
			{
				BBlockingComplete onGetUser = new BBlockingComplete();
				BGetUserCE getUserEvent = new BGetUserCE(this, event.getConnectionID(), onGetUser);
				dispatchEvent(getUserEvent);

				onGetUser.waitForEvent();

				Debug.doPrintln("Got it");
				getUserEvent = (BGetUserCE)onGetUser.getEvent();
				user = getUserEvent.getUser();
				Debug.doPrintln("The winner is: " + user.getFullName());
				if(user.getPrivilege() == BPrivileged.kAdministrator)
				{
					// grant admin full access
					secure = false;
					Debug.doPrintln("Granting admin access");
				}
				else
				{
					Debug.doPrintln("Allowing access with uid = " + user.getUniqueID());
					secure = true;
				}
			}
			else
			{
				// can we handle anything else?
				Debug.doPrintln("This event did not come from the network...");
				// so no security is required...
				secure = false;
			}
		}

		// check file name for security problems...
		if(event instanceof BRepositoryCE)
		{
			BRepositoryCE repositoryEvent = (BRepositoryCE) event;
			String fileName = repositoryEvent.getFileName();
			if(fileName.indexOf("..") != -1)
			{
				Debug.doPrintln("Security risk in file name!");
				repositoryEvent.setError(BRepositoryCE.kErrorInvalidFileName);
				event.doComplete(BCommandComplete.kSuccess);
				return;
			}
		}

		if(event instanceof BStatDirCE)
		{
			BStatDirCE statEvent = (BStatDirCE) event;
			File theFile = makeFile(statEvent.getFileName());
			if( theFile.isDirectory())
			{
				BStringList files = new BStringList();
				enumerateDirectory(statEvent.getFileName(), files);
				for( int i = 0; i < files.getCount(); i++)
				{
					File aFile = makeFile( files.get(i) );
					BMetaData metaData = getMetaData( files.get(i) );
					if ( metaData != null )
					{
						File actualFile = new File( aFile.getPath() + "." + metaData.getVersion() );
						statEvent.addFile( files.get(i), metaData, actualFile.length(), actualFile.lastModified(), actualFile.isDirectory() );
						/*BGetGroupsCE getGroups = new BGetGroupsCE(this, null);
						BBlockingComplete complete = dispatchEventAndWait( getGroups );
						getGroups = (BGetGroupsCE) complete.getEvent();
						statEvent.setGroups( getGroups.getGroups() );
						BGetUsersCE getUsers = new BGetUsersCE( this, null);
						complete = dispatchEventAndWait( getUsers );
						getUsers = (BGetUsersCE) complete.getEvent();
						statEvent.setUsers( getUsers.getUsers() );//*/
					}
				}
			}
			else
			{
				statEvent.setError(BRepositoryCE.kErrorFileNotFound);
			}
			statEvent.doComplete(BCommandComplete.kSuccess);
		}

		// stat does not require security at the moment...
		if(event instanceof BStatFileCE)
		{
			try
			{

				// get the meta-data for the requested file
				BStatFileCE statEvent = (BStatFileCE) event;
				Debug.doPrintln("Got BStatFileCE event, filename = " + statEvent.getFileName());
				File theFile = makeFile(statEvent.getFileName());

				if(theFile.isDirectory())
				{
					statEvent.setDirectory(true);
					// get the file names...
					// from the meta.data.... (too hard at the moment...)
					BMetaData metaData = getMetaData( statEvent.getFileName() );
					if( metaData != null ){ Debug.doPrintln( "Got Dir MetaData" );}
					BStringList files = new BStringList();
					enumerateDirectory(statEvent.getFileName(), files);
					statEvent.setFiles(files);
				}
				else
				{
					BMetaData metaData = getMetaData(statEvent.getFileName());
					if(metaData != null)
					{
						statEvent.setMetaData(metaData);

						// we need to get the latest file version name....
						File actualFile = new File(theFile.getPath() + "." + metaData.getVersion());
						statEvent.setSize(actualFile.length());
						statEvent.setDate(actualFile.lastModified());
						BGetGroupsCE getGroups = new BGetGroupsCE(this, null);
						BBlockingComplete complete = dispatchEventAndWait( getGroups );
						getGroups = (BGetGroupsCE) complete.getEvent();
						statEvent.setGroups( getGroups.getGroups() );
						BGetUsersCE getUsers = new BGetUsersCE( this, null);
						complete = dispatchEventAndWait( getUsers );
						getUsers = (BGetUsersCE) complete.getEvent();
						statEvent.setUsers( getUsers.getUsers() );
						// owner and type need meta-data
					}
					else
					{
						statEvent.setError(BRepositoryCE.kErrorFileNotFound);
					}
				}
				statEvent.doComplete(BCommandComplete.kSuccess);
			}
			catch(Exception E)
			{
				Debug.doPrintln("Exception in stat file", E);
				event.doComplete(BCommandComplete.kError);
			}
		}
		else if(event instanceof BGetFileCE)
		{
			// get needs read access...
			BGetFileCE getEvent = (BGetFileCE) event;
			Debug.doPrintln("Got BGetFileCE for " + getEvent.getFileName() );
			// see what version of the file to get
			// -1 means the latest.

			BMetaData metaData = getMetaData(getEvent.getFileName());
			if(metaData != null)
			{
				boolean proceed = false;
				if(secure)
				{
					// check security...
					if(metaData.checkAccess(user, BDescriptor.kRead))
					{
						proceed = true;
					}
				}
				else
				{
					proceed = true;
				}

				if(proceed)
				{
					// check the version
					int versionToGet = getEvent.getVersion();
					if(versionToGet == -1)
					{
						versionToGet = metaData.getVersion();
					}
					if(versionToGet > 0 && versionToGet <= metaData.getVersion())
					{
						File theFile = makeFile(getEvent.getFileName() + "." + versionToGet);
						if(theFile.exists() && theFile.isFile())
						{
							BFile theData = new BFile(theFile, BFile.kDefaultCompressionRate);
							theData.compress();
							getEvent.setData(theData);
							getEvent.setMetaData(metaData);
						}
						else
						{
							// The specifies version of this file does not exits (on disk);
							getEvent.setError(BRepositoryCE.kErrorSpecifiedVersionDoesNotExist);
						}
					}
					else
					{
						// The specified version of this <i>should</i> does not exist
						getEvent.setError(BRepositoryCE.kErrorSpecifiedVersionShouldNotExist);
					}
				}
				else
				{
					getEvent.setError(BRepositoryCE.kErrorAccessDenied); // access denied
				}
			}
			else
			{
				getEvent.setError(BRepositoryCE.kErrorFileNotFound); // file does not exist
			}


			getEvent.doComplete(BCommandComplete.kSuccess);

		}
		else if(event instanceof BGetFilesCE)
		{
			// get needs read access...
			BGetFilesCE getEvent = (BGetFilesCE) event;
			Debug.doPrintln("Got BGetFilesCE for " + getEvent.getFileName() );

			// see what version of the file to get
			// -1 means the latest.
			//BMetaData metaData = getMetaData(getEvent.getFileName());
			//if(metaData != null)
			if(true)
			{
				/**boolean proceed = false;
				if(secure)
				{
					// check security...
					if(metaData.checkAccess(user, BDescriptor.kRead))
					{
						proceed = true;
					}
				}
				else
				{
					proceed = true;
				}
				//*/
				//if(proceed)
				if(true)
				{
					getTree( getEvent );
					/*for( int i = 0; i < getEvent.getFileCount(); i++)
					{
						// check the version
						int versionToGet = getEvent.getVersion(i);
						if(versionToGet == -1)
						{
							versionToGet = metaData.getVersion();
						}
						if(versionToGet > 0 && versionToGet <= metaData.getVersion())
						{
							File theFile = makeFile(getEvent.getFileName() + "." + versionToGet);
							if(theFile.exists() && theFile.isFile())
							{
								BFile theData = new BFile(theFile, BFile.kDefaultCompressionRate);
								theData.compress();
								getEvent.setData(theData);
								getEvent.setMetaData(metaData);
							}
							else
							{
								// The specifies version of this file does not exits (on disk);
								getEvent.setError(BRepositoryCE.kErrorSpecifiedVersionDoesNotExist);
							}
						}
						else
						{
							// The specified version of this <i>should</i> does not exist
							getEvent.setError(BRepositoryCE.kErrorSpecifiedVersionShouldNotExist);
						}
					}//for//*/
				}
				else
				{
					getEvent.setError(BRepositoryCE.kErrorAccessDenied); // access denied
				}
			}
			else
			{
				getEvent.setError(BRepositoryCE.kErrorFileNotFound); // file does not exist
			}


			getEvent.doComplete(BCommandComplete.kSuccess);

		}
		else if(event instanceof BPutFileCE)
		{
			BPutFileCE putEvent = (BPutFileCE) event;
			// does the file exist on this end?

			BMetaData metaData = getMetaData(putEvent.getFileName());
			if(metaData == null)
			{
				boolean proceed = false;
				boolean abort = false;

				// the file does not exist on this end...

				if(checkDirectory(putEvent.getFileName()))
				{
					Debug.doPrintln("The directory exists");
				}
				else
				{
					Debug.doPrintln("The directory does not exist");
					Debug.doPrintln("About to make directory " + getFilePath(putEvent.getFileName()) + " for file " + putEvent.getFileName());

					if(makeDirectory(getFilePath(putEvent.getFileName()), user))
					{
						Debug.doPrintln("Directory created!");
					}
					else
					{
						Debug.doPrintln("Could not create directory");
						abort = true;
					}
				}

				if(abort == false)
				{
					// check the directory acl for write permissions
					///????
					File theFile = makeFile(putEvent.getFileName());

					String path = getFilePath(putEvent.getFileName());
					BMetaData pathMetaData = null;
					if ( path != null )
					{
						Debug.doPrintln("path = " + path);
						if ( path.equals(".") )
						{
							path = null;
							Debug.doPrintln("Path set to null");
						}
					}
					if(path != null)
					{
						Debug.doPrintln("Looking up metadata for directory " + path);
						pathMetaData = getMetaData(path);
						if(pathMetaData == null)
						{
							Debug.doPrintln("Parent directory does not exit"); // what now...
						}
						else
						{
							if(user.getPrivilege() == BUser.kAdministrator)
							{
								proceed = true;
							}
							else if(pathMetaData.checkAccess(user, BDescriptor.kReadWrite))
							{
								proceed = true;
							}
						}
					}
					else
					{
						Debug.doPrintln("Root directory, admin access only");
						if(user.getPrivilege() == BUser.kAdministrator)
						{
							proceed = true;
						}
					}

					if(proceed)
					{
						metaData = new BMetaData();
						metaData.setFileName(theFile.getName());
						// if the file extention is a registered type, then change the type
						metaData.setType("t"); // these don't mean anything yet

						int extension = theFile.getName().lastIndexOf(".");
						if(extension > -1)
						{
							String ext = theFile.getName().substring(extension + 1);
							Debug.doPrintln("Setting file type to " + ext);
							metaData.setType(ext);
						}



						metaData.setOwner(user.getUniqueID().intValue());

						// make sure that the directory exists
						File actualFile = makeFile(putEvent.getFileName() + "." + metaData.getVersion());

						BFile theData = putEvent.getData();
						theData.expand(actualFile);

						// clear the data so it is not sent back to the client
						putEvent.setData(null);

						writeNewMetaData(putEvent.getFileName(), metaData);
					}
					else
					{
						// error
						putEvent.setError(BRepositoryCE.kErrorAccessDenied);
					}
				}
				else
				{
					Debug.doPrintln("Put aborted");
					putEvent.setError(BRepositoryCE.kErrorInvalidFileName);
				}
			}
			else
			{
				boolean proceed = false;
				if(secure)
				{
					// check security...
					if(metaData.checkAccess(user, BDescriptor.kReadWrite))
					{
						proceed = true;
					}
				}
				else
				{
					proceed = true;
				}

				if(proceed)
				{
					// see if we need to do a merge...
					Debug.doPrintln("Current Version " + metaData.getVersion());
					if( putEvent.getMetaData() == null)
					{
						Debug.doPrintln("put Event meta data is null");
					}
					Debug.doPrintln("Check in Version " + putEvent.getMetaData().getVersion());
					if(metaData.getVersion() == putEvent.getMetaData().getVersion())
					{
						Debug.doPrintln("Version check OK");
						metaData.incrementVersion();
						int newVersion = metaData.getVersion();
						File theFile = makeFile(putEvent.getFileName() + "." + newVersion);

						BFile theData = putEvent.getData();
						if(theData.expand(theFile))
						{
							// the client needs the updated metadata
							Debug.doPrintln("Assigning new metadata");
							putEvent.setMetaData(metaData);
							// write the new metadata
							writeMetaData(putEvent.getFileName(), metaData);
						}
						else
						{
							putEvent.setError(BRepositoryCE.kErrorFileCorrupt);
						}
					}
					else
					{
						Debug.doPrintln("Need to merge version " + putEvent.getMetaData().getVersion() + " to " + metaData.getVersion());
						BRegisterTypeCE typeHandler = (BRegisterTypeCE) registeredTypes.get(metaData.getType());
						if(typeHandler == null)
						{
							Debug.doPrintln("No way to merge this file type");
						}
						else
						{
							Debug.doPrintln("Build merge");
							Vector files = new Vector();

							for(int i=putEvent.getMetaData().getVersion(); i <= metaData.getVersion(); i++)
							{
								File fromVersion = makeFile(putEvent.getFileName() + "." + i);
								files.add(fromVersion.getPath());
							}
							File toVersion = makeFile(putEvent.getFileName() + ".merge");

							BFile theData = putEvent.getData();
							if(theData.expand(toVersion))
							{
								int result = typeHandler.getConflictDetector().doFindConflicts( files, toVersion.getPath());

								if(result != 0)
								{
									// just move the file to the next version
									metaData.incrementVersion();
									int newVersion = metaData.getVersion();
									File theFile = makeFile(putEvent.getFileName() + "." + newVersion);
									toVersion.renameTo(theFile);

									// send the meta-data back
									putEvent.setMetaData(metaData);

									// send the file back...
									theData = new BFile(theFile, BFile.kDefaultCompressionRate);
									theData.compress();
									putEvent.setData(theData);
									// write the new metadata
									writeMetaData(putEvent.getFileName(), metaData);


								}
								else
								{
									putEvent.setMetaData(metaData);

									// send the file back...
									theData = new BFile(toVersion, BFile.kDefaultCompressionRate);
									theData.compress();
									putEvent.setData(theData);
									putEvent.setError(BRepositoryCE.kErrorConflict);
									toVersion.delete();
								}
							}
							else
							{
								putEvent.setError(BRepositoryCE.kErrorFileCorrupt);
							}

						}
					}
				}
				else
				{
					putEvent.setError(BRepositoryCE.kErrorAccessDenied);
				}
			}

			putEvent.doComplete(BCommandComplete.kSuccess);
		}
		else if(event instanceof BSetSecurityCE)
		{
			boolean proceed = false;
			BSetSecurityCE securityEvent = (BSetSecurityCE) event;
			// does the file exist on this end?

			BMetaData metaData = getMetaData(securityEvent.getFileName());
			if(metaData == null)
			{
				securityEvent.setError(BRepositoryCE.kErrorFileNotFound);
			}
			else
			{
				// file exitst
				// allow access to owner only
				if(secure)
				{
					// check security...
					if(metaData.getOwner() == user.getUniqueID().intValue())
					{
						proceed = true;
					}
				}
				else
				{
					proceed = true;
				}

				if(proceed)
				{
					Debug.doPrintln("Updating ACL");
					metaData.setACL(securityEvent.getMetaData().getACL());
					// write the metadata back to file...
					writeMetaData(securityEvent.getFileName(), metaData);
				}
				else
				{
					Debug.doPrintln("Access denied");
					securityEvent.setError(BRepositoryCE.kErrorAccessDenied);
				}
			}
			securityEvent.doComplete(BCommandComplete.kSuccess);

		}

		if(event instanceof BRegisterTypeCE)
		{
			BRegisterTypeCE registerEvent = (BRegisterTypeCE) event;
			registeredTypes.put(registerEvent.getType(), registerEvent);
			registerEvent.doComplete(BCommandComplete.kSuccess);
		}
    }

	private File makeFile(String fileName)
	{
		Debug.doPrintln("BRepository:makeFile:fileName = " + fileName);
		if( fileName.equals("." + File.separator) )
		{
			fileName = ".";
		}
		Debug.doPrintln("In makeFile: makeing file " + repositoryRoot.getPath() +       File.separator + fileName);
		return new File(repositoryRoot.getPath() +      File.separator + fileName);
	}

	private String makeDir(String dirName)
	{
		Debug.doPrintln("IN BRepository:makeDir dir name = " + dirName);
		if ( dirName.lastIndexOf( File.separator ) != -1 )
		{
			return dirName.substring( 0, dirName.lastIndexOf( File.separator ) );
		}
		return dirName;
	}

	private String getFilePath(String fileName)
	{
		int i = fileName.lastIndexOf('/');
		if(i == -1)
		{
			i = fileName.lastIndexOf('\\');
		}
		if(i == -1)
		{
			return null;
		}
		return fileName.substring(0, i);
	}

	private boolean checkDirectory(String fileName)
	{
		// return true if the directory exists
		File theFile = makeFile(fileName);
		if(theFile.isDirectory())
		{
			return true;
		}
		else
		{
			File parentFile = theFile.getParentFile();
			if(parentFile.isDirectory())
			{
				return true;
			}
			else
			{
				return false;
			}
		}
	}

	/**
	 * Checks to see if a file is a directory
	 *
	 * @param dirName the name of the file to be checked
	 */
	private boolean isDir( String dirName )
	{
		File dirFile = makeFile( dirName );
		if( dirFile.isDirectory() )
		{
			Debug.doPrintln("directory " + dirName + " exists");
			return true;
		}
		Debug.doPrintln("directory " + dirName + " does not exist");
		return false;
	}

	private boolean makeDirectory(String directoryName, BUser user)
	{
		Debug.doPrintln( "In BRepository:makeDirectory:directoryName: " + directoryName );
		// we need to make directories that don't exist
		// first checking if we are permitted
		// then adding meta.data for the new directory

		boolean proceed = false;

		// is the parent the root, or does it exist...?
		String parent = getFilePath(directoryName);
		if(parent == null || parent == ".")
		{
			// this is the root...
			Debug.doPrintln( "makeDirectory : parent is root" );
			if(user.getPrivilege() == BUser.kAdministrator)
			{
				proceed = true;
			}
		}
		else
		{
			// we have a parent, does it exist?
			if(isDir(parent))
			{
				Debug.doPrintln("makeDirectory : parent exists\n About to check access on" + directoryName);
				// we are OK to create...
				// if the acl permits

				BMetaData metaData = getMetaData(directoryName);
				if( metaData != null )
				{

					if(metaData.checkAccess(user, BDescriptor.kReadWrite))
					{
						proceed = true;
					}
				}
				else
				{
					proceed = true;
				}
			}
			else
			{
				Debug.doPrintln("makeDirectory : parent does not exist");
				if ( makeDirectory(parent, user))
				{
					proceed = true;
				}
			}
		}
		if(proceed)
		{
			File theFile = makeFile(directoryName);
			Debug.doPrintln("Made File object for " + theFile.getAbsolutePath());
			theFile.mkdir();
			// add the metadata...
			BMetaData metaData = new BMetaData();
			metaData.setOwner(user.getUniqueID().intValue());

			metaData.setFileName(theFile.getName());
			metaData.setType("d");

			writeNewMetaData(directoryName, metaData);

			return true;
		}
		else
		{
			return false;
		}
	}

	BMetaData getMetaData(String fileName)
	{
		// find the directory that the file is in
		File theFile = makeFile(fileName);
		Debug.doPrintln("makeFile returned " + theFile);

		String directory = theFile.getParentFile().getPath();

		Debug.doPrintln("getMetaData:direcotry is " + directory);
		String fileNamePart = theFile.getName();

		String metaFileName = directory + File.separator + kMetaFileName;
		Debug.doPrintln("In getMetaData:meta file = " + metaFileName);
		BTextdb db = new BTextdb(metaFileName);
		db.openFileForRead();
		try
		{
			while(true)
			{
				BTextdbRecord record = db.getRecord();
				if(record == null)
				{
					Debug.doPrintln("no more records");
					break;
				}
				Debug.doPrintln("Reading record");
				if(BMetaData.matchesFile(record, fileNamePart))
				{
					Debug.doPrintln("Metadata found");
					BMetaData metaData = new BMetaData();
					metaData.setRecord(record);
					return metaData;
				}
			}
		}
		finally
		{
			db.close();
		}
		return null;
	}

	boolean writeMetaData(String fileName, BMetaData fMetaData)
	{
		Debug.doPrintln("Writing metadata for " + fileName);
		// find the directory that the file is in
		File theFile = makeFile(fileName);
		String directory = theFile.getParentFile().getPath();
		String fileNamePart = theFile.getName();

		String metaFileName = directory + File.separator + kMetaFileName;
		Debug.doPrintln("In writeMetaData:meta file = " + metaFileName);

		Vector metaRecords = new Vector();

		BTextdb db = new BTextdb(metaFileName);
		db.openFileForRead();
		try
		{
			while(true)
			{
				BTextdbRecord record = db.getRecord();
				if(record == null)
				{
					Debug.doPrintln("no more records");
					break;
				}
				Debug.doPrintln("Reading record");
				if(BMetaData.matchesFile(record, fileNamePart))
				{
					Debug.doPrintln("Metadata found");
					metaRecords.add(fMetaData.getRecord());
				}
				else
				{
					metaRecords.add(record);
				}
			}
		}
		finally
		{
			db.close();
		}

		// now re-write the meta-data
		db.openFileForWrite(false);
		try
		{
			for(int i=0; i<metaRecords.size(); i++)
			{
//                              BMetaData metaData = (BMetaData)metaRecords.get(i);
				BTextdbRecord record = (BTextdbRecord)metaRecords.get(i);
				db.addRecord(record);
			}
		}
		finally
		{
			db.close();
		}
		return false;
	}

	boolean writeNewMetaData(String fileName, BMetaData fMetaData)
	{
		Debug.doPrintln("Writing new metadata for " + fileName);
		// find the directory that the file is in
		File theFile = makeFile(fileName);
		String directory = theFile.getParentFile().getPath();
		String fileNamePart = theFile.getName();

		String metaFileName = directory + File.separator + kMetaFileName;
		Debug.doPrintln("In writeNewMetaData:meta file = " + metaFileName);

		BTextdb db = new BTextdb(metaFileName);
		db.openFileForWrite(true);
		try
		{
			db.addRecord(fMetaData.getRecord());
		}
		finally
		{
			db.close();
		}
		return true;
	}

	boolean enumerateDirectory(String dirName, BStringList files)
	{
		Debug.doPrintln("BRepository:enumerateDirectory:dirName : " + dirName);
		String metaFileName = makeFile(dirName).getPath() + File.separator + kMetaFileName;
		Debug.doPrintln("In enumerateDirectory:meta file = " + metaFileName);
		BTextdb db = new BTextdb(metaFileName);
		db.openFileForRead();
		try
		{
			while(true)
			{
				BTextdbRecord record = db.getRecord();
				if(record == null)
				{
					Debug.doPrintln("no more records");
					break;
				}
				Debug.doPrintln("Reading record");
				BMetaData metaData = new BMetaData();
				metaData.setRecord(record);
				files.add(strip(dirName) + File.separator + metaData.getFileName());
				Debug.doPrintln("BRepository:enumerateDirectory:added File : " + strip(dirName) + File.separator + metaData.getFileName());
			}
		}
		finally
		{
			db.close();
		}
		return true;
	}


	/**
	 * Strips the last \. off direcotry names
	 *
	 * @param dirName the name of the directory to strip
	 */
	private String strip( String dirName )
	{
		Debug.doPrintln("strip :" + dirName);
		int index = dirName.indexOf( File.separator );
		if(index >= 0)
		{
			return dirName.substring(dirName.indexOf( File.separator ), dirName.length() );
		}
		else
		{
			return dirName;
		}
	}

	private void getTree( BGetFilesCE getEvent )
	{
		Debug.doPrintln("In BRepository:getTree:");
		BMetaData metaData = getMetaData(getEvent.getFileName());
		int versionToGet = -1;
		Debug.doPrintln("In getTree: version to get = " + versionToGet);
		if(versionToGet == -1)
		{
			versionToGet = metaData.getVersion();
		}
		if(versionToGet > 0 && versionToGet <= metaData.getVersion())
		{
			//File theFile = makeFile(getEvent.getFileName());
			if(metaData.getType().equals("d") )
			{
				//traverserse directory structure
				getFilesInTree( getEvent.getFileName(), getEvent );
			}
			else
			{
				File theFile = makeFile(getEvent.getFileName() + "." + versionToGet);
				if( theFile.exists() && theFile.isFile() )
				{
					BFile theData = new BFile(theFile, BFile.kDefaultCompressionRate);
					theData.compress();
					//getEvent.setData(theData);
					getEvent.addData(theData, metaData, getEvent.getFileName());
				}
				else
				{
					// The specifies version of this file does not exits (on disk);
					getEvent.setError(BRepositoryCE.kErrorSpecifiedVersionDoesNotExist);
				}
			}//*/

		}
		else
		{
			// The specified version of this <i>should</i> does not exist
			getEvent.setError(BRepositoryCE.kErrorSpecifiedVersionShouldNotExist);
		}
	}

	private boolean getFilesInTree( String dirName, BGetFilesCE event )
	{
		BStringList files = new BStringList();
		Debug.doPrintln( "In getFilesInTree:dir Name = " + dirName );
		enumerateDirectory( dirName, files );
		for( int i = 0; i < files.getCount(); i++)
		{
			BMetaData metaData = getMetaData(files.get(i));
			//File theFile = makeFile(files.get(i));
			Debug.doPrintln("getFilesInTree:file is dir: " + metaData.getType().equals("d"));
			if( metaData.getType().equals("d") )
			{
				Debug.doPrintln("In getFilesInTree:file " + metaData.getFileName() + " is a dir");
				getFilesInTree( files.get(i), event );
			}
			else
			{
				int versionToGet = metaData.getVersion();
				File theFile = makeFile(files.get(i) + "." + versionToGet);
				if( theFile.exists() && theFile.isFile() )
				{
					BFile theData = new BFile(theFile, BFile.kDefaultCompressionRate);
					theData.compress();
					//getEvent.setData(theData);
					event.addData(theData, metaData, files.get(i));
					Debug.doPrintln("In getFilesInTree:file " + theFile.getName() + " has been added to the event");
				}
				else
				{

					// The specifies version of this file does not exits (on disk);
					event.addError(BRepositoryCE.kErrorSpecifiedVersionDoesNotExist);
				}
			}
		}
		return true;
	}



}
