// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/rooms/BRoom.java,v 1.12 1999/08/28 10:20:24 tp079519 Exp $
/**
 * Class created to represent a place of conversation between
 * one or more users logged into the system.
 *
 * @author John Newbigin
 * @author Christian Karolos
 * @copyright
 */
   package au.edu.swin.synergy.plugin.rooms;

import java.util.Vector;
import java.lang.String;
import java.io.Serializable;

import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.debug.Debug;

public class BRoom implements Serializable
{
    String roomName = "Untitled";
    
	/**
	* Doors to other rooms.
	*/
    BStringList doors = new BStringList();

    /**
     * Names of files currently held by this room.
     */
      BStringList files = new BStringList();
   
    /**
     * Users currently logged into this room.  Thrown in here
     * so list of users also returned on BGetRoomCE.
     */
    Vector users = new Vector();
    
	/**
     * Default constructor.
     */    
    public BRoom()
    {
    }

    /**
     * Get the name of the room.
     * @return Name of this room.
     * @pre Room exists.
     */    
    public String getName()
    {
        return roomName;
    }

    /**
     * Set the name of this room.
     * @param fName String to name room.
     * @pre Room exists.
     * @post Room has been named.
     */    
    public void setName( String fName )
    {
        roomName = fName;
    }

    /**
     * Get ajoining rooms.
     * @return List of available doors.
	 * @pre Doors have been created to other rooms.
     */
    public BStringList getDoors()
    {
        return doors;
    }

    /**
     * Get files which are contained in this room.
     * @return List of files stored in this room.
	 * @pre Files have been stored.
     */    
    public BStringList getFiles()
    {
        return files;
    }

    /**
     * Get users currently chatting in this room.
     * @return List of users in this room.
     * @pre Room exists and is occupied by 1 < users.
     */    
    public Vector getUsers()
    {
        return users;
    }

    /**
     * Add a door to another room.
     * @param door Additional door to add.
     * @pre Room has been instanciated.
     * @post Room contains new door.
     */    
    public void addDoor( String door )
    {
        doors.add( door );
    }

    /**
     * Add a user to this room.
     * @param user New user to join chat.
     * @pre Room exists.
     * @post User added to room.
     */
    public void addUser( BChatUser user )
    {
        users.add( user );
    }

    /**
     * Remove user from list of users.
     * @param connectionId Id for user wishing to leave.
     * @pre User exists in room.
     * @post User no longer exists in room.
     */
    public void removeUser( int connectionId )
    {
        boolean found = false;
        BChatUser chatUser;
        int index;
        
        Debug.doPrintln( "Searching for user with connId = " + connectionId );
        for( index = 0; index < users.size() && ! found; index++ )
        {
            chatUser = ( BChatUser ) users.elementAt( index );
         	Debug.doPrintln("BRoom user: " + index + chatUser);
            if( found = chatUser.getConnectionID() == connectionId )
            {
               Debug.doPrintln( "User found! Index = " + index );
            }
         }
      
         if( found )
         {
            // Putting this in coz might suffer by "out-by-one".
            Debug.doPrintln( "Removing user of index = " + ( index - 1 ) );
            users.remove( index - 1 );
        }
    }

    /**
     * Add file to room.
     * @param fileName File for room to own.
     * @pre File has been saved on server.
     */    
    public void addFile( String fileName )
    {
        files.add( fileName );
    }

    /**
     * Edit user residing in room.
     * @param oldUser Old details of user.
	 * @param newUser New details of user.
     * @pre User exists in room.
     * @post User contains new details.
     */    
    public void editUser( BChatUser oldUser, BChatUser newUser )
    {
        Debug.doPrintln( "BRoom:editUser:oldUser: " + oldUser + " newUser: " + newUser );
        Debug.doPrintln( "Index: " + users.indexOf( oldUser ) );
        users.set( users.indexOf( oldUser ), newUser );
    }    
}
