/**
 *
 * @see
 * @concurrency
 * @rref
 */

// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/rooms/BRooms.java,v 1.40 1999/09/07 08:16:05 tp079519 Exp $

   package au.edu.swin.synergy.plugin.rooms;

   import java.lang.String;
   import au.edu.swin.synergy.kernel.BPlugin;
   import au.edu.swin.synergy.kernel.BCommandEvent;
   import au.edu.swin.synergy.kernel.event.BBootCE;

   import au.edu.swin.synergy.plugin.userdb.BInvalidPropertyException;

   import au.edu.swin.synergy.kernel.BKernel;

   import java.util.Hashtable;
   import java.util.Vector;
   import java.util.Enumeration;
   import java.io.File;
   import javax.swing.ImageIcon;
   import java.awt.MediaTracker;
   import java.awt.Toolkit;
   import java.awt.Image;
import au.edu.swin.synergy.lib.BStringList;


   import au.edu.swin.synergy.plugin.rooms.event.BCreateRoomCE;
   import au.edu.swin.synergy.plugin.rooms.event.BGetRoomsCE;
   import au.edu.swin.synergy.plugin.rooms.event.BGetRoomCE;
   import au.edu.swin.synergy.plugin.rooms.event.BEnterRoomCE;
   import au.edu.swin.synergy.plugin.rooms.event.BAddUserCE;
   import au.edu.swin.synergy.plugin.rooms.event.BEditUserCE;
   import au.edu.swin.synergy.plugin.rooms.event.BRemoveUserCE;
   import au.edu.swin.synergy.plugin.rooms.event.BChatMsgCE;
   import au.edu.swin.synergy.plugin.rooms.event.BChatPrivCE;
   import au.edu.swin.synergy.plugin.rooms.event.BRoomChangeCE;
   import au.edu.swin.synergy.plugin.rooms.event.BGetFileCE;
   import au.edu.swin.synergy.plugin.rooms.event.BSendFileCE;
   import au.edu.swin.synergy.plugin.rooms.event.BGetThumbNailCE;
   import au.edu.swin.synergy.plugin.rooms.event.BKickCE;

   import au.edu.swin.synergy.kernel.BCommandComplete;
   import au.edu.swin.synergy.debug.Debug;
   import au.edu.swin.synergy.lib.BFile;
   import au.edu.swin.synergy.plugin.command.event.BRegisterCmdCE;



/**
 * Class taking care of all server side operations.
 * @Author John Newbigin
 * @author Christian Karolos
 * @author Shane Fitzgibbon
 * @Copyright Synergy Software 1999
 */

   public class BRooms extends BPlugin
   {
     /**
     * Name for default room.
     */
      public static final String kDefault = "Court Yard";
   
     /**
     * Maintains all rooms created.
     */
      Hashtable rooms = new Hashtable();
   
     /**
     * Maintains all users logged onto system.
     */
      Vector users = new Vector();
   
     /**
     * Default room.
     */
      BRoom defaultRoom;
   
      BStats stats = new BStats();
   
     /**
     * Height and width of thumbnail images sent to clients.
     */
      private final int thumbwidth = 90;
      private final int thumbheight = 90;
   
      public void setup(BStringList params)
      {
         File file = new File( "files" );
         if (!file.exists())
         {	
            file.mkdir();
         }
      }
    /**
     * Return rooms containing users, files and existing doors.
     * @return Vector containing room info.
     */   
      public Vector getRooms()
      {
         Vector rooms = new Vector();
         Enumeration roomEnumeration = null;
      
        //Get the enumeration
      
         synchronized( rooms )
         {
            roomEnumeration = this.rooms.elements();
         
            //Put all the elements in the database in the vector
            while( roomEnumeration.hasMoreElements() )
            {
               rooms.addElement( roomEnumeration.nextElement() );
            }
         }
      
         return rooms;
      }
   
    /**
     * Get a particular room.
     * @param name Name representing room.
     * @return Requested room.
     * @pre Room has been registered with server.
     */   
      public BRoom getRoom( String name )
      {
         Object o = null;
         synchronized( rooms )
         {
            o = rooms.get( name );
         }
         return( BRoom ) o;
      }
   
    /**
     * Gets room through the connection ID of user.
     * @param id Connection ID.
     * @return Requested room.
     * @pre Room has been registered with server.
     */   
      public BRoom getRoomByID( int id )
      {
         Debug.doPrintln( "Looking for id = " + id );
         synchronized( users )
         {
            for( int i = 0; i < users.size(); i++ )
            {
               BChatUser user = getUser( i );
               if( user.getConnectionID() == id )
               {
                  Debug.doPrintln( "Found id = " + id );
                  return user.getCurrentRoom();
               }
            }
         }
         return null;
      }
   
    /**
     * Returns user corresponding to connection ID
     * @param id Connection ID.
     * @return Requested user.
     * @pre User has logged onto the system.
     */
   
      public BChatUser getUserByID( int id )
      {
         Debug.doPrintln( "Looking for id = " + id );
         synchronized( users )
         {
            for( int i = 0; i < users.size(); i++ )
            {
               BChatUser user = getUser( i );
               if( user.getConnectionID() == id )
               {
                  Debug.doPrintln( "User Found!" );
                  return user;
               }
            }
         }
         return null;
      }
   
    /**
     * Function receiving all messages dispatched.
     * @param source Originating source of event.
    * @param event Event dispatched.
     * @post Event has either been handled or ignored.
     */   
      public void onBCommandEvent( Object source, BCommandEvent event )
      {
         if( event instanceof BBootCE )
         {
            defaultRoom = new BRoom();
            defaultRoom.setName( kDefault );
         
            synchronized( rooms )
            {
                // check for existing rooms...?
               rooms.put( defaultRoom.getName(), defaultRoom );
            }
         
         // add in the stats
            dispatchEvent( new BRegisterCmdCE( this, "stats", stats ) );
         
         }
         else if( event instanceof BCreateRoomCE )
         {
            BCreateRoomCE e = ( BCreateRoomCE ) event;
         
            BRoom newRoom = new BRoom();
            newRoom.setName( e.getName() );
            newRoom.addDoor( kDefault );
         
            defaultRoom.addDoor( e.getName() );
         
            synchronized( rooms )
            {
                // check for existing rooms...?
               rooms.put( newRoom.getName(), newRoom );
            }
         
            e.doComplete( BCommandComplete.kSuccess );
         	stats.createdRooms++;
         
            notifyRoomChange( newRoom );
            notifyRoomChange( defaultRoom );
         }
         else if( event instanceof BGetRoomsCE )
         {
            BGetRoomsCE e = ( BGetRoomsCE ) event;
         
            e.setRooms( getRooms() );
         
            e.doComplete( BCommandComplete.kSuccess );
         }
         else if( event instanceof BEnterRoomCE )
         {
            // find the user and put them into the new room
            // get the room by name
            BEnterRoomCE e = ( BEnterRoomCE ) event;
            BChatUser user = getUserByID( e.getConnectionID() );
            Debug.doPrintln( "User " + e.getConnectionID() + " wants to enter room " + e.getName() );
         
            BRoom newRoom = getRoom( e.getName() );
            BRoom oldRoom = null;
         
            if( user != null && newRoom != null )
            {
               oldRoom = user.getCurrentRoom();
               user.setCurrentRoom( newRoom );
            
                // Hope this works.
               oldRoom.removeUser( user.getConnectionID() );
               newRoom.addUser( user );
            
               e.setRoom( newRoom );
               e.doComplete( BCommandComplete.kSuccess );
            
               notifyRoomChange( oldRoom );
               notifyRoomChange( newRoom );
            }
            else
            {
               e.doComplete( BCommandComplete.kError );
            }
         
         }
         else if( event instanceof BGetRoomCE )
         {
            Debug.doPrintln( "get room" );
         
            BGetRoomCE e = ( BGetRoomCE ) event;
            //BRoom room = getRoom(e.getName());
            BRoom room = getRoomByID( e.getConnectionID() );
            // room = 
         
            e.setRoom( room );
         
            if( room == null )
            {
               Debug.doPrintln( "BRooms:BGetRoomCE:kError" );
               e.doComplete( BCommandComplete.kError );
            }
            else
            {
               Debug.doPrintln( "BRooms:BGetRoomCE:kSuccess" );
               e.doComplete( BCommandComplete.kSuccess );
            }
         }
         else if( event instanceof BGetFileCE )
         {
            BGetFileCE e = ( BGetFileCE ) event;
            File f = new File( "files" + File.separator + e.getFileName() );
            if( f.exists() )
            {
               Debug.doPrintln( "\n\n" + f.getPath() + " exists.\n\n" );
               BFile file = new BFile( f, BFile.kMaxCompressionRate );
            
                // Should test for successful compression.
               file.compress();
               e.setFile( file );
               e.doComplete( BCommandComplete.kSuccess );
            	stats.filesGot++;
            	stats.bytesGot += f.length();
            }
            else
            {
               Debug.doPrintln( "\n\n" + e.getFileName() + "doesn't exist.\n\n" );
               e.doComplete( BCommandComplete.kError );
            }
         }
         else if( event instanceof BSendFileCE )
         {
            // Must find out which room this is being sent from.
            // So we can insert it to rooms list.
         
            BSendFileCE e = ( BSendFileCE ) event;
         
            BRoom room = getRoomByID( e.getConnectionID() );
            room.addFile( e.getFileName() );
         
            BFile file = e.getFile();
            File filename = new File( "files" + File.separator + e.getFileName());
            if( file.expand(filename) )
            {
               Debug.doPrintln( "File has expanded..." );
               e.doComplete( BCommandComplete.kSuccess );
               File f = new File( e.getFileName() );
               if( f.exists() )
               {
                  Debug.doPrintln( e.getFileName() + " does exist in : " + f.getParent() );
               }
               else
               {
                  Debug.doPrintln( "Where the hell has the file gone?" );
               }
               notifyRoomChange( room );
            	stats.filesSent++;
            	stats.bytesSent += filename.length();
            }
            else
            {
               Debug.doPrintln( "File screwed up." );
               e.doComplete( BCommandComplete.kError );
            }
         }
         else if( event instanceof BGetThumbNailCE )
         {
            BGetThumbNailCE e = ( BGetThumbNailCE ) event;
            File f = new File( "files" + File.separator + e.getFileName() );
            if( f.exists() )
            {
               Debug.doPrintln( "\n\n" + f.getPath() + " exists.\n\n" );
               Image image = Toolkit.getDefaultToolkit().getImage( "files" + File.separator + e.getFileName() );
            
               ImageIcon icon = new ImageIcon( image.getScaledInstance( thumbwidth, thumbheight, java.awt.Image.SCALE_FAST ) );
               e.setThumbNail( icon );
               e.doComplete( BCommandComplete.kSuccess );
            	stats.thumbnails++;
            }
            else
            {
               Debug.doPrintln( "\n\n" + e.getFileName() + "doesn't exist.\n\n" );
               e.doComplete( BCommandComplete.kError );
            }
         }
         
         else if(event instanceof BAddUserCE)
         {
            BAddUserCE e = (BAddUserCE) event;
            e.getUser().setConnectionID(e.getConnectionID());
            addUser(e.getUser(), e.getConnectionID(), getRoom(kDefault));
            notifyRoomChange(getRoomByID(e.getConnectionID()));
            e.doComplete(BCommandComplete.kSuccess);
         	stats.logins++;
         }
         else if(event instanceof BRemoveUserCE)
         {
            BRemoveUserCE e = (BRemoveUserCE) event;
            BKickCE kick = new BKickCE(this);
            dispatchEventTo( kick, e.getUser().getConnectionID() );
            BRoom room = getRoomByID(e.getConnectionID());
            removeUser(e.getUser(), e.getUser().getConnectionID());
            notifyRoomChange(room);
            e.doComplete(BCommandComplete.kSuccess);
         	stats.logouts++;
         }
         else if ( event instanceof BChatPrivCE)
         {
         }
         
         else if (event instanceof BEditUserCE)
         {
            BEditUserCE e = (BEditUserCE) event;
         
            BChatUser eUser = e.getEditedUser();
            Debug.doPrintln ( "\n\nUser about to be changed : " + eUser.getName () + "  " + e.getConnectionID () );
            BChatUser user = getUserByID ( e.getConnectionID () );
            Debug.doPrintln ( "\n\nUser retrieved from server : " + user.getName () + "  " + user.getConnectionID () ) ;

            try
            {
                user.setName ( eUser.getName () );
                user.setNickname ( eUser.getNickname () );
                user.setAge ( eUser.getAge () );
                notifyRoomChange(getRoomByID(e.getConnectionID()));//*/
            }
            catch(BInvalidPropertyException p)
            {
                p.printStackTrace();
                e.doComplete( BCommandComplete.kError );
            }
            e.doComplete( BCommandComplete.kSuccess );
         }
         else if( event instanceof BChatMsgCE )
         {
            if( event.getNetworkStatus() == BCommandEvent.kServer )
            {
               Debug.doPrintln( "Got Chat Message" );
               BChatMsgCE e = ( BChatMsgCE ) event;
               int id = e.getConnectionID();
               BRoom room = getRoomByID( id );
            
                // find out the user and the room
                // sent to all people in that room, except who it came from
               synchronized( users )
               {
                  for( int i = 0; i < users.size(); i++ )
                  {
                     BChatUser user = getUser( i );
                     if( user.getCurrentRoom() == room )
                     {
                        if( user.getConnectionID() != id )
                        {
                           BChatMsgCE newEvent = new BChatMsgCE( this, e.getMessage(), e.getName() );
                           dispatchEventTo( newEvent, user.getConnectionID() );
                        }
                     }
                  }
               }
               e.doComplete( BCommandComplete.kSuccess );
            	stats.messagesSent++;
            }
         }
      }
   
    /**
     * Return user held at specified position in vector.
     * @param i Index into vector.
     * @return User at specified index.
     * @pre Vector contains registered users.
     */      
      BChatUser getUser( int i )
      {
         return( BChatUser ) users.get( i );
      }
   
    /**
     * Register user to server giving current room and connection ID.
     * @param user New user to add.
    * @param fConnectionID Connection ID of new user.
    * @param room Room user has entered.
     */   
      void addUser( BChatUser user, int fConnectionID, BRoom room )
      {
         Debug.doPrintln( "add user id = " + fConnectionID );
         user.setConnectionID( fConnectionID );
         user.setCurrentRoom( room );
         room.addUser( user );
         synchronized( users )
         {
            users.add( user );
         }
      }
   
    /**
     * Remove user from server records.
     * @param user User to remove.
    * @param fConnectionID Connection ID of user.
     */   
      void removeUser( BChatUser user, int fConnectionID )
      
      {
         Debug.doPrintln( "remove user id = " + fConnectionID );
         user.setConnectionID( fConnectionID );
         BRoom room = getRoomByID( fConnectionID );
         room.removeUser( fConnectionID );
         synchronized( users )
         {
            if (users.remove( user )) 
            {
               Debug.doPrintln("BRooms:removeUser Success");
            }
            else
            {
               Debug.doPrintln("BRooms:removeUser Failure");
            }
         }
      }
   
    /**
     * Tell anyone who cares a particular room has changed.
     * @param room Room which has changed.
     * @pre Room has been altered.
     * @post Everyone knows about it.
     */   
      void notifyRoomChange( BRoom room )
      {
        // send a BRoomChangeCE to anyone in this room
         synchronized( users )
         {
            for( int i = 0; i < users.size(); i++ )
            {
               BChatUser user = getUser( i );
               if( user.getCurrentRoom() == room )
               {
                  BRoomChangeCE event = new BRoomChangeCE( this, room.getName() );
                  dispatchEventTo( event, user.getConnectionID() );
               }
            }
         }
      }
   
   
   }
