/**
 * Event to store and contain file to be sent across network.
 * Name of file to be retrieved is sent to server and client
 * receives file through the completion object.
 * @see
 * @concurrency
 * @copyright
 * @rref
 * @author Christian Karolos
 */
//$Header

package au.edu.swin.synergy.plugin.rooms.event;

import java.lang.String;

import au.edu.swin.synergy.lib.BFile;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;

public class BGetFileCE extends BCommandEvent
{
    /**
     * Name of file to retrieve from server.
     */
    String fileName = null;
    
    /**
     * File retrieved from server and to be returned to client.
     */
    BFile file = null;

    /**
     * Constructor
     * @param source Source of event.
	 * @param name Name of file to retrieve.
	 * @param complete Completion object of which file will be returned to client.
     */    
    public BGetFileCE( Object source, String name, BCommandComplete complete )
    {
        super( source );
        setCompletionObject( complete );
        setFileName( name );
    }

    /**
     * Get name of file.
     * @return Name of file.
     * @pre File name has been set.
     */    
    public String getFileName()
    {
        return fileName;
    }

    /**
     * Set name of file to be retrieved by this event.
     * @param name Name of file.
     * @pre File name hasn't been set.
     * @post File name is set.
     */    
    public void setFileName( String name )
    {
        fileName = name;
    }

    /**
     * Get file contained in this event.
     * @return File object containing file in compressed form.
     * @pre File has been set.
     */    
    public BFile getFile()
    {
        return file;
    }

    /**
     * Set file to be returned to client.
     * @param file File object.
     * @pre File object hasn't been set.
     */    
    public void setFile( BFile file )
    {
        this.file = file;
    }
}
