//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/rtp/BChgPermPanel.java,v 1.9 1999/10/17 16:18:58 tp079541 Exp $
package au.edu.swin.synergy.plugin.rtp;

import au.edu.swin.synergy.plugin.userdb.BGroup;
import au.edu.swin.synergy.plugin.userdb.BUser;
import au.edu.swin.synergy.plugin.userdb.BPrivileged;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.plugin.repository.BDescriptor;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

import au.edu.swin.synergy.plugin.repository.BDescriptor;

/**
 * This class provides the change permissions panel for changing
 * file permissions in the repository.  The combo box and file list will
 * be disabled if the user is not either the owner or an administrator
 *
 * @author Shane Fitzgibbon
 * @copyright Synergy Software 1999
 */
public class BChgPermPanel extends JPanel
{

	/**
	 * Vector which contains the groups info
	 */
	protected Vector groups;

	/**
	 * Vector which contains the users info
	 */
	protected Vector users;

	/**
	 * Vector which contains the descriptor info
	 */
	protected Vector descriptors;

	/**
	 * Vector containing the old descriptors
	 */
	protected Vector oldDescriptors;

	/**
	 * Filename of file whos permission are being edited
	 */
	protected String filename;

	/**
	 * Owner of the file
	 */
	protected String owner;

	/**
	 * new Owner of the file if the owner has changed
	 */
	protected String newOwner = null;
	/**
	 * UID of the owner of the file
	 */
	protected Long ownerID;

	/**
	 * user who is opeining this dialog
	 */
	protected BUser user;



	//{{DECLARE_CONTROLS
	JLabel fileLabel = new JLabel();
	JLabel ownerLabel = new JLabel();
	JLabel ownerName = new JLabel();
	JLabel groupsLabel = new JLabel();
	JComboBox ownerCombo = new JComboBox();
	BPermissionModel permModel = new BPermissionModel();
	JTable groupList = new JTable(permModel);
	JScrollPane groupPane = new JScrollPane( groupList );
	BOwnerModel ownerModel;


	/**
	 * Constructor for the Change Permissions Panel
	 *
	 * @param file the name of the file
	 * @param ownerName the name of the owner
	 * @param fOwnerID the unique ID o fthe owner of the file
	 * @param currentUser The BUser object of the currently logged in user
	 */
	public BChgPermPanel(String file, String ownerName, Long fOwnerID, BUser currentUser)
	{
		this.filename = file;
		this.owner = ownerName;
		this.descriptors = new Vector();
		this.user = currentUser;
		this.ownerID = fOwnerID;
		doInit();
	}

	private void doInit()
	{
		setSize(340,280);
		setVisible(true);
		this.setLayout(new GridBagLayout());
		fileLabel.setHorizontalAlignment(SwingConstants.CENTER);
		fileLabel.setText(filename);
		this.add(fileLabel,new GridBagConstraints(0,0,5,1,0.0,0.0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(12,12,0,0),251,9));
		fileLabel.setBounds(9,12,300,24);
		ownerLabel.setText("owner");
		this.add(ownerLabel,new GridBagConstraints(1,1,1,1,0.0,0.0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(12,12,0,0),84,9));
		ownerLabel.setBounds(9,48,120,24);
		this.add(ownerCombo,new GridBagConstraints(2,1,1,1,0.0,0.0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(12,12,0,0),16,-1));
		ownerCombo.setBounds(141,48,69,24);
		groupsLabel.setText("groups");
		this.add(groupsLabel,new GridBagConstraints(1,3,1,1,0.0,0.0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(9,12,0,0),80,9));
		groupsLabel.setBounds(9,84,120,24);
		this.add(groupPane,new GridBagConstraints(0,4,5,3,0.0,0.0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(12,12,7,0),204,96));
		groupPane.setBounds(9,120,204,96);

		if( (user.getPrivilege() != BPrivileged.kAdministrator) && (user.getUniqueID() != ownerID) )
		{
			ownerCombo.setEnabled( false );
			permModel.setEnabled( false );
		}
	}


	/**
	 * Sets the groups to be displayed in the groups list
	 *
	 * @param groupsVector Vector containing the group IDs to display
	 * @param members Vector containing the names fo the groups
	 */
	public void setGroups( Vector groupsVector, Vector members )
	{
		Debug.doPrintln("In BChgPermPanel:setGroups:");
		this.groups = groupsVector;
		for( int i = 0; i < groups.size(); i++ )
		{
			Long group = (Long) groups.get(i);
			BDescriptor desc = null;
			desc = getDesc( BDescriptor.kGroup, getGroupID( i ) );
			if( desc.getAccess() == BDescriptor.kNone )
			{
				Debug.doPrintln("desc == BDescriptor.kNone see: " + desc.getAccess());
				Debug.doPrintln("Group ID = " + groups.get(i) + " name " + (String)members.get(i) + "No Access");
				permModel.add( (String)members.get(i), false, false, ((Long)group).intValue(), BDescriptor.kGroup);
			}
			else
			{
				//access = kNone
				Debug.doPrintln("desc != BDescriptor.kNone its: " + desc.getAccess());
				Debug.doPrintln("Group ID = " + groups.get(i) + " name " + (String)members.get(i) + " read " + desc.checkPerm( BDescriptor.kRead ) + " write " + desc.checkPerm( BDescriptor.kReadWrite ));
				permModel.add( (String)members.get(i), desc.checkPerm( BDescriptor.kRead ), desc.checkPerm( BDescriptor.kReadWrite ), ((Long)group).intValue(), BDescriptor.kGroup);
			}
		}
	}

	/**
	 * Sets the users to be displayed in the members list
	 *
	 * @param usersVector Vector containing the users IDs to display
	 * @param members Vector containg the names of the users
	 */
	public void setUsers( Vector usersVector, Vector members )
	{
		Debug.doPrintln( "In BChgPermPanel:setUsers: ");
		this.users = usersVector;
		for( int i = 0; i < users.size(); i++ )
		{
			Long user = (Long) users.get(i);
			BDescriptor desc = getDesc( BDescriptor.kUser, getGroupID( i ) );
			if ( desc.getAccess() == BDescriptor.kNone )
			{
				Debug.doPrintln( "User ID = " + user + " name " + members.get(i) + "No Access");
				permModel.add( (String)members.get(i), false, false, ((Long)user).intValue(), BDescriptor.kUser );
			}
			else
			{
				Debug.doPrintln( "User ID = " + user + " name " + members.get(i) + " read " + desc.checkPerm( BDescriptor.kRead ) + " write " + desc.checkPerm( BDescriptor.kReadWrite ));
				permModel.add( (String)members.get(i), desc.checkPerm( BDescriptor.kRead ), desc.checkPerm( BDescriptor.kReadWrite ), user.intValue(), BDescriptor.kUser);
			}
		}
		ownerModel = new BOwnerModel( members );
		ownerCombo.setModel( ownerModel );
		ownerModel.setSelectedItem( owner );
	}


	/**
	 * Gets the BDescriptors for the groups
	 *
	 * @returns a Vector of BDescriptors
	 */
	public Vector getDescriptors()
	{
		evalDescriptors();
		this.oldDescriptors = this.descriptors;
		return (Vector)this.oldDescriptors.clone();
	}

	public void setDescriptors( Vector desc )
	{
		this.oldDescriptors = desc;
		this.descriptors = (Vector)desc.clone();
	}

	public String getOwner()
	{
		if( newOwner != null )
		{
			return newOwner;
		}
		else
		{
			return owner;
		}
	}

	private int getGroupID( int index )
	{
		if( index < groups.size() )
		{
			return ((Long)groups.get(index)).intValue();
		}
		Debug.doPrintln("Error in GetGroupID");
		return 0;
	}

	class BListSelectionListener implements ListSelectionListener
	{
		public void valueChanged( ListSelectionEvent event )
		{

		}
	}


	private BDescriptor getDesc( int realm, int ID )
	{
		Debug.doPrintln("In BChgPermPanel:getDesc:realm: " + realm + " ID: " + ID);
		BDescriptor desc = new BDescriptor( ID, realm, BDescriptor.kNone );
		Debug.doPrintln("In BChgPermPanel:getDesc:desc: " + desc);
		BDescriptor targetDesc = null;
		for ( int i = 0; i < descriptors.size(); i++)
		{
			BDescriptor	tmpDesc = (BDescriptor)descriptors.get(i);
			if ( tmpDesc.getRealm() == desc.getRealm() && tmpDesc.getId() == desc.getId() )
			{
				targetDesc = tmpDesc;
			}
		}
		if ( targetDesc == null )
		{
			targetDesc = desc;
		}
		Debug.doPrintln("IN BChgPermPanel:getDesc:returning: " + targetDesc );
		return targetDesc;
	}


	private BDescriptor addDesc( BDescriptor desc )
	{
		for ( int i = 0; i < descriptors.size(); i++)
		{
			BDescriptor	tmpDesc = (BDescriptor)descriptors.get(i);
			if ( tmpDesc.getRealm() == desc.getRealm() && tmpDesc.getId() == desc.getId() )
			{
				descriptors.remove(tmpDesc);
			}
		}
		if( desc.getAccess() != BDescriptor.kNone )
		{
			descriptors.add( desc);
		}
		return desc;
	}

	private void evalDescriptors()
	{
		Vector descVector = permModel.getDescriptors();
		for( int i = 0; i < descVector.size(); i++ )
		{
			addDesc( (BDescriptor)descVector.get(i) );
		}
	}

	class BOwnerModel extends AbstractListModel implements ComboBoxModel
	{
		Vector data;
		int selectedIndex;

		public BOwnerModel( Vector owners )
		{
			this.data = owners;
			selectedIndex = 0;
		}

		public Object getElementAt( int index )
		{
			if ( index < data.size() )
			{
				return data.get( index );
			}
			return data.firstElement();
		}

		public int getSize()
		{
			return data.size();
		}

		public Object getSelectedItem()
		{
			Debug.doPrintln( "Selected Index = " + selectedIndex + " data size is " + data.size() );
			newOwner = (String) data.get(selectedIndex);
			return newOwner;
		}

		public void setSelectedItem( Object anItem )
		{
			int index = data.indexOf( anItem );
			if ( index != -1 )
			{
				this.selectedIndex = index;
			}
		}
	}

}