//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/rtp/BPermissionModel.java,v 1.4 1999/10/17 06:45:36 tp079541 Exp $
package au.edu.swin.synergy.plugin.rtp;

import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.plugin.repository.BDescriptor;
import au.edu.swin.synergy.plugin.userdb.BPrivileged;

import javax.swing.table.*;
import java.util.Vector;


/**
 * Provides the data model for the permissions table
 * @author Shane Fitzgibbon
 * @copyright Synergy Software 1999
 */
public class BPermissionModel extends AbstractTableModel
{

	/**
	 * columnNames for the table
	 */
	  final String [] columnNames =
	  {
		 "Group", "Read", "Write"
	  }
	  ;
	/**
	 * The Vector that contains the data for the table
	 */
	  final Vector data = new Vector();

	 /**
	  * Flag used to determis wether this table si editable or not
	  */
	 boolean enabled = true;


	/**
	 * gets the number of columns in the table model
	 * @return the number of columns
	 */
	  public int getColumnCount()
	  {
		 return columnNames.length;
	  }

	/**
	 * Gets the number of rows that current contain data
	 * @returns the number of rows
	 */
	  public int getRowCount()
	  {
		 return data.size();
	  }

	/**
	 * Gets the name of the column for a certain column index
	 * @return the name of the column
	 * @param col the index of the column
	 */
	  public String getColumnName( int col )
	  {
		 return columnNames [ col ];
	  }

	/**
	 * Gets the value of a cell in the table
	 * @return the value of the cell
	 * @param row the row of the cell
	 * @param col the column of the cell
	 */

	  public Object getValueAt( int row, int col )
	  {
		 Vector v = ( Vector ) data.get( row );

		 return v.get( col );
	  }



	/**
	 * Gets the class of the column
	 * @return the class of the column
	 * @param col the index of the column
	 */

	  public Class getColumnClass( int col )
	  {
		 return getValueAt( 0, col ).getClass();
	  }

	/**
	 * adds a row to the table
	 * @param name the name of the member
	 * @param read statu of this members read access
	 * @param write status of the members write access
	 * @param member the member id
	 * @param realm the realm of the member
	 */

	  public void add( String name, boolean read, boolean write, int member, int realm)
	  {
		 Debug.doPrintln( "BPermissionModel:add:name: " + name + " read: " + read + " write: " + write + " id " + member );
		 if ( write )
		 {
		 	read = true;
		 }

		 Vector v = new Vector( 4 );
		 v.add( name );
		 v.add( new Boolean(read) );
		 v.add( new Boolean(write) );
		 v.add( new Integer (member) );
		 v.add( new Integer(realm) );
		 data.add( v );
		 fireTableDataChanged();
	  }


	/**
	 * clears the table model
	 */

	  public void clear()
	  {
		 Debug.doPrintln( "BPeopleTableModel:clear" );
		 data.clear();
		 fireTableDataChanged();
	  }

	  public Vector getDescriptors()
	  {
		 Vector result = new Vector();
		 for( int i = 0; i < data.size(); i++ )
		 {
		 	int access;
		 	Vector row = (Vector)data.get(i);
		 	boolean read = ((Boolean) row.get(1)).booleanValue();
		 	boolean write = ((Boolean) row.get(2) ).booleanValue();
		 	int member = ((Integer) row.get(3)).intValue();
		 	int realm = ((Integer)row.get(4)).intValue();
		 	if( read && write )
		 	{
		 		access = BDescriptor.kReadWrite;
		 	}
		 	else if( read )
		 	{
		 		access = BDescriptor.kRead;
		 	}
		 	else
		 	{
		 		access = BDescriptor.kNone;
		 	}
		 	Debug.doPrintln( "In BPermissionModel:getDescriptors: about to create new BDescriptor " + realm + member + "=" + access);
		 	BDescriptor desc = new BDescriptor( member, realm, access );
		 	Debug.doPrintln( "InBPermissionModel:getDescriptors: created Descriptor: " + desc );
			result.add( desc );

		 }
		 return result;
	 }

	 /**
	  * destermines if a cell in the table model is editable
	  *
	  * @param row the row of the cell
	  * @param col the column of the cell
	  * @returns true if the cell is editable
	  */
	 public boolean isCellEditable(int row, int col)
	 {
		 //Note that the data/cell address is constant,
		 //no matter where the cell appears onscreen.
		 if( enabled )
		 {
			 if (col < 1)
			 {
				 return false;
			 }
			 else
			 {
				 return true;
			 }
		 }
		 else
		 {
		 	return false;
		 }
	 }

	 public void setEnabled( boolean flag )
	 {
	 	this.enabled = flag;
	 }

	 /*
	  * Don't need to implement this method unless your table's
	  * data can change.
	  */
	 public void setValueAt(Object value, int row, int col)
	 {
		Vector rowVector = (Vector)data.get( row );
		rowVector.set( col, value );
		if ( col == 2 )
		{
			if ( ((Boolean)value).booleanValue() )
			{
				rowVector.set( col - 1, value );
			}

		}
		else if ( col == 1 )
		{
			if ( !((Boolean)value).booleanValue() )
			{
				rowVector.set( col + 1, value );
			}
		}
	 	fireTableRowsUpdated(row, row);
     }
}