// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/rtp/BRepositoryClient.java,v 1.35 1999/10/17 21:15:05 tp079541 Exp $

package au.edu.swin.synergy.plugin.rtp;

import java.lang.String;
import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BBlockingComplete;
import au.edu.swin.synergy.kernel.event.BBootCE;

import au.edu.swin.synergy.debug.Debug;
import java.io.File;
import au.edu.swin.synergy.lib.BFile;

import au.edu.swin.synergy.plugin.repository.event.BRepositoryCE;
import au.edu.swin.synergy.plugin.repository.event.BGetFileCE;
import au.edu.swin.synergy.plugin.repository.event.BPutFileCE;
import au.edu.swin.synergy.plugin.repository.event.BStatFileCE;
import au.edu.swin.synergy.plugin.repository.event.BStatDirCE;
import au.edu.swin.synergy.plugin.repository.event.BSetSecurityCE;
import au.edu.swin.synergy.plugin.repository.BMetaData;
import au.edu.swin.synergy.plugin.repository.BACL;
import au.edu.swin.synergy.plugin.repository.BDescriptor;

import au.edu.swin.synergy.plugin.command.BOnCommand;
import au.edu.swin.synergy.plugin.command.event.BRegisterCmdCE;

import au.edu.swin.synergy.plugin.client.event.BLoginCompleteCE;
import au.edu.swin.synergy.plugin.client.event.BLogoutCompleteCE;
import au.edu.swin.synergy.plugin.client.event.BGetUserCE;
import au.edu.swin.synergy.plugin.userdb.BUser;
import au.edu.swin.synergy.plugin.userdb.BGroup;
import au.edu.swin.synergy.plugin.userdb.event.BGetGroupsCE;
import au.edu.swin.synergy.plugin.userdb.event.BGetUsersCE;

import au.edu.swin.synergy.plugin.dialect.BString;
import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.dialect.event.BDialectChangeCE;

import java.io.InputStream;
import java.io.PrintStream;
import au.edu.swin.synergy.lib.io.BTextdb;
import au.edu.swin.synergy.lib.io.BTextdbRecord;
import au.edu.swin.synergy.lib.io.BFileFilter;
import au.edu.swin.synergy.lib.BStringList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.util.Vector;

/**
 * Repository client plugin
 * @Author John Newbigin
 * @author Shane Fitzgibbon
 * @Copyright Synergy Software 1999
 *
 * Files are stored in a directory, under data
 * This is meta-data for each directory stored in the file meta.data
 * This is a BTextDB file, which stores filename to file info
 */

public class BRepositoryClient extends BPlugin
{
	File repositoryRoot = new File("localdata");
	public static final String kMetaFileName = ".meta.data";
	BRepositoryGui gui;



    /**
     * This is where to call constructor like events for classes that extend BPlugin
     *
     * @param params any arguments that were passed to this plugin when it was loaded
     */
    public void setup(BStringList params)
    {
		if(!repositoryRoot.exists())
		{
			repositoryRoot.mkdir();
		}

		// register command line commands
		dispatchEvent( new BRegisterCmdCE( this, "ls", new OnLS() ) );
		dispatchEvent( new BRegisterCmdCE( this, "get", new OnGet() ) );
		dispatchEvent( new BRegisterCmdCE( this, "put", new OnPut() ) );
		dispatchEvent( new BRegisterCmdCE( this, "chown", new OnChown() ) );
		dispatchEvent( new BRegisterCmdCE( this, "chmod", new OnChmod() ) );

		gui  = new BRepositoryGui ( this );

    }

	public File makeFile(String fileName)
	{
		return new File(repositoryRoot.getPath() +	File.separator + fileName);
	}

	/**
	 * This metod is called whenever this plugin recieves an event
	 *
	 * @param source the Object that sent the event
	 * @param event the event the has been recieved
	 */
    public void onBCommandEvent( Object source, BCommandEvent event )
    {
		if( event instanceof BLoginCompleteCE )
		{
			gui.doLogin();
		}

		else if( event instanceof BLogoutCompleteCE )
		{
			gui.doLogout();
		}

		else if( event instanceof BDialectChangeCE )
		{
			doDialect();
		}
    }

	/**
	 * This is the method that is called whenever the locale is changed
	 */
	public void doDialect()
	{
		gui.doDialect();
	}

	public boolean setClientMetaData(String fFileName, BMetaData fMetaData)
	{
		File destFile = makeFile(fFileName);
		return setClientMetaData( destFile, fMetaData );
	}

	public boolean setClientMetaData(File destFile, BMetaData fMetaData)
	{

		File destDir = destFile.getParentFile();
		String fileNamePart = destFile.getName();

		String metaFileName = destDir.getPath() + File.separator + kMetaFileName;
		Debug.doPrintln("Setting meta data in " + metaFileName + " for file " + destFile);
		fMetaData.setClientSide();

		// read in any existing records
		// except this file name

		Vector metaRecords = new Vector();
		boolean found = false;

		BTextdb db = new BTextdb(metaFileName);
		db.openFileForRead();
		try
		{
			while(true)
			{
				BTextdbRecord record = db.getRecord();
				if(record == null)
				{
					Debug.doPrintln("no more records");
					break;
				}
				Debug.doPrintln("Reading record");
				if(BMetaData.matchesFile(record, fileNamePart))
				{
					Debug.doPrintln("Metadata found");
					found = true;
					metaRecords.add(fMetaData.getRecord());
				}
				else
				{
					metaRecords.add(record);
				}
			}
		}
		finally
		{
			db.close();
		}

		if(!found)
		{
			metaRecords.add(fMetaData.getRecord());
		}

		// now write back the new file
		db.openFileForWrite(false);
		try
		{
			for(int i=0; i<metaRecords.size(); i++)
			{
				BTextdbRecord record = (BTextdbRecord)metaRecords.get(i);
				db.addRecord(record);
			}
		}
		finally
		{
			db.close();
		}
		return true;
	}

	public BMetaData getClientMetaData(String fFileName)
	{
		File destFile = makeFile(fFileName);
		File destDir = destFile.getParentFile();
		String fileNamePart = destFile.getName();

		String metaFileName = destDir.getPath() + File.separator + kMetaFileName;
		BTextdb db = new BTextdb(metaFileName);

		BMetaData result = null;
		db.openFileForRead();
		try
		{
			while(true)
			{
				BTextdbRecord record = db.getRecord();
				if(record == null)
				{
					Debug.doPrintln("no more records");
					break;
				}
				Debug.doPrintln("Reading record");
				if(BMetaData.matchesFile(record, fileNamePart))
				{
					Debug.doPrintln("Metadata found");
					result = new BMetaData();
					result.setClientSide();

					result.setRecord(record);
					break;
				}
			}
		}
		finally
		{
			db.close();
		}

		return result;
	}


	/**
	 * gets directory information from the repository
	 * and adds it to the table model
	 *
	 * @param dir the name of the directory to get info on
	 */

	public void getDir( String dir, BResFileModel fileModel )
	{
		fileModel.clear();
		Debug.doPrintln("BRepositoryClient:getDir:dir = " + dir);
		BStatFileCE statEvent = new BStatFileCE(this, dir, new BOnGetDir( dir, fileModel));
		dispatchEventServer(statEvent);
	}

	/**
	 * checks to see if a file si a directory
	 */
	private boolean isDir( String dir )
	{
		BStatFileCE statEvent = new BStatFileCE(this, dir, null);
		BBlockingComplete complete = dispatchEventServerAndWait(statEvent);

		if(complete.getStatus() == BCommandComplete.kSuccess)
		{
			statEvent = (BStatFileCE) complete.getEvent();
			if(statEvent.isDirectory())
			{
				return true;
			}
		}
		return false;
	}


	class BOnGetDir extends BCommandComplete
		{
			String dir;
			BResFileModel fileModel;

			public BOnGetDir( String directory, BResFileModel model )
			{
				this.dir = directory;
				this.fileModel = model;
			}

			public void doComplete( BCommandEvent event, int status )
			{
				if(status == BCommandComplete.kSuccess)
				{
					BStatFileCE statEvent = (BStatFileCE) event;
					if(statEvent.isDirectory())
					{
						/*BStringList files = statEvent.getFiles();
						if(files != null)
						{
							for(int j=0; j<files.getCount(); j++)
							{
								Debug.doPrintln("getDir got file" + files.get(j) );
							}
							for(int j=0; j<files.getCount(); j++)
							{
								/**
								statEvent = new BStatFileCE(this, "./" +files.get(j), null);
								complete = client.dispatchEventServerAndWait( statEvent );
								if ( complete.getStatus() == BCommandComplete.kSuccess )
								{
									Debug.doPrintln( "got file info on " + statEvent.getFileName() + " " + statEvent.getSize() );
								}//*//*
								getDirInfo( files.get(j), fileModel );
								//fileModel.addDir( statEvent.getFileName(), 999999L, 0, "d", 1);
							}
						}*/
						BStatDirCE dirEvent = new BStatDirCE( this, statEvent.getFileName(), new BOnStatDirCC(fileModel));
						dispatchEventServer( dirEvent );
					}
					else
					{
						if(statEvent.getSize() < 0)
						{
							Debug.doPrintln(statEvent.getFileName() + " not found");
						}
						else
						{

							/*BMetaData metaData = statEvent.getMetaData();
							if(metaData != null)
							{
								Debug.doPrintln(statEvent.getFileName() + " (" + statEvent.getSize() + " bytes)");
								Debug.doPrintln("Owner = " + metaData.getOwner());
								Debug.doPrintln("Version = " + metaData.getVersion());
								Debug.doPrintln("Date = " + metaData.getDate());
								Debug.doPrintln("Type = " + metaData.getType());
								if ( fileModel != null )
								{
									fileModel.addFile( statEvent.getFileName(), metaData.getDate(), metaData.getOwner(), metaData.getType(), metaData.getVersion(), statEvent.getSize() );
								}
							}
							else
							{
								Debug.doPrintln("No meta-data");
							}//*/
							Debug.doPrintln("In Get Dir -- Shouldn't be here --");
						}
					}
				}
			}//doComplete

			private void getDirInfo( String dirName, BResFileModel model )
			{
				Debug.doPrintln("In BRepositoryClient:getDirInfo for dir " + dirName);
				BStatFileCE statEvent = new BStatFileCE(this, dirName, new BOnGetDirInfo( dirName, model));
				dispatchEventServer(statEvent);

			}

	}//BOnGetDir


	class BOnStatDirCC extends BCommandComplete
	{
		BResFileModel fileModel;

		public BOnStatDirCC( BResFileModel fFileModel )
		{
			this.fileModel = fFileModel;
		}

		public void doComplete( BCommandEvent event, int status )
		{
			if( status == BCommandComplete.kSuccess )
			{
				BStatDirCE statEvent = (BStatDirCE)event;
				for ( int i = 0; i < statEvent.getCount(); i++ )
				{
					if(statEvent.isDirectory(i))
					{
						BMetaData metaData = statEvent.getMetaData(i);
						Debug.doPrintln( "adding dir " + statEvent.getFileName(i) );
						/*if( metaData != null )
						{
							Debug.doPrintln( "MetaData not null" );
							fileModel.addDir( statEvent.getFileName(), metaData.getDate(), metaData.getOwner(), metaData.getType(), metaData.getVersion() );
						}//*/
						fileModel.addDir( statEvent.getFileName(), 999999L, 0, "d", 1);
					}
					else
					{
						if(statEvent.getSize(i) < 0)
						{
							Debug.doPrintln(statEvent.getFileName() + " not found");
						}
						else
						{

							BMetaData metaData = statEvent.getMetaData(i);
							if(metaData != null)
							{
								Debug.doPrintln(statEvent.getFileName(i) + " (" + statEvent.getSize(i) + " bytes)");
								Debug.doPrintln("Owner = " + metaData.getOwner());
								Debug.doPrintln("Version = " + metaData.getVersion());
								Debug.doPrintln("Date = " + metaData.getDate());
								Debug.doPrintln("Type = " + metaData.getType());
								if ( fileModel != null )
								{
									fileModel.addFile( statEvent.getFileName(i), metaData.getDate(), metaData.getOwner(), metaData.getType(), metaData.getVersion(), statEvent.getSize(i) );
								}
							}
							else
							{
								Debug.doPrintln("No meta-data");
							}
						}
					}
				}
			}
		}
	}

	class BOnGetDirInfo extends BCommandComplete
		{
			String dir;
			BResFileModel fileModel;

			public BOnGetDirInfo( String dirName, BResFileModel model )
			{
				this.dir = dirName;
				this.fileModel = model;
			}

			public void doComplete( BCommandEvent event, int status )
			{


				if(status == BCommandComplete.kSuccess)
				{
					BStatFileCE statEvent = (BStatFileCE) event;
					if(statEvent.isDirectory())
					{
						BMetaData metaData = statEvent.getMetaData();
						Debug.doPrintln( "adding dir " + statEvent.getFileName() );
						/*if( metaData != null )
						{
							Debug.doPrintln( "MetaData not null" );
							fileModel.addDir( statEvent.getFileName(), metaData.getDate(), metaData.getOwner(), metaData.getType(), metaData.getVersion() );
						}//*/
						fileModel.addDir( statEvent.getFileName(), 999999L, 0, "d", 1);
					}
					else
					{
						if(statEvent.getSize() < 0)
						{
							Debug.doPrintln(statEvent.getFileName() + " not found");
						}
						else
						{

							BMetaData metaData = statEvent.getMetaData();
							if(metaData != null)
							{
								Debug.doPrintln(statEvent.getFileName() + " (" + statEvent.getSize() + " bytes)");
								Debug.doPrintln("Owner = " + metaData.getOwner());
								Debug.doPrintln("Version = " + metaData.getVersion());
								Debug.doPrintln("Date = " + metaData.getDate());
								Debug.doPrintln("Type = " + metaData.getType());
								if ( fileModel != null )
								{
									fileModel.addFile( statEvent.getFileName(), metaData.getDate(), metaData.getOwner(), metaData.getType(), metaData.getVersion(), statEvent.getSize() );
								}
							}
							else
							{
								Debug.doPrintln("No meta-data");
							}
						}
					}
				}
			}
	} //BOnGetDirInfo






    class OnLS implements BOnCommand
    {
        public int onCommand( BStringList params, InputStream in, PrintStream out )
        {
			if(params.getCount() == 1)
			{
				params.add(".");
			}
            for( int i = 1; i < params.getCount(); i++ )
			{
				Debug.doPrintln("sending stat for " + params.get(i));
				BStatFileCE statEvent = new BStatFileCE(this, params.get(i), null);
				BBlockingComplete complete = dispatchEventServerAndWait(statEvent);

				if(complete.getStatus() == BCommandComplete.kSuccess)
				{
					statEvent = (BStatFileCE) complete.getEvent();
					if(statEvent.isDirectory())
					{
						out.println(statEvent.getFileName() + " is a directory");
						BStringList files = statEvent.getFiles();
						if(files != null)
						{
							for(int j=0; j<files.getCount(); j++)
							{
								out.println(files.get(j));
							}
						}
					}
					else
					{
						if(statEvent.getSize() < 0)
						{
							out.println(statEvent.getFileName() + " not found");
						}
						else
						{
							out.println(statEvent.getFileName() + " (" + statEvent.getSize() + " bytes)");
							BMetaData metaData = statEvent.getMetaData();
							if(metaData != null)
							{
								out.println("Owner = " + metaData.getOwner());
								out.println("Version = " + metaData.getVersion());
							}
							else
							{
								out.println("No meta-data");
							}
						}
					}
				}
				else if(complete.getStatus() == BCommandComplete.kNetError)
				{
					out.println("Not logged in");
				}
			}
            return 0;
        }

        public String getHelp( BStringList params )
        {
            return "list files in the repository";
        }
    }

    class OnGet implements BOnCommand
    {
        public int onCommand( BStringList params, InputStream in, PrintStream out )
        {
			if(params.getCount() < 1)
			{
				out.println("Please specify a file");
			}
			int i = 1;
			while(i < params.getCount())
			{
				int versionToGet = -1; // default
				if(params.get(i).equals("-v"))
				{
					i++;
					versionToGet = Integer.parseInt(params.get(i));
					i++;
				}
				Debug.doPrintln("getting file" + params.get(i) + " version " + versionToGet);
				BGetFileCE getEvent = new BGetFileCE(this, params.get(i), null);
				i++;
				getEvent.setVersion(versionToGet);
				BBlockingComplete complete = dispatchEventServerAndWait(getEvent);

				if(complete.getStatus() == BCommandComplete.kSuccess)
				{
					getEvent = (BGetFileCE) complete.getEvent();

					BFile data = getEvent.getData();

					if(getEvent.getError() != BRepositoryCE.kErrorNoError)
					{
						out.println("Get failed.  Error = " + getEvent.getErrorString());
					}
					else
					{
						out.println("Got " + getEvent.getFileName());
						File destination = makeFile(getEvent.getFileName());
						File directory = destination.getParentFile();
						if(!directory.exists())
						{
							directory.mkdir();
						}
						data.expand(destination);
						getEvent.getMetaData().setDate(destination.lastModified());
						setClientMetaData(getEvent.getFileName(), getEvent.getMetaData());
					}
				}
			}
            return 0;
        }

        public String getHelp( BStringList params )
        {
            return "get files from the repository\n" +
			       "Usage: get [-v n] file1 [ [-v n ] file2 ... ]";
        }
    }

	class OnPut implements BOnCommand
    {
        public int onCommand( BStringList params, InputStream in, PrintStream out )
        {
			if(params.getCount() < 1)
			{
				out.println("Please specify a file");
			}
			int i = 1;
			while(i < params.getCount())
			{
				Debug.doPrintln("putting file " + params.get(i));

				File source = makeFile(params.get(i));
				if(!source.exists())
				{
					out.println("The file does not exist");
					return 1;
				}
				// look up the metadata....
				BMetaData metaData = getClientMetaData(params.get(i));
				if(metaData == null)
				{
					Debug.doPrintln("Putting this file for the first time");
				}
				else
				{
					Debug.doPrintln("Checking date");
					if(source.lastModified() != metaData.getDate())
					{
						Debug.doPrintln("File has been modified");
					}
					else
					{
						Debug.doPrintln("File has not been modified");
						i++;
						continue;
					}
				}
				BPutFileCE putEvent = new BPutFileCE(this, params.get(i), null);

				// read in the data...
				BFile data = new BFile(source, BFile.kDefaultCompressionRate );
				data.compress();
				putEvent.setData(data);
				putEvent.setMetaData(metaData);


				i++;
				BBlockingComplete complete = dispatchEventServerAndWait(putEvent);
				if(complete.getStatus() == BCommandComplete.kSuccess)
				{
					Debug.doPrintln("OnPutComplete called");
					putEvent = (BPutFileCE) complete.getEvent();

					if(putEvent.getError() == 0)
					{
						out.println("Success");
						// if we got a BFile back, we should write it
						if(putEvent.getData() != null)
						{
							File destination = makeFile(putEvent.getFileName());
							data.expand(destination);
							putEvent.getMetaData().setDate(destination.lastModified());
						}


						// save the new metadata
						Debug.doPrintln("Saving new metadata for " + putEvent.getFileName());
						Debug.doPrintln("New version is  " + putEvent.getMetaData().getVersion());
						File destination = makeFile(putEvent.getFileName());
						putEvent.getMetaData().setDate(destination.lastModified());
						setClientMetaData(putEvent.getFileName(), putEvent.getMetaData());
					}
					else if(putEvent.getError() == BRepositoryCE.kErrorConflict)
					{
						Debug.doPrintln("Conflicts detected");
						// if we got a BFile back, we should write it
						if(putEvent.getData() != null)
						{
							File destination = makeFile(putEvent.getFileName());
							data.expand(destination);
							putEvent.getMetaData().setDate(destination.lastModified());
						}


						// save the new metadata
						Debug.doPrintln("Saving new metadata for " + putEvent.getFileName());
						Debug.doPrintln("New version is  " + putEvent.getMetaData().getVersion());
						File destination = makeFile(putEvent.getFileName());
						putEvent.getMetaData().setDate(destination.lastModified());
						setClientMetaData(putEvent.getFileName(), putEvent.getMetaData());
					}
					else
					{
						out.println("Error " + putEvent.getErrorString());
					}
				}
			}
            return 0;
        }

        public String getHelp( BStringList params )
        {
            return "put files to the repository\n" +
			       "Usage: put file1 [ file2 ... ]";
        }
    }

    class OnChown implements BOnCommand
    {
        public int onCommand( BStringList params, InputStream in, PrintStream out )
        {
			int owner = -1;
			if(params.getCount() < 3)
			{
				out.println("Missing required parameter");
				return 1;
			}
			else
			{
				owner = Integer.parseInt(params.get(1));
				Debug.doPrintln("To owner " + owner);

	            for( int i = 2; i < params.getCount(); i++ )
				{
					String fileName = params.get(i);
					Debug.doPrintln("sending chown for " + fileName);
					out.print(fileName + " ");
					BStatFileCE statEvent = new BStatFileCE(this, fileName, null);
					BBlockingComplete statComplete = dispatchEventServerAndWait(statEvent);
					if(statComplete.getStatus() == BCommandComplete.kSuccess)
					{
						statEvent = (BStatFileCE) statComplete.getEvent();
						if(statEvent.getError() == BRepositoryCE.kErrorNoError)
						{
							BSetSecurityCE securityEvent = new BSetSecurityCE(this, fileName, null);
							securityEvent.setMetaData(statEvent.getMetaData());
							securityEvent.getMetaData().setOwner(owner);
							BBlockingComplete chownComplete = dispatchEventServerAndWait(securityEvent);

							if(chownComplete.getStatus() == BCommandComplete.kSuccess)
							{
								securityEvent = (BSetSecurityCE) chownComplete.getEvent();
								if(securityEvent.getError() == BRepositoryCE.kErrorNoError)
								{
									out.println("OK");
								}
								else
								{
									out.println("Error " + securityEvent.getErrorString());
								}

							}

						}
						else
						{
							out.println("Error " + statEvent.getErrorString());
						}
					}
				}
			}
            return 0;
        }

        public String getHelp( BStringList params )
        {
            return "Change ownership of files in the repository\n" +
			       "usage: chown newowner file1 [file2 ...]";
        }
    }

    class OnChmod implements BOnCommand
    {
        public int onCommand( BStringList params, InputStream in, PrintStream out )
        {
			String action = null;
			boolean add;

			if(params.getCount() < 3)
			{
				out.println("Missing required parameter");
				return 1;
			}
			else
			{
				action = params.get(1);
				if(action.startsWith("+"))
				{
					Debug.doPrintln("Adding descriptor");
					add = true;
					action = action.substring(1);
				}
				else if(action.startsWith("-"))
				{
					Debug.doPrintln("Removing descriptor");
					add = false;
					action = action.substring(1);
				}
				else
				{
					out.println("Invalid action");
					return 1;
				}

				Debug.doPrintln("descriptor " + action);

	            for( int i = 2; i < params.getCount(); i++ )
				{
					String fileName = params.get(i);
					Debug.doPrintln("sending chmod for " + fileName);
					out.print(fileName + " ");
					BStatFileCE statEvent = new BStatFileCE(this, fileName, null);
					BBlockingComplete statComplete = dispatchEventServerAndWait(statEvent);
					if(statComplete.getStatus() == BCommandComplete.kSuccess)
					{
						Debug.doPrintln("got stat info");
						statEvent = (BStatFileCE) statComplete.getEvent();
						Debug.doPrintln("error = " + statEvent.getErrorString());
						if(statEvent.getError() == BRepositoryCE.kErrorNoError)
						{
							BSetSecurityCE securityEvent = new BSetSecurityCE(this, fileName, null);
							securityEvent.setMetaData(statEvent.getMetaData());

							boolean cont = false;

							if(add)
							{
								if(securityEvent.getMetaData().getACL().addDescriptor(action))
								{
									cont = true;
								}
								else
								{
									out.println("malformed descriptor");
								}
							}
							else
							{
								if(securityEvent.getMetaData().getACL().removeDescriptor(action))
								{
									cont = true;
								}
								else
								{
									out.println("descriptor not present");
								}
							}

							if(cont)
							{
								BBlockingComplete chmodComplete = dispatchEventServerAndWait(securityEvent);

								if(chmodComplete.getStatus() == BCommandComplete.kSuccess)
								{
									securityEvent = (BSetSecurityCE) chmodComplete.getEvent();
									if(securityEvent.getError() == BRepositoryCE.kErrorNoError)
									{
										out.println("OK");
									}
									else
									{
										out.println("Error " + securityEvent.getErrorString());
									}
								}
							}
						}
						else
						{
							out.println("Error " + statEvent.getErrorString());
						}
					}
				}
			}
            return 0;
        }

        public String getHelp( BStringList params )
        {
            return "Change permissions of files in the repository\n" +
			       "usage: chmod [+|-][u|g]id=[r|rw] file1 [file2 ...]";
        }
    }









}
