//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/rtp/BResFileModel.java,v 1.15 1999/10/17 22:48:09 tp079541 Exp $
package au.edu.swin.synergy.plugin.rtp;

import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.dialect.event.BDialectChangeCE;
import au.edu.swin.synergy.plugin.dialect.BString;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;

import javax.swing.*;
import javax.swing.table.*;
import java.util.Vector;
import java.util.Date;
import java.io.File;

/**
 * class which supplies the table model for the repository client
 *
 * @author Shane Fitzgibbon
 * @copyright Synergy Software 1999
 */

public class BResFileModel extends AbstractTableModel
{

	/**
	 * The column headers for the table
	 */

	final String [] columnNames = { "Filename", "Date", "Owner", "Type", "Version", "Size" };

	/**
	 * Vector that contains the data
	 */

	Vector data = new Vector();
	Vector fileNames = new Vector();

	ImageIcon fileIcon;
	ImageIcon folderIcon;
	ImageIcon modelIcon;

	public BResFileModel()
	{
		fileIcon = BSwingUtilities.getImage(this, "/au/edu/swin/synergy/plugin/rtp/images/fileIcon.gif");
		folderIcon = BSwingUtilities.getImage(this, "/au/edu/swin/synergy/plugin/rtp/images/folderIcon.gif");
		modelIcon = BSwingUtilities.getImage(this, "/au/edu/swin/synergy/plugin/workspace/icons/model.gif");
	}

	/**
	 * gets the number of rows in the table model
	 *
	 * @returns the number of rows
	 */
	public int getRowCount()
	{
		return data.size();
	}

	/**
	 * gets the number of columns in the table model
	 *
	 * @returns the number of columns
	 */
	public int getColumnCount()
	{
		return columnNames.length;
	}

	/**
	 * gets the name of each of the columns
	 *
	 * @param index the index of the column
	 */
	public String getColumnName( int index )
	{
		return BDialect.getString( this, columnNames[index] );
	}

	/**
	 * gets the value of a specified cell
	 *
	 * @param row the row that the cell is in
	 * @param column the column the cell is in
	 * @returns the data in the cell specified
	 */
	public Object getValueAt( int row, int column )
	{
		Vector value = (Vector) data.get( row );
		if ( column == 3 )
		{
			if( ((String)value.get(column)).equals("d") )
			{
				//return new JLabel(folderIcon);
				return folderIcon;
			}
			else if ( ((String)value.get(column)).equals("cmo") )
			{
				//return model Icon
				return modelIcon;
			}
			else //if( ((String)value.get(column)).equals("t") )
			{
				//return new JLabel(fileIcon);
				return fileIcon;
			}
		}
		else if( column == 1 )
		{
			return new Date( ((Long)value.get(column)).longValue() );
		}//*/
		return value.get( column );
	}

	public Object getFileAt( int row)
	{
		if( row < data.size() )
		{
			Vector value = (Vector) data.get( row );
			return value.get(6);
		}
		Vector value = (Vector) data.get(row -1);
		return value.get(6);
	}

	public String getType(String filename)
	{
		for(int i = 0; i < data.size(); i++)
		{
			Vector record = (Vector)data.get(i);
			if ( ((String)record.get(6)).equals(filename) )
			{
				Debug.doPrintln("BResFileModel:getType:returns: " + record.get(3) );
				return (String)record.get(3);
			}
		}
		return null;
	}

	public int getVersion(String filename)
	{
		for(int i = 0; i < data.size(); i++)
		{
			Vector record = (Vector)data.get(i);
			if ( ((String)record.get(6)).equals(filename) )
			{
				return ((Integer)record.get(4)).intValue();
			}
		}
		return -1;
	}


	/**
	 * Gets the type of class that is in each column of the table model
	 *
	 * @param colIndex the index of the column of the table that contains the requested class
	 * @returns the class of the requested column
	 */
	public Class getColumnClass( int colIndex )
	{
		Object value = getValueAt( 0, colIndex );
		if ( value != null )
		{
			return value.getClass();
		}
		return Object.class;
	}


	/**
	 *  Adds a file to the table model
	 *
	 * @param filename String containing the name of the file
	 * @param date String containing the date teh file was created
	 * @param owner String containing the name of the owner
	 * @param type String containint the type of file that it is
	 * @param verion the version number of the file
	 * @param size the size of the file
	 */
	public void addFile( String filename, long date, int owner, String type, int version, long size )
	{
		if ( !fileExists( filename ) )
		{
			fileNames.add(filename);
			Debug.doPrintln("In BResFileModel:addFile:filename = " + filename);
			Vector value = new Vector( 5 );
			value.add( strip (filename) );
			value.add( new Long(date) );
			value.add( new Integer(owner) );
			value.add( type );
			value.add( new Integer(version) );
			value.add( new Long(size) );
			value.add( filename );
			data.add( value );
			fireTableDataChanged();
		}
	}

	/**
	 * adds a directory to the table model
	 *
	 * @param filename String containing the name of the file
	 * @param date String containing the date teh file was created
	 * @param owner String containing the name of the owner
	 * @param type String containint the type of file that it is
	 * @param verion the version number of the file
	 */
	 public void addDir( String filename, long date, int owner, String type, int version )
	 {
	 	this.addFile( filename, date, owner, type, version, 0);
	 }


	/**
	 * Clears the contents of the table model.
	 * It then advises all listeners of this change
	 */
	public void clear()
    {
		Debug.doPrintln( "BResFileModel:clear" );
		data.clear();
		fireTableDataChanged();
		fileNames.clear();
    }

    private String strip( String filename )
    {
		if ( filename.lastIndexOf( File.separator ) != -1 )
		{
			return filename.substring( filename.lastIndexOf( File.separator )+1, filename.length() );
		}
		return filename;
	}

	private boolean fileExists(String filename)
	{
		if ( fileNames.indexOf( filename) == -1 )
		{
			return false;
		}
		return true;
	}

}