// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/server/BServer.java,v 1.14 1999/09/07 08:16:06 tp079519 Exp $

package au.edu.swin.synergy.plugin.server;

import java.lang.String;
import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.event.BBootCE;


import au.edu.swin.synergy.kernel.BKernel;

import java.net.ServerSocket;
import java.net.Socket;

import java.io.IOException;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.debug.Assertion;
import au.edu.swin.synergy.plugin.command.event.BRegisterCmdCE;
import au.edu.swin.synergy.plugin.command.BOnCommand;
import au.edu.swin.synergy.lib.BStringList;
import java.io.InputStream;
import java.io.PrintStream;
import au.edu.swin.synergy.plugin.userdb.BUser;
import au.edu.swin.synergy.plugin.server.event.BGetUserCE;
import au.edu.swin.synergy.plugin.server.event.BGetSessionsCE;
import au.edu.swin.synergy.plugin.server.event.BGetNetworkStatusCE;
import au.edu.swin.synergy.plugin.server.event.BRegisterFilterCE;



/**
 * Server Redirector plugin
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 * @rref Tristans Book
 * @see BServerSession
 * @concurrency Single instance, multi-threaded
 * @dispatches
 * @handles
 */


public class BServer extends BPlugin
{
    public static final int kPortNumber = 9999;
    ServerSocket socket = null;
    boolean quit = false;
    
    Hashtable connections = new Hashtable();

	private Vector filters = new Vector();
    
    /**
     * @post Pre-boot config complete
     * @pre kernel has been set
     * @throws none
     * @return void
     * @param void
     */
    
    
    public void setup(BStringList params)
    {
        dispatchEvent( new BRegisterCmdCE( this, "who", new OnWho() ) );
        dispatchEvent( new BRegisterCmdCE( this, "kick", new OnKick() ) );
    }
    
    /**
     * @post the event is dispatched
     * @pre the event is of type kServer only
     * @throws none
     * @return void
     * @param an event from a serversession, and the completion object
     */
    
    public void doProcess( BCommandEvent event, BCommandComplete complete )
    {
        // this is an incoming object.
        
        // Re-dispatch the event...
        event.setCompletionObject( complete );
        dispatchEvent( event );
        
        Debug.doPrintln( "Event dispatched" );
        
    }
    
    /**
     * CommandEvent handler
     * @post the command has been handled
     * @pre the plugin has been loaded
     * @throws none
     * @return void
     * @param source object
     * @param the event
     */
    public void onBCommandEvent( Object source, BCommandEvent event )
    {
        if( event instanceof BBootCE )
        {
            startSocket();
        }
        if( event instanceof BGetUserCE )
        {
            BGetUserCE getUserEvent = ( BGetUserCE ) event;
            BServerSession session = getConnection( getUserEvent.getID() );
			if(session == null)
			{
				getUserEvent.setUser( null );
			}
			else
			{
	            getUserEvent.setUser( session.getUser() );
			}
            getUserEvent.doComplete( BCommandComplete.kSuccess );
        }
        else if( event instanceof BGetSessionsCE )
        {
            BGetSessionsCE getSessionEvent = ( BGetSessionsCE ) event;
            
            synchronized( connections )
            {
                Enumeration en = connections.elements();
                while( en.hasMoreElements() )
                {
                    BServerSession session = ( BServerSession ) en.nextElement();
                    getSessionEvent.addSession( session );
                }
            }
            getSessionEvent.doComplete( BCommandComplete.kSuccess );
        }
		else if( event instanceof BGetNetworkStatusCE)
		{
			BGetNetworkStatusCE statusEvent = (BGetNetworkStatusCE) event;
			statusEvent.setStatus(BGetNetworkStatusCE.kServer);
			synchronized(connections)
			{
				statusEvent.setConnections(connections.size());
			}
			statusEvent.doComplete(BCommandComplete.kSuccess);
		}
		else if( event instanceof BRegisterFilterCE)
		{
			Debug.doPrintln("Registering socket filter");
			BRegisterFilterCE filterEvent = (BRegisterFilterCE) event;
			filters.add(filterEvent.getFilter());
			event.doComplete(BCommandComplete.kSuccess);
		}
        else if( event.getNetworkStatus() == BCommandEvent.kClient )
        {
            Debug.doPrintln( "Server redirector" );
            if( event.getConnectionID() == BCommandEvent.kBroadcast )
            {
                Debug.doPrintln( "Broadcast request" );
                // send to all connections...
                // create an enumeration...
                synchronized( connections )
                {
                    Enumeration en = connections.elements();
                    while( en.hasMoreElements() )
                    {
                        BServerSession session = ( BServerSession ) en.nextElement();
                        Debug.doPrintln( "Sending event" );
                        session.sendEvent( event );
                    }
                }
            }
            else
            {
                Debug.doPrintln( "Specific request" );
                BServerSession session = getConnection( event.getConnectionID() );
                if( session != null )
                {
                    session.sendEvent( event );
                }
                else
                {
                    Debug.doPrintln( "Session not found" );
                }
                
            }
        }
    }
    
    /**
     * get connection by connection ID
     * @post the connection is added to the list of connections
     * @pre the list of connections is created
     * @throws none
     * @return void
     * @param a server session object
     */
    BServerSession getConnection( int connectionID )
    {
        BServerSession session;
        Integer i = new Integer( connectionID );
        synchronized( connections )
        {
            session = ( BServerSession ) connections.get( i );
        }
        return session;
    }
    
    /**
     * add connection to list of connections
     * @post the connection is added to the list of connections
     * @pre the list of connections is created
     * @throws none
     * @return void
     * @param a server session object
     */
    
    void addConnection( BServerSession connection )
    {
        Integer i = new Integer( connection.getID() );
        synchronized( connections )
        {
            connections.put( i, connection );
        }
    }
    /**
     * Please complete the missing tags for removeConnection
     * @param
     * @return
     * @pre
     * @post
     */
    
    void removeConnection( BServerSession connection )
    {
        Integer i = new Integer( connection.getID() );
        synchronized( connections )
        {
            connections.remove( i );
        }
        // send out a message?
    }
    
    
    /**
     * Start the socket which accepts incoming connections
     * @post none
     * @pre plugin is running
     * @throws none
     * @return void
     * @param void
     */
    
    private void startSocket()
    {
        try
        {
            socket = new ServerSocket( kPortNumber );
        }
        catch( Exception E )
        {
            Debug.doPrintln( "ServerSocket: " + E );
            socket = null;
        }
        
        if( socket != null )
        {
            while( ! quit )
            {
                try
                {
                    Socket newSocket = socket.accept();
                    addConnection( new BServerSession( newSocket, this ) );
                }
                catch( Exception E )
                {
                    Debug.doPrintln( "Accepting connection: " + E );
                }
            }
        }
    }
    
    class OnWho implements BOnCommand
    {
        public int onCommand( BStringList params, InputStream in, PrintStream out )
        {
            synchronized( connections )
            {
                Enumeration en = connections.elements();
                while( en.hasMoreElements() )
                {
                    BServerSession session = ( BServerSession ) en.nextElement();
                    BUser user = session.getUser();
                    out.println("" + session.getID() + " " + user.getName() + " (" + user.getFullName() + ")" );
                }
            }
            return 0;
        }
        
        public String getHelp( BStringList params )
        {
            return "List all logged in users";
        }
    }

    class OnKick implements BOnCommand
    {
        public int onCommand( BStringList params, InputStream in, PrintStream out )
        {
			if(params.getCount() > 1)
			{
				for(int i=1; i<params.getCount(); i++)
				{
					boolean kicked = false;
					int id = Integer.parseInt(params.get(i));
					Debug.doPrintln("Kicking " + id);
		            synchronized( connections )
        		    {
                		Enumeration en = connections.elements();
		                while( en.hasMoreElements() )
        		        {
                		    BServerSession session = ( BServerSession ) en.nextElement();
							if(session.getID() == id)
							{
								kicked = true;
								session.doKick();
								break;
							}
		                }
        		    }
					if(!kicked)
					{
						out.println("Session " + id + " not found");
					}
				}
			}
			else
			{
				out.println("Missing session_id");
			}
            return 0;
        }
        
        public String getHelp( BStringList params )
        {
            return "Kick of a logged in user\n" +
			       "Usage: kick session_id [session_id ...]";
        }
    }
	
	public Vector getFilters()
	{
		return filters;
	}
    
}
