// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/server/BServerSession.java,v 1.10 1999/09/05 03:40:58 tp079519 Exp $

package au.edu.swin.synergy.plugin.server;

import java.lang.String;
import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.event.BBootCE;

import au.edu.swin.synergy.kernel.BKernel;

import java.net.Socket;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.EOFException;

import au.edu.swin.synergy.plugin.userdb.event.BAuthUserCE;
import au.edu.swin.synergy.plugin.userdb.BUser;

import java.util.Vector;
import au.edu.swin.synergy.kernel.BCommandComplete;

import au.edu.swin.synergy.debug.Debug;

/**
 * Server session
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 * @rref Tristans Book
 * @see BServer
 * @concurrency one instance & thread per client
 */


public class BServerSession extends Thread
{
    Socket socket = null;
    BServer server = null;
    
    static int nextConnectionID = 1;
    int connectionID;
    
    static int nextIndex = 1;
    
    boolean authenticated = false;
    BNetCredentials credentials = null;
    BUser user = null;
    
    Vector sentEvents = new Vector();
    
    private BSocketObjectPipe pipe = null;
    /**
     * Please complete the missing tags for BServerSession
     * @post
     * @pre
     * @throws
     * @return
     * @param
     */
    
    public BServerSession( Socket fSocket, BServer fServer )
    {
        connectionID = nextConnectionID++;
        socket = fSocket;
        server = fServer;
        
        pipe = new BSocketObjectPipe( socket , true, fServer.getFilters());
        
        start();
    }
    /**
     * Please complete the missing tags for getID
     * @param
     * @return
     * @pre
     * @post
     */
    
    public int getID()
    {
        return connectionID;
    }
    /**
     * Please complete the missing tags for getUser
     * @param
     * @return
     * @pre
     * @post
     */
    
    public BUser getUser()
    {
        return user;
    }
    
    /**
     * @pre the event is of type kClient only
     * Please complete the missing tags for sendEvent
     * @param
     * @return
     * @post
     */
    public void sendEvent( BCommandEvent event )
    {
        try
        {
            Debug.doPrintln( "sending event" );
            synchronized( sentEvents )
            {
                // add the event to a list of pending results...
                event.setIndex( nextIndex++ );
                sentEvents.add( event );
                Debug.doPrintln( "event sent " + sentEvents.size() );
            }
            pipe.doNetDataSend( event );
        }
        catch( IOException exception )
        {
            Debug.doPrintln( "Sending event: " + exception );
            event.doComplete( BCommandComplete.kNetError );
        }
    }
    /**
     * Please complete the missing tags for run
     * @post
     * @pre
     * @throws
     * @return
     * @param
     */
    
    public void run()
    {
        // read stuff....
        boolean quit = false;
        
        // read authentication
        
        while( ! quit )
        {
            try
            {
                BCommandEvent event = pipe.doNetDataRead();
                
                if( event == null )
                {
                    Debug.doPrintln( "Got null object" );
                    quit = true;
                    break;
                }
                //				else if(object instanceof BNetCmdCE)
                else if( event.getNetworkStatus() == BCommandEvent.kServer )
                {
                    event.setPipe( pipe );
                    event.setConnectionID( getID() );
                    // process the event inside the object....?
                    
                    // if we are not authenticated then we only allow authentication objects
                    if( authenticated )
                    {
						Debug.doPrintln("Authenticated so processing event");
                        server.doProcess( event, new Completion() );
                    }
                    else
                    {
                        if( event instanceof BAuthUserCE )
                        {
							Debug.doPrintln("Processing authentication event");
                            // hook the completion of the auth object
                            
                            // allow the event
                            server.doProcess( event, new AuthCompletion() );
                        }
                        else
                        {
							Debug.doPrintln("Client has not authenticated");
                            // close the connection?
                            Completion completion = new Completion();
                            completion.doComplete( event, BCommandComplete.kNetError );
                            
                        }
                    }
                }
                else if( event.getNetworkStatus() == BCommandEvent.kClientDone )
                {
                    // find the original completion event...
                    doComplete( event );
                }
                else
                {
                    Debug.doPrintln( "Then event can't be processed here!" );
                }
            }
            catch( Exception e )
            {
                e.printStackTrace();
                quit = true;
            }
        }
        
        // close it all down...
        try
        {
            pipe = null;
            socket.close();
            socket = null;
        }
        catch( Exception e )
        {
            Debug.doPrintln( "Error closing connection: " , e );
            //            e.printStackTrace ();
        }
        // tell the server that we are finished...?
        server.removeConnection( this );
    }

    /**
     * Force the currect connection to close
     * @pre the connection is established
     * @post the connection is comming tumbling down
     */
	public void doKick()
	{
		// this should do it...
		try
		{
			socket.close();
		}
		catch(Exception e)
		{
			// who cares?
		}
	}
	
    /**
     * Please complete the missing tags for doComplete
     * @param
     * @return
     * @pre
     * @post
     */
    
    private void doComplete( BCommandEvent event )
    {
        // and incoming network event (this should be a responce)
        Debug.doPrintln( "Got a responce" );
        // we need to find the original completeion object...
        
        boolean callCompletion = false;
        
        synchronized( sentEvents )
        {
            for( int i = 0; i < sentEvents.size(); i++ )
            {
                BCommandEvent e = ( BCommandEvent ) sentEvents.get( i );
                if( e.getIndex() == event.getIndex() )
                {
                    event.setCompletionObject( e.getCompletionObject() );
                    callCompletion = true;
                    sentEvents.remove( i );
                    Debug.doPrintln( "" + sentEvents.size() + " left in queue" );
                    break;
                }
            }
        }
        if( callCompletion )
        {
            Debug.doPrintln( "Calling original completion object" );
            event.doComplete( event.getCompletionStatus() );
        }
    }
    
    
    class Completion extends BCommandComplete
    {
        public void doComplete( BCommandEvent event, int status )
        {
            Debug.doPrintln( "event complete" );
            // send back to the originating pipe...
            BSocketObjectPipe pipe = event.getPipe();
            event.setPipe( null );
            event.setNetworkStatus( BCommandEvent.kServerDone );
            event.setCompletionStatus( status );
            
            try
            {
                pipe.doNetDataSend( event );
                Debug.doPrintln( " and sent" );
            }
            catch( IOException exception )
            {
                Debug.doPrintln( "sending event: " + exception );
            }
        }
    }
    
    class AuthCompletion extends BCommandComplete
    {
        public void doComplete( BCommandEvent event, int status )
        {
            if( status == BCommandComplete.kSuccess )
            {
                BAuthUserCE e = ( BAuthUserCE ) event;
                if( e.getStatus() == BAuthUserCE.kSuccess )
                {
                    authenticated = true;
                    credentials = e.getCredentials();
                    user = e.getUser();
                    Debug.doPrintln( "User authenticated" );
                }
                else
                {
                    // auth failed
                    authenticated = false;
                    credentials = null;
                    user = null;
                }
            }
            // call original event...
            Completion completion = new Completion();
            completion.doComplete( event, status );
        }
    }
    
}




