// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/server/BSocketObjectPipe.java,v 1.8 1999/10/04 07:02:12 tp079519 Exp $

package au.edu.swin.synergy.plugin.server;

import java.net.Socket;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.EOFException;

import au.edu.swin.synergy.kernel.BCommandEvent;
import java.util.Vector;
import au.edu.swin.synergy.plugin.server.BSessionSetup;
import au.edu.swin.synergy.debug.Debug;

/**
 * A pipe which uses a socket to pass BCommandEvents over the network
 *
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 * @rref Tristans Book
 * @see BServer
 * @concurrency one instance & thread per client
 */


public class BSocketObjectPipe
{
    Socket socket = null;
    
    /**
     * This is the object which we feed the BNetData Objects
     * through to the other end of the socket
     */
    private ObjectOutputStream objectOut = null;
    
    /**
     * This is the object which provides the BNetData objects
     * from the other end of the socket
     */
    private ObjectInputStream objectIn = null;
    /**
     * Please complete the missing tags for BSocketObjectPipe
     * @post
     * @pre
     * @throws
     * @return
     * @param
     */
    
    public BSocketObjectPipe( Socket fSocket, boolean isServer, Vector filters )
    {
        socket = fSocket;
        
        try
        {
            //Get the low level streams
            OutputStream outputStream = socket.getOutputStream();
            InputStream inputStream = socket.getInputStream();

			// start the filters
			for(int i=0; i<filters.size(); i++)
			{
				Debug.doPrintln("Starting filter no " + i);
				BSessionSetup filter = (BSessionSetup)filters.get(i);
				BStreamSet streams = filter.setStreams(isServer, inputStream, outputStream);
				if(streams != null)
				{
					inputStream = streams.getInputStream();
					outputStream = streams.getOutputStream();
				}
				else
				{
					Debug.doPrintln("Filter failed to perform session setup");
				}
			}
            
            //Initialise the object in and out streams
            objectOut = new ObjectOutputStream( outputStream );
            objectIn = new ObjectInputStream( inputStream );
        }
        catch( Exception e )
        {
            e.printStackTrace();
        }
    }
    /**
     * Please complete the missing tags for doNetDataSend
     * @post
     * @pre
     * @throws
     * @return
     */
    
    /**
     * Sends the given BNetData object to the connected
     * host.
     *
     * @param   data  The data to send
     */
    public void doNetDataSend( BCommandEvent data ) throws IOException
    {
        synchronized( objectOut )
        {
			objectOut.flush();
            objectOut.writeObject( data );
            objectOut.reset();
            // flush the pipe
			objectOut.flush();
        }
        
        
    }
    /**
     * Please complete the missing tags for doPortClose
     * @post
     * @pre
     * @throws
     * @return
     * @param
     */
    
    /**
     * Closes this port so that anyone waiting on a read
     * will be notified that it is no longer in use.
     */
    public void doPortClose()
    {
        try
        {
            //Determine what type of port this is
            if( this.objectIn != null )
            {
                //Close the input stream
                this.objectIn.close();
            }
            else if( this.objectOut != null )
            {
                //Close the output stream
                this.objectOut.close();
            }
        }
        catch( IOException ioException )
        {
            ioException.printStackTrace();
        }
    }
    /**
     * Please complete the missing tags for doNetDataRead
     * @post
     * @pre
     * @throws
     * @param
     */
    
    
    /**
     * Reads the next object from the input stream. If the object
     * sent is not a BNetData object, this method will return null
     *
     * @return The BNetData object sent from the host
     */
    public BCommandEvent doNetDataRead() throws IOException
    {
        Object readObject = null;
        
        try
        {
            //Read the object from the socket
            synchronized( objectIn )
            {
                readObject = objectIn.readObject();
            }
        }
        catch( Exception e )
        {
			//Debug.doPrintln("BSocketObjectPipe:doNetDataRead", e);
            return null;
        }
        
        return( BCommandEvent ) readObject;
    }
}




