// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/splash/BSplashScreen.java,v 1.6 1999/09/08 11:36:26 tp079519 Exp $

package au.edu.swin.synergy.plugin.splash;

import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.event.BBootCE;
import au.edu.swin.synergy.kernel.BKernel;
import au.edu.swin.synergy.lib.BSleep;
import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.debug.Debug;
import java.awt.*;
import javax.swing.*;


/**
 * Splash Screen
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 * @concurrency All AWT is done in the AWT thread
 * @rref
 * @see
 */

public class BSplashScreen extends BPlugin
{
    protected String splashImageLocation = "/au/edu/swin/synergy/plugin/splash/";
    protected String splashImageFile = "beaker.gif";
    protected ImageIcon splashIcon = null;
    
    /**
     * How long to wait before returning.
     * The delay is to give Swing time to display the splash screen
     */
    protected int kInitalWait = 2000;
    
    /**
     * How long to wait before closing the splash screen.
     */
    protected int kFinalWait = 2000;
    
    JLabel splashLabel = null;
    JWindow frame = new JWindow();
	
    /**
     * On setup, the splash screen is shown
     * @param params if a file is supplied, it is used
     */
    
    public void setup( BStringList params )
    {
        if( params.getCount() == 2 )
        {
            Debug.doPrintln( "loading " + params.get( 1 ) );
            splashIcon = new ImageIcon( getClass().getResource( splashImageLocation + params.get( 1 ) ) );
        }
        
        if( splashIcon == null )
        {
            splashIcon = new ImageIcon( getClass().getResource( splashImageLocation + splashImageFile ) );
        }
        
        splashLabel = new JLabel( splashIcon );
        
        try
        {
            SwingUtilities.invokeAndWait( new ShowSplash() );
            BSleep.doSleep( kInitalWait );
            // give it some time to display the window 
        }
        catch( Exception E )
        {
            Debug.doPrintln( "Error showing splash screen" );
        }
    }
	
    /**
     * When the boot event is received, close the splash screem
     * @param source the plugin sending the event
	 * @param event the event object
     */
    
    public void onBCommandEvent( Object source, BCommandEvent event )
    {
        if( event instanceof BBootCE )
        {
            // hide the splash screen
            BSleep.doSleep( kFinalWait );
            // give it some time to display the window manager
            SwingUtilities.invokeLater( new HideSplash() );
            
            // we no longer need any events
            getKernel().doUnRegister( this );
        }
        
    }
    
    public class ShowSplash implements Runnable
    {
        public void run()
        {
            Thread.currentThread().setPriority( Thread.MAX_PRIORITY );
            // show the splash screen
            frame.getContentPane().add( splashLabel );
            // set the size
            frame.setSize( splashIcon.getIconWidth(), splashIcon.getIconHeight() );
            
            Dimension sd = frame.getToolkit().getScreenSize();
            int x = ( sd.width - splashIcon.getIconWidth() ) / 2;
            int y = ( sd.height - splashIcon.getIconHeight() ) / 2;
            
            frame.setLocation( x, y );
            
            frame.show();
            splashLabel.repaint();
        }
    }
    
    
    public class HideSplash implements Runnable
    {
        public void run()
        {
            frame.setVisible( false );
            frame.dispose();
        }
    }
}
