

package au.edu.swin.synergy.plugin.userdb;

/**
 * This exception should be thrown whenever a value is set
 * in an object that doesn't fall within the required paramaters.
 * <p>
 * For example, when you set the privilege of a BPrivileged item
 * the value must be one of a predefined set.
 *
 * @author Tristan Austin
 */
public class BInvalidPropertyException extends Exception
{
    /**
     * Indicates that a String provided is too short
     */
    public static final int kTooShort = 1;

    /**
     * Indicates that the number provided is out of range
     */
    public static final int kInvalidNumber = 2;

    /**
     * The indicator of what was wrong
     */
    private int state;

    /**
     * Creates a new invalid property exception with the
     * given state indicating what went wrong.
     *
     * @param   newState    kTooShort, kInvalidNumber
     */
    public BInvalidPropertyException(int newState)
    {
        this.state = newState;
    }

    /**
     * Gets the state of the exception which indicates
     * the type of problem that ocurrred.
     *
     * @return The state of the exception kTooShort or kInvalidNumber
     */
    public int getState()
    {
        return this.state;
    }
}
