package au.edu.swin.synergy.plugin.userdb;

import java.util.Comparator;

/**
 * This is used to compare two BPrivileged items to determine
 * whether the order they should come in. The way in which they
 * are ordered may be set as either by unique id, or by name.
 *
 * @author Tristan Austin
 */
public class BItemSorter implements Comparator
{
    /**
     * Indicates whether to compare ids or compare user names, true for ids
     */
    private boolean compareIDs = true;

    /**
     * Default constructor creates a Comparator with the
     * sort ids option selected by default.
     */
    public BItemSorter()
    {
    }


    /**
     * Creates the Comparator with the compareIDs flag set to
     * the given value.
     *
     * @param   compareIDs True to compare by ids, false to compare by usernames
     */
    public BItemSorter( boolean compareIDs )
    {
        this.compareIDs = compareIDs;
    }
        
    /**
     * This will compare the two objects on the assumption. If item1 is
     * less than item2 it will return a negative number, if it is the same
     * it will return 0 and if it is greater than the second, it will return a positive
     * number.
     *
     * @param   item1   The first privilege to compare
     * @param   item2   The second item to compare the first one to
     */
    public int compare( Object item1, Object item2 )
    {
        BPrivileged first = ( BPrivileged ) item1;
        BPrivileged second = ( BPrivileged ) item2;
            
        if( this.compareIDs )
        {
            return( int ) ( first.getUniqueID().longValue() - second.getUniqueID().longValue() );
        }
        else
        {
            return first.getName().compareToIgnoreCase( second.getName() );
        }
    }
}
