

package au.edu.swin.synergy.plugin.userdb;

import au.edu.swin.synergy.kernel.BCommandEvent;

import au.edu.swin.synergy.plugin.command.BOnCommand;

import au.edu.swin.synergy.lib.io.BTextdb;
import au.edu.swin.synergy.lib.io.BTextdbRecord;
import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;

import au.edu.swin.synergy.plugin.userdb.event.BDBUpdateCE;

import au.edu.swin.synergy.debug.Debug;

import java.util.Vector;
import java.util.Hashtable;


/**
 * This encapsulates all the file to memory work
 * required by the user database class.
 *
 * @author Tristan Austin
 */
public class BPermanentStorage
{
    private BUserDB userDB;

    /**
     * This 
     */
    public BPermanentStorage(BUserDB userDB)
    {
        this.userDB = userDB;
    }

    /**
     * This will take the contents of the user hashtable and
     * write them to the user file on disk for permanent storage.
     * @post    The file on disk will reflect the contents of memory
     */
    public void writeUserTextdb(Hashtable userDatabase)
    {
        Vector users = this.userDB.getPrivilegedItems(userDatabase);
        Vector groups = null;
        BUser user = null;
        String currentGroup;
        BTextdbRecord record = null;
        BTextdb userTextdb = new BTextdb( BUserDB.kUserDBFileName );
        userTextdb.openFileForWrite( false );

        for( int i = 0; i < users.size(); i++ )
        {
            user = ( BUser ) users.elementAt( i );
            record = new BTextdbRecord();

            //Set all the fields of the record
            record.addElement( user.getUniqueID().toString() );
            record.addElement( user.getName() );
            record.addElement( user.getFullName() );
            record.addElement( user.getAlias() );
            record.addElement( user.getPassword() );
            record.addElement( user.getEmailAddress() );
            record.addElement( String.valueOf( user.getPrivilege() ) );
            record.addElement( user.getComments() );

            //Run through the user and add all their groups
            groups = user.getMembers();

            if( groups != null )
            {
                //Runs through the groups and adds them to the record
                for( int j = 0; j < groups.size(); j++ )
                {
                    currentGroup = ( String ) groups.elementAt( j );
                    record.addElement( currentGroup );
                }
            }

            //Add the record to the database
            userTextdb.addRecord( record );
        }
    }

    /**
     * Writes all the groups to the groups database
     * Please complete the missing tags for writeGroupTextdb
     * @param
     * @return
     * @pre
     * @post
     */
    public void writeGroupTextdb(Hashtable groupDatabase)
    {
        Vector groups = this.userDB.getPrivilegedItems(groupDatabase);
        Vector members = null;
        BTextdbRecord record = null;
        BGroup group = null;
        String currentUser = null;
        BTextdb groupTextdb = new BTextdb( BUserDB.kGroupDBFileName );
        groupTextdb.openFileForWrite( false );


        //Run through the list of groups and add each one to the file
        for( int i = 0; i < groups.size(); i++ )
        {
            group = ( BGroup ) groups.elementAt( i );
            record = new BTextdbRecord();
            record.addElement( group.getUniqueID().toString() );
            record.addElement( group.getName() );
            record.addElement( String.valueOf( group.getPrivilege() ) );
            record.addElement( group.getComments() );

            //Now need to get the members of the group and add those
            members = group.getMembers();

            //Run through and
            if( members != null )
            {

                //Run through all the members of the group and add the to the record
                for( int j = 0; j < members.size(); j++ )
                {
                    currentUser = ( String ) members.elementAt( j );
                    record.addElement( currentUser );
                }
            }

            //Add the record to the database file
            groupTextdb.addRecord( record );
        }
    }

    /**
     * Reads the users details from permanent storage on disk
     * into memory for instant use.
     *
     * @post    The contents of the file are represented in memory
     * @pre The file exists
     */
    public void readUserTextdb(Hashtable userDatabase, Hashtable groupDatabase)
    {
        BTextdb userTextdb = new BTextdb( BUserDB.kUserDBFileName );
        BStandardOutCE standardOut = null;
        Vector groups = null;
        String currentGroupName = null;
        userTextdb.openFileForRead();
        BUser user = null;
        String userName;
        Long uniqueID;
        int privilege;
        int unamedCount = 0;

        Debug.doPrintln( "Reading the user database." );

        for (int i = 0; ; i++)
        {
            try
            {
                BTextdbRecord record = userTextdb.getRecord();
                if( record == null )
                {
                    break;
                }

                user = new BUser( );

                //Need to do privileged stuff first
                uniqueID = Long.valueOf( record.getElement( 0 ));
                userName = record.getElement( 1 ) ;
                privilege = Integer.parseInt( record.getElement( 6 ) );

                //Make sure it has a unique id
                if (this.userDB.locateItem(uniqueID, userDatabase) != null)
                {
                    uniqueID = this.userDB.getNewID(userDatabase);
                }

                //Need to make sure the user name is valid
                try
                {
                    user.setName(userName);
                }
                catch(BInvalidPropertyException invalidException)
                {
                    //Set to a default name
                    user.setName("Unamed " + uniqueID);

                    //The user name was too short
                    standardOut = new BStandardOutCE( this, "User " + i + " in the user database file has an invalid user name. It has been renamed: " + user.getName());

                    //Tells the user about the situation
                    this.userDB.dispatchEvent(standardOut);
                }

                try
                {
                    user.setPrivilege(privilege);
                }
                catch(BInvalidPropertyException invalidException)
                {
                    //Set to the lowest privilege available
                    user.setPrivilege(BPrivileged.kUser);

                    //The user name was too short
                    standardOut = new BStandardOutCE( this, "User " + user.getName() + " in the user database file has an invalid privilege. It has been reset to the lowest privilege");

                    //Tells the user about the situation
                    this.userDB.dispatchEvent(standardOut);
                }

                //Do the rest
                user.setUniqueID(uniqueID);
                user.setFullName( record.getElement( 2 ) );
                user.setAlias( record.getElement( 3 ) );
                user.setPassword( record.getElement( 4 ) );
                user.setEmailAddress( record.getElement( 5 ) );
                user.setComments( record.getElement( 7 ) );

                Debug.doPrintln("UserName: " + user.getName());
                Debug.doPrintln("Password: " + user.getPassword());

                //See if there are any groups to get
                if( record.getCount() >= 8 )
                {
                    groups = new Vector( record.getCount() - 7 );

                    //Runs through the groups and adds them to the record
                    for( int j = 8; j < record.getCount(); j++ )
                    {
                        currentGroupName = record.getElement( j );
                        groups.addElement( currentGroupName );
                    }

                    //Set the groups in the user
                    user.setMembers( groups );
                }

                //Need to make sure that the user and group db are consistant
                BReferentialIntegrity.doCrossReference(user, groupDatabase);

                //Add to the database both under unique id and user name
                userDatabase.put( user.getName(), user );

                Debug.doPrintln( "Adding user " + user.getName() + " password " + record.getElement( 2 ) );
            }
            catch(BInvalidPropertyException invalidException)
            {
                //It should never come to this
                //The user name was too short
                standardOut = new BStandardOutCE( this, "User " + i + " in the user database file had an unrecoverable error. It has been removed from the database.");

                //Tells the user about the situation
                this.userDB.dispatchEvent(standardOut);
            }
        }
        userTextdb.close();
        userTextdb = null;
    }



    /**
     * This will read the groups from the group.txt file and
     * load them into memory
     * Please complete the missing tags for readGroupTextdb
     * @param
     * @return
     * @pre
     * @post
     */
    public void readGroupTextdb(Hashtable groupDatabase, Hashtable userDatabase)
    {
        BTextdb groupTextdb = new BTextdb( BUserDB.kGroupDBFileName );
        BStandardOutCE standardOut = null;
        BGroup group = null;
        String userName = null;
        Long uniqueID;
        int privilege;
        BUser currentUser = null;
        int recordCount;
        String groupName;

        groupTextdb.openFileForRead();

        //Loop through indefinitely
        for (int i = 0; ; i++)
        {
            try
            {
                BTextdbRecord record = groupTextdb.getRecord();
                if( record == null )
                {
                    break;
                }
                //Determine how many columns in the record
                recordCount = record.getCount();

                group = new BGroup( );

                //Need to do privileged stuff first
                uniqueID = Long.valueOf( record.getElement( 0 ));
                groupName = record.getElement( 1 ) ;
                privilege = Integer.parseInt( record.getElement( 2) );

                //Make sure it has a unique id
                if (this.userDB.locateItem(uniqueID, groupDatabase) != null)
                {
                    uniqueID = this.userDB.getNewID(groupDatabase);
                }

                //Need to make sure the user name is valid
                try
                {
                    group.setName(groupName);
                }
                catch(BInvalidPropertyException invalidException)
                {
                    //Set to a default name
                    group.setName("Unamed " + uniqueID);

                    //The user name was too short
                    standardOut = new BStandardOutCE( this, "Group " + i + " in the group database file has an invalid user name. It has been renamed: " + group.getName());

                    //Tells the user about the situation
                    this.userDB.dispatchEvent(standardOut);
                }

                try
                {
                    group.setPrivilege(privilege);
                }
                catch(BInvalidPropertyException invalidException)
                {
                    //Set to the lowest privilege available
                    group.setPrivilege(BPrivileged.kUser);

                    //The user name was too short
                    standardOut = new BStandardOutCE( this, "Group " + group.getName() + " in the group database file has an invalid privilege. It has been reset to the lowest privilege");

                    //Tells the user about the situation
                    this.userDB.dispatchEvent(standardOut);
                }

                group.setUniqueID(uniqueID);
                group.setComments( record.getElement( 3 ) );

                //Need to run through the rest of the record to get the users
                for( int j = 4; j < recordCount; j++ )
                {
                    //Get the user id
                    userName = record.getElement( j );

                    //Get the user from the database
                    currentUser = ( BUser ) userDatabase.get( userName );

                    //Add the user to the database
                    group.addMember( currentUser.toString() );
                }

                //Need to make sure that the user and group db are consistant
                BReferentialIntegrity.doCrossReference(group, userDatabase);

                //Put the group in the group database
                groupDatabase.put( group.getName(), group );
            }
            catch(BInvalidPropertyException invalidException)
            {
                //Set the success state to error and indicate why
                if (invalidException.getState() == BInvalidPropertyException.kTooShort)
                {
                    //The user name was too short
                    standardOut = new BStandardOutCE( this, "Group " + i + " in the group database file has an invalid group name. It will be removed from the database.");
                }
                else if (invalidException.getState() == BInvalidPropertyException.kInvalidNumber)
                {
                    //The privilege given was wrong
                    standardOut = new BStandardOutCE( this, "Group \"" + group.getName() + "\" in the group database file has an invalid privilege. It will be removed from the database.");
                }

                //Dispatch the standard out event
                if (standardOut != null)
                {
                    this.userDB.dispatchEvent(standardOut);
                }
            }
            catch(Exception e)
            {
                e.printStackTrace();
            }
        }
        groupTextdb.close();
    }
}
