// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/userdb/BPrivileged.java,v 1.16 1999/09/16 05:13:58 tp079210 Exp $


package au.edu.swin.synergy.plugin.userdb;

import java.util.Vector;
import au.edu.swin.synergy.debug.Debug;

import java.io.Serializable;

/**
 * Anything that has privileges extends this class. It contains
 * various generic privilege settings that may be applied to
 * such classes as users and groups.
 *
 * @author Tristan Austin
 * Please complete these missing tags
 * @copyright
 */
public abstract class BPrivileged implements Serializable
{
    /**
     * The constant identifying the privilage "user"
     */
    public static final int kUser = 0;
    
    /**
     * The constant identifying the privilage "administrator"
     */
    public static final int kAdministrator = 1;
    
    /**
     * The unique id
     */
    private Long uniqueID;
    
    /**
     * The name of the privileged item
     */
    private String name = null;
    
    /**
     * The privilage associated with this item
     */
    private int privilege = kUser;
    
    /**
     * Indicates whether it requires a unique id or not
     */
    private boolean isNew = false;
    
    /**
     * Comments are anything the creator wishes to include as a description
     */
    private String comments = null;
    
    /**
     * The list of group names this user is a member of or the list of user names in the group
     */
    private Vector members = new Vector();
    
    /**
     * Creates a new privileged object with no values.
     * Please complete the missing tags for BPrivileged
     * @param
     * @return
     * @pre
     * @post
     */
    public BPrivileged()
    {
    }
    
    /**
     * Creates a privileged object with the given name and
     * the ip address of the client machine
     * Please complete the missing tags for BPrivileged
     * @param
     * @return
     * @pre
     * @post
     */
    public BPrivileged( String name ) throws BInvalidPropertyException
    {
        this( null, name );
        this.isNew = true;
    }

    /**
     * Creates an object with the given name and
     * the ip address of the client machine
     * Please complete the missing tags for BPrivileged
     * @param
     * @return
     * @pre
     * @post
     */
    public BPrivileged( Long uniqueID, String name ) throws BInvalidPropertyException
    {
        this.setName(name);
        this.uniqueID = uniqueID;
        this.isNew = false;
    }

    /**
     * Gets the name of this item
     *
     * @return The name
     * Please complete the missing tags for getName
     * @param
     * @pre
     * @post
     */
    public final String getName()
    {
        return this.name;
    }

    /**
     * Sets the name to that specified
     *
     * @param   name The name of this item
     * Please complete the missing tags for setName
     * @return
     * @pre
     * @post
     */
    public final void setName( String name ) throws BInvalidPropertyException
    {
        //Make sure the name is ok
        if ((name != null) && (name.trim().length() > 0))
        {
            //Name is ok
            this.name = name;
        }
        else
        {
            //The name has no length
            throw new BInvalidPropertyException(BInvalidPropertyException.kTooShort);
        }
    }

    /**
     * Gets the unique id of this item
     *
     * @return The unique id identifying this item
     * Please complete the missing tags for getUniqueID
     * @param
     * @pre
     * @post
     */
    public final Long getUniqueID()
    {
        return this.uniqueID;
    }

    /**
     * Sets the unique id of this item
     *
     * @param The unique id identifying this item
     * Please complete the missing tags for setUniqueID
     * @return
     * @pre
     * @post
     */
    public final void setUniqueID( Long newID )
    {
        this.uniqueID = newID;
    }

    /**
     * Supplies the privilage associated with this
     *
     * @return The Privilage as defined in this class
     * Please complete the missing tags for getPrivilege
     * @param
     * @pre
     * @post
     */
    public final int getPrivilege()
    {
        return this.privilege;
    }

    /**
     * Sets the privilage of this to that specified
     *
     * @param   privilage The new privilage of this
     * Please complete the missing tags for setPrivilege
     * @return
     * @pre
     * @post
     */
    public final void setPrivilege( int privilege ) throws BInvalidPropertyException
    {
        //Make sure the value given is ok
        if ((privilege == kAdministrator) || (privilege == kUser))
        {
            //The privilege is ok
            this.privilege = privilege;
        }
        else
        {
            //The privilege given doesn't match the required values
            throw new BInvalidPropertyException(BInvalidPropertyException.kInvalidNumber);
        }
    }

    /**
     * Supplies the comments associated with this
     *
     * @return The comments regarding this user
     * Please complete the missing tags for getComments
     * @param
     * @pre
     * @post
     */
    public final String getComments()
    {
        return this.comments;
    }

    /**
     * Sets the comments of this to that specified
     *
     * @param   comments The new comments of this
     * Please complete the missing tags for setComments
     * @return
     * @pre
     * @post
     */
    public final void setComments( String comments )
    {
        this.comments = comments;
    }

    /**
     * Returns the Vector containing the list of memberships
     * relating to this item. This may be a list of groups a user
     * is a member of, or a list of users a group has as members.
     *
     * @return The list of members names
     * Please complete the missing tags for getMembers
     * @param
     * @pre
     * @post
     */
    public final Vector getMembers()
    {
        return this.members;
    }

    /**
     * Takes the Vector containing the list of memberships
     * relating to this item. This may be a list of groups a user
     * is a member of, or a list of users a group has as members.
     *
     * @return The list of members names
     * Please complete the missing tags for setMembers
     * @param
     * @pre
     * @post
     */
    public final void setMembers( Vector members )
    {
        this.members = members;
    }

    /**
     * Indicates whether this requires a unique ID or not. This is
     * the case when the item is being added to the database.
     *
     * @return True if it does, false otherwise
     * Please complete the missing tags for isNew
     * @param
     * @pre
     * @post
     */
    public boolean isNew()
    {
        return this.isNew;
    }

    /**
     * Adds the new member to the list of members
     *
     * @param   newMember   The new member for this privileged item
     * Please complete the missing tags for addMember
     * @return
     * @pre
     * @post
     */
    public final void addMember( String newMember )
    {
        //Make sure we have a list to add to
        if( this.members == null )
        {
            this.members = new Vector();
        }

        //Only if it's not there already
        if( ! this.members.contains( newMember ) )
        {
            //Add the new member
            this.members.addElement( newMember );
        }
    }

    /**
     * This will remove all references to other members
     * in this privileged item.
     *
     * @post    Vector of members will be either null or empty
     * Please complete the missing tags for removeAllMembers
     * @param
     * @return
     * @pre
     */
    public final void removeAllMembers()
    {
        this.members.removeAllElements();
    }

    /**
     * Removes the member from the list of members. If the member
     * doesn't already exist it will be ignored.
     *
     * @param   oldMember   The member to be removed from this privileged item
     * Please complete the missing tags for removeMember
     * @return
     * @pre
     * @post
     */
    public final void removeMember( String oldMember )
    {
        //Add the new member
        this.members.removeElement( oldMember );
     }

    /**
     * Returns a representation of this as a string
     *
     * @return The name of the this item
     * Please complete the missing tags for toString
     * @param
     * @pre
     * @post
     */
    public String toString()
    {
        return this.name;
    }

    /**
     * Determines whether this item is equal to
     * the item passed in.
     *
     * @param   user  The user to compare
     * Please complete the missing tags for equals
     * @return
     * @pre
     * @post
     */
    public final boolean equals( Object item )
    {
        BPrivileged targetItem = ( BPrivileged ) item;
        Debug.doPrintln( "Target Unique ID: " + targetItem.getUniqueID() );

        return this.getUniqueID().equals( targetItem.getUniqueID() );
    }
}

