

package au.edu.swin.synergy.plugin.userdb;

import au.edu.swin.synergy.debug.Debug;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * This class handles all the maintenance
 * of the integrity of the cross references between
 * the users and groups
 *
 * @author Tristan Austin
 */
public class BReferentialIntegrity
{

    /**
     * This will run through all of the members of the given privileged item,
     * locate them in the appropriate database and make sure that
     * they exist in each others member lists.
     *
     * @param   item    The item to update the members of
     * @param   itemDatabase    The database to look in to find the members
     * Please complete the missing tags for doCrossReference
     * @return
     * @pre
     * @post
     */
    protected static void doCrossReference( BPrivileged item, Hashtable itemDatabase )
    {
        Debug.doPrintln( "doCrossReference: item: " + item.toString() + " itemDatabase: " + itemDatabase.toString() );
        Enumeration items = null;
        BPrivileged currentItem = null;
        Vector members = item.getMembers();
        int targetIndex = - 1;

        //Need to remove any duplicates in this items members
        doDuplicateFilter(members);
        
        //Only need to do this if the item has any members at all
        if( members != null )
        {
            items = itemDatabase.elements();
            
            //Run through all the items in the database
            while( items.hasMoreElements() )
            {
                //Get the item
                currentItem = ( BPrivileged ) items.nextElement();
                
                Debug.doPrintln( "Current Item is: " + currentItem.toString() );
                
                //Determine whether the current item is supposed to be a member or not
                if( members.contains( currentItem.getName() ) )
                {
                    Debug.doPrintln( "Adding " + item.getName() + " to group " + currentItem.getName() );
                    Debug.doPrintln( "Members of " + currentItem + ": " + currentItem.getMembers() );
                    //Make sure this item is a member of the current item
                    currentItem.addMember( item.getName() );
                }
                else
                {
                    Debug.doPrintln( "Removing " + item.getName() + " to group " + currentItem.getName() );
                    //Make sure this item is not a member of the current item
                    currentItem.removeMember( item.getName() );
                }
            }
        }
    }

    /**
     * This will take all the groups this user is a member of, as
     * defined by the user itself, and translate the group names to
     * group ids. This new list is stored in the BUser object itself
     * for use later on.
     *
     * @param   user    The user to resolve the groups for
     */
	protected static void resolveGroupNumbers(Hashtable groupDatabase, BUser user)
	{
		Vector ids = new Vector();
		Vector names = user.getMembers();
        String name = null;
        Long id = null;
        BPrivileged item;

		if(names != null)
		{
			for(int i=0; i<names.size(); i++)
			{
				name = (String) names.get(i);
                item = locateItemByName(name, groupDatabase);

                if (item != null)
                {
                    id = item.getUniqueID();
                    ids.add(id);
				}
			}

		}
		user.setGroups(ids);
	}

    /**
     * This method will locate the given name in the given
     * hashtable and return the privileged item it maps to.
     *
     * @param   name    The name of the item to locate in the given hashtable
     * @param   database    The database to search through
     */
    private static BPrivileged locateItemByName( String name, Hashtable database )
    {
		BPrivileged result = (BPrivileged)database.get(name);
        return result;
    }

    /**
     * This will take a vector of items and remove any
     * elements that appear more than once, leaving only
     * one copy of each element in the Vector.
     *
     * @param   items   The Vector to remove duplicates from
     */
    private static void doDuplicateFilter(Vector items)
    {
        Object currentItem;
        int index = -1;

        //Make sure there are items to check
        if (items != null)
        {
            //Run through the items and remove duplicates
            for (int i = 0; i < items.size(); i++)
            {
                currentItem = items.elementAt(i);

                //See if this item exists after this item
                index = items.indexOf(currentItem, i+1);

                //Do the check
                if (index >= 0)
                {
                    //Remove the element and allow it to check this element again
                    items.removeElementAt(index);
                    i--;
                }
            }
        }
    }
}