/**
 * Please complete these missing tags
 * @see
 * @concurrency
 * @rref
 */
// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/userdb/BUserDB.java,v 1.40 1999/10/12 10:57:47 tp079210 Exp $

package au.edu.swin.synergy.plugin.userdb;

import java.lang.String;
import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.event.BBootCE;

import au.edu.swin.synergy.kernel.BKernel;

import au.edu.swin.synergy.lib.UnixCrypt;
import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Collections;
import java.util.Collection;
import java.util.Iterator;
import java.util.AbstractList;

import au.edu.swin.synergy.plugin.userdb.event.BGetUsersCE;
import au.edu.swin.synergy.plugin.userdb.event.BGetGroupsCE;
import au.edu.swin.synergy.plugin.userdb.event.BAuthUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BAddUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BAddGroupCE;
import au.edu.swin.synergy.plugin.userdb.event.BUpdateUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BUpdateGroupCE;
import au.edu.swin.synergy.plugin.userdb.event.BUpdatePrivilegedCE;
import au.edu.swin.synergy.plugin.userdb.event.BChangePasswordCE;
import au.edu.swin.synergy.plugin.userdb.event.BRemoveUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BRemoveGroupCE;
import au.edu.swin.synergy.plugin.userdb.event.BUserDBCE;
import au.edu.swin.synergy.plugin.userdb.event.BAdminCE;
import au.edu.swin.synergy.plugin.userdb.event.BOwnerCE;
import au.edu.swin.synergy.plugin.userdb.event.BUpdateNotificationCE;
import au.edu.swin.synergy.plugin.userdb.event.BGroupUpdatedCE;
import au.edu.swin.synergy.plugin.userdb.event.BUserUpdatedCE;
import au.edu.swin.synergy.plugin.userdb.event.BDBUpdateCE;

import au.edu.swin.synergy.plugin.command.BOnCommand;
import au.edu.swin.synergy.plugin.command.event.BRegisterCmdCE;

import au.edu.swin.synergy.lib.io.BTextdb;
import au.edu.swin.synergy.lib.io.BTextdbRecord;
import au.edu.swin.synergy.lib.BStringList;

import au.edu.swin.synergy.debug.Debug;

import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.plugin.server.BNetCredentials;
import au.edu.swin.synergy.lib.BSleep;


/**
 * User database.
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 */

public class BUserDB extends BPlugin
{
    /**
     * The store of registered users
     */
    private Hashtable userDatabase = new Hashtable();
    
    /**
     * The store of registered groups
     */
    private Hashtable groupDatabase = new Hashtable();
    
    /**
     * The file name of the group database
     */
    protected static final String kGroupDBFileName = "group.txt";
    
    /**
     * The file name of the user database
     */
    protected static final String kUserDBFileName = "user.txt";
    
    /**
     * This is the minimum id for users and groups
     */
    protected static final Long kMinimumID = new Long( 0 );

    /**
     * The security manager handles all the security checking
     */
    private BSecurityManager securityManager = new BSecurityManager(this, this.userDatabase, this.groupDatabase);

    /**
     * This is used to maintain the user and group databases on disk
     */
    private BPermanentStorage storage = new BPermanentStorage(this);
    
    /**
     * Gets all the users in the database and returns
     * them in a Vector
     *
     * @return A Vector of BPrivileged objects
     */
    protected Vector getPrivilegedItems(Hashtable database)
    {
        Vector items = new Vector();
        Enumeration itemEnumeration = null;

        //Get the enumeration
        itemEnumeration = database.elements();

        //Put all the elements in the database in the vector
        while( itemEnumeration.hasMoreElements() )
        {
            items.addElement( itemEnumeration.nextElement() );
        }

        return items;
    }

    /**
     * Please complete the missing tags for setup
     * @post
     * @pre
     * @throws
     * @return
     * @param
     */
    
    public void setup(BStringList params)
    {
        //Load all the users and groups from file
        storage.readUserTextdb(this.userDatabase, this.groupDatabase);
        storage.readGroupTextdb(this.groupDatabase, this.userDatabase);

        //Because the reference check may have changed things, need to save them again
        storage.writeUserTextdb(this.userDatabase);
        storage.writeGroupTextdb(this.groupDatabase);
    }

    /**
     * Any events dispatched by the local kernel will be received through here.
     * The user database only handles events that extend BUserDBCE.
     *
     * @param   source  The source of the event
     * @param   event   The event to handle
     */
    public synchronized void onBCommandEvent( Object source, BCommandEvent event )
    {
        boolean isAdmin = false;
        boolean isOwner = false;
        BUser targetUser = null;

        //See if this is a user database event
        if (event instanceof BUserDBCE)
        {
            //See if any of these events require security
            if (event instanceof BAdminCE)
            {
                //Determine whether the user is an administrator
                isAdmin = this.securityManager.isAdministrator(event);

                //If not admin, see if owner access will suffice
                if (!isAdmin && (event instanceof BOwnerCE))
                {
                    targetUser = (BUser)(((BOwnerCE)event).getItem());

                    //This task can be done by an owner, see if they are the owner
                    isOwner = this.securityManager.isOwner(targetUser, event.getConnectionID());
                }

                //These events may only be processed if the user is authorised
                if (isAdmin || isOwner)
                {
                    //The required status is attained, process events
                    if( event instanceof BRemoveUserCE )
                    {
                        //Remove the given user
                        this.doUserRemove( ( BRemoveUserCE ) event );
                    }
                    else if( event instanceof BAddUserCE )
                    {
                        //Add the new user to the database
                        doUserAdd( ( BAddUserCE ) event );
                    }
                    else if( event instanceof BAddGroupCE )
                    {
                        //Add the group to the database
                        this.doGroupAdd( ( BAddGroupCE ) event );
                    }
                    else if( event instanceof BRemoveGroupCE )
                    {
                        //Remove the group from the database
                        this.doGroupRemove( ( BRemoveGroupCE ) event );
                    }
                    else if( event instanceof BUpdateGroupCE )
                    {
                        //Update the given group
                        this.doGroupUpdate( ( BUpdateGroupCE ) event );
                    }
                    else if( event instanceof BChangePasswordCE )
                    {
                        //Change the password
                        this.securityManager.doPasswordChange( ( BChangePasswordCE ) event);
                    }
                    else if( event instanceof BUpdateUserCE )
                    {
                        //Update the user, indicate whether we have admin or owner access
                        doUserUpdate( ( BUpdateUserCE ) event, isAdmin );
                    }
                }
                else
                {
                    //The user does not have the required access to perform this operation
                    ((BOwnerCE)event).setStatus(BUpdatePrivilegedCE.kLowPrivilege);
                    event.doComplete(BCommandComplete.kError);
                }
            }
            //The reamining services require no privilege at all
            else if( event instanceof BAuthUserCE )
            {
                //Send the authenticate event to the security manager
                this.securityManager.doAuthentication(( BAuthUserCE )event);
            }
            else if( event instanceof BGetGroupsCE )
            {
                //Just get the groups and send back
                BGetGroupsCE e = ( BGetGroupsCE ) event;
                e.setGroups( this.getPrivilegedItems(this.groupDatabase) );
                e.doComplete( BCommandComplete.kSuccess );
            }
            else if( event instanceof BGetUsersCE )
            {
                //Just get the users and send back
                BGetUsersCE e = ( BGetUsersCE ) event;
                e.setUsers( this.getPrivilegedItems(this.userDatabase) );
                e.doComplete( BCommandComplete.kSuccess );
            }
        }
        else if (event instanceof BBootCE)
        {
            //This is the boot event, here we need to register commands with the command line
            BRegisterCmdCE registerCMD = new BRegisterCmdCE(this, "reloaduserdb", new BUserDBCommand(this));
            this.dispatchEvent(registerCMD);
        }
    }

    /**
     * Adds the given group to the group database file.
     *
     * @param   group   The group to add
     * Please complete the missing tags for addGroup
     * @return
     * @pre
     * @post
     */
    private boolean addGroup( BGroup group )
    {
        boolean success = false;

        // if the user name already exists, then return false...
        if( ! groupDatabase.containsKey( group.getName() ) )
        {
            groupDatabase.put( group.getName(), group );
            storage.writeGroupTextdb(this.groupDatabase);
            
            //Flag success state
            success = true;
        }
        
        //Indicate the success of the operation
        return success;
    }
    
    /**
     * This will run through the given hashtable of BPrivileged
     * items and determine the next available unique id assuming
     * that the minimum is kMinimumID
     * Please complete the missing tags for getNewID
     * @param
     * @return
     * @pre
     * @post
     */
    protected Long getNewID( Hashtable itemHashtable )
    {
        Collection items = itemHashtable.values();
        Vector itemList = null;
        BItemSorter sorter = new BItemSorter();
        Iterator itemIterator = null;
        BPrivileged currentItem = null;
        Long newID = new Long( kMinimumID.longValue() );
        
        //Sort the items by unique id
        itemList = new Vector( items );
        Collections.sort( itemList, sorter );
        itemIterator = itemList.iterator();
        
        //Need to run through the items in order to find
        //the first available unique id
        while( itemIterator.hasNext() )
        {
            currentItem = ( BPrivileged ) itemIterator.next();
            
            Debug.doPrintln( "Current existing id is: " + currentItem.getUniqueID() );
            Debug.doPrintln( "Current proposed new id is: " + newID );
            
            //See if the current id is greater than the new id
            if( currentItem.getUniqueID().longValue() > newID.longValue() )
            {
                break;
            }
            else
            {
                newID = new Long( newID.longValue() + 1 );
            }
        }
        
        //Return then new id
        return newID;
    }

    /**
     * This will take the data from the remove user
     * event and attempt to remove the given user and
     * all references to it from the databases.
     *
     * @param event   The remove user event
     * Please complete the missing tags for doUserRemove
     * @return
     * @pre
     * @post
     */
    private void doUserRemove( BRemoveUserCE event )
    {
        BUser targetUser = event.getUser();
        BUser existingUser = ( BUser ) this.userDatabase.get( targetUser.getName() );
        BUserUpdatedCE userNotifyCE;

        if (!securityManager.isLoggedIn(targetUser))
        {
            //Make sure that the two users do in fact match
            if( existingUser != null && existingUser.equals( targetUser ))
            {
                //Clean up all references to user
                existingUser.removeAllMembers();

                //Remove all references to this user in the group database
                BReferentialIntegrity.doCrossReference( existingUser, groupDatabase );

                //Remove the user from the user database
                this.userDatabase.remove( existingUser.getName() );

                //Update the persistant storage
                this.storage.writeGroupTextdb(this.groupDatabase);
                this.storage.writeUserTextdb(this.userDatabase);

                event.setStatus( BUpdateUserCE.kSuccess );
                event.doComplete( BCommandComplete.kSuccess );

                //Notify everyone that the user was removed
                userNotifyCE = new BUserUpdatedCE(this, existingUser, BUpdateNotificationCE.kRemoved);
                this.dispatchEventTo(userNotifyCE, BCommandEvent.kBroadcast);
            }
            else
            {
                //The user wasn't found to remove
                event.setStatus( BUpdatePrivilegedCE.kNotFound );
                event.doComplete( BCommandComplete.kError );
            }
        }
        else
        {
            //The user is trying to delete themselves
            event.setStatus( BUpdatePrivilegedCE.kDeleteSelf );
            event.doComplete( BCommandComplete.kError );
        }
    }



    /**
     * This will find the given user in the database
     * and update their details. It will also update
     * the groups to reflect the fact that this user
     * is a member of them.
     *
     * @param   user    The user to update
     * Please complete the missing tags for doUserUpdate
     * @return
     * @pre
     * @post
     */
    private void doUserUpdate( BUpdateUserCE userUpdateCE, boolean isAdministrator )
    {
        BUser user = userUpdateCE.getUser();
        int connectionID = userUpdateCE.getConnectionID();
        Long id = user.getUniqueID();
        BUser existingUser = ( BUser ) this.locateItem( id, userDatabase );
        BUserUpdatedCE userNotifyCE;

        //If the user exists in the database
        if( existingUser != null )
        {
            try
            {
                //The user name, privilege and groups may only be changed by an administrator
                if (isAdministrator)
                {
                    //Need to see if the user name has changed
                    if( ! existingUser.getName().equals( user ) )
                    {
                        //If not, need to update all the references to this user
                        this.doNameUpdate( existingUser.getName(), user.getName(), groupDatabase );
                    }

                    //Only administrators can change privileges and groups
                    existingUser.setPrivilege(user.getPrivilege());
                    existingUser.setName(user.getName());
                    existingUser.setMembers(user.getMembers());
                }

                //Make sure all the groups this user is in contains it
                BReferentialIntegrity.doCrossReference( user, groupDatabase );

                //Need to update the existing user with the new details
                //Only the full name, email address, alias and comments may be editted by owner
                existingUser.setFullName(user.getFullName());
                existingUser.setEmailAddress(user.getEmailAddress());
                existingUser.setAlias(user.getAlias());
                existingUser.setComments(user.getComments());

                //Update both databases
                this.storage.writeUserTextdb(this.userDatabase);
                this.storage.writeGroupTextdb(this.groupDatabase);

                //Set the success state to true
                userUpdateCE.setStatus( BUpdateUserCE.kSuccess );
                userUpdateCE.doComplete( BCommandComplete.kSuccess );

                //Notify everyone that the user was updated
                userNotifyCE = new BUserUpdatedCE(this, existingUser, BUpdateNotificationCE.kUpdated);
                this.dispatchEventTo(userNotifyCE, BCommandEvent.kBroadcast);
            }
            catch(BInvalidPropertyException invalidException)
            {
                //Set the success state to error and indicate why
                if (invalidException.getState() == BInvalidPropertyException.kTooShort)
                {
                    //The user name was too short
                    userUpdateCE.setStatus( BUpdatePrivilegedCE.kNameTooShort );
                }
                else if (invalidException.getState() == BInvalidPropertyException.kInvalidNumber)
                {
                    //The privilege given was wrong
                    userUpdateCE.setStatus( BUpdatePrivilegedCE.kInvalidPrivilege );
                }

                //Call the completion method
                userUpdateCE.doComplete( BCommandComplete.kError );
            }
        }
        else
        {
            //Set the success state to error and indicate why
            userUpdateCE.setStatus( BUpdatePrivilegedCE.kNotFound );
            userUpdateCE.doComplete( BCommandComplete.kError );
        }
    }

    /**
     * This will run through all of the items in the given database and replace
     * all references to the old name with the new one in each items member list.
     *
     * @param   oldName The name to update
     * @param   newName The name to update it to
     * @param   itemDatabase    The database to look in to find the members
     * Please complete the missing tags for doNameUpdate
     * @return
     * @pre
     * @post
     */
    private void doNameUpdate( String oldName, String newName, Hashtable itemDatabase )
    {
        Enumeration items = itemDatabase.elements();
        BPrivileged currentItem = null;
        Vector members = null;
        int targetIndex = - 1;
        
        //Run through all the items in the database
        while( items.hasMoreElements() )
        {
            //Get the item
            currentItem = ( BPrivileged ) items.nextElement();
            
            //Get the members of the current item
            members = currentItem.getMembers();
            
            //Make sure there are members to run through
            if( members != null )
            {
                //Get the members of the item and update references to the old name
                targetIndex = members.indexOf( oldName );
                
                //This will replace the existing name with the new name
                if( targetIndex >= 0 )
                {
                    members.setElementAt( newName, targetIndex );
                }
            }
        }
    }
    

    
    /**
     * This will add the group supplied in the
     * add group command event to the database as
     * well as update the user database with
     * any user data required.
     *
     * @param groupEvent   The add user event
     * Please complete the missing tags for doGroupAdd
     * @return
     * @pre
     * @post
     */
    private void doGroupAdd( BAddGroupCE groupEvent )
    {
        BGroup group = groupEvent.getGroup();
        BGroupUpdatedCE groupNotifyCE;
        String groupName = group.getName();
        

        // if the user name already exists, then return false...
        if( ! groupDatabase.containsKey( group.getName() ) )
        {
            //Need to insert the new id for the user
            group.setUniqueID( this.getNewID( groupDatabase ) );
            groupDatabase.put( group.getName(), group );

            //Make sure all the groups this user is in contains it
            BReferentialIntegrity.doCrossReference( group, userDatabase );

            //Update the persistant storage
            storage.writeUserTextdb(this.userDatabase);
            storage.writeGroupTextdb(this.groupDatabase);
            Debug.doPrintln( "Group " + group.getName() + " added" );

            //Indicate that it was ok
            groupEvent.setStatus( BUpdatePrivilegedCE.kSuccess );
            groupEvent.doComplete( BCommandComplete.kSuccess );

            //Indicate that a new group was added
            groupNotifyCE = new BGroupUpdatedCE(this, group, BUpdateNotificationCE.kAdded);
            this.dispatchEventTo(groupNotifyCE, BCommandEvent.kBroadcast);
        }
        else
        {
            groupEvent.setStatus( BAddGroupCE.kNonUniqueGroupName );
            groupEvent.doComplete( BCommandComplete.kSuccess );
            Debug.doPrintln( "User already exists in database." );
        }
    }
    
    /**
     * This will take the data from the remove user
     * event and attempt to remove the given user and
     * all references to it from the databases.
     *
     * @param event   The remove user event
     * Please complete the missing tags for doGroupRemove
     * @return
     * @pre
     * @post
     */
    private void doGroupRemove( BRemoveGroupCE event )
    {
        BGroup targetGroup = event.getGroup();
        BGroup existingGroup = ( BGroup ) this.groupDatabase.get( targetGroup.getName() );
        BGroupUpdatedCE groupNotify;

        //Make sure that the two users do in fact match
        if( existingGroup != null && existingGroup.equals( targetGroup ) )
        {
            //Clean up all references to user
            existingGroup.removeAllMembers();

            //Remove all references to this user in the group database
            BReferentialIntegrity.doCrossReference( existingGroup, userDatabase );

            //Remove the user from the user database
            this.groupDatabase.remove( existingGroup.getName() );

            //Update the persistant storage
            this.storage.writeGroupTextdb(this.groupDatabase);
            this.storage.writeUserTextdb(this.userDatabase);

            event.setStatus( BUpdatePrivilegedCE.kSuccess );
            event.doComplete( BCommandComplete.kSuccess );

            //Notify everyone that the group was removed
            groupNotify = new BGroupUpdatedCE(this, existingGroup, BUpdateNotificationCE.kRemoved);
            this.dispatchEventTo(groupNotify, BCommandEvent.kBroadcast);
        }
        else
        {
            //The user wasn't found to remove
            event.setStatus( BUpdatePrivilegedCE.kNotFound );
            event.doComplete( BCommandComplete.kError );
        }
    }
    
    /**
     * This will find the given user in the database
     * and update their details. It will also update
     * the groups to reflect the fact that this user
     * is a member of them.
     *
     * @param   user    The user to update
     * Please complete the missing tags for doGroupUpdate
     * @return
     * @pre
     * @post
     */
    private void doGroupUpdate( BUpdateGroupCE groupUpdateCE )
    {
        BGroup group = groupUpdateCE.getGroup();
        Long id = group.getUniqueID();
        BGroup existingGroup = ( BGroup ) this.locateItem( id, groupDatabase );
        BGroupUpdatedCE groupNotifyCE;
        
        //If the user exists in the database
        if( existingGroup != null )
        {
            //Need to see if the user name has changed
            if( ! existingGroup.getName().equals( group ) )
            {
                //If not, need to update all the references to this user
                this.doNameUpdate( existingGroup.getName(), group.getName(), userDatabase );
            }

            //Make sure all the groups this user is in contains it
            BReferentialIntegrity.doCrossReference( group, userDatabase );

            //Remove them and replace with the new key
            this.groupDatabase.remove( existingGroup.getName() );
            this.groupDatabase.put( group.getName(), group );

            //Update both databases
            this.storage.writeUserTextdb(this.userDatabase);
            this.storage.writeGroupTextdb(this.groupDatabase);

            //Set the success state to true
            groupUpdateCE.setStatus( BUpdatePrivilegedCE.kSuccess );
            groupUpdateCE.doComplete( BCommandComplete.kSuccess );

            //Notify everyone that the group was updated
            groupNotifyCE = new BGroupUpdatedCE(this, group, BUpdateNotificationCE.kUpdated);
            this.dispatchEventTo(groupNotifyCE, BCommandEvent.kBroadcast);
        }
        else
        {
            //Set the success state to error and indicate why
            groupUpdateCE.setStatus( BUpdatePrivilegedCE.kNotFound );
            groupUpdateCE.doComplete( BCommandComplete.kError );
        }
    }

    /**
     * This will add the user supplied in the
     * add user command event to the database as
     * well as update the groups database with
     * any group data required.
     *
     * @param userEvent   The add user event
     * Please complete the missing tags for doUserAdd
     * @return
     * @pre
     * @post
     */
    private void doUserAdd( BAddUserCE userEvent )
    {
        BUser user = userEvent.getUser();
        BUserUpdatedCE userNotifyCE;

        // if the user name already exists, then return false...
        if( ! userDatabase.containsKey( user.getName() ) )
        {
            //Need to insert the new id for the user
            user.setUniqueID( this.getNewID( userDatabase ) );
            userDatabase.put( user.getName(), user );

            //Need to encrypt their new password
            user.setPassword( this.securityManager.doPasswordEncrypt( userEvent.getInitialPassword() ) );

            //Make sure all the groups this user is in contains it
            BReferentialIntegrity.doCrossReference( user, groupDatabase );

            storage.writeUserTextdb(this.userDatabase);
            storage.writeGroupTextdb(this.groupDatabase);
            Debug.doPrintln( "User " + user.getName() + " added" );

            //Indicate that it was ok
            userEvent.setStatus( BUpdateUserCE.kSuccess );
            userEvent.doComplete( BCommandComplete.kSuccess );

            //Notify everyone that the user was added
            userNotifyCE = new BUserUpdatedCE(this, user, BUpdateNotificationCE.kAdded);
            this.dispatchEventTo(userNotifyCE, BCommandEvent.kBroadcast);
        }
        else
        {
            userEvent.setStatus( BAddUserCE.kNonUniqueUserName );
            userEvent.doComplete( BCommandComplete.kSuccess );
            Debug.doPrintln( "User already exists in database." );
        }
    }
    /**
     * This will run through the user database looking
     * for the user with the given id. When it finds it,
     * it will return it.
     *
     * @param   userID  The id of the user to locate
     * @return The user with the given id if they exist, null otherwise
     * Please complete the missing tags for locateItem
     * @pre
     * @post
     */
    protected BPrivileged locateItem( Long uniqueID, Hashtable database )
    {
        Enumeration items = database.elements();
        BPrivileged currentItem = null;

        //Run through all users looking for unique id
        while( items.hasMoreElements() )
        {
            currentItem = ( BPrivileged ) items.nextElement();

            //Check if we have found a match
            if( currentItem.getUniqueID().equals( uniqueID ) )
            {
                //Return the user
                return currentItem;
            }
        }

        //Indicate failure
        return null;
    }
 }
