

package au.edu.swin.synergy.plugin.userdb;

import au.edu.swin.synergy.kernel.BCommandEvent;

import au.edu.swin.synergy.plugin.command.BOnCommand;

import au.edu.swin.synergy.lib.BStringList;

import au.edu.swin.synergy.plugin.userdb.event.BDBUpdateCE;



/**
 * This will simply allow the user to refresh
 * the users and groups in memory with what
 * is currently in the file.
 *
 * @author Tristan Austin
 */
public class BUserDBCommand implements BOnCommand
{
    /**
     * The user database to communicate with
     */
    private BUserDB userDB;

    /**
     * Creates a new user command object with the given
     * user database to tell what to do
     *
     * @param   userDB  The database to talk to 
     */
    public BUserDBCommand(BUserDB userDB)
    {
        this.userDB = userDB;
    }


    /**
     * This method is called by the command line when the
     * user types "help userdbreload"
     *
     * @param   paramaters  The paramaters to the help command
     * @returns The instructions for use as a String
     */
    public String getHelp(BStringList paramaters)
    {
        return "Reload the user and group database from disk";
    }

    /**
     * When the user types the command associated with this on command
     * class, this method will be called. It contains a list of the
     * paramaters the user presented as well as the input and output
     * streams to workw with.
     *
     * @param   params  The parmaters given with the command
     * @param   in  The input stream to read from if required
     * @param   out The output stream to write to
     * @return An integer in the range Integer'range
     */
    public int onCommand(BStringList params, java.io.InputStream in, java.io.PrintStream out)
    {
        BDBUpdateCE refreshNotifyCE = new BDBUpdateCE(this);
        out.println("Reloading the users and groups from file...");
        this.userDB.setup(null);
        out.println("Done.");

        //Need to notify everyone to refresh if they want to
        this.userDB.dispatchEventTo(refreshNotifyCE, BCommandEvent.kBroadcast);

        //Indicate done
        return 0;
    }
}
