

package au.edu.swin.synergy.plugin.userdb.event;

import au.edu.swin.synergy.plugin.userdb.BPrivileged;

/**
 * When a user or group is updated in a manner that
 * other plugins may be interested, this event is fired.
 * <p>
 * Plugins may be interested in situations where the user name
 * changes, so they can update the interface. Other situations
 * may be when a group is added or a simple change occurs which
 * results in others changing. For example, if you add a user
 * to a group, both the user and group are updated to reflect
 * this.
 * <p>
 * Similarly, when a users status is changed, the clients
 * may wish to update immediately so as to reflect the change.
 *
 * @author Tristan Austin
 */
public class BUpdateNotificationCE extends BUserDBCE
{
    /**
     * Indicates that the item was added
     */
    public static final int kAdded = 1;

    /**
     * Indicates that the item was updated
     */
    public static final int kUpdated = 2;

    /**
     * Indicates that the item was removed
     */
    public static final int kRemoved = 3;

    /**
     * This is the item that was updated
     */
    private BPrivileged updatedItem = null;

    /**
     * The type of update performed
     */
    private int updateType = -1;

    /**
     * Creates a new update notification event with
     * the source specified.
     *
     * @param   source  The source of the event
     */
    public BUpdateNotificationCE(Object source, BPrivileged updatedItem, int updateType)
    {
        super(source);
        this.updatedItem = updatedItem;
        this.updateType = updateType;
    }

    /**
     * Gets the item that was updated.
     *
     * @return The item updated
     */
    public BPrivileged getUpdatedItem()
    {
        return this.updatedItem;
    }

    /**
     * This will get the type of update that was performed
     * on this item.
     *
     * @return The type of update: kAdded, kRemoved, kUpdated
     */
    public int getUpdateType()
    {
        return this.updateType;
    }
}