// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/userman/BCRUDGroup.java,v 1.14 1999/10/17 15:58:14 tp079210 Exp $

//Title:        User Manager
//Version:
//Copyright:    Copyright (c) 1998
//Author:       Tristan Austin
//Company:      Synergy
//Description:  The user manager for CRUDing users

package au.edu.swin.synergy.plugin.userman;

import java.awt.*;
import javax.swing.*;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Vector;

import au.edu.swin.synergy.debug.Debug;

import au.edu.swin.synergy.lib.BGridBagConstraints;
import au.edu.swin.synergy.lib.BSelectionListener;
import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;

import au.edu.swin.synergy.plugin.mod13.event.BAddWindowCE;
import au.edu.swin.synergy.plugin.mod13.event.BCloseWindowCE;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowAdapter;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowEvent;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;

import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.dialect.BString;

import au.edu.swin.synergy.plugin.userdb.BGroup;
import au.edu.swin.synergy.plugin.userdb.BUser;
import au.edu.swin.synergy.plugin.userdb.BPrivileged;
import au.edu.swin.synergy.plugin.userdb.BInvalidPropertyException;


/**
 * This form provides the fields mapping to
 * the properties of a BGroup object allowing
 * the user to edit them.
 *
 * @author Tristan Austin
 * Please complete these missing tags
 * @copyright
 */
public class BCRUDGroup extends JPanel
{
    JLabel groupLabel = new JLabel();
    JLabel privilegeLabel = new JLabel();
    JPanel usersPanel = new JPanel();
    JLabel usersLabel = new JLabel();
    JLabel commentsLabel = new JLabel();
    JButton removeButton = new JButton();
    JButton selectButton = new JButton();
    JButton okButton = new JButton();
    JButton cancelButton = new JButton();
    JTextField nameField = new JTextField();
    JScrollPane jScrollPane1 = new JScrollPane();
    JScrollPane usersScrollPane = new JScrollPane();
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    GridBagLayout gridBagLayout2 = new GridBagLayout();
    JComboBox privilegeCombo = new JComboBox();
    JList userList = new JList();
    JTextArea commentsTextArea = new JTextArea();
    
    /**
     * The name of the administrator privilege to display
     */
    private String adminDisplay = "Administrator";

    /**
     * The name of the user privilege to display
     */
    private String userDisplay = "User";

    /**
     * When the privilege is invalid, this is shown
     */
    private String invalidPrivilege = "Invalid Privilege";
    
    /**
     * The list of available users to select from
     */
    private Vector availableUsers = null;
    
    /**
     * The group we're editting
     */
    private BGroup group = null;
    
    /**
     * Indicates whether this group is ok to upload
     */
    private boolean isOk = false;
    
    /**
     * This is the list model that we will use
     */
    private DefaultListModel userListModel = new DefaultListModel();
    
    /**
     * Handles program flow when registered buttons are clicked
     */
    private BButtonListener buttonListener = new BButtonListener();
    
    /**
     * The user manager itself
     */
    private BUserMan userManager;
    
    /**
     * Indicates whether we're editting or not
     */
    private boolean editting = false;

    /**
     * Indicates whether we're an administrator editting or not
     */
    private boolean administrator = false;

    /**
     * The member select dialog
     */
    BMemberSelect memberSelect = null;
    
    /**
     * Creates a new CRUD user panel with the user manager
     * passed in to allow events to be dispatched from here.
     *
     * @param   userManager The user manager plugin
     * Please complete the missing tags for BCRUDGroup
     * @return
     * @pre
     * @post
     */
    public BCRUDGroup( BUserMan userManager, boolean editting, boolean administrator )
    {
        this.userManager = userManager;
        this.editting = editting;
        this.administrator = administrator;
        jbInit();
    }
    
    
    /**
     * The constructor of the CRUDGroup class
     * Please complete the missing tags for BCRUDGroup
     * @param
     * @return
     * @pre
     * @post
     */
    public BCRUDGroup()
    {
        try
        {
            jbInit();
        }
        catch( Exception ex )
        {
            ex.printStackTrace();
        }
    }
    
    /**
     * Provides a means for the available users the user can
     * select from to be passed in rather than going to the
     * server to get them.
     *
     * @param   availableUsers A Vector of BUser objects
     * Please complete the missing tags for setAvailableUsers
     * @return
     * @pre
     * @post
     */
    public void setAvailableUsers( Vector availableUsers )
    {
        this.availableUsers = availableUsers;
    }
    
    /**
     * This will set the given user as the one the
     * CRUD User panel will display and make available
     * for editting if required.
     *
     * @param   currentGroup The user to display
     * Please complete the missing tags for setBGroup
     * @return
     * @pre
     * @post
     */
    public void setBGroup( BGroup currentGroup, boolean edit )
    {
        Vector groups = null;
        String groupName = currentGroup.getName();
        String comments = currentGroup.getComments();
        int privilege = currentGroup.getPrivilege();
        
        //Set the currentGroup
        this.group = currentGroup;
        this.isOk = false;
        
        //Set the state of all the fields
        this.nameField.setEnabled( edit && this.administrator);
        this.privilegeCombo.setEnabled( edit && this.administrator );
        this.commentsTextArea.setEnabled( edit && this.administrator );

        if( groupName != null )
        {
            //Set the contents of the field
            this.nameField.setText( groupName );
        }

        if( comments != null )
        {
            this.commentsTextArea.setText( comments );
        }

        //Get the members
        groups = this.group.getMembers();
        this.userListModel.removeAllElements();

        //Make sure there are groups to set
        if( groups != null )
        {
            for( int i = 0; i < groups.size(); i++ )
            {
                this.userListModel.addElement( groups.elementAt( i ) );
            }
        }

        //See if there's a privilege to set
        if( privilege >= 0 )
        {
            //Determine their privilege level
            this.privilegeCombo.setSelectedItem( this.getPrivilege( privilege ) );
        }
    }
    
    /**
     * Indicates whether the user object was updated
     * by the panel and needs to be updated on the server
     *
     * @return True if it was, false otherwise
     * Please complete the missing tags for isOk
     * @param
     * @pre
     * @post
     */
    public boolean isOk()
    {
        return this.isOk;
    }

    /**
     * When called, this will make sure all the text components have
     * the correct text as per the current Locale;
     */
    protected void doDialect()
    {
        this.adminDisplay = BDialect.getString(this, "admindisplay");
        this.userDisplay = BDialect.getString(this, "userdisplay");
        this.invalidPrivilege = BDialect.getString(this, "invalidpriv");

        //Do all the labels
        BSwingUtilities.setJLabelText(groupLabel, BDialect.getString(this, "name") );
        BSwingUtilities.setJLabelText(privilegeLabel, BDialect.getString(this, "privilege") );
        BSwingUtilities.setJLabelText(commentsLabel, BDialect.getString(this, "comments") );
        BSwingUtilities.setJLabelText(usersLabel, BDialect.getString(this, "users") );

        //Do all the buttons
        BSwingUtilities.setJButtonText(removeButton, BDialect.getString(this, "remove") );
        BSwingUtilities.setJButtonText(selectButton, BDialect.getString(this, "add") );
        BSwingUtilities.setJButtonText(okButton, BDialect.getString("ok") );
        BSwingUtilities.setJButtonText(cancelButton, BDialect.getString("cancel") );

		if (privilegeCombo.getItemCount() > 0)
		{
	        //Set the combo contents
    	    BSwingUtilities.doJComboRemoveAllItems(privilegeCombo);
		}

		synchronized(privilegeCombo)
		{
        	BSwingUtilities.doJComboAddItem(privilegeCombo, this.adminDisplay);
	        BSwingUtilities.doJComboAddItem(privilegeCombo, this.userDisplay);
		}
    }
    
    /**
     * Initialises the user interface
     * Please complete the missing tags for jbInit
     *
     * @param
     * @return
     * @pre
     * @post
     */
    void jbInit()// throws Exception
    {
        usersPanel.setLayout( gridBagLayout2 );
        userList.setModel( this.userListModel );
        commentsTextArea.setLineWrap( true );
        commentsTextArea.setWrapStyleWord( true );
        removeButton.setEnabled( false );
        this.setLayout( gridBagLayout1 );
        this.add( groupLabel, new BGridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( privilegeLabel, new BGridBagConstraints( 0, 1, 1, 1, 0.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( usersPanel, new BGridBagConstraints( 0, 3, 2, 3, 1.0, 1.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 0, 0, 0, 0 ), 0, 0 ) );
        usersPanel.add( usersScrollPane, new BGridBagConstraints( 0, 1, 2, 1, 1.0, 1.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        usersScrollPane.getViewport().add( userList, null );
        usersPanel.add( usersLabel, new BGridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
        , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 5, 16, 0, 0 ), 0, 0 ) );
        usersPanel.add( selectButton, new BGridBagConstraints( 1, 2, 1, 1, 0.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        usersPanel.add( removeButton, new BGridBagConstraints( 0, 2, 1, 1, 1.0, 0.0
        , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( commentsLabel, new BGridBagConstraints( 2, 3, 1, 1, 0.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( okButton, new BGridBagConstraints( 3, 5, 1, 1, 0.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( cancelButton, new BGridBagConstraints( 2, 5, 1, 1, 1.0, 0.0
        , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( nameField, new BGridBagConstraints( 1, 0, 3, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( jScrollPane1, new BGridBagConstraints( 2, 4, 2, 1, 1.0, 1.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        jScrollPane1.getViewport().add( commentsTextArea, null );
        this.add( privilegeCombo, new BGridBagConstraints( 1, 1, 3, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets( 5, 5, 5, 5 ), 0, 0 ) );

        //Set the actionlisteners
        this.okButton.addActionListener( this.buttonListener );
        this.cancelButton.addActionListener( this.buttonListener );
        this.selectButton.addActionListener( this.buttonListener );
        this.removeButton.addActionListener( this.buttonListener );

        //Set the selection listeners
        this.userList.addListSelectionListener( new BSelectionListener( this.userList.getSelectionModel(), this.removeButton, this.administrator));

        //Make sure the appropriate fields are available
        this.doFieldEnabling();
        this.doDialect();
    }

    /**
     * This will make sure all the fields are enabled in
     * the respect to the administrative status of the currently
     * authenticated user.
     */
    private void doFieldEnabling()
    {
        //These are administrator only fields
        nameField.setEnabled(this.administrator);
        privilegeCombo.setEnabled(this.administrator);
        selectButton.setEnabled(this.administrator);
        commentsTextArea.setEnabled(this.administrator);
        okButton.setEnabled(this.administrator);
    }

    /**
     * This will accept the privilege as defined in the
     * BPrivileged class and translate it into a string
     * for display purposes.
     *
     * @param   privilegeConstant
     * @return The privilege in a String representation
     * Please complete the missing tags for getPrivilege
     * @pre
     * @post
     */
    private int getPrivilege( Object privilegeConstant )
    {
        if( privilegeConstant == BCRUDGroup.this.adminDisplay )
        {
            return BPrivileged.kAdministrator;
        }
        else if( privilegeConstant == BCRUDGroup.this.userDisplay )
        {
            return BPrivileged.kUser;
        }
        else
        {
            return - 1;
        }
    }
    
    /**
     * This will accept the privilege as defined in the
     * BPrivileged class and translate it into a string
     * for display purposes.
     *
     * @param   privilegeConstant
     * @return The privilege in a String representation
     * Please complete the missing tags for getPrivilege
     * @pre
     * @post
     */
    private String getPrivilege( int privilegeConstant )
    {
        if( privilegeConstant == BPrivileged.kAdministrator )
        {
            return this.adminDisplay;
        }
        else if( privilegeConstant == BPrivileged.kUser )
        {
            return this.userDisplay;
        }
        else
        {
            return this.invalidPrivilege;
        }
    }
    
    /**
     * Will check that the fields have been entered correctly
     * and set the values of the fields into the group object.
     * It will then indicate that it was succesfull and close the dialog
     * Please complete the missing tags for doOK
     * @param
     * @return
     * @pre
     * @post
     */
    private void doOK()
    {
        Vector users = new Vector( this.userListModel.getSize() );
        BStandardOutCE standardOutCE = null;

        if( this.nameField.isEnabled() )
        {
            try
            {
                //Set the contents of the field
                this.group.setName( this.nameField.getText() );
                this.group.setComments( this.commentsTextArea.getText() );

                //Get the members
                for( int i = 0; i < userListModel.getSize(); i++ )
                {
                    users.addElement( userListModel.getElementAt( i ) );
                }

                //set the members of the user
                this.group.setMembers( users );

                //Determine their privilege level
                this.group.setPrivilege( this.getPrivilege( this.privilegeCombo.getSelectedItem() ) );

                //Set the success state
                this.isOk = true;
                }
            catch(BInvalidPropertyException invalidException)
            {
                //Determine the source of the error
                if (invalidException.getState() == BInvalidPropertyException.kInvalidNumber)
                {
                    standardOutCE = new BStandardOutCE(this, BDialect.getString(this, "invalPriv"));
                }
                else if (invalidException.getState() == BInvalidPropertyException.kTooShort)
                {
                    standardOutCE = new BStandardOutCE(this, BDialect.getString(this, "nogroup"));
                }

                //Dispatch the event
                if (standardOutCE != null)
                {
                    this.userManager.dispatchEvent(standardOutCE);
                }
            }
        }
        
        //Sends the close window event to the window manager
        this.doClose();
    }
    
    /**
     * When called, this method will send a close window event
     * to the window manager which should result in the dialog
     * or window this resides on being closed. Any listeners
     * to this event should be notified of this as well.
     * Please complete the missing tags for doClose
     * @param
     * @return
     * @pre
     * @post
     */
    protected void doClose()
    {
        BCloseWindowCE closeWindowCE = new BCloseWindowCE( this, this );
        this.userManager.dispatchEvent( closeWindowCE );

        if (this.memberSelect != null)
        {
            this.memberSelect.doClose();
        }
    }
    
    /**
     * This will simply close the dialog without
     * saving any changes. It will set the isOk indicating
     * that the data isn't to be updated.
     * Please complete the missing tags for doCancel
     * @param
     * @return
     * @pre
     * @post
     */
    private void doCancel()
    {
        this.isOk = false;
        this.doClose();
    }
    
    /**
     * This will remove the currently selected users
     * from the list of users in this group
     * Please complete the missing tags for removeUsers
     * @param
     * @return
     * @pre
     * @post
     */
    private void removeUsers()
    {
        int first = this.userList.getMinSelectionIndex();
        int last = this.userList.getMaxSelectionIndex();
        
        this.userListModel.removeRange( first, last );
    }
    
    /**
     * This will display the show users dialog allowing
     * the user to select the users that should be apart
     * of this group.
     * Please complete the missing tags for selectUsers
     * @param
     * @return
     * @pre
     * @post
     */
    private void selectUsers()
    {
        memberSelect = new BMemberSelect( this.userManager );
        BAddWindowCE addWindowCE = new BAddWindowCE( this, memberSelect, new BString(this, "memselect") );

        //Add the window listener
        addWindowCE.addWindowListener( new BMemberListener() );
        addWindowCE.setDialog( true );
        addWindowCE.setParentDialog( this );
        
        //Set the groups in the memberSelect
        memberSelect.setMembers( this.availableUsers );
        
        //Send the event to the window manager
        this.userManager.dispatchEvent( addWindowCE );
    }
    
    /**
     * Handles program flow when the user clicks on a registered
     * button.
     *
     * @author Tristan Austin
     */
    private class BButtonListener implements ActionListener
    {
        /**
         * Called when the user clicks on either the cancel or
         * the ok button.
         *
         * @param   actionEvent The event fired
         */
        public void actionPerformed( ActionEvent actionEvent )
        {
            Object source = actionEvent.getSource();

            if( source == BCRUDGroup.this.okButton )
            {
                BCRUDGroup.this.doOK();
            }
            else if( source == BCRUDGroup.this.cancelButton )
            {
                BCRUDGroup.this.doCancel();
            }
            else if( source == BCRUDGroup.this.selectButton )
            {
                BCRUDGroup.this.selectUsers();
            }
            else if( source == BCRUDGroup.this.removeButton )
            {
                BCRUDGroup.this.removeUsers();
            }
        }
    }
    
    /**
     * This will listen to the window the member dialog is on and
     * take control when the window has been closed.
     *
     * @author Tristan Austin
     */
    private class BMemberListener extends BWindowAdapter
    {
        /**
         * This method is called by the window manager when
         * the window the member select panel is on is closed.
         */
        public void windowClosed( BWindowEvent windowEvent )
        {
            BMemberSelect memberSelect = ( BMemberSelect ) windowEvent.getWindowContent();
            Vector selectedUsers = null;
            
            //Make sure the user didn't press cancel
            if( memberSelect.isOk() )
            {
                selectedUsers = memberSelect.getSelectedMembers();
                BCRUDGroup.this.userListModel.removeAllElements();
                
                for( int i = 0; i < selectedUsers.size(); i++ )
                {
                    //Don't add duplicates
                    if (!BCRUDGroup.this.userListModel.contains(selectedUsers.elementAt( i )))
                    {
                        BCRUDGroup.this.userListModel.addElement( selectedUsers.elementAt( i ) );
                    }

                }
            }
        }
    }
}




