// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/userman/BCRUDUser.java,v 1.30 1999/10/12 13:34:21 tp079210 Exp $

//Title:        User Manager
//Version:
//Copyright:    Copyright (c) 1998
//Author:       Tristan Austin
//Company:      Synergy
//Description:  The user manager for CRUDing users

package au.edu.swin.synergy.plugin.userman;

import java.awt.*;
import javax.swing.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Vector;

import au.edu.swin.synergy.lib.BSelectionListener;
import au.edu.swin.synergy.lib.BGridBagConstraints;
import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;
import au.edu.swin.synergy.plugin.userdb.BUser;
import au.edu.swin.synergy.plugin.userdb.BPrivileged;
import au.edu.swin.synergy.plugin.userdb.BInvalidPropertyException;
import au.edu.swin.synergy.plugin.mod13.event.BCloseWindowCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddWindowCE;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowAdapter;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowEvent;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;
import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.dialect.BString;

import au.edu.swin.synergy.debug.Debug;

/**
 * This dialog allows the user to edit the details of a given
 * BUser object. This is not responsible for creating or deleting
 * anything, it just provides a means of editting the details.
 *
 * @author Tristan Austin
 * Please complete these missing tags
 * @copyright
 */
public class BCRUDUser extends JPanel
{
    JLabel userNameLabel = new JLabel();
    JLabel fullNameLabel = new JLabel();
    JLabel aliasLabel = new JLabel();
    JLabel groupsLabel = new JLabel();
    JLabel privilegeLabel = new JLabel();
    JLabel commentsLabel = new JLabel();
    JTextField userNameField = new JTextField();
    JTextField fullNameField = new JTextField();
    JTextField aliasField = new JTextField();
    JScrollPane groupsScrollPane = new JScrollPane();
    JScrollPane commentsScrollPane = new JScrollPane();
    JComboBox privilegeCombo = new JComboBox();
    JButton okButton = new JButton();
    JButton cancelButton = new JButton();
    JTextArea commentsTextArea = new JTextArea();
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    
    /**
     * The button listener
     */
    private BButtonListener buttonListener = new BButtonListener();
    
    /**
     * The name of the administrator privilege to display
     */
    private String adminDisplay = "Administrator";

    /**
     * The name of the user privilege to display
     */
    private String userDisplay = "User";

    /**
     * When the privilege is invalid, this is shown
     */
    private String invalidPrivilege = "Invalid Privilege";
    
    /**
     * The user this CRUD Panel displays
     */
    private BUser user = null;
    
    /**
     * This is the list model that we will use
     */
    private DefaultListModel groupListModel = new DefaultListModel();
    
    /**
     * The list of groups
     */
    private JList groupList = new JList();
    
    /**
     * The available groups to choose from
     */
    private Vector availableGroups = null;
    
    /**
     * Indicates whether the contents were updated succesfully
     */
    private boolean isOk = false;
    
    /**
     * The user manager itself
     */
    private BUserMan userManager;

    /**
     * The member select dialog
     */
    BMemberSelect memberSelect = null;

    
    JButton changePasswordButton = new JButton();
    JButton selectGroupButton = new JButton();
    JButton removeButton = new JButton();

    /**
     * The email address field
     */
    private JTextField emailField = new JTextField();

    /**
     * The email address label
     */
    private JLabel emailLabel = new JLabel();

    /**
     * Indicates whether we're editting or not
     */
    private boolean editting = false;

    /**
     * Indicates whether the current user is an administrator
     */
    private boolean administrator = false;

    /**
     * The password label
     */
    private JLabel passwordLabel = null;

    /**
     * The password field
     */
    private JPasswordField passwordField = null;

    /**
     * The change password dialog
     */
    BChangePassword changePassword = null;

    /**
     * Creates a new CRUD user panel with the user manager
     * passed in to allow events to be dispatched from here.
     *
     * @param   userManager The user manager plugin
     * Please complete the missing tags for BCRUDUser
     * @return
     * @pre
     * @post
     */
    public BCRUDUser( BUserMan userManager, boolean editting, boolean administrator )
    {
        this.userManager = userManager;
        this.editting = editting;
        this.administrator = administrator;
        jbInit();
    }

    /**
     * When called, this will make sure all the text components have
     * the correct text as per the current Locale;
     */
    protected void doDialect()
    {
        this.adminDisplay = BDialect.getString(this, "admindisplay");
        this.userDisplay = BDialect.getString(this, "userdisplay");
        this.invalidPrivilege = BDialect.getString(this, "invalidpriv");

        //Do all the labels
        BSwingUtilities.setJLabelText(userNameLabel, BDialect.getString(this, "username") );
        BSwingUtilities.setJLabelText(fullNameLabel, BDialect.getString(this, "fullname") );
        BSwingUtilities.setJLabelText(emailLabel, BDialect.getString(this, "email") );
        BSwingUtilities.setJLabelText(aliasLabel, BDialect.getString(this, "alias") );
        BSwingUtilities.setJLabelText(groupsLabel, BDialect.getString(this, "groups") );
        BSwingUtilities.setJLabelText(privilegeLabel, BDialect.getString(this, "privilege") );
        BSwingUtilities.setJLabelText(commentsLabel, BDialect.getString(this, "comments") );

        //Only need to do this if there is one
        if (this.passwordLabel != null)
        {
            BSwingUtilities.setJLabelText(this.passwordLabel, BDialect.getString(this, "passwd") );
        }

        //Do all the buttons
        BSwingUtilities.setJButtonText(okButton, BDialect.getString("ok") );
        BSwingUtilities.setJButtonText(cancelButton, BDialect.getString("cancel") );
        BSwingUtilities.setJButtonText(removeButton, BDialect.getString(this, "remove") );
        BSwingUtilities.setJButtonText(selectGroupButton, BDialect.getString(this, "add") );
        BSwingUtilities.setJButtonText(changePasswordButton, BDialect.getString(this, "changepw") );

        //Set the combo contents
		if (privilegeCombo.getItemCount() > 0)
		{
        	BSwingUtilities.doJComboRemoveAllItems(privilegeCombo);
		}

        BSwingUtilities.doJComboAddItem(privilegeCombo, this.userDisplay);
        BSwingUtilities.doJComboAddItem(privilegeCombo, this.adminDisplay);
    }

    /**
     * Please complete the missing tags for BCRUDUser
     * @param
     * @return
     * @pre
     * @post
     */
    
    public BCRUDUser()
    {
        try
        {
            jbInit();
        }
        catch( Exception e )
        {
        }
    }
    
    /**
     * Provides a means for the available groups the user can
     * select from to be passed in rather than going to the
     * server to get them.
     *
     * @param   availableGroups A Vector of BGroup objects
     * Please complete the missing tags for setAvailableGroups
     * @return
     * @pre
     * @post
     */
    public void setAvailableGroups( Vector availableGroups )
    {
        this.availableGroups = availableGroups;
    }
    
    /**
     * This will set the given user as the one the
     * CRUD User panel will display and make available
     * for editting if required.
     *
     * @param   currentUser The user to display
     * Please complete the missing tags for setBUser
     * @return
     * @pre
     * @post
     */
    public void setBUser( BUser currentUser, boolean edit )
    {
        Vector groups = null;
        String userName = currentUser.getName();
        String fullName = currentUser.getFullName();
        String alias = currentUser.getAlias();
        String email = currentUser.getEmailAddress();
        String comments = currentUser.getComments();
        int privilege = currentUser.getPrivilege();
        
        //Set the currentUser
        this.user = currentUser;
        this.isOk = false;
        
        //Set the state of all the fields
        this.userNameField.setEnabled( edit );
        this.fullNameField.setEnabled( edit );
        this.aliasField.setEnabled( edit );
        this.emailField.setEnabled( edit );
        this.groupList.setEnabled( edit );
        this.privilegeCombo.setEnabled( edit );
        this.commentsTextArea.setEnabled( edit );
        
        if( userName != null )
        {
            //Set the contents of the field
            this.userNameField.setText( userName );
        }
        if( fullName != null )
        {
            this.fullNameField.setText( fullName );
        }
        
        if( alias != null )
        {
            this.aliasField.setText( alias );
        }
        
        if( email != null )
        {
            this.emailField.setText( email );
        }
        
        if( comments != null )
        {
            this.commentsTextArea.setText( comments );
        }
        
        //Get the members
        groups = this.user.getMembers();
        this.groupListModel.removeAllElements();
        
        //Make sure there are groups to set
        if( groups != null )
        {
            for( int i = 0; i < groups.size(); i++ )
            {
                BCRUDUser.this.groupListModel.addElement( groups.elementAt( i ) );
            }
        }
       
		Debug.doPrintln("Privilege in set is: " + this.getPrivilege(privilege));
		Debug.doPrintln("The number of items in the combo are: " + this.privilegeCombo.getItemCount());
        //See if there's a privilege to set
        if( privilege >= 0 )
        {
            //Determine their privilege level
			BSwingUtilities.doJComboSetItem(this.privilegeCombo, this.getPrivilege(privilege));
        }

        //Make sure fields are only enabled as required
        this.doFieldEnabling();
    }
    
    /**
     * Indicates whether the user object was updated
     * by the panel and needs to be updated on the server
     *
     * @return True if it was, false otherwise
     * Please complete the missing tags for isOk
     * @param
     * @pre
     * @post
     */
    public boolean isOk()
    {
        return this.isOk;
    }
    /**
     * Please complete the missing tags for jbInit
     * @param
     * @return
     * @pre
     * @post
     */
    
    void jbInit()// throws Exception
    {
        this.setSize( 500, 400 );

        commentsTextArea.setLineWrap( true );
        commentsTextArea.setWrapStyleWord( true );
        removeButton.setEnabled( false );
        changePasswordButton.setEnabled( this.editting );
        groupList.setSelectionMode( ListSelectionModel.SINGLE_INTERVAL_SELECTION );
        groupList.setModel( this.groupListModel );
        this.setLayout( gridBagLayout1 );
        this.add( userNameLabel, new BGridBagConstraints( 0, 0, 2, 1, 1.0, 0.0
        , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( fullNameLabel, new BGridBagConstraints( 0, 1, 2, 1, 1.0, 0.0
        , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( commentsLabel, new BGridBagConstraints( 2, 6, 1, 1, 1.0, 0.0
        , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( privilegeLabel, new BGridBagConstraints( 0, 5, 2, 1, 1.0, 0.0
        , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( groupsLabel, new BGridBagConstraints( 0, 6, 2, 1, 1.0, 0.0
        , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( emailLabel, new BGridBagConstraints( 0, 2, 2, 1, 1.0, 0.0
        , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( emailField, new BGridBagConstraints( 2, 2, 4, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( aliasLabel, new BGridBagConstraints( 0, 3, 2, 1, 1.0, 0.0
        , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( userNameField, new BGridBagConstraints( 2, 0, 4, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( aliasField, new BGridBagConstraints( 2, 3, 4, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( fullNameField, new BGridBagConstraints( 2, 1, 4, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( groupsScrollPane, new BGridBagConstraints( 0, 7, 2, 1, 1.0, 1.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( commentsScrollPane, new BGridBagConstraints( 2, 7, 3, 1, 1.0, 1.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( privilegeCombo, new BGridBagConstraints( 2, 5, 4, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( okButton, new BGridBagConstraints( 4, 8, 1, 1, 0.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( cancelButton, new BGridBagConstraints( 3, 8, 1, 1, 0.0, 0.0
        , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( changePasswordButton, new BGridBagConstraints( 2, 8, 1, 1, 1.0, 0.0
        , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( selectGroupButton, new BGridBagConstraints( 1, 8, 1, 1, 0.0, 0.0
        , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( removeButton, new BGridBagConstraints( 0, 8, 1, 1, 1.0, 0.0
        , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        groupsScrollPane.getViewport().add( this.groupList );
        commentsScrollPane.getViewport().add( this.commentsTextArea );

        //Set the actionlisteners
        this.okButton.addActionListener( this.buttonListener );
        this.cancelButton.addActionListener( this.buttonListener );
        this.selectGroupButton.addActionListener( this.buttonListener );
        this.removeButton.addActionListener( this.buttonListener );
        this.changePasswordButton.addActionListener( this.buttonListener );

        //Set the selection listeners
        this.groupList.addListSelectionListener( new BSelectionListener( this.groupList.getSelectionModel(), removeButton) );

        //Do some more setup
        if( ! this.editting )
        {
            //Create the password label and field
            this.passwordLabel = new JLabel();
            this.passwordField = new JPasswordField();

            //Add the password label and field
            this.add( passwordLabel, new BGridBagConstraints( 0, 4, 2, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
            this.add( passwordField, new BGridBagConstraints( 2, 4, 4, 1, 1.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        }

        //Set up the language text
        this.doDialect();
    }

    /**
     * This will make sure all the fields are enabled in
     * the respect to the administrative status of the currently
     * authenticated user.
     */
    private void doFieldEnabling()
    {
        BButtonEnabler buttonEnabler = new BButtonEnabler();
        buttonEnabler.start();
    }

    /**
     * Work around so I don't have to block in the awt thread
     */
    protected class BButtonEnabler extends Thread
    {
        public void run()
        {
            boolean isCurrentUser = userManager.getCurrentUser().equals(user);

            //These are administrator only fields
            userNameField.setEnabled(administrator);
            groupList.setEnabled(administrator);
            selectGroupButton.setEnabled(administrator);
            privilegeCombo.setEnabled(administrator);

            //These are current user and administrator only fields
            fullNameField.setEnabled(administrator || isCurrentUser);
            emailField.setEnabled(administrator || isCurrentUser);
            aliasField.setEnabled(administrator || isCurrentUser);
            changePasswordButton.setEnabled(editting && (administrator || isCurrentUser));
            commentsTextArea.setEnabled(administrator || isCurrentUser);
            okButton.setEnabled(administrator || isCurrentUser);
        }
    }

    /**
     * This will accept the privilege as defined in the
     * BPrivileged class and translate it into a string
     * for display purposes.
     *
     * @param   privilegeConstant
     * @return The privilege in a String representation
     * Please complete the missing tags for getPrivilege
     * @pre
     * @post
     */
    private int getPrivilege( Object privilegeConstant )
    {
        if( privilegeConstant == BCRUDUser.this.adminDisplay )
        {
            return BPrivileged.kAdministrator;
        }
        else if( privilegeConstant == BCRUDUser.this.userDisplay )
        {
            return BPrivileged.kUser;
        }
        else
        {
            return - 1;
        }
    }
    
    /**
     * This will accept the privilege as defined in the
     * BPrivileged class and translate it into a string
     * for display purposes.
     *
     * @param   privilegeConstant
     * @return The privilege in a String representation
     * Please complete the missing tags for getPrivilege
     * @pre
     * @post
     */
    private String getPrivilege( int privilegeConstant )
    {
        if( privilegeConstant == BPrivileged.kAdministrator )
        {
            return BCRUDUser.this.adminDisplay;
        }
        else if( privilegeConstant == BPrivileged.kUser )
        {
            return BCRUDUser.this.userDisplay;
        }
        else
        {
            return BCRUDUser.this.invalidPrivilege;
        }
    }
    
    /**
     * When called, this method will send a close window event
     * to the window manager which should result in the dialog
     * or window this resides on being closed. Any listeners
     * to this event should be notified of this as well.
     * Please complete the missing tags for doClose
     * @param
     * @return
     * @pre
     * @post
     */
    protected void doClose()
    {
        BCloseWindowCE closeWindowCE = new BCloseWindowCE( this, this );
        this.userManager.dispatchEvent( closeWindowCE );

        if (this.changePassword != null)
        {
            this.changePassword.doClose();
        }

        if (this.memberSelect != null)
        {
            this.memberSelect.doClose();
        }
    }
    
    /**
     * Called when the user clicks on the ok button
     * Please complete the missing tags for doOK
     * @param
     * @return
     * @pre
     * @post
     */
    private void doOK()
    {
        Debug.doPrintln( "OK Pressed." );
        Vector groups = new Vector( this.groupListModel.getSize() );
        BStandardOutCE standardOutCE = null;

        if( this.userNameField.isEnabled() )
        {
            try
            {
                //Set the contents of the field
                this.user.setName( this.userNameField.getText() );
                this.user.setFullName( this.fullNameField.getText() );
                this.user.setAlias( this.aliasField.getText() );
                this.user.setEmailAddress( this.emailField.getText() );
                this.user.setComments( this.commentsTextArea.getText() );

                //If adding, set the password
                if( ! this.editting )
                {
                    this.user.setPassword( ( new String( ( this.passwordField.getPassword() ) ) ).trim() );
                }

                //Get the members
                for( int i = 0; i < groupListModel.getSize(); i++ )
                {
                    groups.addElement( groupListModel.getElementAt( i ) );
                }

                //set the members of the user
                this.user.setMembers( groups );

                //Determine their privilege level
                this.user.setPrivilege( this.getPrivilege( this.privilegeCombo.getSelectedItem() ) );

                //Set the success state
                this.isOk = true;
            }
            catch(BInvalidPropertyException invalidException)
            {
                //Determine the source of the error
                if (invalidException.getState() == BInvalidPropertyException.kInvalidNumber)
                {
                    standardOutCE = new BStandardOutCE(this, "The privilege selected is not valid");
                }
                else if (invalidException.getState() == BInvalidPropertyException.kTooShort)
                {
                    standardOutCE = new BStandardOutCE(this, "The user name is empty");
                }

                //Dispatch the event
                if (standardOutCE != null)
                {
                    this.userManager.dispatchEvent(standardOutCE);
                }
            }
        }

        //Sends the close window event to the window manager
        this.doClose();
    }

    /**
     * Called when the user clicks on the cancel button
     * Please complete the missing tags for doCancel
     * @param
     * @return
     * @pre
     * @post
     */
    private void doCancel()
    {
        //Sends the close window event to the window manager
        this.doClose();
    }
    
    /**
     * This will display the change password dialog
     * which handles all the processing itself.
     * Please complete the missing tags for doPasswordChange
     * @param
     * @return
     * @pre
     * @post
     */
    private void doPasswordChange()
    {
        changePassword = new BChangePassword( this.userManager, this.user, this.administrator);
        BAddWindowCE addWindow = new BAddWindowCE( this, changePassword, new BString(this, "changepw") );
        
        //Set the dialog properties
        addWindow.setDialog( true );
        addWindow.setParentDialog( this );
        
        //Send the event to the window manager
        this.userManager.dispatchEvent( addWindow );
    }
    
    /**
     * This will remove all the currently selected groups
     * in the group list.
     * Please complete the missing tags for removeGroups
     * @param
     * @return
     * @pre
     * @post
     */
    private void removeGroups()
    {
        int first = this.groupList.getMinSelectionIndex();
        int last = this.groupList.getMaxSelectionIndex();
        
        Debug.doPrintln( "The list model: " + this.groupListModel );
        
        this.groupListModel.removeRange( first, last );
    }
    
    /**
     * This will display the group selection dialog allowing the user
     * to select the groups this member is a member of.
     * Please complete the missing tags for selectGroup
     * @param
     * @return
     * @pre
     * @post
     */
    private void selectGroup()
    {
        memberSelect = new BMemberSelect( this.userManager );
        BAddWindowCE addWindowCE = new BAddWindowCE( this, memberSelect, new BString(this, "MemberSelect") );
        
        //Add the window listener
        addWindowCE.addWindowListener( new BMemberListener() );
        addWindowCE.setDialog( true );
        addWindowCE.setParentDialog( this );
        
        //Set the groups in the memberSelect
        memberSelect.setMembers( this.availableGroups );
        
        //Send the event to the window manager
        this.userManager.dispatchEvent( addWindowCE );
    }
    
    /**
     * This will listen to the window the member dialog is on and
     * take control when the window has been closed.
     *
     * @author Tristan Austin
     */
    private class BMemberListener extends BWindowAdapter
    {
        /**
         * This method is called by the window manager when
         * the window the member select panel is on is closed.
         */
        public void windowClosed( BWindowEvent windowEvent )
        {
            BMemberSelect memberSelect = ( BMemberSelect ) windowEvent.getWindowContent();
            Vector selectedGroups = null;
            
            //Make sure the user didn't press cancel
            if( memberSelect.isOk() )
            {
                selectedGroups = memberSelect.getSelectedMembers();

                for( int i = 0; i < selectedGroups.size(); i++ )
                {
                    //Don't add duplicates
                    if (!BCRUDUser.this.groupListModel.contains(selectedGroups.elementAt( i )))
                    {
                        BCRUDUser.this.groupListModel.addElement( selectedGroups.elementAt( i ) );
                    }
                }
            }
        }
    }
    
    /**
     * Handles program flow when the user clicks on a registered
     * button.
     *
     * @author Tristan Austin
     */
    private class BButtonListener implements ActionListener
    {
        /**
         * Called when the user clicks on either the cancel or
         * the ok button.
         *
         * @param   actionEvent The event fired
         */
        public void actionPerformed( ActionEvent actionEvent )
        {
            Object source = actionEvent.getSource();
            
            Debug.doPrintln( "Button Pressed." );
            
            if( source == BCRUDUser.this.okButton )
            {
                BCRUDUser.this.doOK();
            }
            else if( source == BCRUDUser.this.cancelButton )
            {
                BCRUDUser.this.doCancel();
            }
            else if( source == BCRUDUser.this.selectGroupButton )
            {
                BCRUDUser.this.selectGroup();
            }
            else if( source == BCRUDUser.this.removeButton )
            {
                BCRUDUser.this.removeGroups();
            }
            else if( source == BCRUDUser.this.changePasswordButton )
            {
                BCRUDUser.this.doPasswordChange();
            }
        }
    }
}

