// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/userman/BCRUDUserManager.java,v 1.32 1999/10/12 10:57:28 tp079210 Exp $

//Title:        User Manager
//Version:
//Copyright:    Copyright (c) 1998
//Author:       Tristan Austin
//Company:      Synergy
//Description:  The user manager for CRUDing users

package au.edu.swin.synergy.plugin.userman;

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.border.*;

import java.util.Vector;

import au.edu.swin.synergy.debug.Debug;


import au.edu.swin.synergy.lib.BGridBagConstraints;
import au.edu.swin.synergy.lib.BSelectionListener;
import au.edu.swin.synergy.lib.BListModel;

import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.plugin.userdb.BPrivileged;
import au.edu.swin.synergy.plugin.userdb.BUser;
import au.edu.swin.synergy.plugin.userdb.BGroup;
import au.edu.swin.synergy.plugin.userdb.event.BUpdatePrivilegedCE;
import au.edu.swin.synergy.plugin.userdb.event.BUpdateUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BUpdateGroupCE;
import au.edu.swin.synergy.plugin.userdb.event.BAddUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BAddGroupCE;
import au.edu.swin.synergy.plugin.userdb.event.BRemoveUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BRemoveGroupCE;

import au.edu.swin.synergy.plugin.mod13.event.BAddWindowCE;
import au.edu.swin.synergy.plugin.mod13.event.BCloseWindowCE;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;

import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.dialect.BString;

/**
 * This represents the user manager from the users perspective. It
 * provides a list of all users and groups in the system and
 * a means of editting them all, given the correct privilages.
 *
 * @author Tristan Austin
 * Please complete these missing tags
 * @copyright
 */
public class BCRUDUserManager extends JPanel
{
    GridBagLayout mainGridBagLayout = new GridBagLayout();
    JPanel userPanel = new JPanel();
    JPanel buttonPanel = new JPanel();
    JLabel userLabel = new JLabel();
    JScrollPane userScrollPane = new JScrollPane();
    JList userList = new JList();
    JButton openUserButton = new JButton();
    JButton addUserButton = new JButton();
    JButton deleteUserButton = new JButton();
    JButton okButton = new JButton();
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    JPanel groupPanel = new JPanel();
    JButton addGroupButton = new JButton();
    JLabel groupLabel = new JLabel();
    JScrollPane groupScrollPane = new JScrollPane();
    JButton deleteGroupButton = new JButton();
    JList groupList = new JList();
    JButton openGroupButton = new JButton();
    GridBagLayout gridBagLayout2 = new GridBagLayout();

    /**
     * These listeners enable/disable the delete buttons depending
     * on the privilege of the current user
     */
    BSelectionListener deleteUserSelectionListener = null;
    BSelectionListener deleteGroupSelectionListener = null;

    /**
     * These listeners enable/disable the open buttons depending
     * on the privilege of the current user
     */
    BSelectionListener openUserSelectionListener = null;
    BSelectionListener openGroupSelectionListener = null;

    /**
     * The user editting dialog
     */
    BCRUDUser editUser = null;

    /**
     * The group editting dialog
     */
    BCRUDGroup editGroup = null;

    /**
     * Indicates whether the current user is an administrator
     */
    private boolean isAdministrator = false;
    
    /**
     * The button listener
     */
    private BButtonListener buttonListener = new BButtonListener();
    
    /**
     * The mouse listener for the lists
     */
    private BMouseListener mouseListener = new BMouseListener();
    
    /**
     * The user manager plugin
     */
    private BUserMan userManager = null;
    
    /**
     * The users
     */
    private Vector users = null;

    /**
     * The groups
     */
    private Vector groups = null;
    
    /**
     * This is where the group data is stored
     */
    private BListModel groupListModel = new BListModel();

    /**
     * This is where the user data is stored
     */
    private BListModel userListModel = new BListModel();

    /**
     * Creates a new BCRUDUserManager object providing
     * the user manager plugin as a paramater.
     *
     * @param   userManager The user manager
     * Please complete the missing tags for BCRUDUserManager
     * @return
     * @pre
     * @post
     */
    public BCRUDUserManager( BUserMan userManager )
    {
        this.userManager = userManager;
        jbInit();
    }
    
    /**
     * This accepts a Vector of BUser objects to place in the
     * user list on the user manager.
     *
     * @param   users   The Vector of BUser objects
     * Please complete the missing tags for setUsers
     * @return
     * @pre
     * @post
     */
    public void setUsers( Vector users )
    {
        this.users = users;

        //Send to the awt thread
        BSwingUtilities.setBListModelContents(this.userListModel, this.users );
    }
    
    /**
     * This accepts a Vector of BGroup objects to place in the
     * group list on the user manager.
     *
     * @param   users   The Vector of BGroup objects
     * Please complete the missing tags for setGroups
     * @return
     * @pre
     * @post
     */
    public void setGroups( Vector groups )
    {
        this.groups = groups;

        //Send to the event thread
        BSwingUtilities.setBListModelContents(this.groupListModel, this.groups );
    }

    /**
     * This is called to set the appropriate text of the user manager
     * when the user manager is first initialised or when the Locale changes.
     */
    protected void doDialect()
    {
        BSwingUtilities.setJLabelText(groupLabel, BDialect.getString(this, "groups") );
        BSwingUtilities.setJLabelText(userLabel, BDialect.getString(this, "users") );

        BSwingUtilities.setJButtonText(addGroupButton, BDialect.getString(this, "add") );
        BSwingUtilities.setJButtonText(deleteGroupButton, BDialect.getString(this, "remove") );
        BSwingUtilities.setJButtonText(openGroupButton, BDialect.getString(this, "open") );
        BSwingUtilities.setJButtonText(openUserButton, BDialect.getString(this, "open") );
        BSwingUtilities.setJButtonText(addUserButton, BDialect.getString(this, "add") );
        BSwingUtilities.setJButtonText(deleteUserButton, BDialect.getString(this, "remove") );
        BSwingUtilities.setJButtonText(okButton, BDialect.getString("close") );
    }

    /**
     * Please complete the missing tags for jbInit
     * @param
     * @return
     * @pre
     * @post
     */

    void jbInit()
    {
        this.setLayout( mainGridBagLayout );
        this.setSize( 500, 400 );

        groupPanel.setLayout( gridBagLayout2 );
        groupPanel.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED ) );
        deleteGroupButton.setEnabled(false);
        openGroupButton.setEnabled(false);
        openUserButton.setEnabled(false);
        deleteUserButton.setEnabled(false);

        userPanel.setLayout( gridBagLayout1 );
        userList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        groupList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        this.add( userPanel, new BGridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 0, 0, 0, 0 ), 0, 0 ) );
        userPanel.add( userLabel, new BGridBagConstraints( 0, 0, 1, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        userPanel.add( userScrollPane, new BGridBagConstraints( 0, 1, 3, 1, 1.0, 1.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        userScrollPane.getViewport().add( userList, null );
        userPanel.add( openUserButton, new BGridBagConstraints( 2, 2, 1, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        userPanel.add( addUserButton, new BGridBagConstraints( 1, 2, 1, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        userPanel.add( deleteUserButton, new BGridBagConstraints( 0, 2, 1, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        this.add( groupPanel, new BGridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 0, 0, 0, 0 ), 0, 0 ) );
        groupPanel.add( groupLabel, new BGridBagConstraints( 0, 0, 1, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        groupPanel.add( groupScrollPane, new BGridBagConstraints( 0, 1, 3, 1, 1.0, 1.0
        , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        groupScrollPane.getViewport().add( groupList, null );
        groupPanel.add( openGroupButton, new BGridBagConstraints( 2, 2, 1, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        groupPanel.add( addGroupButton, new BGridBagConstraints( 1, 2, 1, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );
        groupPanel.add( deleteGroupButton, new BGridBagConstraints( 0, 2, 1, 1, 1.0, 0.0
        , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets( 5, 5, 5, 5 ), 0, 0 ) );

        this.add( buttonPanel, new BGridBagConstraints( 0, 1, 2, 1, 1.0, 0.0
        , GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets( 0, 0, 0, 0 ), 0, 0 ) );

        buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
        buttonPanel.add(okButton);

        buttonPanel.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED ) );
        userPanel.setBorder( BorderFactory.createBevelBorder( BevelBorder.RAISED ) );
        groupScrollPane.getViewport();

        //Set the list data models
        this.userList.setModel( this.userListModel );
        this.groupList.setModel( this.groupListModel );

        //Add action listeners
        this.openUserButton.addActionListener( this.buttonListener );
        this.addUserButton.addActionListener( this.buttonListener );
        this.deleteUserButton.addActionListener( this.buttonListener );

        this.openGroupButton.addActionListener( this.buttonListener );
        this.addGroupButton.addActionListener( this.buttonListener );
        this.deleteGroupButton.addActionListener( this.buttonListener );

        this.okButton.addActionListener( this.buttonListener );

        //Create and assign the selection listeners
        deleteUserSelectionListener = new BSelectionListener( this.userList.getSelectionModel(), deleteUserButton);
        this.userList.addListSelectionListener( deleteUserSelectionListener );

        //Create the selection listener
        openUserSelectionListener = new BSelectionListener( this.userList.getSelectionModel(), openUserButton);
        this.userList.addListSelectionListener(openUserSelectionListener);

        //Set up the selection listener for the delete group button
        deleteGroupSelectionListener = new BSelectionListener( this.groupList.getSelectionModel(), deleteGroupButton);
        this.groupList.addListSelectionListener( deleteGroupSelectionListener);

        //Set up the selection listener for the open group button
        openGroupSelectionListener = new BSelectionListener( this.groupList.getSelectionModel(), openGroupButton);
        this.groupList.addListSelectionListener( openGroupSelectionListener );

        //Initialise the enabled states of the buttons
        this.doButtonEnabling();
        this.doDialect();

        //Add the mouse listeners
        this.userList.addMouseListener( this.mouseListener );
        this.groupList.addMouseListener( this.mouseListener );
    }

    /**
     * When called, this method will determine the
     * privilege of the user and set the buttons
     * appropriately.
     */
    protected void doButtonEnabling()
    {
        BButtonEnabler buttonEnabler = new BButtonEnabler();
        buttonEnabler.start();
    }

    /**
     * Work around so I don't have to block in the awt thread
     */
    protected class BButtonEnabler extends Thread
    {
        public void run()
        {
            //Initialise the administrator flags
            BCRUDUserManager.this.isAdministrator = BCRUDUserManager.this.userManager.isAdministrator();

            //Only administrators can modify the groups
            BSwingUtilities.setJComponentEnabled(BCRUDUserManager.this.addGroupButton, BCRUDUserManager.this.isAdministrator);

            //Only administrators can modify all users
            BSwingUtilities.setJComponentEnabled(BCRUDUserManager.this.addUserButton, BCRUDUserManager.this.isAdministrator);

            //Create and assign the selection listeners
            deleteUserSelectionListener.setSelectedState(BCRUDUserManager.this.isAdministrator);

            //Set up the selection listener for the delete group button
            deleteGroupSelectionListener.setSelectedState(BCRUDUserManager.this.isAdministrator);
        }
    }

    /**
     * Whenever the add button is pressed, this method
     * will be called. It will display a dialog containing
     * all the fields for the user to edit the new user.
     *
     * @pre none
     * Please complete the missing tags for doAddUser
     * @param
     * @return
     * @post
     */
    private void doAddUser()
    {
        editUser = new BCRUDUser( this.userManager, false, this.isAdministrator );
        BUser selectedUser = new BUser();
        BUserComplete dialogComplete = new BUserComplete( selectedUser, true );
        BAddWindowCE addDialog = new BAddWindowCE( this, editUser, new BString(this, "createuser"), dialogComplete );
        
        //Set up the event
        editUser.setBUser( selectedUser, true );
        editUser.setAvailableGroups( this.groups );
        addDialog.setDialog( true );
        addDialog.setParentDialog( this );
        this.userManager.dispatchEvent( addDialog );
    }
    
    /**
     * Whenever the edit button is pressed, this method
     * will be called. It will display a dialog containing
     * all the details of the selected user and provide a means
     * for the user to edit it.
     *
     * @pre There is a user already selected
     * Please complete the missing tags for doEditUser
     * @param
     * @return
     * @post
     */
    private void doEditUser()
    {
        BUser selectedUser = ( BUser ) this.userList.getSelectedValue();

        //Only do this if something was selected
        if (selectedUser != null)
        {
            editUser = new BCRUDUser( this.userManager, true, this.isAdministrator );
            BUserComplete dialogComplete = new BUserComplete( selectedUser, false );
            BAddWindowCE addDialog = new BAddWindowCE( this, editUser, new BString(this, "edituser"), dialogComplete );

            //Set up the event
            editUser.setBUser( selectedUser, true );
            editUser.setAvailableGroups( this.groups );
            addDialog.setDialog( true );
            addDialog.setParentDialog( this );
            this.userManager.dispatchEvent( addDialog );
        }
    }

    /**
     * Whenever the add button is pressed, this method
     * will be called. It will display a dialog containing
     * all the fields for the user to edit the new user.
     *
     * @pre none
     * Please complete the missing tags for doAddGroup
     * @param
     * @return
     * @post
     */
    private void doAddGroup()
    {
        editGroup = new BCRUDGroup( this.userManager, false, this.isAdministrator );
        BGroup selectedGroup = new BGroup();
        BGroupComplete dialogComplete = new BGroupComplete( selectedGroup, true );
        BAddWindowCE addDialog = new BAddWindowCE( this, editGroup, new BString(this, "creategroup"), dialogComplete );

        //Set up the event
        editGroup.setBGroup( selectedGroup, true );
        editGroup.setAvailableUsers( this.users );
        addDialog.setDialog( true );
        addDialog.setParentDialog( this );
        this.userManager.dispatchEvent( addDialog );
    }
    
    /**
     * This method will display a dialog containing
     * all the details of the selected group and provide a means
     * for the user to edit it.
     *
     * @pre There is a group already selected
     * Please complete the missing tags for doEditGroup
     * @param
     * @return
     * @post
     */
    private void doEditGroup()
    {
        BGroup selectedGroup = ( BGroup ) this.groupList.getSelectedValue();
        //Make sure a group was selected
        if (selectedGroup != null)
        {
            editGroup = new BCRUDGroup( this.userManager, true, this.isAdministrator );

            BGroupComplete dialogComplete = new BGroupComplete( selectedGroup, false );
            BAddWindowCE addDialog = new BAddWindowCE( this, editGroup, new BString(this, "editgroup"), dialogComplete );

            //Set up the event
            editGroup.setBGroup( selectedGroup, true );
            editGroup.setAvailableUsers( this.users );
            addDialog.setDialog( true );
            addDialog.setParentDialog( this );
            this.userManager.dispatchEvent( addDialog );
        }
    }
    
    /**
     * This will delete the selected user from the
     * database on the storage.
     *
     * @pre The client is connected to the server
     * @pre A user is currently selected
     * @post    The user is removed from storage on the server
     * @post    The user is removed from the list on the client
     * Please complete the missing tags for doDeleteGroup
     * @param
     * @return
     */
    private void doDeleteGroup()
    {
        BGroup selectedGroup = ( BGroup ) this.groupList.getSelectedValue();
        BServerUpdateCompleteCC commandComplete = new BServerUpdateCompleteCC(this.userManager);
        BRemoveGroupCE removeGroupCE = new BRemoveGroupCE( this, selectedGroup, commandComplete );
        
        //Send the remove user event to the server
        this.userManager.dispatchEventServer( removeGroupCE );
    }
    
    /**
     * This will delete the selected user from the
     * database on the storage.
     *
     * @pre The client is connected to the server
     * @pre A user is currently selected
     * @post    The user is removed from storage on the server
     * @post    The user is removed from the list on the client
     * Please complete the missing tags for doDeleteUser
     * @param
     * @return
     */
    private void doDeleteUser()
    {
        BUser selectedUser = ( BUser ) this.userList.getSelectedValue();
        BServerUpdateCompleteCC commandComplete = new BServerUpdateCompleteCC( this.userManager);
        BRemoveUserCE removeUserCE = new BRemoveUserCE( this, selectedUser, commandComplete );
        
        //Send the remove user event to the server
        this.userManager.dispatchEventServer( removeUserCE );
    }

    /**
     * When called, this will send an event to the window manager
     * telling it to close this window.
     */
    protected void doCloseWindow()
    {
        BCloseWindowCE closeWindow = new BCloseWindowCE(this, this);

        //Dispatch the event to the window manager
        this.userManager.dispatchEvent(closeWindow);

        if (this.editGroup != null)
        {
            this.editGroup.doClose();
        }

        if (this.editUser != null)
        {
            this.editUser.doClose();
        }
    }

    /**
     * This class is called by the window manager when
     * the dialog the user is on is closed.
     *
     * @author Tristan Austin
     */
    public class BGroupComplete extends BCommandComplete
    {
        /**
         * The user being updated
         */
        private BGroup group = null;

        /**
         * Indicates whether to add or update
         */
        private boolean adding = false;
        
        /**
         * Creates a new BUserComplete object with the user to
         * update if required and the complete object to send
         * to the server along with the update event.
         *
         * @param   user    The user to update if required
         * @param   adding  True to add the user, false to update it
         */
        public BGroupComplete( BGroup group, boolean adding )
        {
            this.group = group;
            this.adding = adding;
        }
        
        /**
         * Called by the window manager when the dialog the CRUD User
         * panel was on is closed.
         *
         * @param   event   The original event sent to the window manager
         * @param   status  The success status of the event
         */
        public void doComplete( BCommandEvent event, int status )
        {
            BServerUpdateCompleteCC serverComplete = null;
            BCRUDGroup groupPanel = null;
            BCommandEvent sendGroup = null;
            
            groupPanel = ( BCRUDGroup ) ( ( BAddWindowCE ) ( event ) ).getWindowContent();
            
            //See if the user succesfully updated the user
            if( groupPanel.isOk() )
            {
                serverComplete = new BServerUpdateCompleteCC( BCRUDUserManager.this.userManager);
                
                //Determine what to do
                if( this.adding )
                {
                    //Create an add user event
                    sendGroup = new BAddGroupCE( this, this.group, serverComplete );
                }
                else
                {
                    //Create an update user event
                    sendGroup = new BUpdateGroupCE( this, this.group, serverComplete );
                }
                
                //Send the updated user to the server
                BCRUDUserManager.this.userManager.dispatchEventServer( sendGroup );
            }
            //*/
        }
    }
    
    /**
     * This class is called by the window manager when
     * the dialog the user is on is closed.
     *
     * @author Tristan Austin
     */
    public class BUserComplete extends BCommandComplete
    {
        /**
         * The user being updated
         */
        private BUser user = null;
        
        /**
         * Indicates whether to add or update
         */
        private boolean adding = false;
        
        /**
         * Creates a new BUserComplete object with the user to
         * update if required and the complete object to send
         * to the server along with the update event.
         *
         * @param   user    The user to update if required
         * @param   adding  True to add the user, false to update it
         */
        public BUserComplete( BUser user, boolean adding )
        {
            this.user = user;
            this.adding = adding;
        }
        
        /**
         * Called by the window manager when the dialog the CRUD User
         * panel was on is closed.
         *
         * @param   event   The original event sent to the window manager
         * @param   status  The success status of the event
         */
        public void doComplete( BCommandEvent event, int status )
        {
            BServerUpdateCompleteCC serverComplete = null;
            BCRUDUser userPanel = null;
            BCommandEvent sendUser = null;
            
            userPanel = ( BCRUDUser ) ( ( BAddWindowCE ) ( event ) ).getWindowContent();
            
            //See if the user succesfully updated the user
            if( userPanel.isOk() )
            {
                serverComplete = new BServerUpdateCompleteCC( BCRUDUserManager.this.userManager);
                
                //Determine what to do
                if( this.adding )
                {
                    //Create an add user event
                    sendUser = new BAddUserCE( this, this.user, this.user.getPassword(), serverComplete );
                }
                else
                {
                    //Create an update user event
                    sendUser = new BUpdateUserCE( this, this.user, serverComplete );
                }
                
                //Send the updated user to the server
                BCRUDUserManager.this.userManager.dispatchEventServer( sendUser );
            }
        }
    }
    
    
    /**
     * This will detect double clicks on the lists
     * and redirect the program flow to the appropriate
     * edit method for either groups or users.
     *
     * @author Tristan Austin
     */
    private class BMouseListener extends MouseAdapter
    {
        /**
         * When the user clicks the mouse on the list
         * this method is called. It checks to see if
         * it was a double click and if so, sends program
         * flow to the appropriate place.
         *
         * @param   mouseEvent  The mouse event fired
         */
        public void mouseClicked( MouseEvent mouseEvent )
        {
            Object source = mouseEvent.getSource();
            
            //See if it was a double click
            if( mouseEvent.getClickCount() == 2 )
            {
                //Determine whether to edit a user or group
                if( source == BCRUDUserManager.this.userList )
                {
                    //Edit a user
                    BCRUDUserManager.this.doEditUser();
                }
                else if( source == BCRUDUserManager.this.groupList )
                {
                    //Edit a group
                    BCRUDUserManager.this.doEditGroup();
                }
            }
        }
    }
    
    
    /**
     * This will handle all program flow when the user
     * clicks on the buttons on this panel.
     *
     * @author Tristan Austin
     */
    private class BButtonListener implements ActionListener
    {
        /**
         * Called when the registered button is clicked on
         *
         * @param   actionEvent The event fired
         */
        public void actionPerformed( ActionEvent actionEvent )
        {
            Object source = actionEvent.getSource();

            if( source == BCRUDUserManager.this.openUserButton )
            {
                BCRUDUserManager.this.doEditUser();
            }
            else if( source == BCRUDUserManager.this.addUserButton )
            {
                BCRUDUserManager.this.doAddUser();
            }
            else if( source == BCRUDUserManager.this.deleteUserButton )
            {
                BCRUDUserManager.this.doDeleteUser();
            }
            else if( source == BCRUDUserManager.this.openGroupButton )
            {
                BCRUDUserManager.this.doEditGroup();
            }
            else if( source == BCRUDUserManager.this.addGroupButton )
            {
                BCRUDUserManager.this.doAddGroup();
            }
            else if( source == BCRUDUserManager.this.deleteGroupButton )
            {
                BCRUDUserManager.this.doDeleteGroup();
            }
            else if (source == BCRUDUserManager.this.okButton)
            {
                BCRUDUserManager.this.doCloseWindow();
            }
        }
    }
}

