

package au.edu.swin.synergy.plugin.userman;

import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BPlugin;

import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;

import au.edu.swin.synergy.plugin.userdb.event.BUpdatePrivilegedCE;
import au.edu.swin.synergy.plugin.userdb.event.BRemoveUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BRemoveGroupCE;
import au.edu.swin.synergy.plugin.userdb.event.BAddUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BAddGroupCE;
import au.edu.swin.synergy.plugin.userdb.event.BUpdateUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BUpdateGroupCE;

import au.edu.swin.synergy.plugin.dialect.BDialect;

import au.edu.swin.synergy.lib.BListModel;

import au.edu.swin.synergy.debug.Debug;

import javax.swing.*;

/**
 * This class is called by the window manager when
 * the dialog the user is on is closed.
 *
 * @author Tristan Austin
 */
public class BServerUpdateCompleteCC extends BCommandComplete
{
    /**
     * This is the plugin to use to dispatch events via
     */
    private BPlugin dispatcher;

    /**
     * Creates a new server udpate complete object with the
     * given plugin to be used as the dispatcher.
     *
     * @param   dispatcher  The plugin to dispatch events via
     */
    public BServerUpdateCompleteCC(BPlugin dispatcher)
    {
        this.dispatcher = dispatcher;
    }

    /**
     * Called by the window manager when the dialog the CRUD User
     * panel was on is closed.
     *
     * @param   event   The original event sent to the window manager
     * @param   status  The success status of the event
     */
    public void doComplete( BCommandEvent event, int status )
    {
        BUpdatePrivilegedCE userEvent = ( ( BUpdatePrivilegedCE ) event );
        int updateStatus = userEvent.getStatus();
        BStandardOutCE standardOut = null;

        Debug.doPrintln( "Received completion for event: " + event );
        Debug.doPrintln( "Status is: " + updateStatus );

        //Determine the type of update that went on
        if (updateStatus == BUpdateUserCE.kSuccess)
        {
            standardOut = new BStandardOutCE(this, BDialect.getString(this, "success"));
            standardOut.setMessageType(BStandardOutCE.kInfo);
        }
        else if( updateStatus == BUpdatePrivilegedCE.kLowPrivilege )
        {
            standardOut = new BStandardOutCE(this, BDialect.getString(this, "lowPrivilege"));
            standardOut.setMessageType(BStandardOutCE.kError);
        }
        else if( updateStatus == BUpdatePrivilegedCE.kNotFound )
        {
            standardOut = new BStandardOutCE(this, BDialect.getString(this, "noLongerExists"));
            standardOut.setMessageType(BStandardOutCE.kError);
        }
        else if( updateStatus == BUpdatePrivilegedCE.kFailed || (updateStatus == BCommandComplete.kError))
        {
            standardOut = new BStandardOutCE(this, BDialect.getString(this, "unknownError"));
            standardOut.setMessageType(BStandardOutCE.kError);
        }
        else if (updateStatus == BAddGroupCE.kNonUniqueGroupName)
        {
            standardOut = new BStandardOutCE(this, BDialect.getString(this, "alreadyExists"));
            standardOut.setMessageType(BStandardOutCE.kError);
        }
        else if (updateStatus == BUpdatePrivilegedCE.kDeleteSelf)
        {
            standardOut = new BStandardOutCE(this, BDialect.getString(this, "deleteSelf"));
            standardOut.setMessageType(BStandardOutCE.kError);
        }
        else if (updateStatus == BCommandComplete.kNetError)
        {
            standardOut = new BStandardOutCE(this, BDialect.getString(this, "networkError"));
            standardOut.setMessageType(BStandardOutCE.kError);
        }

        //Make sure there is a message to send
        if (standardOut != null)
        {
            //Dispatch the event
            this.dispatcher.dispatchEvent(standardOut);
        }
    }


}
