// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/userman/BUserMan.java,v 1.40 1999/10/17 06:37:45 tp079511 Exp $

package au.edu.swin.synergy.plugin.userman;

import java.lang.String;
import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.event.BBootCE;

import au.edu.swin.synergy.kernel.BKernel;
import au.edu.swin.synergy.debug.Debug;

import java.net.URL;
import java.util.Vector;
import au.edu.swin.synergy.plugin.userdb.BUser;
import au.edu.swin.synergy.plugin.userdb.BGroup;
import au.edu.swin.synergy.plugin.userdb.BPrivileged;

import au.edu.swin.synergy.plugin.userdb.event.BGetUsersCE;
import au.edu.swin.synergy.plugin.userdb.event.BGetGroupsCE;
import au.edu.swin.synergy.plugin.userdb.event.BUserDBCE;
import au.edu.swin.synergy.plugin.userdb.event.BDBUpdateCE;
import au.edu.swin.synergy.plugin.userdb.event.BUpdateNotificationCE;
import au.edu.swin.synergy.plugin.userdb.event.BUserUpdatedCE;
import au.edu.swin.synergy.plugin.userdb.event.BGroupUpdatedCE;
import au.edu.swin.synergy.plugin.userdb.event.BAddUserCE;

import au.edu.swin.synergy.plugin.mod13.event.BAddMenuItemCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddWindowCE;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowAdapter;
import au.edu.swin.synergy.plugin.mod13.bwt.BWindowEvent;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;
import au.edu.swin.synergy.plugin.client.event.BGetUserCE;
import au.edu.swin.synergy.plugin.client.event.BLoginCompleteCE;
import au.edu.swin.synergy.plugin.client.event.BLogoutCompleteCE;

import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.dialect.event.BDialectChangeCE;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JMenuItem;
import javax.swing.tree.DefaultMutableTreeNode;

import au.edu.swin.synergy.plugin.help.BHelpItem;
import au.edu.swin.synergy.plugin.help.BHelpUtil;
import au.edu.swin.synergy.plugin.help.BHelpTreeModel;
import au.edu.swin.synergy.plugin.help.event.BRegisterHelpCE;

import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BBlockingComplete;

import au.edu.swin.synergy.plugin.command.BOnCommand;
import au.edu.swin.synergy.plugin.command.event.BRegisterCmdCE;

import au.edu.swin.synergy.plugin.server.event.BGetNetworkStatusCE;

import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;

import au.edu.swin.synergy.plugin.dialect.BString;

import java.io.InputStream;
import java.io.PrintStream;


/**
 * User manager GUI
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 */

public class BUserMan extends BPlugin
{
    JMenuItem menuItem = new JMenuItem();

    /**
     * The user manager
     */
    BCRUDUserManager userManager = null;

    /**
     * The user manager command line module for users
     */
    private BUserManCL userCommandLine = null;

    /**
     * The window title for the user manager
     */
    private BString windowTitle = null;

    /**
     * The message to display when the user is not connected
     */
    private String notConnectedMsg = null;

    /**
     * The user manager command line module for groups
     */
    private BGroupManCL groupCommandLine = null;

    /**
     * This is the groups to check for administrative privileges
     */
    private Vector availableGroups = new Vector();

    /**
     * Indicates whether we have got users and groups or not
     */
    private boolean isInitialised = false;

    /**
     * This method will be called whenever the current locale
     * changes as we need to refill the text on the interface
     * with values from the appropriate locale.
     */
    protected void doDialect()
    {
        BSwingUtilities.setJMenuItemText(this.menuItem, BDialect.getString(this, "menuitem"));
        this.windowTitle = new BString(this, "windowtitle");
        this.notConnectedMsg = BDialect.getString(this, "notconnected");
    }

    /**
     * Please complete the missing tags for setup
     * @post
     * @pre
     * @throws
     * @return
     * @param
     */
    
    public void setup(BStringList params)
    {
        //Make sure we have some initial values for things
        this.doDialect();
        //Initialise and register the command line
        this.userCommandLine = new BUserManCL(BUserMan.this);
        this.groupCommandLine = new BGroupManCL(BUserMan.this);
        dispatchEvent( new BRegisterCmdCE( this, BDialect.getString(this, "userscmd"), BUserMan.this.userCommandLine ) );
        dispatchEvent( new BRegisterCmdCE( this, BDialect.getString(this, "groupscmd"), BUserMan.this.groupCommandLine ) );

        //Register the menu items
        registerMenu();
    }

    /**
     * Please complete the missing tags for onBCommandEvent
     * @post
     * @pre
     * @throws
     * @return
     * @param
     */

    public void onBCommandEvent( Object source, BCommandEvent event )
    {
		if ( event instanceof BBootCE )
		{
			// Register help.
			registerHelp ();
		}
        else if (event instanceof BUserDBCE)
        {
            if (event instanceof BUpdateNotificationCE)
            {
                //It may be necessary to handle this at a lower level rather
                //than refreshing the entire list of users and groups
                if (event instanceof BUserUpdatedCE)
                {
                    if (this.userManager != null)
                    {
                        //Need to make sure the user manager reflects the privilege of the current user
                        this.userManager.doButtonEnabling();
                    }
                }
                /*else if (event instanceof BGroupUpdatedCE)
                {
                    //We are being notified of an update of some sort
                    Debug.doPrintln("Got a group update event", Debug.kWarningLevel);
                }//*/

                //Refresh the list of users and groups
                this.doRefresh();
            }
            else if (event instanceof BDBUpdateCE)
            {
                //Debug.doPrintln("Got a group update event", Debug.kWarningLevel);
                //The user database on the server has been refreshed, need to refresh
                this.doRefresh();
            }
        }
        else if (event instanceof BDialectChangeCE)
        {
            this.doDialect();
        }
        else if (event instanceof BLoginCompleteCE)
        {
			BSwingUtilities.setJComponentEnabled(this.menuItem, true);
        }
        else if (event instanceof BLogoutCompleteCE)
        {
			BSwingUtilities.setJComponentEnabled(this.menuItem, false);

            //Make sure it's not showing
            if (this.userManager != null)
            {
                this.userManager.doCloseWindow();
                BStandardOutCE standardOut = new BStandardOutCE(this, BDialect.getString(this, "networkError"));
                this.dispatchEvent(standardOut);
            }
        }
    }
    /**
     * Creates and registers the user manager menu item with the window manager
     *
     * @post
     * @pre
     * Please complete the missing tags for registerMenu
     * @param
     * @return
     */
    private void registerMenu()
    {
        menuItem.addActionListener( new UserManagerMenuListener() );
        BAddMenuItemCE event = new BAddMenuItemCE( this, menuItem, new BString(this, "userman"), new BString(this, "adminmenu" ));
        event.setPreferredMenuMnemonic(new BString(this, "menumn"));
        menuItem.setEnabled(this.isLoggedIn());
        dispatchEvent( event );
    }

    /**
     * This will determine whether the user is logged in
     * so that an appropriate message can be displayed
     * to the user if they are not.
     *
     * @returns True if they are, false otherwise
     */
    protected boolean isLoggedIn()
    {
        BBlockingComplete onLoggedIn = new BBlockingComplete();
        BGetNetworkStatusCE networkStatusCE = new BGetNetworkStatusCE(this, onLoggedIn);

        //Dispatch the request
		this.dispatchEvent(networkStatusCE);

        //Block until we get a response
		onLoggedIn.waitForEvent();

        //Return the connection status
        return networkStatusCE.isConnected();
    }

    /**
    * This will indicate whether the currently authenticated
    * user is an administrator or not.
    *
    * @post The isAdministrator and isUser flags will be set in BCRUDUserManager
    * @return True if they are, false otherwise
    */
   protected boolean isAdministrator()
   {
        int privilege;
        boolean isAdmin;
        BGroup currentGroup;
        String currentGroupName;
        BUser currentUser = this.getCurrentUser();
        Vector groups;

        //First check to see if they're logged in at all
        isAdmin = currentUser == null;

        if (!isAdmin)
        {
            //Gets the users privilege
            privilege = currentUser.getPrivilege();

            //If there are no groups available, refresh
            if (!isInitialised)
            {
                this.doRefresh();
            }

            //Do initial check
            isAdmin = (privilege == BPrivileged.kAdministrator);

            //Check the groups this user is a member of
            if (!isAdmin)
            {
                groups = currentUser.getMembers();

                //Run through all the groups and see if any of them have admin privileges
                for (int i = 0; i < groups.size(); i++)
                {
                    currentGroupName = (String)groups.elementAt(i);

                    //Need to locate the group
                    for (int j = 0; j < this.availableGroups.size(); j++)
                    {
                        currentGroup = (BGroup)this.availableGroups.elementAt(j);

                        //If the group matches, do it
                        if (currentGroup.getName().equals(currentGroupName))
                        {
                            //See if this group has admin privileges
                            isAdmin = (currentGroup.getPrivilege() == BPrivileged.kAdministrator);

                            //If we find an admin group, return
                            if (isAdmin)
                            {
                                break;
                            }
                        }
                    }

                    //If we find an admin group, return
                    if (isAdmin)
                    {
                        break;
                    }
                 }
            }            
        }

        //Determine the privilege
        return isAdmin;
   }

    /**
     * This will get the user who is currently logged in
     * to a server. This will be the user that is doing the editting.
     *
     * @returns The currently authenticted user
     */
    protected BUser getCurrentUser()
    {
        BGetUserCE getCurrentUserCE = new BGetUserCE(this, null);

        //Dispatch the event
        BBlockingComplete complete = this.dispatchEventAndWait(getCurrentUserCE);

        if (complete.getStatus() != BBlockingComplete.kSuccess)
        {
            return null;
        }
        else
        {
            //Return the user
            return getCurrentUserCE.getUser();
        }
    }

    /**
     * This will get all the users and groups from the server
     */
    protected void doRefresh()
    {
        BGetUsersCE getUsersCE = null;
        BGetGroupsCE getGroupsCE = null;
        BGetUsersCC usersComplete = new BGetUsersCC();
        BGetGroupsCC groupsComplete = new BGetGroupsCC();

        //Just set it, don't need to check it
        this.isInitialised = true;

        //Create the event to get the users from the server
        getUsersCE = new BGetUsersCE( this, usersComplete );

        //Create the event to get the groups from the server
        getGroupsCE = new BGetGroupsCE( this, groupsComplete );

        //Send the events
        dispatchEventServer( getUsersCE );
        dispatchEventServer( getGroupsCE );

        //If we're not in the AWT thread, block
        if (Thread.currentThread().getName().indexOf("AWT") == -1)
        {
            usersComplete.waitForEvent();
            groupsComplete.waitForEvent();
        }
    }

    /**
     * This will register the help files associated
     * with the user manager to the help plugin
     */
	public void registerHelp ()
	{
		try
		{
			BHelpTreeModel tree = new BHelpTreeModel ( "User Admin" );
			DefaultMutableTreeNode users;
			DefaultMutableTreeNode groups;
			BHelpItem cu = new BHelpItem ( "Create User", "/au/edu/swin/synergy/plugin/userman/help/addUser.html" );
			BHelpItem eu = new BHelpItem ( "Edit User", "/au/edu/swin/synergy/plugin/userman/help/editUser.html" );
			BHelpItem ru = new BHelpItem ( "Remove User", "/au/edu/swin/synergy/plugin/userman/help/removeUser.html" );
			BHelpItem cp = new BHelpItem ( "Change Password", "/au/edu/swin/synergy/plugin/userman/help/changePassword.html" );
			BHelpItem sg = new BHelpItem ( "Select Groups", "/au/edu/swin/synergy/plugin/userman/help/group.html" );
			BHelpItem cg = new BHelpItem ( "Create Group", "/au/edu/swin/synergy/plugin/userman/help/addGroup.html" );
			BHelpItem eg = new BHelpItem ( "Edit Group", "/au/edu/swin/synergy/plugin/userman/help/editGroup.html" );
			BHelpItem rg = new BHelpItem ( "Remove Group", "/au/edu/swin/synergy/plugin/userman/help/removeGroup.html" );
			BHelpItem priv = new BHelpItem ( "Privileges", "/au/edu/swin/synergy/plugin/userman/help/privilege.html" );
			BHelpItem ua = new BHelpItem ( "User Admin", "/au/edu/swin/synergy/plugin/userman/help/userAdmin.html" );

			tree.addHelpItem ( null, ua );
			tree.addHelpItem ( null, priv );
			users = tree.addBranch ( null, "Users" );
			tree.addHelpItem ( users, cu );
			tree.addHelpItem ( users, eu );
			tree.addHelpItem ( users, ru );
			tree.addHelpItem ( users, cp );
			tree.addHelpItem ( users, sg );

			groups = tree.addBranch ( null, "Groups" );
			tree.addHelpItem ( groups, cg );
			tree.addHelpItem ( groups, eg );
			tree.addHelpItem ( groups, rg );

			BRegisterHelpCE event = new BRegisterHelpCE ( this, tree );
			dispatchEvent ( event );
		}
		catch ( Exception e )
		{
			e.printStackTrace();
		}
	}

    /**
     * When the user selects the user manager option from the menu
     */
    class UserManagerMenuListener implements java.awt.event.ActionListener
    {
        /**
         * When the user selects the User Manager menu item, this
         * method is called. It will display the user manager and populate
         * it with the list of users and groups on the server.
         *
         * @param   e   The event fired
         */
        public void actionPerformed( ActionEvent e )
        {
            BAddWindowCE addDialog = null;

            //Always start with a new user manager
            BUserMan.this.userManager = new BCRUDUserManager( BUserMan.this );

            //Get the user and group list
            BUserMan.this.doRefresh();

            //Create the add dialog event
            addDialog = new BAddWindowCE( this, BUserMan.this.userManager, BUserMan.this.windowTitle);
			addDialog.addWindowListener(new WindowListener());
            BUserMan.this.menuItem.setEnabled(false);

            dispatchEvent( addDialog );
        }

    }

	/**
	 * Listens for the crud user manager window closing. When it's closed
	 * it will be set to null.
	 */
	public class WindowListener extends BWindowAdapter
	{	
		/**
		 * Called when the CRUDUserManager is closed. This will set
		 * the variable to null so that we don't refer to it again.
		 *
		 * @param	windowEvent	The event fired when the window is closed
		 */
		public void windowClosed(BWindowEvent windowEvent)
		{
			BUserMan.this.userManager = null;
            BUserMan.this.menuItem.setEnabled(true);
		}
	}

    /**
     * An instance of this class is passed along with the BGetUserCE event
     * so that it's <code>doComplete</code> method may be called by the
     * target plugin when the list of available users has been obtained.
     * <p>
     * At this point the available users are displayed in the BUserManagerGUI dialog.
     *
     * @author John Newbigin
     */
    class BGetGroupsCC extends BBlockingComplete
    {
        /**
         * Call the super class of this event
         */
        public BGetGroupsCC()
        {
            super();
        }

        /**
         * When the users have been retrieved this method is called
         *
         * @param   event   The event that was originally fired
         * @param   status  The success status of the operation
         */
        public void doComplete( BCommandEvent event, int status )
        {
            BGetGroupsCE getGroupsCE = ( BGetGroupsCE ) event;
            Vector groups = getGroupsCE.getGroups();

            if (groups != null)
            {
                //Make sure there is a user manager to set the groups on
                if (BUserMan.this.userManager != null)
                {
                    BUserMan.this.userManager.setGroups( groups );
                }

                //Make sure we synchronize on the available groups
                synchronized(BUserMan.this.availableGroups)
                {
                    //Update the existing groups
                    BUserMan.this.availableGroups.removeAllElements();
                    BUserMan.this.availableGroups.addAll(groups);
                }

                //Make sure the command line has the latest data
                BUserMan.this.userCommandLine.setAvailableGroups(groups);
                BUserMan.this.groupCommandLine.setAvailableGroups(groups);
            }

            //Make sure the blocking object notifies its peers
            super.doComplete(event, status);
        }
    }

    /**
     * An instance of this class is passed along with the BGetUserCE event
     * so that it's <code>doComplete</code> method may be called by the
     * target plugin when the list of available users has been obtained.
     * <p>
     * At this point the available users are displayed in the BUserManagerGUI dialog.
     *
     * @author John Newbigin
     */
    class BGetUsersCC extends BBlockingComplete
    {
        /**
         * Call the super class of this event
         */
        public BGetUsersCC()
        {
            super();
        }

        /**
         * When the users have been retrieved this method is called
         *
         * @param   event   The event that was originally fired
         * @param   status  The success status of the operation
         */
        public void doComplete( BCommandEvent event, int status )
        {
            BGetUsersCE e = ( BGetUsersCE ) event;
            Vector users = e.getUsers();

            if (users != null)
            {
                for( int i = 0; i < users.size(); i++ )
                {
                    Debug.doPrintln( users.elementAt( i ).toString() );
                }

                //Make sure there is a user manager to set the groups on
                if (BUserMan.this.userManager != null)
                {
                    BUserMan.this.userManager.setUsers( users );
                }

                //Set the users in the command line
                BUserMan.this.userCommandLine.setAvailableUsers(users);
                BUserMan.this.groupCommandLine.setAvailableUsers(users);
            }

            //Make sure the blocking object notifies its peers
            super.doComplete(event, status);
        }
    }
}








