//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/workspace/BNewModelWindow.java,v 1.9 1999/10/17 17:14:22 tp082177 Exp $

package au.edu.swin.synergy.plugin.workspace;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

import au.edu.swin.synergy.plugin.mod13.event.BCloseWindowCE;
import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.lib.io.BFileFilter;
import au.edu.swin.synergy.lib.BGridBagConstraints;
import au.edu.swin.synergy.plugin.modeller.BModel;
import au.edu.swin.synergy.plugin.dialect.BString;
import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.plugin.mod13.event.BGetFilesCE;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;
import au.edu.swin.synergy.plugin.packager.event.BOpenPackageCE;
	
/**
 * This class provides a GUI for users to add new models to the 
 * current project in the workspace.
 *
 * @author Matthew Evans
 */
class BNewModelWindow extends JPanel implements ActionListener
{
		JLabel nameLabel = new JLabel();
		JLabel packageLabel = new JLabel();
		JTextField modelNameField = new JTextField();
		JTextField packageNameField = new JTextField();
		JButton browseButton = new JButton();
		JButton okButton = new JButton();
		JButton cancelButton = new JButton();

		BWorkspace parent;
		BWorkspacePanel workspaceGUI;
		
		/**
		 * constructing the GUI for new models
		 *
		 * @param parent The workspace object the new model will be added to. 
		 */
		public BNewModelWindow(BWorkspace parent, BWorkspacePanel workspaceGUI)
		{
			this.parent = parent;
			this.workspaceGUI = workspaceGUI;
			this.setSize(400, 200);
			this.setLayout(new GridBagLayout());

			this.add(nameLabel, new BGridBagConstraints(0, 0, 1, 1, 0.0, 0.0
              ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
			
			this.add(modelNameField, new BGridBagConstraints(1, 0, 1, 1, 1.0, 1.0
              ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
			
			this.add(packageLabel, new BGridBagConstraints(0, 1, 1, 1, 0.0, 0.0
              ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
			
			add(packageNameField,new BGridBagConstraints(1, 1, 1, 1, 1.0, 1.0
              ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
			add(browseButton, new BGridBagConstraints(2, 1, 1, 1, 0.0, 0.0
              ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

			add(okButton, new BGridBagConstraints(3, 2, 1, 1, 0.0, 0.0
              ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
			add(cancelButton, new BGridBagConstraints(3, 3, 1, 1, 0.0, 0.0
              ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

			browseButton.addActionListener(this);
			okButton.addActionListener(this);
			cancelButton.addActionListener(this);

         //Set the text
         this.doDialect();
		}

        /**
         * This will set the text of all the buttons and
         * labels on the screen in a manner suitable for the
         * current locale.
         */
        protected void doDialect()
        {
            nameLabel.setText(BDialect.getString(this, "name"));
            packageLabel.setText(BDialect.getString(this, "package"));
            browseButton.setText(BDialect.getString("browse"));
            okButton.setText(BDialect.getString("ok"));
            cancelButton.setText(BDialect.getString("cancel"));
        }

		public void actionPerformed(ActionEvent event)
		{
			Object source = event.getSource();

			if (source == okButton)
			{
				createModel();
			}
			else if (source == cancelButton)
			{
				closeWindow();
			}
			else if (source == browseButton)
			{
				getPackage();
			}

		}

		private void closeWindow()
		{
			BCloseWindowCE closeEvent = new BCloseWindowCE(parent, this);
			parent.dispatchEvent(closeEvent);
		}

		private void getPackage()
		{
			/* BFileFilter filter =new BFileFilter("not", BDialect.getString(this, "modelFileFilter")); 
			 BGetFilesCE getFile = new BGetFilesCE(".");
      	 
			 getFile.setTitle(BDialect.getString(this, "openworkspace"));
			 getFile.setFileFilter(filter);
			 getFile.setCompletionObject(new OnNewModelComplete());
			 parent.dispatchEvent(getFile);*/
			 BOpenPackageCE openEvent = new BOpenPackageCE(this, new OnNewModelComplete());
			 parent.dispatchEvent(openEvent);
		}

		private void createModel()
		{
		   BStandardOutCE errorMessage = null;
		   	
		  if (modelNameField.getText().trim().length() == 0 )
		  {
			 errorMessage = new BStandardOutCE(this, BDialect.getString(this, "nameError"));
		  }
		  else if (packageNameField.getText().trim().length() <= 0)
		  {
			 errorMessage = new BStandardOutCE(this, BDialect.getString(this, "noPackage"));
          }
		  else
		  {
			 workspaceGUI.doCreateNewModel(new File(modelNameField.getText()), packageNameField.getText());
			 closeWindow();
		  }

		  if (errorMessage != null)
		  {
				errorMessage.setUserResponseOption(BStandardOutCE.kDefaultOption);
				errorMessage.setMessageType(BStandardOutCE.kError);
				parent.dispatchEvent(errorMessage);
		  }
	   }

	class OnNewModelComplete extends BCommandComplete
	{
	   public void doComplete( BCommandEvent event, int status )
		{
			if(status == BCommandComplete.kSuccess)
			{
/*	   			BGetFilesCE getFile = (BGetFilesCE) event;
				
				if(getFile.getSelectedFile() != null)
				{
				   Debug.doPrintln("Loading file");
		 		   File packageFile = getFile.getSelectedFile(); 
				   packageNameField.setText(packageFile.getPath());
				}*/
				BOpenPackageCE openEvent = (BOpenPackageCE) event;
				if(openEvent.getFileName() != null)
				{
				   BSwingUtilities.setJText(packageNameField, openEvent.getFileName());
				}
			}
			else
			{
				Debug.doPrintln("Error opening file");
			}

		}
	}
		
}
