//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/workspace/BProject.java,v 1.32 1999/10/17 22:08:00 tp082177 Exp $

package au.edu.swin.synergy.plugin.workspace;

import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.lib.io.BDataFile;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.tree.TreePath;
import au.edu.swin.synergy.lib.BTreeNode;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.plugin.packager.BPackageModel;
import au.edu.swin.synergy.plugin.modeller.BModel;
import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;
import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.plugin.dialect.BDialect;

import java.io.File;
import java.util.Vector;

/**
 * Represents a project of models.  Can be saved, and is used by the BWorkspace
 * to create BWorkspaceTreeBranch objects
 *
 * @author Matthew Evans
 */
public class BProject implements BTreeNode, BSaveable
{

   	// for BSaveable
	protected static final String kKeyName = "name";
	protected static final String kDefaultName = "untitled";

	protected static final String kKeyModel = "model_path";


  /**
   * The parent of this branch in the tree
   */
   private BWorkspaceTreeModel parent = null;

   /**
   * The name of the element being represented by the branch
   */
  private String name = null;

  private String path = null;

  /**
   * Collection of models contained in a project
   */
  private Vector models = null;


  /**
   * The file extension for projects files
   */
  //public static final String kFileExtension = "cpr";

  /**
   * Construct a new project with name set to 'name', and the
   * models Vector created
   *
   * @param name the name of the model
   */
  public BProject(BWorkspaceTreeModel parent, String name, String path)
  {
     this.parent = parent;
     this.name = name;
     this.path = path;
     this.models = new Vector();
  }

	/**
	 * Default constructor for BSaveable
	 */
  public BProject()
  {
  	this(null, null, null);
  }

  public void setParent(BWorkspaceTreeModel parent)
  {
  	this.parent = parent;
  }

  public void setName(String name)
  {
	 this.name = name;
  }
  	
  /**
   * Add a model to the project, so that it is accessible via this project
   * from the cohesion workspace
   *
   * @param model the model to be included in this project
   */
  public BModel addModel(File modelFile, String packageFileName)
  {
	boolean alreadyInProject= false;

	//check the file is a model file
	if (!modelFile.getPath().endsWith("." + BModel.kFileExtension))
	{
		String path = modelFile.getPath();
		modelFile = new File(path + "." + BModel.kFileExtension);
	}

    String fileName = modelFile.getName();
    String modelName = fileName.substring(0, fileName.length()-4);

    // check that a model with 'modelName' already exists
	for(int i = 0; i < models.size(); i++)
    {
        BModel searchModel = (BModel) models.elementAt(i);
          
        if (searchModel.getName().equals(modelName))
           alreadyInProject = true;
    }
     
    if (alreadyInProject)
    {
		return null;
    }
    else
    {
		BModel model = null;

		if (packageFileName != null)
		{
			BDataFile packageFile = new BDataFile( packageFileName );

			BPackageModel packageModel = BPackageModel.loadFromFile(packageFileName);
		
			//check to see if an error occured..
			if ( packageModel == null )
			{
				return null;
			}
			else
			{
				model = new BModel( this, modelName, modelFile.getPath(), packageModel  );
			}
		 }
		 else
		 {
			//check if it is an existing model, or one the user is creating
			if (modelFile.exists())
			{
			   Debug.doPrintln("model exists...loading1");
			   BDataFile modelData = new BDataFile(modelFile.getPath());
			   modelData.openFileForRead();
			   model = (BModel) modelData.autoLoad();
			   modelData.close();

			    if (model != null)
			    {
		   		   model.setParent(this);
			    }
				else
				{
					return null;
				}

			}
		 }	

		 if(model != null)
		 {
			this.models.addElement(model);

			// this line still causes Null Pointer Exceptions when loading models from disk
			Debug.doPrintln("parent = " + this.parent);
			Debug.doPrintln("models = " + this.models);
			
			if(parent != null)
			{
				this.parent.nodeWasInserted( this, this.models.size() - 1, model );
			}
		 }

	  return model;
	}
  }

  public void nodeWasInserted(BTreeNode model, int index, BTreeNode elementAdded)
  {
		this.parent.nodeWasInserted(model, index, elementAdded);
  }

  public void nodeWasRemoved(BTreeNode model, int index, BTreeNode elementRemoved)
  {
		this.parent.nodeWasRemoved(model, index, elementRemoved);
  }

  /**
   * Remove a specified model form the project
   *
   * @param model the model to remove
   */
  public BModel removeModel(BModel model)
  {
     for (int i = 0; i < models.size(); i++)
     {
        BModel aModel = (BModel) models.elementAt(i);

        if(aModel == model)
        {
		   int index = models.indexOf(model);	
           models.removeElementAt(i);
		   this.parent.nodeWasRemoved(this, i, aModel);
		 
		   return model;
        }
     }

	  return null;
  }

  public Vector getModels()
  {
     return this.models;
  }

  public String getPath()
  {
     return this.path;
  }

  public String getName()
  {
     return this.name;
  }


	public boolean isLeaf()
   {
      return this.models.size() <= 0;
   }

   public Object getChildAt( int index )
   {
      return this.models.elementAt( index );
   }

   public int getChildCount()
   {
      return this.models.size();
   }

   public Object getParent()
   {
      return this.parent;
   }

   public int getIndexOf( Object node )
   {
      return this.models.indexOf( node );
   }

   public void setParent( Object parent )
   {
      this.parent = (BWorkspaceTreeModel)parent;
   }

   public void removeAllChildren()
   {
      Debug.doPrintln("about to clear the children of " + this.name );
      this.models.clear();
   }

   public boolean load(BDataFile file)
   {
      Debug.doPrintln("BProject.load");
      this.name = file.getData(kKeyName, kDefaultName);
	  path = file.getFileName();
      Debug.doPrintln("Name is : " + name);

      while(true)
      {
         String modelPath = file.getData(kKeyModel, null);

         if(modelPath == null)
         {
            Debug.doPrintln("Load Finished", Debug.kWarningLevel);
            break;
         }
         else
         {
            Debug.doPrintln("Getting project: " + modelPath);
            File modelFile = new File(modelPath);
			if(modelFile.exists())
			{
	            this.addModel(modelFile, null);
			}
			else
			{
				Debug.doPrintln("model does not exist");
			}
         }
      }

      return true;
   }

   public boolean save(BDataFile file)
   {
      Debug.doPrintln("BProject.save");
      file.addData(kKeyName, this.name, kDefaultName);

      for(int i = 0; i < models.size(); i++)
      {
         //file.save((BSaveable)projects.elementAt(i));
         BModel model = (BModel) models.elementAt(i);
         String path = model.getPath();
         file.addData(kKeyModel, path);

		 Debug.doPrintln("Saving the model");

		 //save the model to its own file too
		 BDataFile modelData = new BDataFile(path);
		 modelData.openFileForWrite(false);
		 modelData.save(model);
		 modelData.close();
      }
      return true;
   }

    public void saveModifiedModels(BWorkspace workspacePlugin)
    {
        for (int i = 0; i < models.size(); i++)
        {
            BModel aModel = (BModel) models.elementAt(i);

            if (aModel.hasBeenModified())
            {
                BStandardOutCE saveRequest = new BStandardOutCE(this, aModel.getName() + "\n" + BDialect.getString(this, "saveFileRequest"), true, new BSaveChangesCC(i));
                saveRequest.setUserResponseOption(BStandardOutCE.kYesNoOption);
                saveRequest.setMessageType(BStandardOutCE.kQuestion);
                workspacePlugin.dispatchEvent(saveRequest);
            }
        }
    }

  public String toString()
  {
    return this.name;
  }


    /**
     * This completion class is defined to complete the necessary steps
     * after a model has been selected to save or not by the user.
     */
   class BSaveChangesCC extends BCommandComplete
   {
       /**
        * the position of a model in the models array in BProject
        */
        int modelNumber;

      /**
       * Creates the save changes completion object with a reference to 
       * the position of the model being saved in the project's collection of models
       *
       * @param modelNumber the position of a model in the projcet's model collection
       */
      public BSaveChangesCC(int modelNumber)
      {
         this.modelNumber = modelNumber;
      }

    /**
     * When the user has chosen whether to save the method or not, this method is called
     *
     * @param   event   The event that was originally fired
     * @param   status  The success status of the operation
     */
    public void doComplete( BCommandEvent event, int status )
    {
        Debug.doPrintln( "got list of users" );
        BStandardOutCE saveRequest = (BStandardOutCE) event;

        if (saveRequest.getUserResponse() == BStandardOutCE.kYes)
        {
            BModel model = (BModel) models.elementAt(modelNumber);
            
            String path = model.getPath();

            Debug.doPrintln("Saving the model");

		    //save the model to its own file too
		    BDataFile modelData = new BDataFile(path);
		    modelData.openFileForWrite(false);
		    modelData.save(model);
		    modelData.close();

            model.setModified(false);
        }
    }

   }
}
