//$

package au.edu.swin.synergy.plugin.workspace;

import java.util.Vector;

import au.edu.swin.synergy.lib.BTreeNode;
import au.edu.swin.synergy.debug.Debug;

/**
 * An expandable branch on a tree
 *
 * @author Matthew Evans
 */
public class BWorkspaceModelBranch implements BTreeNode
{
  /**
   * The parent of this branch in the tree
   */
	private BWorkspaceTreeModel parent = null;

	/**
   * The name of the element being represented by the branch
   */
  private String name = null;

  /**
   * The collection of items in this model branch
   */
  private Vector list = null;


  /**
   * Creates the branch with a reference to the parent, as well as giving it a name
   *
   * @param parent
   * @param name
   */
  public BWorkspaceModelBranch( BWorkspaceTreeModel parent, String name )
  {
    this.name = name;
    this.parent = parent;
	 this.list = new Vector();
  }

  /**
   * Add's a tree node to the list
   *
   * @param item the tree node to add to the branch
   */
	public void addNode( BTreeNode item )
	{
			this.list.add( item );
			Debug.doPrintln("Adding a new tree node!", Debug.kWarningLevel );
	}

  /**
   * Removes the specified node from the branch
   *
   * @param item the tree node to remove
   */
  public boolean removeNode (BTreeNode item)
  {
    return list.removeElement(item);
  }

	public boolean isLeaf()
	{
		return this.list.size() <= 0;
	}

	public Object getChildAt( int index )
	{
		return this.list.elementAt( index );
	}

	public int getChildCount()
	{
		return this.list.size();
	}

	public Object getParent()
	{
		return this.parent;
	}

	public int getIndexOf( Object node )
	{
		return this.list.indexOf( node );
	}

	public void setParent( Object parent )
	{
		this.parent = (BWorkspaceTreeModel)parent;
	}

	public void removeAllChildren()
	{
		Debug.doPrintln("about to clear the children of " + this.name );
		this.list.clear();
	}

	public String getName()
	{
		return this.name;
	}

	public void setName(String name)
	{
		this.name = name;
	}

    public String toString()
    {
       return this.name;
    }
}
