//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/workspace/propertybag/BPropertyBag.java,v 1.13 1999/10/17 15:36:10 tp080510 Exp $
package au.edu.swin.synergy.plugin.workspace.propertybag;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import java.util.Vector;

import au.edu.swin.synergy.plugin.dialect.BDialect;

/**
 * Displays a Delphi "Object Inspector" type property editor. Items are stored
 * in a hash table using the property name as a key. Items are stored as
 * BPropertyBagItem, which is an interface for retreiving and editing the
 * items in the editor. 
 */
public class BPropertyBag extends JPanel implements MouseListener, MouseMotionListener, ComponentListener
{
	/**
	 * Used for editing members of Vector-based items.
	 */
	protected class BListItemStorageContainer implements BPropertyBagItem
	{
		protected BPropertyBagItem parentItem;
		protected int index;

		public Object getPropertyValue (String name)
		{
			Vector vec = ((Vector)(parentItem.getPropertyValue(name)));
			return vec.elementAt(index);
		}

		public boolean isChangeAllowed (String name, Object oldValue, Object newValue)
		{
			Vector vec = ((Vector)(parentItem.getPropertyValue(name)));
			return parentItem.isChangeAllowed((name + ":" + index), vec.elementAt(index), newValue);
			
		}
	};

	/**
	 * Used to store all relevant information for each item in the BPropertyBag.
	 */
	protected class BPropertyBagContainer
	{
		/**
		 * The name of the item (same as key).
		 */
		protected String name;
		/**
		 * The BPropertyBagItem to retrive the value from (and check edits with).
		 */
		protected BPropertyBagItem item;
		/**
		 * The editor to use with this item.
		 */
		protected BPropertyBagItemEditor editor;
		/**
		 * The bounding rectangle of the item (relative to the BPropertyBag).
		 */
		protected Rectangle bounds;
		/**
		 * Controls whether or not this property is read only.
		 */
		protected boolean editable;
	};

	/**
	 * Stores BPropertyBagContainer objects for the BPropertyBag using the
	 * property name as the key.
	 */
	protected Hashtable data;
	/**
	 * Stores the keys for BPropertyBagContainer objects in some rational order.
	 */
	protected Vector keys; 
	/**
	 * Stores the height of each item in the BPropertyBag.
	 */
	protected int textHeight;
	/**
	 * The font used to draw the BPropertyBag.
	 */
	protected Font propertyFont = new Font("MS Sans Serif", Font.PLAIN, 11);
	/**
	 * Controls where the property name/value divider appears.
	 */
	protected int dividerLocation = 80; // arbitary ititial value
	/**
	 * Controls when to move the divider.
	 */
	protected boolean movingDivider = false;
	/**
	 * Stores the item which is focussed.
	 */
	protected int focusProperty = -1;
	/**
	 * The default editor for string items.
	 */
	protected BPropertyBagStringItemEditor stringEditor;
	/**
	 * The current editor being displayed.
	 */
	protected BPropertyBagItemEditor currentEditor;
	/**
	 * Stores the item in a vector whilst it is begin edited.
	 */
	protected BListItemStorageContainer itemContainer;
	
	/**
	 * Creates a new, empty property bag.
	 */
	public BPropertyBag ()
	{
		data = new Hashtable();
		keys = new Vector();
		addMouseListener(this);
		addMouseMotionListener(this);
		addComponentListener(this);

		stringEditor = new BPropertyBagStringItemEditor(this, propertyFont);
		currentEditor = null;
		itemContainer = new BListItemStorageContainer();

		stringEditor.registerKeyboardAction(new OnUpArrow(), KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0), JComponent.WHEN_FOCUSED);
		stringEditor.registerKeyboardAction(new OnDownArrow(), KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0), JComponent.WHEN_FOCUSED);
		stringEditor.registerKeyboardAction(new OnEscape(), KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_FOCUSED);
	}

	/**
	 * Moves up one item in the properties...
	 */
	class OnUpArrow extends AbstractAction
	{
		public void actionPerformed(ActionEvent e)
		{
		}
	}

	/**
	 * Moves down one item in the properties...
	 */
	class OnDownArrow extends AbstractAction
	{
		public void actionPerformed(ActionEvent e)
		{
			
		}
	}

	class OnEscape extends AbstractAction
	{
		public void actionPerformed(ActionEvent e)
		{
			if (currentEditor != null) // this should never be the case
			{
				currentEditor.cancel();
			}
		}
	}

	/**
	 * Removes all the properties (and associated editors) from the BPropertyBag.
	 */
	public void removeAllProperties ()
	{
		data.clear();
      keys.removeAllElements();
		focusProperty = -1;
		if (currentEditor != null && currentEditor.getItem() != null)
		{
			// try and save the current edit...
			currentEditor.getItem().isChangeAllowed (currentEditor.getName(), currentEditor.getItem().getPropertyValue(currentEditor.getName()), currentEditor.getValue());
			currentEditor.cancel();
			currentEditor = null;
		}
      repaint();
	}

	/**
	 * Adds a generic property to the BPropertyBag.
	 * @param name The displayed name of the property, this must be unique within
	 * the BPropertyBag object.
	 * @param item The BPropertyBagItem to retrieve the value from.
	 * @param editor The BPropertyBagEditor to run when the user edits the item.
	 * @return True if the operation was successful, false otherwise.
	 */
	public boolean addProperty (String name, BPropertyBagItem item, BPropertyBagItemEditor editor)
	{
		if (!data.containsKey(name))
		{
			BPropertyBagContainer newProperty = new BPropertyBagContainer();
			newProperty.name = name;
			newProperty.item = item;
			newProperty.editor = editor;
			newProperty.bounds = null;
			
			data.put(name, newProperty);
			keys.addElement(name);
			repaint(); // trigger repaint.

			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Adds a string property to the BPropertyBag. This uses the default string
	 * editor for editing.
	 * @param name The displayed name of the property, this must be unique within
	 * the BPropertyBag object.
	 * @param item The BPropertyBagItem to retrieve the value from.
	 * @return True if the operation was successful, false otherwise.
	 */
	public boolean addStringProperty (String name, BPropertyBagItem item)
	{
		if (!data.containsKey(name))
		{
			BPropertyBagContainer newProperty = new BPropertyBagContainer();
			newProperty.name = name;
			newProperty.item = item;
			newProperty.editor = stringEditor;
			newProperty.bounds = null;
			
			data.put(name, newProperty);
			keys.addElement(name);
			repaint(); // trigger repaint.

			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Removes the named property from the BPropertyBag.
	 * @param name The name of the item to be removed.
	 */
	public void removeProperty (String name)
	{
		data.remove(name);
		repaint(); // trigger repaint.
	}

	/**
	 * Retrieves the bounding rectangle of the value of a given property.
	 * @param name The name of the property.
	 * @return The bounding rectangle, or null if not found.
	 */
	public Rectangle getPropertyBounds (String name)
	{
		BPropertyBagContainer property = (BPropertyBagContainer)(data.get(name));
		if (property != null)
		{
			if (property.item.getPropertyValue(name) instanceof Vector)
			{
				return new Rectangle(property.bounds.x, property.bounds.y + (itemContainer.index * textHeight), property.bounds.width, textHeight);
			}
			else
			{
				return property.bounds;
			}
		}
		else
		{
			return null;
		}
	}

	/**
	 * Paints the BPropertyBag object.
	 */
	public void paintComponent (Graphics g)
	{
		try
		{
			g.setFont(propertyFont);
			textHeight = g.getFontMetrics().getHeight();
			int width = getWidth();
			int maxWidth = -1;

			Dimension oldDimensions = getPreferredSize();

			g.setColor(Color.lightGray);
			g.fillRect(0, 0, width, getHeight()); // fill entire (viewable) area
			int y = 0;
			for (int i = 0; i < keys.size(); i++)
			{
				String key = keys.elementAt(i).toString();
				BPropertyBagContainer property = (BPropertyBagContainer)(data.get(key));
				Object value = property.item.getPropertyValue(key);

				if (value instanceof Vector) // if is a list of values...
				{
					Vector vec = (Vector)value;

					int vecHeight = vec.size();
					if (vecHeight == 0)
					{
						vecHeight = 1;
					}

					g.setColor(Color.lightGray);
					g.draw3DRect(0, y, dividerLocation, (textHeight * vecHeight) - 1, !(i == focusProperty));
					g.setColor(Color.black);
					drawConstrainedString(g, key, 4, y + textHeight - 3, dividerLocation - 18);
					drawAddSymbol(g, dividerLocation - 12, y + 3);

					property.bounds = new Rectangle(dividerLocation + 2, y + 1, width - dividerLocation - 1, textHeight * vecHeight);

					for (int j = 0; j < vec.size(); j++)
					{
						g.setColor(Color.gray);
						g.drawRect(dividerLocation + 2, y, width - dividerLocation - 3, textHeight);
						g.setColor(Color.black);
						if (vec.elementAt(j) != null)
						{
							g.drawString(vec.elementAt(j).toString(), dividerLocation + 4, y + textHeight - 3);
							int w = g.getFontMetrics().stringWidth(vec.elementAt(j).toString());
							if ((w + dividerLocation + 4) > maxWidth)
							{
								maxWidth = w + dividerLocation + 4;
							}
						}
						else
						{
							g.drawString("", dividerLocation + 4, y + textHeight - 3);
						}
						y += textHeight;
					}
					if (vec.size() == 0) // no items, so we need to make the item higher
					{
						g.setColor(Color.black);
						g.drawString(BDialect.getString(this, "noitems"), dividerLocation + 4, y + textHeight - 3);
						y += textHeight;
					}
				}
				else
				{
					g.setColor(Color.lightGray);
					g.draw3DRect(0, y, dividerLocation, textHeight - 1, !(i == focusProperty));
					g.setColor(Color.gray);
					g.drawRect(dividerLocation + 2, y, width - dividerLocation - 3, textHeight);
					g.setColor(Color.black);
					drawConstrainedString(g, key, 4, y + textHeight - 3, dividerLocation - 4);
					if (value != null)
					{
						g.drawString(value.toString(), dividerLocation + 4, y + textHeight - 3);
						int w = g.getFontMetrics().stringWidth(value.toString());
						if ((w + dividerLocation + 4) > maxWidth)
						{
							maxWidth = w + dividerLocation + 4;
						}
					}
					else
					{
						g.drawString("", dividerLocation + 4, y + textHeight - 3);
					}
					property.bounds = new Rectangle(dividerLocation + 2, y + 1, width - dividerLocation - 1, textHeight);
					y += textHeight;
				}
			}
			if (currentEditor != null && currentEditor.getItem() != null)
			{
				currentEditor.setBounds(getPropertyBounds(currentEditor.getName()));
				currentEditor.repaint();
			}
			Dimension newDimensions = new Dimension(maxWidth, textHeight * getNumRows());
			if (!newDimensions.equals(getPreferredSize())) // need a repaint, cause dimensions have changed.
			{
				setPreferredSize(newDimensions);
				revalidate();
				repaint();
			}
		}
		catch(Exception E)
		{
		}
	}

	/** 
	 * Draws the symbol to indicate addition to a vector property.
	 * @param g The graphics context to draw to.
	 * @param x The X coodinate of the symbol's leftmost point.
	 * @param y The Y coodinate of the symbol's topmost point.
	 */
	protected void drawAddSymbol (Graphics g, int x, int y)
	{
		g.setColor(Color.lightGray);
		g.draw3DRect(x, y, 8, 8, true);
		g.setColor(Color.black);
		g.drawLine(x + 4, y + 1, x + 4, y + 7);
		g.drawLine(x + 1, y + 4, x + 7, y + 4);
	}

	/**
	 * This draws as much of a string as will fit for makWidth.
	 * @param g The grpahics context.
	 * @param text The text to draw.
	 * @param x The x location to draw at.
	 * @param y The y location to draw at.
	 * @param maxWidth The maximum width of the string.
	 */
	public void drawConstrainedString(Graphics g, String text, int x, int y, int maxWidth)
	{
		FontMetrics fm = g.getFontMetrics();
		String subString = text;
		String totalString = text;
		boolean worthPrinting = true;

		while (worthPrinting && fm.stringWidth(totalString) > maxWidth)
		{
			if (subString.length() > 1)
			{
				subString = subString.substring(0, subString.length() - 1).trim(); // shorten by one char
				totalString = subString + "...";
			}
			else
			{
				worthPrinting = false;
			}
		}
		if (worthPrinting)
		{
			g.drawString(totalString, x, y);
		}
	}


	public void mousePressed (MouseEvent e)
	{
		if (e.getX() >= dividerLocation - 2 && e.getX() <= dividerLocation + 2)
		{
			movingDivider = true;
		}
	}

	/*
	 * This method returns the number of rows that are in the property editor.
	 * @return The number of rows.
	 */
	public int getNumRows()
	{
		int rowCount = 0;
		for (int i = 0; i < keys.size(); i++)
		{
			String key = (String)(keys.elementAt(i));
			BPropertyBagContainer property = (BPropertyBagContainer)(data.get(key));
			Object obj = property.item.getPropertyValue(key);

			if (obj instanceof Vector)
			{
				rowCount += ((Vector)obj).size();
			}
			else
			{
				rowCount++;
			}
		}
		return rowCount;
	}
	
	public Point getItemIndex (int y)
	{
		int itemIndex = -1;
		int listIndex = -1;
		boolean found = false;

		while ((itemIndex + 1) < keys.size() && !found)
		{
			itemIndex++;
			String key = (String)(keys.elementAt(itemIndex));
			BPropertyBagContainer property = (BPropertyBagContainer)(data.get(key));
			Object obj = property.item.getPropertyValue(key);

			if (y < (property.bounds.height + property.bounds.y)) // this is it...
			{
				found = true;
				if (obj instanceof Vector)
				{
					Vector vec = ((Vector)(obj));
					if (vec.size() > 0)
					{
						listIndex = (y - property.bounds.y) / textHeight;
					}
				}
			}
		}
		if (!found)
		{
			itemIndex = -1;
		}
		return new Point(itemIndex, listIndex);
	}
	
	public void mouseReleased (MouseEvent e)
	{
		Point itemData = getItemIndex(e.getY());
		int itemIndex = itemData.x; // X is the list index...
		boolean canMoveItem = true;
		
		if (!movingDivider && itemIndex >= 0) 
		{
			String key = (String)(keys.elementAt(itemIndex));
			BPropertyBagContainer property = (BPropertyBagContainer)(data.get(key));
			Object obj = property.item.getPropertyValue(key);

			if (currentEditor != null && currentEditor.getItem() != null) // if we are editing
			{
				if ((!key.equals(currentEditor.getName()) || (e.getX() < dividerLocation)) //if we select another property
				|| (currentEditor.getItem() == itemContainer && itemContainer.index != itemData.y)) // or another item in a vector
				{
					if (currentEditor.getItem().isChangeAllowed (currentEditor.getName(), currentEditor.getItem().getPropertyValue(currentEditor.getName()), currentEditor.getValue()))
					{
						currentEditor.cancel(); // stop editing
						currentEditor = null;
					}
					else
					{
						canMoveItem = false;
					}
				}
			}
			if (canMoveItem)
			{
				focusProperty = itemIndex;
				if (e.getX() > dividerLocation)
				{
					if (property.editor != null)
					{
						if (obj instanceof Vector) // if this is a list element
						{
							Vector vec = (Vector)obj;
							if (vec.size() > 0)
							{
								itemContainer.parentItem = property.item;
								itemContainer.index = itemData.y;
								property.editor.getNewValue(key, itemContainer);
								currentEditor = property.editor;
							}
						}
						else
						{
							property.editor.getNewValue(key, property.item);
							currentEditor = property.editor;
						}
					}
				}
				else // clicking on item name... check if clicking on add symbol of Vector item.
				{
					if (obj instanceof Vector)
					{
						Vector vec = (Vector)obj;
						int xOff = e.getX();
						int yOff = e.getY() - property.bounds.y;

						if ((yOff > 3 && yOff < 11) && (xOff >= (dividerLocation - 12) && xOff < (dividerLocation - 4)))
						{
							property.item.isChangeAllowed((key + ':' + vec.size()), "", ""); // adding a new item...
						}
					}
				}
			}
			repaint(); // trigger repaint.
		}
		else
		{
			movingDivider = false;
		}
	}

	public void mouseDragged (MouseEvent e)
	{
		if (movingDivider)
		{
			if (e.getX() > 2 && e.getX() < getWidth() - 2)
			{
				dividerLocation = e.getX();
				repaint();
			}
		}
	}

	public void mouseMoved (MouseEvent e)
	{
		if (e.getX() >= dividerLocation - 2 && e.getX() <= dividerLocation + 2)
		{
			setCursor(Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
		}
		else
		{
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}

	public void componentShown (ComponentEvent e)
	{
		repaint();
	}
	
	public void componentResized (ComponentEvent e)
	{
		repaint();
	}

	public void mouseClicked (MouseEvent e) { }
	public void mouseEntered (MouseEvent e) { }
	public void mouseExited (MouseEvent e) { }
	public void componentHidden (ComponentEvent e) { }
	public void componentMoved (ComponentEvent e) { }
};
