//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/workspace/propertybag/BPropertyBagItem.java,v 1.2 1999/10/06 11:30:47 tp082177 Exp $
package au.edu.swin.synergy.plugin.workspace.propertybag;

/**
 * Provides the interface allowing items to be "editable" in a BPropertyBag
 * component. So many properties with different names can be stored inside one
 * class implementing BPropertyBagItem, the name of the specific property is
 * passed to all methods in this interface.
 */
public interface BPropertyBagItem
{
	/**
	 * This method is called by the BPropertyBag to obtain the value that is
	 * to be displayed as the property value for the named property.
	 * @param propertyName The name of the property for which the value
	 * is being obtained.
	 * @return The Object to be displayed in the BPropertyBag. Its toString()
	 * method is the displayed value of this property.
	 */
	public abstract Object getPropertyValue (String propertyName);

	/**
	 * After a property has been edited by a user in the BPropertyBag object,
	 * this method is called to check if the new value should be allowed.
	 * @param name The name of the property.
	 * @param oldValue The old value of the property.
	 * @param newValue The new value of the property (as returned from the
	 * appropriate BPropertyBagItemEditor calling this method).
	 * @return If true is returned, the edit is completed. If false is returned,
	 * then the editor should force the user rechoose a value.
	 */
	public abstract boolean isChangeAllowed (String name, Object oldValue, Object newValue);
};

