//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/workspace/propertybag/BPropertyBagItemEditor.java,v 1.4 1999/10/15 11:16:09 tp080510 Exp $
package au.edu.swin.synergy.plugin.workspace.propertybag;

import java.awt.Rectangle;

/**
 * This interface is used to obtain a new value for items is a BPropertyBag.
 * Default editors are provided for obatining strings and numeric values.
 */
interface BPropertyBagItemEditor
{
	/**
	 * This method is called when the user attempts to edit an item in
	 * a BPropertyBag. This method will generally show a dialog where the
	 * user can input/select a new value. This method should return immediately.
	 * Before the edit is completed, the editor should call item.isChangeAllowed()
	 * to see if the new value is  valid. After this method returns true
	 * (valid change), the edit completed, and any dialogs should be closed.
	 * Finally, the editor must call the editFinished() method of BPropertyBag
	 * to allow other items to be edited.
	 * @param name The name of the property to be edited.
	 * @param item The BPropertyBagItem interface associated with this name.
	 */
	public abstract void getNewValue (String name, BPropertyBagItem item);

	/**
	 * Cancels the edit. This method should hide the editor and set it's item and name values to empty values.
	 */
	public abstract void cancel ();


	/**
	 * If the property bag needs to know the current value, it calls this method.
	 *	@return The current value of the editor.
	 */
	 public abstract Object getValue();

	/**
	 * If the property bag needs to know the name of the current item, it calls this method.
	 *	@return The name of the current item in the editor.
	 */
	 public abstract String getName();

	/**
	 * If the editor needs to know the current item, it calls this method.
	 *	@return The current item in the editor.
	 */
	 public abstract BPropertyBagItem getItem();

	/**
	 * If the property bag needs to resize the editor, it calls this method.
	 * @param newBounds The new bounding rectangle of the editor.
	 */
	public abstract void setBounds (Rectangle newBounds);

	/**
	 * If the property bag needs to repaint the editor, it calls this method.
	 */
	public abstract void repaint ();
};
