//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/workspace/propertybag/BPropertyBagStringItemEditor.java,v 1.5 1999/10/17 07:39:49 tp082177 Exp $
package au.edu.swin.synergy.plugin.workspace.propertybag;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This class is used to obtain a new value for string items is a BPropertyBag.
 */
public class BPropertyBagStringItemEditor extends JTextField implements BPropertyBagItemEditor, ActionListener
{
	/**
	 * The name of the property being edited.
	 */
	protected String name = "";
	/**
	 * The item being edited.
	 */
	protected BPropertyBagItem item = null;
	/**
	 * The property bag that is the parent of this editor.
	 */
	protected BPropertyBag propBag = null;
	
	/**
	 * Create a new string editor.
	 * @param propBag The property bag that is the parent of this editor.
	 * @param displayFont The font to display the text in.
	 */
	public BPropertyBagStringItemEditor (BPropertyBag propBag, Font displayFont)
	{
		super();
		this.propBag = propBag;
		setFont(displayFont);
		setVisible(false);
		propBag.add(this);
		this.addActionListener(this);
	}

	/**
	 * Tells the editor to display for a given item.
	 * @param name The name of the property to be edited.
	 * @param item The BPropertyBagItem interface associated with this name.
	 */
	public void getNewValue (String name, BPropertyBagItem item)
	{
		this.name = name;
		this.item = item;
		setText(item.getPropertyValue(name).toString());
		setBounds(propBag.getPropertyBounds(name));
		setVisible(true);
		selectAll();
		repaint();
		requestFocus();
	}

	/**
	 * Cancels the edit, and hides the editor.
	 */
	public void cancel ()
	{
		setVisible(false);
		name = "";
		item = null;
	}

	/**
	 * Returns the current string.
	 * @return The current string in the editor.
	 */
	public Object getValue()
	{
		return getText();
	}

	public void actionPerformed (ActionEvent e)
	{
		if (item.isChangeAllowed(name, item.getPropertyValue(name), getText()))
		{
			setVisible(false);
			name = "";
			item = null;
			propBag.repaint();
		}
	}

	public BPropertyBagItem getItem()
	{
		return item;
	}

	public String getName()
	{
		return name;
	}
};
