/**
 * BEllipse - class for drawing an ellipse to the screen.
 * @author Imron Alston
 * @copyright team synergy 1999
 */
//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/shapes/BEllipse.java,v 1.9 1999/10/06 03:28:32 tp079555 Exp $

package au.edu.swin.synergy.shapes;

import java.awt.*;


import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.shapes.BRectShape;
import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.debug.Debug;
import java.io.*;



public class BEllipse extends BRectShape implements BSaveable
{
	public static final String kName = "Ellipse";

	public BEllipse( int x, int y, int w, int h )
	{
		super( x, y, w, h );
		this.name = BEllipse.kName;
		this.stroke = new BasicStroke( 1.0f );
	}

	public BEllipse()
	{
		this( 0, 0, 0, 0 );
	}

	protected BEllipse( BEllipse ellipse )
	{
		this();
		this.copy( ellipse );
	}

	private void readObject(java.io.ObjectInputStream in)
     throws IOException, ClassNotFoundException
	 {
		in.defaultReadObject();
		this.stroke = new BasicStroke( 1.0f );
	 }

	public void renderFill ( Graphics2D g2 )
	{
		//check to see if we have to set the colour of the graphics context..
		if ( this.useFillColour )
		{
			g2.setColor( this.fillColour );
		}
		//render the filled ellipse to the screen.
		g2.fillOval( this.xformed.x, this.xformed.y, this.xformed.width, this.xformed.height );
	}

    /**
	 * Renders the transformed coordinates of the drawable
	 * object to a Graphics2D
	 *
	 * @param Graphics2D g2 - the place to render the object to.
	 */
	public void renderShape ( Graphics2D g2 )
	{
		//check to see if we have to outline this shape...
		if ( this.doOutline )
		{
			//check to see if we have to set the colour of the graphics context..
			if ( this.useOutlineColour )
			{
				g2.setColor( this.outlineColour );
			}
			//render the stroked outline to the screen..
			g2.setStroke( this.stroke );
			g2.drawOval( this.xformed.x, this.xformed.y, this.xformed.width, this.xformed.height );
		}
	}

	/**
	 * Creates and returns a new instance of the BDrawable object
	 *
	 * @return a new instance of this BDrawable object.
	 */
	public BDrawable newInstance()
	{
		return new BEllipse( this );
	}

	/**
	 * Returns the name of this drawable object
	 * @return String - the name of the object..
	 */
	public String getName()
	{
		return BEllipse.kName;
	}

	/**
	 * Saves the ellipse to a datafile
	 *
	 * @param file the datafile to save the ellipse to
	 * @returns the success of the save
	 */
	 public boolean save( BDataFile file )
	 {
		Debug.doPrintln("BEllipse save");
		super.save(file);
		return true;
	 }

	 /**
	  * Loads the ellipse from a datafile
	  *
	  * @param file the datafile to load the ellipse from
	  * @returns the success fo the load
	  */
	  public boolean load( BDataFile file )
	  {
		Debug.doPrintln("BEllipse load");
		super.load(file);
		return true;
	  }

}
