//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/shapes/BLine.java,v 1.22 1999/10/13 08:05:21 tp079541 Exp $
/**
 * superclass for all primitive shapes that
 * are lines
 * @author Imron Alston
 * @copyright team synergy 1999
 */


package au.edu.swin.synergy.shapes;

import java.awt.*;
import java.awt.geom.*;

import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.plugin.diagram.BSizer;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.lib.io.BStrokeIO;

import au.edu.swin.synergy.plugin.diagram.BView;
import au.edu.swin.synergy.shapes.BPropPanel;
import au.edu.swin.synergy.shapes.BLineProperties;
import au.edu.swin.synergy.shapes.BLineEditorPanel;

import au.edu.swin.synergy.math.BScalarPoint;
import au.edu.swin.synergy.math.BScalarRect;
import au.edu.swin.synergy.math.BScalar;

public class BLine extends BDrawable implements BSaveable
{
	private static final String kDefaultName = "badname";
	private static final String kName = "Line";	
	private static final int kHitLen = 6;
	private static final int kHalfHitLen = kHitLen >> 1;

	private BScalarRect localCoords = null;

	protected boolean useLineColour = true;
	protected Color lineColour = Color.black;
	protected BasicStroke stroke = null;
	protected Integer style = BLineEditorPanel.kSingle;
	
	// Temp members used when rendering multiple strokes.
	private Rectangle tmpXformedCoords = null;
	private int xDirection;
	private int yDirection;
	private int dist;

	//don't save these vars...
	protected int lineSize;
	protected BScalarRect viewCoords = null;
	private Rectangle xformedCoords = null;
	private Rectangle hitRect = null;
	private BSizer startSizer = null;
	private BSizer endSizer = null;
	private int sizer = kNotHit;
	

	public BLine( int x1, int y1, int x2, int y2 )
	{
		super();


		this.localCoords = new BScalarRect( x1, y1, x2, y2 );

		this.viewCoords = new BScalarRect();
		this.xformedCoords = new Rectangle();
		this.tmpXformedCoords = new Rectangle ();


		this.stroke = new BasicStroke( 1.0f );
		this.name = kName;

		this.startSizer = new BSizer();
		this.endSizer = new BSizer();
		this.hitRect = new Rectangle();
	}
	
	public BLine( BLine line )
	{
		this();
		this.copy( line );
	}

	public BLine()
	{
		this( 0, 0, 0, 0 );
	}

	public Integer getLineStyle ()
	{
		return this.style;
	}

	public void setLineStyle ( Integer s )
	{
		this.style = s;
	}

    public Color getLineColour ()
    {
        return this.lineColour;
    }

    public void setLineColour ( Color colour )
    {
        lineColour = colour;
    }

    public BasicStroke getStroke ()
    {
        return this.stroke;
    }

    public void setStroke ( BasicStroke str )
    {
        stroke = str;
    }


    public BPropPanel getPropertyPanel ()
    {
        return new BLineProperties ( this );
    }

	/**
	 * Renders the transformed coordinates of the line
	 * to a Graphics2D
	 *
	 * @param Graphics2D g2 - the place to render the object to.
	 */
	public void render( Graphics2D g2 )
	{
		tmpXformedCoords.setBounds ( xformedCoords );
		lineSize = ( int ) Math.round ( stroke.getLineWidth () );
		dist = lineSize * 2;
		this.calcDirection ();

		// Before rendering line we have to calculate dist to move along and
		// direction.
		if ( useLineColour )
		{
			g2.setColor( this.lineColour );
		}
		g2.setStroke ( this.stroke );
		renderLine ( g2, style.intValue () );

		if ( this.isSelected() )
		{
			renderOutlineAndSizers( g2 );
		}
	}

	private void renderLine ( Graphics2D g2, int style )
	{
		if ( style == BLineEditorPanel.kSingle.intValue () )
		{
			g2.drawLine( xformedCoords.x, xformedCoords.y, xformedCoords.width, xformedCoords.height );
			xformedCoords.setBounds ( tmpXformedCoords );
		}
		else
		{
			// Might have a little troubles with this....see how we go!
			g2.drawLine( xformedCoords.x, xformedCoords.y, xformedCoords.width, xformedCoords.height );

			xformedCoords.x += dist * xDirection;
			xformedCoords.y += dist * yDirection;
			xformedCoords.width += dist * xDirection;
			xformedCoords.height += dist * yDirection;

			renderLine ( g2, style - 1 );
		}
	}

	/** 
	* Calculates the direction of the line... comes in handy when
	* rendering multiple line styles.
	*/
	private void calcDirection ()
	{
		if ( xformedCoords.x <= xformedCoords.width )
		{
			this.yDirection = BDrawable.kForward;
		}
		else
		{
			this.yDirection = BDrawable.kBackward;
		}

		if ( xformedCoords.y >= xformedCoords.height )
		{
			this.xDirection = BDrawable.kForward;
		}
		else
		{
			this.xDirection = BDrawable.kBackward;
		}
	}

	public void renderOutlineAndSizers( Graphics2D g2 )
	{
		g2.setColor( BSizer.kSizerColour );
		this.startSizer.render( g2 );
		this.endSizer.render( g2 );
	}


	/**
	 * Creates and returns a new instance of the BDrawable object
	 *
	 * @return a new instance of this BDrawable object.
	 */
	public BDrawable newInstance()
	{
		return new BLine( this );
	}

	
	public void startAdd()
	{
		this.localCoords.setBounds( 0, 0, 0, 0 );	
//		this.localCoords.setXY( this.pos );//Bounds( 0, 0, 0, 0 );
//		this.localCoords.setWH( this.pos );
	}


	/**
	 * called when the user is drag adding the line...
	 *
	 */
	public void dragAdd( BScalarPoint deltaXY )
	{
		this.localCoords.incWH( deltaXY );
	}


	/**
	 * Transforms the coordinates of the rectshape
	 * to a new coordinate space, as defined by the
	 * viewport and parent pos.
	 *
	 * @param BView viewport - the viewport that this object is going to be drawn in
	 * @param Point parentPos - coordinates of any parent object
	 */
	public void transform( BView viewport, BScalarPoint parentPos, int frameCounter )
	{
		BScalar zoom = viewport.getOOZ();


		if ( this.lastFrame != frameCounter )
		{
			this.viewCoords.setBounds( this.localCoords );			
			this.viewCoords.incBounds( this.pos );
			this.viewCoords.incBounds( parentPos );
			this.viewCoords.decBoundsByXY( viewport.getViewCoords() );
			this.viewCoords.scale( zoom );
			this.viewCoords.incBounds( viewport.getHalfWH() );
			this.viewCoords.getBounds( this.xformedCoords );

			this.startSizer.setPos( this.xformedCoords.x, this.xformedCoords.y );
			this.endSizer.setPos( this.xformedCoords.width, this.xformedCoords.height );
			this.lastFrame = frameCounter;
		}		

	}

	public void paintIcon( Component c, Graphics g, int x, int y )
	{
		this.xformedCoords.x = x - c.getWidth() / 4;
		this.xformedCoords.y = y - c.getHeight() / 4;
		this.xformedCoords.width = this.xformedCoords.x + (c.getWidth() / 2);
		this.xformedCoords.height = this.xformedCoords.y + (c.getHeight() / 2);

		render( (Graphics2D)g );
	}

	protected static final String kKeyUseLineColour = "uselinecol";
	protected static final String kDefaultUseLineColour = "true";

	protected static final String kKeyLineColour = "linecol";
	protected static final String kDefaultLineColour = "" + Color.black.getRGB();

	protected static final String kKeyStyle = "style";
	protected static final String kDefaultStyle = "1";



	/**
	 * Loads the BDrawable from a file
	 * @param BDataFile file - the file to load from
	 * @return boolean - true on success, false on failure.
	 */
	public boolean load(BDataFile file)
	{
		Debug.doPrintln("BLine.load");
		super.load( file );

		this.useLineColour = new Boolean( file.getData( kKeyUseLineColour, kDefaultUseLineColour) ).booleanValue();
		this.lineColour = new Color( new Integer( file.getData( kKeyLineColour, kDefaultLineColour ) ).intValue() );	
		this.style = Integer.valueOf ( file.getData ( kKeyStyle, kDefaultStyle ) );

		this.stroke = BStrokeIO.load( file );


		this.localCoords = (BScalarRect)file.autoLoad();
		
		return true;
	}

	/**
	 * Saves the BDrawable to a file
	 * @param BDataFile file - the file to save to
	 * @return boolean - true on success, false on failure.
	 */
	public boolean save(BDataFile file)
	{
		Debug.doPrintln("BLine.save");
		super.save(file);

		
		file.addData( kKeyUseLineColour, useLineColour ? "true" : "false" );
		file.addData( kKeyLineColour, Integer.toString( this.lineColour.getRGB() ) );
		file.addData( kKeyStyle, this.style.toString() );
		

		BStrokeIO.save( file, this.stroke );
		
		file.save( (BSaveable)localCoords );

		return true;
	}

	public void startResize( int sizerNum )
	{
		this.sizer = sizerNum;
	}

	public void dragResize( BScalarPoint deltaXY )
	{
		switch( this.sizer )
		{
			case kSizerStartArc:
				this.localCoords.incXY( deltaXY );
				break;
			case kSizerEndArc:
				this.localCoords.incWH( deltaXY );
				break;
		}
	}

	public void stopResize()
	{
		this.sizer = kNotHit;
		this.updateBounds();
	}

	public void stopAdd()
	{
		this.updateBounds();		
	}

	/**
	 * updates the bounding region of the drawable
	 * Called when something requires the bounds of
	 * the drawable to be updated.  Subclasses should
	 * determine a suitable way to update themselves.
	 */
	public void updateBounds()
	{
		this.bounds.setBoundsFromLine( this.localCoords );

		Debug.doPrintln("Updated the bounds of the line to " + this.bounds );
	}


	/**
	 * copies the data from the src BRectShape into this one.
	 * @param BRectShape src - the sourc BRectShape..
	 */
	public void copy( BLine src )
	{
		super.copy( src );
		this.localCoords.setBounds( src.localCoords );
		
		this.useLineColour = src.useLineColour;

		this.lineColour = src.lineColour;
		this.style = src.style;

		this.stroke = new BasicStroke( src.stroke.getLineWidth(),
									 src.stroke.getEndCap(),
									 src.stroke.getLineJoin(),
									 src.stroke.getMiterLimit(),
									 src.stroke.getDashArray(),
									 src.stroke.getDashPhase() );
	}


	/**
	 * Returns the name of this drawable object
	 * @return String - the name of the object..
	 */
	public String getName()
	{
		return BLine.kName;
	}


	/**
	 * Checks to see if a point has hit the transformed bounds
	 * of this object
	 * @param Point pt - the point to check the hit against..
	 * @return kNotHit - not hit
	 * @return kHitBounds - hit the main bounds
	 */
	public int checkXFormBounds( Point pt )
	{				
		this.hitRect.setBounds( pt.x - kHalfHitLen, pt.y - kHalfHitLen, kHitLen, kHitLen );
		

		if ( this.hitRect.intersectsLine( (double)this.xformedCoords.x, (double)this.xformedCoords.y, (double)this.xformedCoords.width, (double)this.xformedCoords.height ) )
		{		
			return kHitBounds;
		}
		else
		{
			return kNotHit;
		}		
	}


	/**
	 * Checks to see if a sizer has been hit by a point
	 * Works on xformed values.
	 * Subclasses should determine appropriate places for
	 * the sizer..
	 *
	 * @param Point pt - the point to check the hit against
	 * @return kNotHit - not hit
	 * @return kHitSizer + XX hit the XX sizer where XX is any number.
	 */
	public int checkSizerHit( Point pt )
	{
		if ( this.startSizer.contains( pt ) ) return kSizerStartArc;
		else if ( this.endSizer.contains( pt ) ) return kSizerEndArc;
		
		return kNotHit;
	}


}
