package au.edu.swin.synergy.shapes;

/**
*
* @author Christian Karolos
* @date 1/9/99
* @bugs If another instance created, listener of previous could still be registered.
*		So when changes are made, unwanted effects could apply to objects.
*/

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.TitledBorder;
import javax.swing.border.EtchedBorder;
import java.awt.*;
import java.awt.event.*;

import java.util.Vector;
import java.lang.Float;
import java.lang.Integer;
import java.lang.Math;

import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.plugin.packager.event.BIStrokeChangeListener;
import au.edu.swin.synergy.lib.BColorCombo;

import au.edu.swin.synergy.plugin.dialect.BDialect;

public class BLineEditorPanel extends JPanel
{
	public final static Integer kNone = new Integer ( 0 );
	public final static Integer kSingle = new Integer ( 1 );
	public final static Integer kDouble = new Integer ( 2 );
	public final static Integer kTripple = new Integer ( 3 );

	/**
	* Contains BIStrokeChange where event are fired on change of stroke.
	*/
	Vector strokeEvents;
	Vector colourEvents;
	Vector styleEvents;

	/**
	* Where I keep latest record of submitted data.
	*/
	BasicStroke myStroke;
	Color myColour;
	Integer myStyle;

	/**
	* Labels used for signifying uses of interactive elements.
	*/
	TitledBorder border;
	JLabel colour;
	JLabel dash;
	JLabel style;
	JLabel weight;

	/**
	* Combo box listing possible colours.
	*/
	BColorCombo colourBox;

	/**
	* Combo box listing possible preset dashed styles.
	*/
	JComboBox dashBox;

	/**
	* Combo box listing possible preset styles.
	*/
	JComboBox styleBox;

	/**
	* Where user may specify weight of line.
	*/
	JComboBox weightBox;

	/**
	* Arrays defined for preset dashed and line styles.
	* Static if anyone outside this class would like to use them.
	*/
	static final float[] dotteda = { 2.0f, 20.0f };
	static final float[] dasheda = { 10.0f };
	static final float[] ldasheda = { 20.0f };
	static final float[] lsa = { 20.0f, 20.0f, 10.0f, 20.0f };
	static final float[] lssa = { 25.0f, 20.0f, 10.0f, 20.0f, 10.0f, 20.0f };

	Integer[] lineStyles = { kNone, kSingle, kDouble, kTripple };

	/**
	* Preset dashed strokes.
	*/
	BasicStroke	straight = new BasicStroke ( 2 );
	BasicStroke	dotted = new BasicStroke ( 2, BasicStroke.CAP_BUTT, BasicStroke.CAP_BUTT, 1.0f, dotteda, 0.0f );
	BasicStroke	dashed = new BasicStroke ( 2, BasicStroke.CAP_BUTT, BasicStroke.CAP_BUTT, 1.0f, dasheda, 0.0f );
	BasicStroke	longDashed = new BasicStroke ( 2, BasicStroke.CAP_BUTT, BasicStroke.CAP_BUTT, 1.0f, ldasheda, 0.0f );
	BasicStroke	longShort = new BasicStroke ( 2, BasicStroke.CAP_BUTT, BasicStroke.CAP_BUTT, 1.0f, lsa, 0.0f );
	BasicStroke	longShortShort = new BasicStroke ( 2, BasicStroke.CAP_BUTT, BasicStroke.CAP_BUTT, 1.0f, lssa, 0.0f );

	/**
	* Array containing all preset strokes.
	*/
	BasicStroke[] strokes = { straight, dotted, dashed, longDashed, longShort, longShortShort };

	/**
	* Array containig all preset arc widths.
	*/
	Float[] arcWidths = { new Float ( 1.0 ), new Float ( 2.0 ), new Float ( 3.0 ), new Float ( 4.0 ), new Float ( 5.0 ), 
		new Float ( 6.0 ), new Float ( 7.0 ), new Float ( 8.0 ), new Float ( 9.0 ) }; 
	
	public BLineEditorPanel ( BasicStroke stroke, Color colour, Integer sStyle )
	{
		border = new TitledBorder ( new EtchedBorder ( EtchedBorder.LOWERED ), BDialect.getString ( this, "line" ) );
		this.colour = new JLabel ( BDialect.getString ( this, "colour" ) );
		dash = new JLabel ( BDialect.getString ( this, "dash" ) );
		style = new JLabel ( BDialect.getString ( this, "style" ) );
		weight = new JLabel ( BDialect.getString ( this, "weight" ) );

		setLayout ( new GridBagLayout () );
	
		strokeEvents = new Vector ();
		colourEvents = new Vector ();
		styleEvents = new Vector ();
		myStroke = stroke;
		myColour = colour;

		GridBagConstraints c = new GridBagConstraints ();
		
        setBorder ( border );

		setConst ( c, 0, 1, 1, 1, GridBagConstraints.EAST );
		add ( this.colour, c );
		colourBox = new BColorCombo ();
		colourBox.addItemListener ( new colourListener () );
		setConst ( c, 1, 1, 1, 1, GridBagConstraints.WEST );
		add ( colourBox, c );

		setConst ( c, 0, 2, 1, 1, GridBagConstraints.EAST );
		add ( dash, c );
		dashBox = new JComboBox ( strokes );
		dashBox.setRenderer ( new comboBoxRenderer () );
		dashBox.addItemListener ( new dashListener () ); 
		setConst ( c, 1, 2, 1, 1, GridBagConstraints.WEST );
		add ( dashBox, c );

		setConst ( c, 2, 1, 1, 1, GridBagConstraints.EAST );
		add ( style, c );
		styleBox = new JComboBox ( lineStyles );
		styleBox.addItemListener ( new styleListener () );
		styleBox.setRenderer ( new styleBoxRenderer () );
		setConst ( c, 3, 1, 1, 1, GridBagConstraints.WEST );
		add ( styleBox, c );

		setConst ( c, 2, 2, 1, 1, GridBagConstraints.EAST );
		add ( weight, c );
		weightBox = new JComboBox ( arcWidths );
		weightBox.addItemListener ( new weightListener () );
		setConst ( c, 3, 2, 1, 1, GridBagConstraints.WEST );
		add ( weightBox, c );

		initStrokePart ();
		initColourPart ();
		
		// Do this a better way for style.
		this.myStyle = sStyle;
		styleBox.setSelectedItem ( sStyle );
	}

	public void enable ( boolean e )
	{
		weightBox.setEnabled (e);
		styleBox.setEnabled (e);
		dashBox.setEnabled (e);
		colourBox.setEnabled (e);
	}

	/**
	* Custom made function for plotting labels and combo boxes on a panel.
	* Panel shouldn'?t be expandable, so concerning attribs are fixed.
	*/
    public static void setConst( GridBagConstraints gbc, int x, int y, int w, int h, int anc )
    {
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.gridwidth = w;
        gbc.gridheight = h;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = anc;
        gbc.insets = new Insets ( 10, 10, 10, 10 );

		// Only a light weight so things don't expand too much.
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbc.ipady = 10;
        gbc.ipadx = 10;
    }

	/**
	* Initialises all combo boxes and the like to correct values
	* contained by this panel.
	*
	* @pre BasicStroke and Color members of this class must be set.
	*/
	public void initStrokePart ()
	{
		BasicStroke stroke;
		int i = 0;
        int index = 0;
		boolean found = false;
		Float w = new Float ( myStroke.getLineWidth () );
		// Setting the weight field.
		weightBox.setSelectedItem ( w );

        if ( myStroke.getDashArray () != null )
        {
		    // Go through all possibles in dash list until we find matching.
		    for ( i = 0; !found && i < dashBox.getModel ().getSize (); i++ )
		    {
			    stroke = ( BasicStroke ) dashBox.getItemAt ( i );

                found = compArray ( myStroke.getDashArray (), stroke.getDashArray () );

			    if ( found )
			    {
                    Debug.doPrintln ( "Correct dash found..." );
				    index = i;
			    }
		    }
        }

        dashBox.setSelectedIndex ( index );

	}

    /**
    * Compares "cmp" to "const".  Returns true only if cmp == const!
    */
    public boolean compArray ( float[] cnst, float[] cmp )
    {
        boolean same;

        if ( ( cnst == null ) || ( cmp == null ) )
        {
            // If both are null, still equates to a valid stroke.
            same = ( cnst == null ) && ( cmp == null );    
        }
        else if ( ( same = cnst.length == cmp.length ) )
        {
            for ( int i = 0; i < cnst.length && same; i++ )
            {
                same = cnst[ i ] == cmp[ i ];
            }
        }

        return same;
    }

	public void initColourPart ()
	{
		colourBox.setSelectedItem ( myColour );
	}

	public void setLineStroke ( BasicStroke stroke )
	{
		myStroke = stroke;
		if ( stroke != null )
		{
			initStrokePart ();
		}
		else
		{
			Debug.doPrintln ( "stroke is null" );
		}
	}

	public void setColour ( Color colour )
	{
		myColour = colour;
		initColourPart ();
	}

	public void setStyle ( Integer sStyle )
	{
		this.myStyle = sStyle;
		styleBox.setSelectedItem ( sStyle );
	}

	public BasicStroke getLineStroke ()
	{
		return myStroke;
	}

	public Integer getStyle ()
	{
		return ( Integer ) styleBox.getSelectedItem ();
	}

	public Color getColour ()
	{
		return myColour;
	}

	public void addStrokeChangeListener ( BIStrokeChangeListener cl )
	{
		strokeEvents.add ( cl );		
	}

	public void addColourChangeListener ( BIStrokeChangeListener cl )
	{
		colourEvents.add ( cl );
	}

	public void addStyleChangeListener ( BIStrokeChangeListener cl )
	{
		styleEvents.add ( cl );
	}

	protected void fireColourChangeEvents ( Color colour )
	{
		BIStrokeChangeListener cl;

		for ( int i = 0; i < colourEvents.size (); i++ )
		{
			cl = ( BIStrokeChangeListener ) colourEvents.elementAt ( i );
			cl.update ( colour );	
		}
	}

	protected void fireStrokeChangeEvents ( BasicStroke stroke )
	{
		BIStrokeChangeListener cl;

		Debug.doPrintln ( "firing " + strokeEvents.size () + " events" );

		for ( int i = 0; i < strokeEvents.size (); i++ )
		{
			cl = ( BIStrokeChangeListener ) strokeEvents.elementAt ( i );
			cl.update ( stroke );
		}
	}

	protected void fireStyleChangeEvents ( Integer style )
	{
		BIStrokeChangeListener cl;

		for ( int i = 0; i < styleEvents.size (); i++ )
		{
			cl = ( BIStrokeChangeListener ) styleEvents.elementAt ( i );
			cl.update ( style );	
		}
	}


	public class styleBoxRenderer extends JPanel implements ListCellRenderer
	{
		BasicStroke stroke;
		FontMetrics fm;
		String noneStr;
		int strokeStyle;
		int height;
		int width;
		int lineSpacing;
		int xPos;
		int yPos;
		Dimension d;

		public styleBoxRenderer ()
		{
			setPreferredSize ( new Dimension ( 60, 20 ) );
			stroke = new BasicStroke ( 1.0f );
			noneStr = new String ( "-None-" );
			lineSpacing = 2;
		}

		public void paintComponent ( Graphics g )
		{
			super.paintComponent ( g );
			d = this.getPreferredSize ();

			width = ( int ) d.getWidth ();
			height = ( int ) d.getHeight ();

			Rectangle rect = this.getVisibleRect ();
			if ( strokeStyle == kNone.intValue () )
			{
				fm = g.getFontMetrics ( g.getFont () );
				xPos = ( width - fm.stringWidth ( noneStr ) ) / 2;
				yPos = fm.getHeight () - 2;

				Debug.doPrintln ( "x:"+xPos+"y"+yPos+" h:"+height+"w"+width );

				g.drawString ( noneStr, xPos, yPos );
/**/
			}
			else
			{
				xPos = ( ( int ) d.getHeight () - ( lineSpacing * strokeStyle ) ) / 2;
				int tmp = lineSpacing * strokeStyle;

				for ( int i = 0; i < strokeStyle; i++, xPos += lineSpacing )
				{
					g.drawLine ( 0, xPos, width, xPos );
				}
			}
		}

		public Component getListCellRendererComponent ( JList list, Object value, 
			int index, boolean isSelected, boolean cellHasFocus )
		{
			setBackground(isSelected ? list.getSelectionBackground() : Color.lightGray);

			strokeStyle = ( ( Integer ) value ).intValue ();

			return this;
		}
	}

	public class comboBoxRenderer extends JPanel implements ListCellRenderer
	{
		BasicStroke stroke = null;
		Dimension d;
		int panelH;
		int panelW;
		int length;
		int x = 0;
		int y = 0;

		public comboBoxRenderer ()
		{
			setPreferredSize ( new Dimension ( 60, 15 ) );
		}

		public void paintComponent ( Graphics g )
		{
			super.paintComponent ( g ); 
			Graphics2D g2 = ( Graphics2D ) g;
			if ( stroke != null )
			{
				g2.setColor ( Color.black );
				d = getPreferredSize ();
				panelH = ( int ) d.getHeight ();
				panelW = ( int ) d.getWidth ();

				g2.setStroke ( stroke );
				float lineWidth = stroke.getLineWidth ();
				// We want to draw line through middle of pannel.
				y = ( int ) ((( float ) panelH - lineWidth ) / 2.0f);
				length = x + panelW;
				g2.drawLine ( x, y, length, y );
			}
		}

		public Component getListCellRendererComponent ( JList list, Object value, 
			int index, boolean isSelected, boolean cellHasFocus )
		{
			if ( isSelected )
			{
				setBackground ( list.getSelectionBackground () );
				setForeground ( list.getSelectionForeground () );
			}
			else
			{
				setBackground ( list.getBackground () );
				setForeground ( list.getForeground () );
			}
			stroke = ( BasicStroke ) value;

			return this;
		}
	}

	public class colourListener implements ItemListener
	{
		public void itemStateChanged ( ItemEvent e )
		{
			if ( e.getStateChange () == ItemEvent.SELECTED )
			{
				myColour = ( Color ) e.getItem ();

				fireColourChangeEvents ( myColour );
			}
		}
	}

	public class dashListener implements ItemListener
	{
		public void itemStateChanged ( ItemEvent e )
		{
			BasicStroke newStroke;
			// get current selected item...
			if ( e.getStateChange () == ItemEvent.SELECTED )
			{
				BasicStroke bs = ( BasicStroke ) e.getItem ();

				// Set up the new stroke.
				myStroke = new BasicStroke ( myStroke.getLineWidth (), myStroke.getEndCap (), myStroke.getLineJoin (),
					myStroke.getMiterLimit (), bs.getDashArray (), bs.getDashPhase () );
	
				fireStrokeChangeEvents ( myStroke );
			}
		}
	}	

	private class weightListener implements ItemListener
	{
		public void itemStateChanged ( ItemEvent e )
		{
			if ( e.getStateChange () == ItemEvent.SELECTED )
			{
				Float weight = ( Float ) e.getItem ();

				// Update our version of stroke.
				myStroke = new BasicStroke ( weight.floatValue (), myStroke.getEndCap (), myStroke.getLineJoin (),
					myStroke.getMiterLimit (), myStroke.getDashArray (), myStroke.getDashPhase () );

				// tell everyone about it.
				fireStrokeChangeEvents ( myStroke );				
			}
		}
	}

	private class styleListener implements ItemListener
	{
		public void itemStateChanged ( ItemEvent e )
		{
			if ( e.getStateChange () == ItemEvent.SELECTED )
			{
				myStyle = ( Integer ) e.getItem ();
				Debug.doPrintln ( "myStyle = " + myStyle.intValue () );

				// If no dash has been selected, we should disable all other 
				// selectable items.
				if ( ( Integer ) e.getItem () == kNone )
				{
					weightBox.setEnabled ( false );
					dashBox.setEnabled ( false );
					colourBox.setEnabled ( false );
				}
				else
				{
					weightBox.setEnabled ( true );
					dashBox.setEnabled ( true );
					colourBox.setEnabled ( true );
				}
				fireStyleChangeEvents ( myStyle );
			}
		}
	}
}
