//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/shapes/BPolygon.java,v 1.8 1999/10/17 21:24:23 tp079568 Exp $
/**
 * are lines
 * @author Imron Alston
 * @copyright team synergy 1999
 */


package au.edu.swin.synergy.shapes;

import java.awt.*;
import java.awt.geom.*;

import java.util.Vector;

import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.plugin.diagram.BSizer;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.lib.io.BStrokeIO;
import au.edu.swin.synergy.lib.io.BVectorIO;
import au.edu.swin.synergy.lib.BColorCombo;

import au.edu.swin.synergy.plugin.diagram.BView;
import au.edu.swin.synergy.shapes.BPropPanel;
import au.edu.swin.synergy.shapes.BLineProperties;
import au.edu.swin.synergy.shapes.BLineEditorPanel;

import au.edu.swin.synergy.math.BScalarPoint;
import au.edu.swin.synergy.math.BScalarRect;
import au.edu.swin.synergy.math.BScalar;

public class BPolygon extends BDrawable implements BSaveable
{
	private static final String kDefaultName = "badname";
	private static final String kName = "Polygon";	
	private static final int kHitLen = 6;
	private static final int kHalfHitLen = kHitLen >> 1;	

	private static final int kDefArraySize = 10;
	private static final int kDefIncSize = 5;

	protected boolean doFill = true;
	protected boolean doOutline = true;
	protected boolean useFillColour = true;
	protected boolean useOutlineColour = true;

	protected Color fillColour = Color.white;
	protected Color outlineColour = Color.black;
    protected Color lineColour = Color.black;

	protected BasicStroke stroke = null;

	protected Vector points = null;
	
	
	//don't save these vars...

	protected BScalarPoint tmpView = null;
	protected int [] xVals = null;
	protected int [] yVals = null;
	private boolean adding = false;
	private Rectangle xformedBounds = null;
	private BScalarRect tmpViewBounds = null;
	private BSizer sizer = null;
	protected int sizerNum = kNotHit;
	
	

	public BPolygon( int x1, int y1, int x2, int y2 )
	{
		super();



		this.stroke = new BasicStroke( 1.0f );
		this.name = kName;

		points = new Vector();
		tmpView = new BScalarPoint();
		xVals = new int[kDefArraySize];
		yVals = new int[kDefArraySize];
		xformedBounds = new Rectangle();
		tmpViewBounds = new BScalarRect();
		sizer = new BSizer();
	}
	
	public BPolygon( BPolygon line )
	{
		this();
		this.copy( line );
	}

	public BPolygon()
	{
		this( 0, 0, 0, 0 );
	}

	public void setDoFill( boolean doFill )
	{
		this.doFill = doFill;
	}

    public void setFillColour ( Color colour )
    {		
        this.fillColour = colour;
		if ( colour == BColorCombo.kNoColour )
		{
			this.doFill = false;
		}
		else
		{
			this.doFill = true;
		}
    }

    public Color getFillColour ()
    {
        return this.fillColour;
    }

    public void setOutlineColour ( Color colour )
    {
        this.outlineColour = colour;
    }

	public void setLineColour( Color colour )
	{
		this.lineColour = colour;
	}

    public Color getLineColour ()
    {
        return this.lineColour;
    }

	public void setStroke( BasicStroke stroke )
	{
		this.stroke = stroke;
	}

	public void setStyle ( Integer style )
	{
//		this.shapeStyle = style;
//		this.doOutline = style.intValue () != BLineEditorPanel.kNone.intValue ();
	}

	public Integer getStyle ()
	{
		return new Integer(0);//this.shapeStyle;
	}

    public BasicStroke getStroke ()
    {
        return this.stroke;
    }

    public BPropPanel getPropertyPanel ()
    {        
        return new BPolygonProperties ( this );
    }

	/**
	 * Renders the transformed coordinates of the line
	 * to a Graphics2D
	 *
	 * @param Graphics2D g2 - the place to render the object to.
	 */
	public void render( Graphics2D g2 )
	{
		int size = points.size();
		

		if ( this.adding )
		{
			if ( this.doOutline )
			{
				//check to see if we have to set the colour of the graphics context..
				if ( this.useOutlineColour )
				{
					g2.setColor( this.outlineColour );
				}
				//render the stroked outline to the screen..
				g2.setStroke( this.stroke );
			
				g2.drawPolyline( xVals, yVals, size );
			}
		}
		else
		{
			if ( this.doFill )
			{
				this.renderFill ( g2 );
			}
			this.renderShape ( g2 );
		}
		

		if ( this.isSelected() )
		{
			renderOutlineAndSizers( g2 );
		}
	}

	protected void renderFill( Graphics2D g2 )
	{
		int size = points.size();
		//check to see if we have to set the colour of the graphics context..
		if ( this.useFillColour )
		{
			g2.setColor( this.fillColour );
		}
		//render the filled rect to the screen.
		g2.fillPolygon( xVals, yVals, size );
	}

	protected void renderShape( Graphics2D g2 )
	{
		int size = points.size();
		//check to see if we have to outline this shape...
		if ( this.doOutline )
		{
			//check to see if we have to set the colour of the graphics context..
			if ( this.useOutlineColour )
			{
				g2.setColor( this.outlineColour );
			}
			//render the stroked outline to the screen..
			g2.setStroke( this.stroke );
			g2.drawPolygon( xVals, yVals, size );
		}
	}


	public void renderOutlineAndSizers( Graphics2D g2 )
	{
		int i, size;
		
		size = this.points.size();
		g2.setColor( BSizer.kSizerColour );
		for ( i = 0; i < size; i++ )
		{
			this.sizer.setPos( xVals[i], yVals[i] );
			this.sizer.render( g2 );
		}
	}


	/**
	 * Creates and returns a new instance of the BDrawable object
	 *
	 * @return a new instance of this BDrawable object.
	 */
	public BDrawable newInstance()
	{
		return new BPolygon( this );
	}

	
	/**
	 * called when the drawable is about to be added..
	 *
	 */
	public void startAdd()
	{
		this.adding = true;
		this.points.add( new BScalarPoint( 0, 0  ));
	}

	
	/**
	 * called when the user is drag adding the line...
	 *
	 */
	public void dragAdd( BScalarPoint deltaXY )
	{

	}

	/**
	 *continue adding this drawable...
	 *
	 */
	public void continueAdd( BScalarPoint pt )
	{
		BScalarPoint tmp = new BScalarPoint( pt );

		tmp.decXY( this.pos );
		this.points.add( tmp );		
	}

	/**
	 * called when the drawable has finished being added..
	 *
	 */
	public void stopAdd()
	{		
		this.adding = false;
		this.updateBounds();
	}


	/**
	 * Transforms the coordinates of the rectshape
	 * to a new coordinate space, as defined by the
	 * viewport and parent pos.
	 *
	 * @param BView viewport - the viewport that this object is going to be drawn in
	 * @param Point parentPos - coordinates of any parent object
	 */
	public void transform( BView viewport, BScalarPoint parentPos, int frameCounter )
	{
		BScalar zoom = viewport.getOOZ();
		int i, size;
		int [] tmpX;
		int [] tmpY;

		if ( this.lastFrame != frameCounter )
		{
			size= points.size();

			//growthe arrays if needed
			if ( xVals.length < size )
			{
				tmpX = new int[size + kDefIncSize];
				tmpY = new int[size + kDefIncSize];

				xVals = null;
				yVals = null;

				xVals =tmpX;
				yVals = tmpY;
				
			}

			for( i=0; i<size; i++ )
			{
				tmpView.setLocation( (BScalarPoint)points.elementAt( i ) );
				tmpView.incXY( this.pos );
				tmpView.incXY( parentPos );
				tmpView.decXY( viewport.getViewCoords() );
				tmpView.scale( zoom );
				tmpView.incXY( viewport.getHalfWH() );

				xVals[i] = tmpView.getX();
				yVals[i] = tmpView.getY();


			}					

			//transform the bounds to viewspace...
			tmpViewBounds.setBounds( this.bounds );
			tmpViewBounds.incXY( this.pos );
			tmpViewBounds.incXY( parentPos );
			tmpViewBounds.decXY( viewport.getViewCoords() );

			tmpViewBounds.scale( zoom );

			tmpViewBounds.incXY( viewport.getHalfWH() );

			tmpViewBounds.getBounds( this.xformedBounds );

			this.lastFrame = frameCounter;
		}		

	}

	public void paintIcon( Component c, Graphics g, int x, int y )
	{
		int i;

		xVals[0] = 3; yVals[0] = 15;
		xVals[1] = 14; yVals[1] = 22;
		xVals[2] = 10; yVals[2] = 28;
		xVals[3] = 25; yVals[3] = 18;
		xVals[4] = 15; yVals[4] = 7;

		g.setColor( Color.white );
		g.fillPolygon( xVals, yVals, 5 );
		g.setColor( Color.black );
		g.drawPolygon( xVals, yVals, 5 );
		

	}

	protected static final String kKeyDoFill = "doFill";
	protected static final String kDefaultDoFill = "true";

	protected static final String kKeyUseFillColor = "useFillColour";
	protected static final String kDefaultUseFillColor = "true";

	protected static final String kKeyFillColor = "fillColour";
	protected static final String kDefaultFillColor = "" + Color.white.getRGB();

	protected static final String kKeyDoOutline = "doOutline";
	protected static final String kDefaultDoOutline = "true";

	protected static final String kKeyUseOutlineColor = "useOutlineColour";
	protected static final String kDefaultUseOutlineColor = "true";

	protected static final String kKeyOutlineColor = "outlineColour";
	protected static final String kDefaultOutlineColor = "" + Color.black.getRGB();

	protected static final String kKeyShapeStyle = "shapeStyle";
	protected static final String kDefaultShapeStyle = "1";

	protected static final String kKeyLineColor = "lineColour";
	protected static final String kDefaultLineColor = "" + Color.black.getRGB();



	/**
	 * Loads the BDrawable from a file
	 * @param BDataFile file - the file to load from
	 * @return boolean - true on success, false on failure.
	 */
	public boolean load(BDataFile file)
	{
		Debug.doPrintln("BPolygon.load");
		super.load( file );

		doFill = new Boolean( file.getData( kKeyDoFill, kDefaultDoFill) ).booleanValue();
		useFillColour = new Boolean( file.getData( kKeyUseFillColor, kDefaultUseFillColor ) ).booleanValue();
		fillColour = new Color( new Integer( file.getData( kKeyFillColor, kDefaultFillColor ) ).intValue() );

		doOutline = new Boolean( file.getData( kKeyDoOutline, kDefaultDoOutline ) ).booleanValue();
		useOutlineColour = new Boolean( file.getData( kKeyUseOutlineColor, kDefaultUseOutlineColor ) ).booleanValue();
		outlineColour = new Color( new Integer( file.getData( kKeyOutlineColor, kDefaultOutlineColor ) ).intValue() );
		lineColour = new Color( new Integer( file.getData( kKeyLineColor, kDefaultLineColor ) ).intValue() );


//		shapeStyle = Integer.valueOf ( file.getData ( kKeyShapeStyle, kDefaultShapeStyle ) );

		// load the stroke bits...
		stroke = BStrokeIO.load ( file );

		points = BVectorIO.load( file, "points");

		
		return true;
	}

	/**
	 * Saves the BDrawable to a file
	 * @param BDataFile file - the file to save to
	 * @return boolean - true on success, false on failure.
	 */
	public boolean save(BDataFile file)
	{
		Debug.doPrintln("BPolygon.save");
		super.save(file);


		file.addData( kKeyDoFill, new Boolean(doFill).toString(), kDefaultDoFill );
		file.addData( kKeyUseFillColor, new Boolean(useFillColour).toString(), kDefaultUseFillColor );
		file.addData( kKeyFillColor, Integer.toString( fillColour.getRGB() ), kDefaultFillColor );

		file.addData( kKeyDoOutline, new Boolean(doOutline).toString(), kDefaultDoOutline );
		file.addData( kKeyUseOutlineColor, new Boolean(useOutlineColour).toString(), kDefaultUseOutlineColor );
		file.addData( kKeyOutlineColor, Integer.toString( outlineColour.getRGB() ), kDefaultOutlineColor );

//		file.addData ( kKeyShapeStyle, shapeStyle.toString (), kDefaultShapeStyle );
		
		// the stroke...
		BStrokeIO.save ( file, stroke );		

		BVectorIO.save( file, points, "points");
		

		

		return true;
	}

	public void startResize( int sizerNum )
	{
		this.sizerNum = sizerNum - kHitSizer;
	}

	public void dragResize( BScalarPoint deltaXY )
	{
		((BScalarPoint)points.elementAt( sizerNum )).incXY( deltaXY );
	}

	public void stopResize()
	{
		this.sizerNum = kNotHit;
		this.updateBounds();
	}

	

	/**
	 * updates the bounding region of the drawable
	 * Called when something requires the bounds of
	 * the drawable to be updated.  Subclasses should
	 * determine a suitable way to update themselves.
	 */
	public void updateBounds()
	{
		this.bounds.setBoundsFromPoly( points );
	}


	/**
	 * copies the data from the src BRectShape into this one.
	 * @param BRectShape src - the sourc BRectShape..
	 */
	public void copy( BPolygon src )
	{
		int i, size;
		BScalarPoint cur = null;

		super.copy( src );

		size = src.points.size();
		for ( i = 0; i < size; i++ )
		{
			cur = (BScalarPoint)src.points.elementAt(i);
			this.points.add( new BScalarPoint( cur ) );
		}



		this.doFill = src.doFill;
		this.doOutline = src.doOutline;
		this.useFillColour = src.useFillColour;
		this.useOutlineColour = src.useOutlineColour;

		this.fillColour = src.fillColour;
		this.outlineColour = src.outlineColour;
        this.lineColour = src.lineColour;

		this.stroke = new BasicStroke( src.stroke.getLineWidth(),
									 src.stroke.getEndCap(),
									 src.stroke.getLineJoin(),
									 src.stroke.getMiterLimit(),
									 src.stroke.getDashArray(),
									 src.stroke.getDashPhase() );

	}


	/**
	 * Returns the name of this drawable object
	 * @return String - the name of the object..
	 */
	public String getName()
	{
		return BPolygon.kName;
	}


	/**
	 * Checks to see if a point has hit the transformed bounds
	 * of this object
	 * @param Point pt - the point to check the hit against..
	 * @return kNotHit - not hit
	 * @return kHitBounds - hit the main bounds
	 */
	public int checkXFormBounds( Point pt )
	{	
		int result = kNotHit;			
	
		if ( this.xformedBounds.contains( pt ) )
		{
			result = kHitBounds;
		}
		
		return result;		
	}


	/**
	 * Checks to see if a sizer has been hit by a point
	 * Works on xformed values.
	 * Subclasses should determine appropriate places for
	 * the sizer..
	 *
	 * @param Point pt - the point to check the hit against
	 * @return kNotHit - not hit
	 * @return kHitSizer + XX hit the XX sizer where XX is any number.
	 */
	public int checkSizerHit( Point pt )
	{
		int i, size;
		
		size = this.points.size();
		for ( i = 0; i < size; i++ )
		{
			this.sizer.setPos( xVals[i], yVals[i] );
			if ( this.sizer.contains( pt ) )
			{
				return kHitSizer + i;
			}
		}
		return kNotHit;
	}


}
