/**
 * BRectangle - class for drawing a rectangular
 * shape to the screen.
 * @author Imron Alston
 * @copyright team synergy 1999
 */
//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/shapes/BRectangle.java,v 1.12 1999/10/06 03:28:32 tp079555 Exp $

package au.edu.swin.synergy.shapes;

import java.awt.*;
import java.io.*;
import java.lang.Math;

import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.shapes.BRectShape;
import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.debug.Debug;


public class BRectangle extends BRectShape implements BSaveable
{
	public static final String kName = "Rectangle";
	public static final float kDefaultStrokeSize = 1.0f;

	public BRectangle( int x, int y, int w, int h )
	{
		super( x, y, w, h );
		this.name = BRectangle.kName;
		this.stroke = new BasicStroke( kDefaultStrokeSize );
	}

	public BRectangle()
	{
		this( 0, 0, 0, 0 );
	}

	protected BRectangle( BRectangle rect )
	{
		this();
		this.copy( rect );
	}

	private void readObject(java.io.ObjectInputStream in)
     throws IOException, ClassNotFoundException
	 {
		in.defaultReadObject();
		this.stroke = new BasicStroke( kDefaultStrokeSize );
	 }


	public void renderFill ( Graphics2D g2 )
	{
		//check to see if we have to set the colour of the graphics context..
		if ( this.useFillColour )
		{
			g2.setColor( this.fillColour );
		}
		//render the filled rect to the screen.
		g2.fillRect( this.xformed.x, this.xformed.y, this.xformed.width, this.xformed.height );
	}

    /**
	 * Renders the transformed coordinates of the drawable
	 * object to a Graphics2D
	 *
	 * @param Graphics2D g2 - the place to render the object to.
	 */
	public void renderShape ( Graphics2D g2 )
	{
		//check to see if we have to outline this shape...
		if ( this.doOutline )
		{
			//check to see if we have to set the colour of the graphics context..
			if ( this.useOutlineColour )
			{
				g2.setColor( this.outlineColour );
			}
			//render the stroked outline to the screen..
			g2.setStroke( this.stroke );
			g2.drawRect( this.xformed.x, this.xformed.y, this.xformed.width, this.xformed.height );
		}
	}

	/**
	 * Creates and returns a new instance of the BDrawable object
	 *
	 * @return a new instance of this BDrawable object.
	 */
	public BDrawable newInstance()
	{
		return new BRectangle( this );
	}

	/**
	 * loads the rectangle from a data file
	 *
	 * @param file the file to load the rectangle from
	 * @returns the success of the load
	 */
	 public boolean load( BDataFile file )
	 {
		Debug.doPrintln("BRectangle load");
		super.load(file);
		return true;
	 }

	 /**
	  * saves the rectangle to a data file
	  *
	  * @param file the file to save the rectangle to
	  * @returns the success of the save
	  */
	  public boolean save( BDataFile file )
	  {
		Debug.doPrintln("BRectangle save");
		super.save(file);
		return true;
	  }

}
