/**
 * BRoundRect - class for drawing a rounded rectangular
 * shape to the screen.
 * @author Imron Alston
 * @copyright team synergy 1999
 */
//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/shapes/BRoundRect.java,v 1.9 1999/10/06 03:28:32 tp079555 Exp $

package au.edu.swin.synergy.shapes;



import java.awt.*;
import java.awt.geom.*;
import java.io.*;


import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.shapes.BRectShape;
import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.debug.Debug;



public class BRoundRect extends BRectShape implements BSaveable
{
	public static final String kName = "Round Rectangle";

	private Point arcWH = null;

	public BRoundRect( int x, int y, int w, int h )
	{
		super( x, y, w, h );
		this.arcWH = new Point(5, 5 );
		this.name = BRoundRect.kName;
		this.stroke = new BasicStroke( 1.0f );
	}

	public BRoundRect()
	{
		this( 0, 0, 0, 0 );
	}

	protected BRoundRect( BRoundRect rect )
	{
		this();
		this.copy( rect );
		this.arcWH.x = rect.arcWH.x;
		this.arcWH.y = rect.arcWH.y;
	}

	private void readObject(java.io.ObjectInputStream in)
     throws IOException, ClassNotFoundException
	 {
		in.defaultReadObject();
		this.stroke = new BasicStroke( 1.0f );
	 }

	public void renderFill ( Graphics2D g2 )
	{
		//check to see if we have to set the colour of the graphics context..
		if ( this.useFillColour )
		{
			g2.setColor( this.fillColour );
		}
		//render the filled rect to the screen.
		g2.fillRoundRect( this.xformed.x, this.xformed.y, this.xformed.width, this.xformed.height, this.arcWH.x, this.arcWH.y );
	}

    /**
	 * Renders the transformed coordinates of the drawable
	 * object to a Graphics2D
	 *
	 * @param Graphics2D g2 - the place to render the object to.
	 */
	public void renderShape ( Graphics2D g2 )
	{
		//check to see if we have to outline this shape...
		if ( this.doOutline )
		{
			//check to see if we have to set the colour of the graphics context..
			if ( this.useOutlineColour )
			{
				g2.setColor( this.outlineColour );
			}
			//render the stroked outline to the screen..
			g2.setStroke( this.stroke );
			g2.drawRoundRect( this.xformed.x, this.xformed.y, this.xformed.width, this.xformed.height, this.arcWH.x, this.arcWH.y );
		}		
	}

	/**
	 * Creates and returns a new instance of the BDrawable object
	 *
	 * @return a new instance of this BDrawable object.
	 */
	public BDrawable newInstance()
	{
		return new BRoundRect( this );
	}

	/**
	 * Returns the name of this drawable object
	 * @return String - the name of the object..
	 */
	public String getName()
	{
		return BRoundRect.kName;
	}

	/**
	 * loads a round rect form a data file
	 *
	 * @param file the file to load the round rect from
	 * @returns the success of the load
	 */

	 protected static final String kKeyX = "arcWHx";
	 protected static final String kDefaultX = "5";
	 
	 protected static final String kKeyY = "arcWHy";
	 protected static final String kDefaultY = "5";
	 
	 public boolean load( BDataFile file )
	 {
		Debug.doPrintln("BRoundRect load");
		super.load(file);
		arcWH.x = new Integer( file.getData( kKeyX, kDefaultX ) ).intValue();
		arcWH.y = new Integer( file.getData( kKeyY, kDefaultY ) ).intValue();
		return true;
	 }

	 /**
	  * saves a round rect to a data file
	  *
	  * @param file the file to save the round rect to
	  * @returns the success of the save
	  */
	 public boolean save( BDataFile file )
	 {
		Debug.doPrintln("BRoundRect save");
		super.save(file);
		file.addData( kKeyX, Integer.toString(arcWH.x), kDefaultX );
		file.addData( kKeyY, Integer.toString(arcWH.y), kDefaultY );
		return true;
	 }


}
