package au.edu.swin.synergy.shapes;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import java.lang.Math;
import java.util.Vector;

import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.plugin.diagram.BView;
import au.edu.swin.synergy.plugin.diagram.BViewport;

import au.edu.swin.synergy.shapes.BRectShape;
import au.edu.swin.synergy.plugin.packager.BPackageArc;

import au.edu.swin.synergy.lib.BTreeNode;
import au.edu.swin.synergy.plugin.workspace.propertybag.BPropertyBagItem;

import au.edu.swin.synergy.math.BScalarPoint;
import au.edu.swin.synergy.math.BScalarRect;
import au.edu.swin.synergy.math.BScalar;

import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.debug.Debug;

public class BTextArea extends BRectShape implements BSaveable, BPropertyBagItem, BTreeNode
{
   public final static String kName = "Text Area";
	public final static int kTextSize = 2;

	protected boolean visible;
	private Object parent = null;
	protected volatile Vector strings;

    /**
    * Used to calc positisons of text within this area
    * depending on the font.
    */
    protected FontMetrics metric;
    protected Font textFont;
    protected Color textColour;
    protected int textSize;

    /**
    * Stuff used when rendering text area.
    */
    int x;
    int y;
    String currStr;
    Rectangle oldClip;
    Font oldFont;
    Color oldColour;
    int viewH;
    int viewW;
    float oozValue;	

    /**
    * 
    */
    Vector visibleStr;
	BViewport view = null;

	/**
	* Following attributes are only used when transforming and rendering
	* when this text area is associated with arcs...
	*/
	// % area is placed across from init arc pos.  mental note: must save / load...
	BPackageArc pArc = null;
	BScalarPoint dumbyParent;
	/**
	* this delta being the distance from the center pt of the arc.
	*/
	BScalarPoint deltaXY;
	BScalarPoint influence;

	public BTextArea ( int x, int y, int w, int h )
	{
		// I really want to put the default constructor in here
		// as well, but it wont let me.  Why???
		super ( x, y, w, h );		
		strings = new Vector ();
        textColour = Color.black;

        // Default font stuff.
        textSize = kTextSize;
        visibleStr = new Vector ();
        oldClip = new Rectangle ();
        metric = null;

		visible = true;
		dumbyParent = new BScalarPoint ();
		deltaXY = new BScalarPoint ();
		influence = new BScalarPoint ();

        stroke = new BasicStroke ( 1.0f );
        this.name = kName;
	}	

	public BTextArea ( String str )
	{
		this ( 0, 0, 20, 10 );
		this.name = str;
		this.setString ( str );

		// This is assuming this will be the only constructor
		// used for properties.
		this.visible = false;
	}

	public BTextArea ( BTextArea ta )
	{
		this ( 0, 0, 0, 0 );
		this.copy ( ta );
	}

	public BTextArea ()
	{
        this ( 0, 0, 0, 0 );
	}

	/**
	* Append a string onto the existing list of strings to
	* be displayed in text area.
	*
	* @param str String to be appended onto existing vector.
	*/
	public void addString ( String str )
	{		
		strings.add ( str );
	}

	/**
	* Used when vector may be retrieved from text box.
	*
	* @param strs Vector of strings for text area to display.
	*/
	public void setMultipleStrings ( Vector strs )
	{
		strings = strs;
	}

	/**
	* For display of one line text areas.
	*
	* @param str String to be displayed.
	*/
	public void setString ( String str )
	{
		// Do something about this later on.
		strings.insertElementAt ( str, 0 );
	}

	/**
	* Clears all strings held within this text area.
	*/
	public void clearArea ()
	{
		strings.clear ();
	}

	public Object getPropertyValue ( String property )
	{
		return strings;
	}
	
	public boolean isChangeAllowed (String name, Object oldValue, Object newValue)
	{
		int charIndex = name.lastIndexOf(':');
		String pureName = name.substring(0, charIndex);
		int index = Integer.parseInt(name.substring(charIndex + 1));

		if (index >= strings.size()) // adding a new item
		{
//			strings.addElement("New " + name + " " + strings.size());
			strings.addElement ( "" );
		}
		else if (newValue.toString().length() == 0) // need at least one item
		{
			strings.removeElementAt(index);
		}
		else
		{
			strings.setElementAt(newValue, index);
		}
		if ( this.view != null )
		{
			this.view.repaint ();
		}
		return true;
	}

    public FontMetrics getMetric ()
    {
        return metric;
    }

    public Font getFont ()
    {
        return textFont;
    }

	/**
	* Creates and returns a complete copy of our string vector.
	*/
	public Vector getStrings ()
	{
        Vector result = new Vector ();

        for ( int i = 0; i < strings.size (); i++ )
        {
            result.add ( new String ( ( String ) strings.elementAt ( i ) ) );
        }
		
		return result;
	}


	public Vector getRefToStrings()
	{
		return this.strings;
	}

	/**
	* Go through our list of strings return vector containing
	* those which are visible.
	*/
	public void calcVisibleStrs ( FontMetrics fm )
	{
        String tmpString;
        boolean moreText = true;
        int localH = ( int ) xformed.getHeight ();
        int acumm = 0;
        int height = fm.getHeight ();

        visibleStr.clear ();

		int size = strings.size ();
		for ( int i = 0; ( i < strings.size () ) && moreText; i++ )
		{
            // Do it this way so we fit half the string which just misses
            // the cut.
            if ( acumm < localH )
            {
                tmpString = ( String ) strings.elementAt ( i );
                visibleStr.add ( tmpString );				
            }
            else
            {
                moreText = false;
            }

            acumm += height;
		}
	}

    public BScalarPoint getDeltaXY ()
    {
        return this.deltaXY;
    }
	/**
	* Used to calculate distance for our accosicated arcs midPt.
	* @pre pos MUST be set.  Important to note when adding...
	*/
	public void calcDelta ( BScalarPoint startPt, BScalarPoint endPt )
	{
//<<<<<<< BTextArea.java
        influence.setLocation ( endPt );
        influence.decXY ( startPt );

        deltaXY.setLocation ( this.pos );
//        deltaXY.decXY ( startPt );

        deltaXY.div ( influence );
/*=======
        influence.setLocation ( endPt );
        influence.decXY ( startPt );
        Debug.doPrintln ( "arc length : " +influence);

        deltaXY.setLocation ( this.pos );
//        deltaXY.decXY ( startPt );
        Debug.doPrintln ( "delta : "+deltaXY );

        deltaXY.div ( influence );
        Debug.doPrintln ( "Scaled : " +deltaXY );

		// Distance from the center point of the arc.
        if ( pArc == null )
        {
            Debug.doPrintln ( "PARC IS NULL" );
            return;
        }
        else
        {
		    influence = pArc.calcDirection ();		
        }

        if ( pos == null )
        {
            Debug.doPrintln ( "OUR POS IS NULL" );
            return;
        }
        else
        {
		    deltaXY.setLocation ( pos );
        }
		deltaXY.decXY ( centerPt );
		deltaXY.scale ( influence );
>>>>>>> 1.37*/
	}

	public void setArc ( BPackageArc arc )
	{
		this.pArc = arc;
	}

	public void dragMove( BScalarPoint deltaXY )
    {
        super.dragMove ( deltaXY );
		if ( pArc != null )
		{
			this.calcDelta ( pArc.getPos (), pArc.getEndPos () );
            Debug.doPrintln ( "ARC start: "+pArc.getPos ()+" ARC end: "+pArc.getEndPos () );
		}
    }

	public void transform ( BView viewport, BScalarPoint parentPos, int frameCounter )
	{
        if ( lastFrame != frameCounter )
        {
		    // keep this for it sets up stuff for clipping bounds.
		    oozValue = viewport.getOOZ().getFloatValue ();
		    viewH = viewport.getHeight ();
		    viewW = viewport.getWidth ();

		    // Needed for when updating text on the fly...
		    if ( viewport instanceof BViewport )
		    {
			    this.view = ( BViewport ) viewport;
		    }


            super.transform ( viewport, parentPos, frameCounter );
        }
	}

	/**
	* A little specialised transform function which is only called when this
	* property belongs to an arc...
	* @param parentRect start pt, length and width of arc.
	*/
	public void arcTransform ( BView viewport, BScalarPoint centerPt, int frameCounter )
	{        
		// First when clicked, 

		this.transform ( viewport, centerPt, frameCounter );
	}

	public void renderFill ( Graphics2D g2 )
	{
		if ( visible )
		{
			if ( this.useFillColour )
			{
				g2.setColor( this.fillColour );
			}
			//render the filled rect to the screen.
			g2.fillRect( this.xformed.x, this.xformed.y, this.xformed.width, this.xformed.height );		
		}
	}

	/**
	 * Renders the transformed coordinates of the drawable
	 * object to a Graphics2D
	 * Left to subclasses to implement as needed
	 * Subclasses should check to make sure if they are supposed
	 * to fill or outline the shape, as well as whether or not
	 * they should use their own colours.
	 *
	 * @param Graphics2D g2 - the place to render the object to.
	 */
	public void renderShape ( Graphics2D g2 )
	{
		if ( visible )
		{
			// Draw outline of text area.
			if ( this.doOutline )
			{				
				g2.setStroke ( stroke );
				g2.setColor ( outlineColour );
				g2.drawRect ( xformed.x, xformed.y, xformed.width, xformed.height );
			}
			
			// Set up font and metric stuff...
			textFont = scaleFont ( oozValue );
			metric = g2.getFontMetrics ( textFont );

			// What we want this to do is draw and clip the text that we contain.
			g2.getClipBounds ( oldClip );
			setClippedRect ( g2 );


			// This could screw a couple things up...just remember to check
			// come back and correct.
			oldFont = g2.getFont ();
			oldColour = g2.getColor ();

			g2.setFont ( textFont );
			g2.setColor ( textColour );

			x = xformed.x + metric.charWidth ( 'I' ); 
			y = xformed.y + metric.getHeight ();


	//      Calc visible text.
			calcVisibleStrs ( metric );

	//      Render all visible text.
			for ( int i = 0; i < visibleStr.size (); i++ )
			{
				currStr = ( String ) visibleStr.elementAt ( i );
				g2.drawString ( currStr, x, y );
				y += metric.getHeight ();
			}

			g2.setClip ( oldClip );
			g2.setFont ( oldFont );
			g2.setColor ( oldColour );
		}
	}

	public void setVisible ( boolean show )
	{
		this.visible = show;
	}

	public boolean isVisible ()
	{
		return this.visible;
	}

	public void setLocation ( BScalarRect loc )
	{
		local.setBounds ( loc );
	}

	public BScalarRect getLocation ()
	{
		return local;
	}

    /**
    * Calculates the clipped area of rect against the viewport.
    *
    * Must credit java source Rectangle.insection () function for the
    * algorithm used.
    */
    protected void setClippedRect ( Graphics2D g2 )
    {
        // Assumes height and width of viewport have been set.
        int x = Math.max ( 0, xformed.x );
        int y = Math.max ( 0, xformed.y );

        int tw = Math.min ( viewW, xformed.x + xformed.width );
        int th = Math.min ( viewH, xformed.y + xformed.height );
		
        g2.setClip ( x, y, tw - x, th - y );
    }

    /**
    *
    */
    public Font scaleFont ( float oozVal )
    {
        float size = ( float ) textSize * oozVal;
        return new Font ( "system", Font.PLAIN, ( int ) size );
    }

	/**
	 * Creates and returns a new instance of the BDrawable object
	 *
	 * @return a new instance of this BDrawable object.
	 */
	public BDrawable newInstance()
	{
		return new BTextArea ( this );
	}

	public void paintIcon( Component c, Graphics g, int x, int y )
	{
        Graphics2D g2 = ( Graphics2D ) g;
        g2.setColor ( Color.black );
		g2.drawString ( "abc", 5, 19 );
	}

	public void copy ( BTextArea src )
	{
		super.copy ( src );
        
		this.parent = src.parent;
        this.strings = src.getStrings ();

		this.pArc = src.pArc;
        this.textFont = src.getFont ();
		this.visible = src.visible;

		if ( src.deltaXY != null )
		{
			this.deltaXY.setLocation ( src.deltaXY );
		}
		else
		{
			this.deltaXY.setLocation( 1, 1 );
		}

    }
	
	protected static final String kKeyText = "AreaString";
	protected static final String kKeyVisible = "Visible";

    public boolean load ( BDataFile file )
    {
        Debug.doPrintln ( "BTextArea load..." );
		String str;
        super.load ( file );
        strings = new Vector ();
		visible = ( Boolean.valueOf ( file.getData ( kKeyVisible, null ) ) ).booleanValue ();
		influence = ( BScalarPoint ) file.autoLoad ();
		deltaXY = ( BScalarPoint ) file.autoLoad ();
        while ( true )
        {
            str = file.getData ( kKeyText, null );
            if ( str == null )
            {
                break;
            }

            strings.add ( str );
        }

        return true;
    }

    public boolean save ( BDataFile file )
    {
        Debug.doPrintln ( "BTextArea save..." );
        super.save ( file );
		file.addData ( kKeyVisible, this.visible ? "true" : "false" );
		file.save ( ( BSaveable ) influence );
		file.save ( ( BSaveable ) deltaXY );
        for ( int i = 0; i < strings.size (); i++ )
        {
            file.addData ( kKeyText, ( String ) strings.elementAt ( i ) );
        }

        return true;
    }

///// Tree node stuff
	public boolean isLeaf()
	{
		return true;
	}

	public Object getChildAt( int index )
	{
		return null;
	}

	public int getChildCount()
	{
		return 0;
	}

	public Object getParent()
	{
		return this.parent;
	}

	public int getIndexOf( Object node )
	{
		return -1;
	}

	public void setParent( Object parent )
	{
		this.parent = parent;
	}

	public void removeAllChildren()
	{
	}

	public String getName()
	{
		return this.name;
	}
}
