//$Header: /teamproj/cvsroot/teamb/debugsrc/au/edu/swin/synergy/debug/Assertion.java,v 1.2 1999/09/05 03:20:13 tp079519 Exp $

package au.edu.swin.synergy.debug;

/**
 * This class allows the coders to put in code that simply
 * checks a boolean value or statement and logs the results
 * accordingly.
 *
 * @author Tristan Austin
 */
public class Assertion
{
   /**
    * The assertion failed message
    */
   private static final String kAssertionFailedMessage = "Assertion failed";

   /**
    * This method will check the given condition and, if
    * it fails, will generate a fatal error, log it in the log
    * file and exit the system.
    *
    * @param   assertion   The condition to check
    * @returns True if the assertion succeeded, false otherwise
    */
   /*deprecated*/ /*public static boolean doAssert(boolean assertion)
   {
      return Assertion.doAssert(assertion, kAssertionFailedMessage);
   }*/

   /**
    * This will accept a given condition and if it failed, log
    * the message in the log file as a fatal error and exit
    * the system.
    *
    * @param   assertion   The condition to check
    * @param   logMessage  The message to put in the log file
    * @returns True if the assertion succeeded, false otherwise
    */
   public static boolean doAssert(boolean assertion, String logMessage)
   {
   	
      //Check the assertion
      if (!assertion)
      {
	  	System.err.println(kAssertionFailedMessage + ": " + logMessage);
		Thread.currentThread().dumpStack();

         //Log the fatal error
         Log.doFatalError(logMessage);
      }

      //Return the result
      return assertion;
	  
	  
   }

   /**
    * This method will check the given condition and, if
    * it fails, will generate a warning, log it in the log
    * file and return the value of the assertion.
    *
    * @param   assertion   The condition to check
    * @returns True if the assertion succeeded, false otherwise
    */
   public static boolean doWarning(boolean assertion)
   {
      return Assertion.doWarning(assertion, kAssertionFailedMessage);
   }

   /**
    * This will accept a given condition and if it failed, log
    * the message in the log file as a warning.
    *
    * @param   assertion   The condition to check
    * @param   logMessage  The message to put in the log file
    * @returns True if the assertion succeeded, false otherwise
    */
   public static boolean doWarning(boolean assertion, String logMessage)
   {
      //Check the assertion
      if (!assertion)
      {
         //Log the fatal error
         Log.doWarning(logMessage);
      }

      //Return the result
      return assertion;
   }
}
