//$Header: /teamproj/cvsroot/teamb/nodebugsrc/au/edu/swin/synergy/debug/Debug.java,v 1.5 1999/09/05 13:14:57 tp079519 Exp $

   package au.edu.swin.synergy.debug;

   import java.text.SimpleDateFormat;
   import java.io.*;
   import java.util.*;

/**
 * The debug class provides an alternative to the System.out.println and print
 * methods. It provides a means for the Quality Assurance engineers to determine
 * problem areas in the code and focus their testing on that section.
 * <p>
 * <B>Debug.prp</B>
 * <p>
 * The debug properties are stored in a text file by the name of "Debug.prp"
 * which must be located in the same directory as the java VM was run in.
 * This file will provide a means of setting various parameters for the Debug class.
 * <p>
 * When the doPrintln or print method is called, there is an associated debug level
 * provided with it. By default, the debugLevel is 0. This level is used to determine
 * the importance of the statement. The default level is simply general output
 * by the developer and does not necessarily indicate a problem area.
 * <p>
 * The valid values for this include:<p>
 * 0: print level (general message)<p>
 * 1: warning level (problem area in code, requires attention)<p>
 * 2: error level (a genuine error has occurred in the code)<p>
 * 3: fatal error (a fatal error has occured and the system is exiting)<p>
 * <p>
 * There are a number of options that can be set for the Debug class in the Debug.prp
 * file. These include:
 * <p>
 * <b>timestampformat = dd.MM.yyyy hh.mm.ss</b>
 * <p>
 * This specifies the format of the time stamp. The above example indicates
 * that the time stamp will be in the form day/month/year hours/minutes/seconds. The following
 * is taken from the java.text.SimpleDateFormat java documentation:
 * <p>
 * To specify the time format use a time pattern string. In this pattern, all ASCII letters are reserved as pattern letters, which are defined as the following:
 *
 * <blockquote>
 * <pre>
 * Symbol   Meaning                 Presentation        Example
 * ------   -------                 ------------        -------
 * G        era designator          (Text)              AD
 * y        year                    (Number)            1996
 * M        month in year           (Text & Number)     July & 07
 * d        day in month            (Number)            10
 * h        hour in am/pm (1~12)    (Number)            12
 * H        hour in day (0~23)      (Number)            0
 * m        minute in hour          (Number)            30
 * s        second in minute        (Number)            55
 * S        millisecond             (Number)            978
 * E        day in week             (Text)              Tuesday
 * D        day in year             (Number)            189
 * F        day of week in month    (Number)            2 (2nd Wed in July)
 * w        week in year            (Number)            27
 * W        week in month           (Number)            2
 * a        am/pm marker            (Text)              PM
 * k        hour in day (1~24)      (Number)            24
 * K        hour in am/pm (0~11)    (Number)            0
 * z        time zone               (Text)              Pacific Standard Time
 * '        escape for text         (Delimiter)
 * ''       single quote            (Literal)           '
 * </pre>
 * </blockquote>
 * <p>
 *  <strong>Examples Using the US Locale:</strong>
 * <blockquote>
 * <pre>
 * Format Pattern                         Result
 * --------------                         -------
 * "yyyy.MM.dd G 'at' hh:mm:ss z"    ->>  1996.07.10 AD at 15:08:56 PDT
 * "EEE, MMM d, ''yy"                ->>  Wed, July 10, '96
 * "h:mm a"                          ->>  12:08 PM
 * "hh 'o''clock' a, zzzz"           ->>  12 o'clock PM, Pacific Daylight Time
 * "K:mm a, z"                       ->>  0:00 PM, PST
 * "yyyyy.MMMMM.dd GGG hh:mm aaa"    ->>  1996.July.10 AD 12:08 PM
 * </pre>
 * </blockquote>
 * <p>
 * <b>debuglevel = 0</b>
 * <p>
 * The debug level indicates what sort of errors to display. As mentioned above, the
 * debug level of a print statement can have a given value, by setting the debug level
 * in the properties file, you can specify what sort of errors you want to see. For example,
 * a debug level of 1 would show only warning level, error and fatal error messages, a debug level of 2
 * would show all error messages and fatal error messages.
 * NB. You can not turn off fatal error messages.
 *
 * @author Tristan Austin
 */
   public class Debug
   {
   /**
    * The debug level this is at
    */
      public static int debugLevel = 0;
   
   /**
    * The lowest debug level, least important
    */
      public static final int kPrintLevel = 0;
   
   /**
    * Moderately important debug levels
    */
      public static final int kWarningLevel = 1;
   
   /**
    * Highly important debug levels
    */
      public static final int kErrorLevel = 2;
   
   /**
    * Anything above this value will always be logged
    */
      public static final int kFatalLevel = 3;
   
   /**
    * The default date format
    */
      public static String dateFormat = "dd.MM.yyyy hh.mm.ss";
   
   /**
    * The format date class
    */
      public static SimpleDateFormat dateFormatter = new SimpleDateFormat(dateFormat);
   
   /**
    * The name of the file with the debug properties in it
    */
      public static final String kPropertiesFileName = "Debug.prp";
   
   /**
    * The constant name of the run level property
    */
      public static final String kRunLevel = "debuglevel";
   
   /**
    * The constant name of the time stamp format property
    */
      public static final String kTimeStampFormat = "timestampformat";
   
      /**
       * Set the debug output stream
       *
       * @param   fOut The PrintStream to write to 
       */
         public static final void setOutputStream(PrintStream fOut)
		 {
		 }
   /**
    * This writes the given message to standard out and provides the
    * runLevel in which the debug message should be presented.
    *
    * @param   message  The message to log and print
    * @param   runLevel The debug level to log the message in
    */
      public static void doPrintln(String message, int runLevel)
      {
      }

   /**
    * Performs the same functionality as the System.err.println() method
    *
    * @param   message  The message to output
    */
      public static void doPrintln(String message)
      {
      }
   
   /**
    * Performs the same functionality as the System.err.print() method
    *  
    * @param   message  The message to output
    */
      public static void doPrint(String message)
      {
      //Print the message with the lowest run level
      }

      /**
       * This writes the given message to standard out and provides the
       * runLevel in which the debug message should be presented.
       *
       * @param   message  The message to log and print
       * @param   runLevel The debug level to log the message in
       */
         public static void doPrint(String message, int runLevel)
         {
         }
   /**
    * This writes the given message to standard out and then does a stack
	* dump of the exception
    *
    * @param   message  The message to log and print
    * @param   exception The exception to stackTrace
    */
      public static final void doPrintln(String message, Exception exception)
      {
      /*      System.err.println(message);
			exception.printStackTrace(System.err);*/
      }

   /**
    * This writes the given message to standard out and provides the
    * runLevel in which the debug message should be presented. This
    * method is used by all other print methods in the class so that
    * it can do either print or println.
    *
    * @param   message  The message to log and print
    * @param   runLevel The debug level to log the message in
    * @param   newLine  Indicates whether to do a println, or print
    */
      private static void doPrint(String message, int runLevel, boolean newLine)
      {
      }

   /**
    * This method will initialise the Debug class with
    * the defaults, or the text file provided.
    */
      public static void doInitialisation()
      {
      }
   
   /**
    * This method will indicate whether the Debug class has been
    * intialised.
    *
    * @returns True if it is, false otherwise
    */
      public static boolean isInitialised()
      {
         return true;
      }
   }
